/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
Ext.namespace('org.ametys.odf.catalog');

org.ametys.odf.catalog.CatalogsTool = function(id, pluginName) 
{
    org.ametys.odf.catalog.CatalogsTool.superclass.constructor.call(this, id, pluginName);
};

Ext.extend(org.ametys.odf.catalog.CatalogsTool, org.ametys.uitool.tool.Tool, {});

org.ametys.odf.catalog.CatalogsTool.prototype._refreshing = false;
org.ametys.odf.catalog.CatalogsTool.prototype._gridMask = null;

org.ametys.odf.catalog.CatalogsTool.prototype.createPanel = function()
{
    // The mapping for XML doc
    var mapping = Ext.data.Record.create([
       {name: 'id', mapping: '@id'},    
       {name: 'title', mapping: 'title'},
       {name: 'code', mapping: 'code'},
       {name: 'nbPrograms', mapping: 'nbPrograms', type: 'int'}
    ]);
    
    // Data store
    this._xmlStore = new Ext.data.Store({
        // HTTP proxy
        proxy: new Ext.data.HttpProxy({
            url: getPluginDirectUrl(this.getPluginName()) + "/catalogs.xml",
            method: 'post',
            timeout: 0
        }),
        reader: new Ext.data.XmlReader(
            {
                record: 'item',
                totalProperty: "total",
                id: '@id'
            },
            mapping
        ),
        sortInfo: {field: 'title', direction: 'ASC'}
    });
    this._xmlStore.on('load', this._loadCallback, this);
    
    this._grid = new Ext.grid.GridPanel({
        region: 'center',
        
        store : this._xmlStore,
        
        columns: [
             {header: "<i18n:text i18n:key="PLUGINS_ODF_UITOOL_CATALOG_COLUMN_TITLE"/>", width: 250, sortable: true, dataIndex: 'title', renderer: this._renderTitle},
             {header: "<i18n:text i18n:key="PLUGINS_ODF_UITOOL_CATALOG_COLUMN_CODE"/>", width: 200, sortable: true, dataIndex: 'code'},
             {header: "<i18n:text i18n:key="PLUGINS_ODF_UITOOL_CATALOG_COLUMN_PROGRAMS"/>", width: 100, sortable: true, dataIndex: 'nbPrograms', type: 'numbercolumn'}
        ],
        
        // Single selection only
        sm: new Ext.grid.RowSelectionModel ({
        	singleSelect: true
        })
    });
    
    this._grid.getSelectionModel().on('selectionchange', this._onSelectionChange, this);
    
    this._mainPanel =  new Ext.Panel({
        layout: 'border',
        items : [this._grid]
    });
    
    this._mainPanel.addListener('focus', this._onFocus, this);
    this._mainPanel.addListener('close', this._onClose, this);
    
    return this._mainPanel;
}

org.ametys.odf.catalog.CatalogsTool.prototype._renderTitle = function(value, p, record)
{
	var title = '&lt;img src="' + getPluginResourcesUrl('odf') + '/img/catalog/catalog_16.png' + '" style="float:left; margin-right: 5px"/&gt;';
	title += value;
	
	return title;
}

/**
 * Fill the grid with the contents
 */
org.ametys.odf.catalog.CatalogsTool.prototype._refreshData = function()
{  
	if (this._gridMask)
	{
		this._gridMask.hide();
	}
	
    this._grid.getStore().load({
        callback: this._loadCallback, 
        scope: this
    });
}

org.ametys.odf.catalog.CatalogsTool.prototype._loadCallback = function (records, opt, success)
{
    if (!success)
    {
        Ext.Msg.alert("", "<i18n:text i18n:key="PLUGINS_ODF_UITOOL_CATALOG_TOOL_ERROR" i18n:catalogue="plugin.odf"/>");
        return;
    }
    
    if (records.length == 0)
    {
        // No catalogs
        if (this._gridMask != null)
        {
            this._gridMask.hide();
        }
        this._gridMask = new org.ametys.msg.Mask(this._grid.getEl(), "<i18n:text i18n:key='PLUGINS_ODF_UITOOL_CATALOG_TOOL_NO_RESULT' i18n:catalogue='plugin.odf'/>", true);
    }
}

org.ametys.odf.catalog.CatalogsTool.prototype.onMessage = function (messages)
{
    for (var i = messages.length - 1; i >= 0; i--)
    {
        if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_CREATED || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_METADATACHANGED)
        {
             var allTargets = messages[i].getTargets();
             var targets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, this._testTarget)
             
             if (targets.length &gt; 0)
             {
                 this.outOfDate();
             }
        }
        else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_DELETED)
        {
        	var allTargets = messages[i].getTargets();
            var targets = org.ametys.messagebus.message.MessageTargetHelper.findAll(allTargets, this._testTarget);
            
            for (var j=0; j &lt; targets.length;j++)
            {
                var id = allTargets[j].getParameters().id;
                var record = this._grid.getStore().getById(id);
                if (record != null)
                {
                    this._grid.getStore().remove(record);
                }
            } 
        }
    }
}

org.ametys.odf.catalog.CatalogsTool.prototype._testTarget = function(target)
{
    return target.getType() == 'catalog';
}

org.ametys.odf.catalog.CatalogsTool.prototype.setParams = function (params)
{
	this._params = params;
	this.refresh();
}

/**
 * Refresh the tool when it is out of date.
 * @params {boolean} manual The refresh is manual. (default is false)
 */
org.ametys.odf.catalog.CatalogsTool.prototype.refresh = function (manual)
{
	this.refreshing();
    this._refreshData();
    
    this.upToDate();
}


org.ametys.odf.catalog.CatalogsTool.prototype._sendSelection = function()
{
	var records = this._grid.getSelectionModel().getSelections();
	var targets = [];
    for (var i=0; i &lt; records.length; i++)
    {
        targets.push(org.ametys.messagebus.bus.MessageBuilder.getInstance().createTarget('catalog', {'id': records[i].data.id, 'title': records[i].data.title}));
    }
    
    var event = org.ametys.messagebus.bus.MessageBuilder.getInstance().createMessage(org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED, null, targets);
    org.ametys.messagebus.MessageBus.getInstance().fireMessages([event]);
}

org.ametys.odf.catalog.CatalogsTool.prototype._onSelectionChange = function(sm)
{
    this._sendSelection();
}

org.ametys.odf.catalog.CatalogsTool.prototype._onFocus = function()
{
    this._sendSelection();
}

/**
 * @private
 */
org.ametys.odf.catalog.CatalogsTool.prototype._onClose = function()
{
	var event = org.ametys.messagebus.bus.MessageBuilder.getInstance().createMessage(org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED, null, []);
    org.ametys.messagebus.MessageBus.getInstance().fireMessages([event]);
}
