/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.odf.content.DeleteButton');

/**
 * Create the control instance.
 * @param {String} id The unique identifier
 * @param {Map<id, value>} configuration The configuration of the control instance.
 * @param {String} pluginName The name of the declaring plugin
 * @class This class implements the behavior for an action.
 * @extends org.ametys.ribbon.control.RibbonControl
 */
org.ametys.odf.content.DeleteButton = function(id, configuration, pluginName)
{
	org.ametys.odf.content.DeleteButton.superclass.constructor.call(this, id, configuration, pluginName);
}

Ext.extend(org.ametys.odf.content.DeleteButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property _lastParams The params received on the last refresh
 */
org.ametys.odf.content.DeleteButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentContentsId The content id currently selected or null. 
 */
org.ametys.odf.content.DeleteButton.prototype._currentContentsId;

/**
 * @private
 * @property {String[]} _currentToDeleteContentsId The id to delete currently selected. 
 */
org.ametys.odf.content.DeleteButton.prototype._currentToDeleteContentsId;


/**
 * @private
 * @property {String[]} _currentReferencedContentsId The contents id currently selected and referenced. 
 */
org.ametys.odf.content.DeleteButton.prototype._currentReferencedContentsId;


org.ametys.odf.content.DeleteButton.prototype.createControl = function(colspan)
{
	var elmt = org.ametys.odf.content.DeleteButton.superclass.createControl.call(this, colspan);
	elmt.disable();
	return elmt;
}

/**
 * Creates the toggle button
 * @param {integer} colspan The configured colspan. Must be transmited to the control.
 * @return {Ext.form.field} The toggle button created. 
 */
org.ametys.odf.content.DeleteButton.prototype.onMessage = function(messages)
{
	// SELECTION
	for (var i=messages.length - 1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var allTargetsObject = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
			var allTargets = this._convertToId(allTargetsObject);
			
			// No content selected
			if (allTargets.length == 0)
			{
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["no-content"]);
				
				this._currentContentsId = [];
			}
			else
			{
				if (!this._areTheSame(this._currentContentsId, allTargets))
				{
					this._currentContentsId = allTargets;
					this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
					this.updateTooltip(this.getConfiguration()["default-description"]);
					org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: allTargets});
				}
			}
			
			return;
		}
	}
}


org.ametys.odf.content.DeleteButton.prototype._areTheSame = function (tab1, tab2)
{
	if (tab1 == tab2)
	{
		return true;
	}

	if (tab1 == null || tab2 == null || tab1.length != tab2.length)
	{
		return false;
	}	
	
	for (var i=0; i < tab1.length; i++)
	{
		if (tab2.indexOf(tab1[i]) == -1)
		{
			return false;
		}
	}
	
	return true;
}

org.ametys.odf.content.DeleteButton.prototype._isInSelection = function (tab, elmt)
{
	if (tab == null || tab.length == 0)
	{
		return false;
	}	
	
	for (var i=0; i < tab.length; i++)
	{
		if (tab[i] == elmt)
		{
			return true;
		}
	}
	
	return false;
	
}

org.ametys.odf.content.DeleteButton.prototype._convertToId = function (targets)
{
	var targetsIds = [];
	for (var i = 0; i < targets.length; i++)
	{
		targetsIds.push(targets[i].getParameters()['id']);
	}
	return targetsIds;
}

org.ametys.odf.content.DeleteButton.prototype.checkTarget = function ()
{
	return function (target) { 
		if(target.getType() == 'content')
		{
			if(target.getParameters().type == "org.ametys.plugins.odf.Content.program" 
				|| target.getParameters().type == "org.ametys.plugins.odf.Content.subProgram"
					|| target.getParameters().type == "org.ametys.plugins.odf.Content.container"
						|| target.getParameters().type == "org.ametys.plugins.odf.Content.course"
							|| target.getParameters().type == "org.ametys.plugins.odf.Content.orgunit"
								|| target.getParameters().type == "org.ametys.plugins.odf.Content.person")
				return true;
		}
		
		return false;
	};
}

org.ametys.odf.content.DeleteButton.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.odf.content.DeleteButton.prototype.configurationChanged = function(params)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });
	
	var tooltip = this._getTooltip("", params);
	
	var hasOneContent = tooltip.length > 0; 
	if (!hasOneContent)
	{
		tooltip += this.getConfiguration()['no-content'];
	}
	
	var disabledState = this._isDisabled(params);
	this.getElements().each(function(element) { element.setDisabled(disabledState); });
	this.updateTooltip(tooltip);
	
	this._lastParams = params;
}

org.ametys.odf.content.DeleteButton.prototype._isDisabled = function(params)
{
	var nbContents = parseInt(params['program-content-size']) + parseInt(params['container-content-size']) + parseInt(params['course-content-size'])+ parseInt(params['orgunit-content-size']) + parseInt(params['person-content-size']);
	if (nbContents == 0)
	{
		return true;
	}
	return false;
}

org.ametys.odf.content.DeleteButton.prototype._getTooltip = function(tooltip, params)
{
	this._currentToDeleteContentsId = [];
	this._currentReferencedContentsId = [];
	
	var count = parseInt(params['person-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["person-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["person-content-description-" + i];
			this._currentToDeleteContentsId.push(params["person-content-" + i]);
		}
		tooltip += this.getConfiguration()["person-content-end-description"];
	}	
	var count = parseInt(params['orgunit-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["orgunit-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["orgunit-content-description-" + i];
			this._currentToDeleteContentsId.push(params["orgunit-content-" + i]);
		}
		tooltip += this.getConfiguration()["orgunit-content-end-description"];
	}
	var count = parseInt(params['unauthorized-orgunit-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		tooltip += this.getConfiguration()["unauthorized-orgunit-description"];
	}	
	var count = parseInt(params['program-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["program-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["program-content-description-" + i];
			this._currentToDeleteContentsId.push(params["program-content-" + i]);
		}
		tooltip += this.getConfiguration()["program-content-end-description"];
	}
	
	var count = parseInt(params['container-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["container-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["container-content-description-" + i];
			this._currentToDeleteContentsId.push(params["container-content-" + i]);
		}
		tooltip += this.getConfiguration()["container-content-end-description"];
	}
	var count = parseInt(params['course-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["course-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["course-content-description-" + i];
			this._currentToDeleteContentsId.push(params["course-content-" + i]);
		}
		tooltip += this.getConfiguration()["course-content-end-description"];
	}	
	var count = parseInt(params['unauthorized-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["unauthorized-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["unauthorized-content-description-" + i];
			this._currentReferencedContentsId.push(params["unauthorized-content-" + i]);
		}
		tooltip += this.getConfiguration()["unauthorized-content-end-description"];
	}
	
	var count = parseInt(params['locked-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["locked-content-start-description"];
		for (var i=0; i < count; i++)
		{
			tooltip += "<br/>";
			tooltip += params["locked-content-description-" + i];
			this._currentReferencedContentsId.push(params["locked-content-" + i]);
		}
		tooltip += this.getConfiguration()["locked-content-end-description"];
	}
	var count = parseInt(params['referenced-content-size']);
	if (count > 0)
	{
		if (tooltip != "")
		{
			tooltip += "<br/><br/>";
		}
		
		tooltip += this.getConfiguration()["referenced-content-start-description"];
		for (var i=0; i < count; i++)
		{
			if (i != 0) 
			{
				tooltip += ", ";
			}
			tooltip += params["referenced-content-description-" + i];
			this._currentReferencedContentsId.push(params["referenced-content-" + i]);
		}
		tooltip += this.getConfiguration()["referenced-content-end-description"];
	}
	
	
	return tooltip;
}
