/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf;

import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;

import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.cms.workflow.CreateContentFunction;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.OrgUnitFactory;
import org.ametys.odf.orgunit.RootOrgUnitProvider;
import org.ametys.odf.workflow.CreateOrgUnitFunction;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.ModifiableTraversableAmetysObject;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.workflow.Workflow;
import org.ametys.runtime.config.Config;

/**
 * Odf plugin initialization class
 */
public class Init extends AbstractLogEnabled implements org.ametys.runtime.plugin.Init, Serviceable, Configurable
{
    /** The odf root node name */
    public static final String _ODF_ROOT_NODE = RepositoryConstants.NAMESPACE_PREFIX + ":odf";

    /** The odf contents root node name */
    public static final String _ODF_CONTENTS_ROOT_NODE = RepositoryConstants.NAMESPACE_PREFIX + ":contents";
    
    /** The root OrgUnit default name */
    public static final String _ODF_ORGUNIT_DEFAULT_NAME = "Etablissement";

    /** The Ametys object resolver */
    protected AmetysObjectResolver _resolver;
    /** The orgunit provider */
    protected RootOrgUnitProvider _orgUnitProvider;
    /** The workflow */
    protected Workflow _workflow;

    /** The workflow name for orgunit */
    private String _orgUnitWorkflowName;

    /** The workflow id to create orgunit */
    private String _orgUnitWorkflowId;


    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) manager.lookup(AmetysObjectResolver.ROLE);
        _workflow = (Workflow) manager.lookup(Workflow.ROLE);
        _orgUnitProvider = (RootOrgUnitProvider) manager.lookup(RootOrgUnitProvider.ROLE);
    }

    @Override
    public void configure(Configuration configuration) throws ConfigurationException
    {
        _orgUnitWorkflowName = configuration.getChild("orgunit-workflow-name").getValue(null);
        _orgUnitWorkflowId = configuration.getChild("orgunit-workflow-init-action-id").getValue(null);
    }

    @Override
    public void init() throws Exception
    {
        ModifiableTraversableAmetysObject root = _resolver.resolve("/");

        // Creates odf space at "/ametys:odf" if needed
        if (!root.hasChild(_ODF_ROOT_NODE))
        {
            root.createChild(_ODF_ROOT_NODE, "ametys:unstructured");
        }

        ModifiableTraversableAmetysObject odfNode = root.getChild(_ODF_ROOT_NODE);
        
        // Creates a odfContents space at "/ametys:odf/ametys:contents" if needed
        if (!odfNode.hasChild(_ODF_CONTENTS_ROOT_NODE))
        {
            odfNode.createChild(_ODF_CONTENTS_ROOT_NODE, "ametys:collection");
        }

        ModifiableTraversableAmetysObject odfContentsNode = odfNode.getChild(_ODF_CONTENTS_ROOT_NODE);

        if (root.needsSave())
        {
            root.saveChanges();
        }
        
        // Create the main OrgUnit node, if needed
        if (!odfContentsNode.hasChild(CreateOrgUnitFunction.CONTENT_NAME_PREFIX + OrgUnitFactory._ODF_ORGUNIT_ROOT_NODE))
        {
            Map<String, Object> inputs = new HashMap<String, Object>();
            inputs.put(AbstractContentWorkflowComponent.RESULT_MAP_KEY, new HashMap<String, Object>());
            inputs.put(CreateContentFunction.CONTENT_NAME_KEY, OrgUnitFactory._ODF_ORGUNIT_ROOT_NODE);
            inputs.put(CreateContentFunction.CONTENT_TITLE_KEY, _ODF_ORGUNIT_DEFAULT_NAME);
            inputs.put(CreateContentFunction.CONTENT_TYPE_KEY, OrgUnitFactory.ORGUNIT_CONTENT_TYPE);
            inputs.put(CreateContentFunction.CONTENT_LANGUAGE_KEY, Config.getInstance().getValueAsString("odf.programs.lang"));

            _workflow.initialize(_orgUnitWorkflowName, Integer.parseInt(_orgUnitWorkflowId), inputs);

            Map<String, Object> workflowResult = (Map<String, Object>) inputs.get(AbstractContentWorkflowComponent.RESULT_MAP_KEY);
            String contentId = (String) workflowResult.get("contentId");
            
            OrgUnit rootOrgUnit = _resolver.resolveById(contentId);
            rootOrgUnit.setUAICode(Config.getInstance().getValueAsString("odf.root-orgunit.uaiCode"));
            
            rootOrgUnit.saveChanges();

            _orgUnitProvider.setRootId(contentId);
        }
        else
        {
            OrgUnit orgUnit = odfContentsNode.getChild(CreateOrgUnitFunction.CONTENT_NAME_PREFIX + OrgUnitFactory._ODF_ORGUNIT_ROOT_NODE);
            _orgUnitProvider.setRootId(orgUnit.getId());
            
            // Set UAI code in case it changed
            orgUnit.setUAICode(Config.getInstance().getValueAsString("odf.root-orgunit.uaiCode"));
            orgUnit.saveChanges();
        }
    }
}
