/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf;

import java.util.Date;

import javax.jcr.Node;

import org.ametys.cms.contenttype.MetadataType;
import org.ametys.cms.repository.DefaultContent;
import org.ametys.cms.repository.ModifiableContentFactory;
import org.ametys.cms.repository.ModifiableDefaultContent;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.metadata.BinaryMetadata;
import org.ametys.plugins.repository.metadata.RichText;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;

/**
 * Synchronizable Content
 * 
 * @param <F> the actual type of factory.
 */
public class SynchronizableContent<F extends ModifiableContentFactory> extends ModifiableDefaultContent<F>
{
    /** sync suffix */
    public static final String SYNC_SUFFIX = "_sync";

    /** remote suffix */
    public static final String REMOTE_SUFFIX = "_remote";

    /**
     * Constructor
     * 
     * @param node
     * @param parentPath
     * @param factory
     */
    public SynchronizableContent(Node node, String parentPath, F factory)
    {
        super(node, parentPath, factory);
    }
    
    @Override
    public String getTitle() throws UnknownMetadataException, AmetysRepositoryException
    {
        return getString(DefaultContent.METADATA_TITLE);
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public String getString (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getString(metadataName + REMOTE_SUFFIX, "");
        }
        else
        {
            return getMetadataHolder().getString(metadataName, "");
        }
    }
    
    /**
     * Get the remote metadata value
     * @param metadataName  the metadata name
     * @return the metadata value
     */
    public String getRemoteString (String metadataName)
    {
        return getMetadataHolder().getString(metadataName + REMOTE_SUFFIX, "");
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public String[] getStringArray (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getStringArray(metadataName + REMOTE_SUFFIX, new String[] {});
        }
        else
        {
            return getMetadataHolder().getStringArray(metadataName, new String[] {});
        }
    }
    
    /**
     * Get the remote metadata value
     * @param metadataName the metadata 
     * @return the metadata value
     */
    public String[] getRemoteStringArray (String metadataName)
    {
        return getMetadataHolder().getStringArray(metadataName + REMOTE_SUFFIX, new String[] {});
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or -1 if not set
     */
    public double getDouble (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getDouble(metadataName + REMOTE_SUFFIX, -1);
        }
        else
        {
            return getMetadataHolder().getDouble(metadataName, -1);
        }
    }
    
    /**
     * Get the remote metadata value 
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public double getRemoteDouble (String metadataName)
    {
        return getMetadataHolder().getDouble(metadataName + REMOTE_SUFFIX, -1);
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or -1 if not set
     */
    public double[] getDoubleArray (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getDoubleArray(metadataName + REMOTE_SUFFIX, new double[]{});
        }
        else
        {
            return getMetadataHolder().getDoubleArray(metadataName, new double[]{});
        }
    }
    
    /**
     * Get the remote metadata value 
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public double[] getRemoteDoubleArray (String metadataName)
    {
        return getMetadataHolder().getDoubleArray(metadataName + REMOTE_SUFFIX, new double[]{});
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or -1 if not set
     */
    public long getLong (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getLong(metadataName + REMOTE_SUFFIX, -1);
        }
        else
        {
            return getMetadataHolder().getLong(metadataName, -1);
        }
    }
    
    /**
     * Get the remote metadata value 
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public long getRemoteLong (String metadataName)
    {
        return getMetadataHolder().getLong(metadataName + REMOTE_SUFFIX, -1);
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or -1 if not set
     */
    public long[] getLongArray (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getLongArray(metadataName + REMOTE_SUFFIX, new long[]{});
        }
        else
        {
            return getMetadataHolder().getLongArray(metadataName, new long[]{});
        }
    }
    
    /**
     * Get the remote metadata value 
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public long[] getRemoteLongArray (String metadataName)
    {
        return getMetadataHolder().getLongArray(metadataName + REMOTE_SUFFIX, new long[]{});
    }
    
    /**
     * Get the metadata binary value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata binary value or null if not set
     */
    public BinaryMetadata getBinary (String metadataName)
    {
        try
        {
            if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
            {
                return getMetadataHolder().getBinaryMetadata(metadataName + REMOTE_SUFFIX, false);
            }
            else
            {
                return getMetadataHolder().getBinaryMetadata(metadataName, false);
            }
        }
        catch (UnknownMetadataException e)
        {
            return null;
        }
    }
    
    /**
     * Get the remote metadata value 
     * @param metadataName the metadata name
     * @param createNew <code>true</code> to create automatically the {@link BinaryMetadata} when missing.
     * @return the metadata value or <code>null</code> if not set
     */
    public BinaryMetadata getRemoteBinary (String metadataName, boolean createNew)
    {
        try
        {
            return getMetadataHolder().getBinaryMetadata(metadataName + REMOTE_SUFFIX, createNew);
        }
        catch (UnknownMetadataException e)
        {
            return null;
        }
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or null if not set
     */
    public boolean getBoolean (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getBoolean(metadataName + REMOTE_SUFFIX, false);
        }
        else
        {
            return getMetadataHolder().getBoolean(metadataName, false);
        }
    }
    
    /**
     * Get the remote metadata value 
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public boolean getRemoteBoolean (String metadataName)
    {
        return getMetadataHolder().getBoolean(metadataName + REMOTE_SUFFIX, false);
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or null if not set
     */
    public Date getDate (String metadataName)
    {
        if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getDate(metadataName + REMOTE_SUFFIX, null);
        }
        else
        {
            return getMetadataHolder().getDate(metadataName, null);
        }
    }
    
    /**
     * Get the remote metadata value 
     * @param metadataName the metadata name
     * @return the metadata value
     */
    public Date getRemoteDate (String metadataName)
    {
        return getMetadataHolder().getDate(metadataName + REMOTE_SUFFIX, null);
    }
    
    /**
     * Get the rich text value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or null if not set
     */
    public RichText getRichText (String metadataName)
    {
        return getRichText(metadataName, false);
    }
    
    /**
     * Get the rich text value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @param createNew <code>true</code> to create automatically the {@link BinaryMetadata} when missing.
     * @return the metadata value or null if not set
     */
    public RichText getRichText (String metadataName, boolean createNew)
    {
        try
        {
            if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
            {
                return getMetadataHolder().getRichText(metadataName + REMOTE_SUFFIX, createNew);
            }
            else
            {
                return getMetadataHolder().getRichText(metadataName,  createNew);
            }
        }
        catch (UnknownMetadataException e)
        {
            return null;
        }
    }

    /**
     * Get the rich text value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @param createNew <code>true</code> to create automatically the {@link BinaryMetadata} when missing.
     * @return the metadata value or null if not set
     */
    public RichText getRemoteRichText (String metadataName, boolean createNew)
    {
        try
        {
            return getMetadataHolder().getRichText(metadataName + REMOTE_SUFFIX, createNew);
        }
        catch (UnknownMetadataException e)
        {
            return null;
        }
    }
    
    /**
     * Get the rich text value : if synchronized, get the remote value, otherwise the local one.
     * @param metadataName the metadata name
     * @return the metadata value or null if not set
     */
    public BinaryMetadata getBinaryMetadata (String metadataName)
    {
        try
        {
            if (getMetadataHolder().getBoolean(metadataName + SYNC_SUFFIX, false))
            {
                return getMetadataHolder().getBinaryMetadata(metadataName + REMOTE_SUFFIX);
            }
            else
            {
                return getMetadataHolder().getBinaryMetadata(metadataName);
            }
        }
        catch (UnknownMetadataException e)
        {
            return null;
        }
    }
    
    /**
     * Get the metadata value : if synchronized, get the remote value, otherwise
     * the local one.
     * 
     * @param metadataName the metadata name
     * @param type the metadata type
     * @return Object
     */
    public Object getMetadata(String metadataName, MetadataType type)
    {
        switch (type)
        {
            case STRING:
                if (getMetadataHolder().hasMetadata(metadataName) && getMetadataHolder().isMultiple(metadataName))
                {
                    return getStringArray(metadataName);
                }
                else
                {
                    return getString(metadataName);
                }

            case LONG:
                return getLong(metadataName);

            case DOUBLE:
                return getDouble(metadataName);

            case BOOLEAN:
                return getBoolean(metadataName);

            case DATE:
                return getDate(metadataName);

            case RICH_TEXT:
                return getRichText(metadataName);

            case FILE:
            case BINARY:
                return getBinaryMetadata(metadataName);

            default:
                throw new IllegalStateException("This type of metadata " + metadataName + " is not supported.");
        }
    }

    /**
     * Set the synchronization status
     * 
     * @param name
     * @param sync
     */
    public void synchronizeMetadata(String name, boolean sync)
    {
        getMetadataHolder().setMetadata(name + SYNC_SUFFIX, sync);
    }
    
    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param value
     */
    public void setRemoteMetadata(String name, boolean value)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, value);
    }

    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param values
     */
    public void setRemoteMetadata(String name, boolean[] values)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, values);
    }

    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param value
     */
    public void setRemoteMetadata(String name, double value)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, value);
    }

    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param values
     */
    public void setRemoteMetadata(String name, double[] values)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, values);
    }

    /**
     * Set the local metadata value
     * 
     * @param name
     * @param value
     */
    public void setRemoteMetadata(String name, long value)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, value);
    }

    /**
     * Set the remote metadata value
     * @param name the metadata name
     * @param values the value to set
     */
    public void setRemoteMetadata(String name, long[] values)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, values);
    }

    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param value
     */
    public void setRemoteMetadata(String name, String value)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, value);
    }

    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param values
     */
    public void setRemoteMetadata(String name, String[] values)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, values);
    }

    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param value
     */
    public void setRemoteMetadata(String name, Date value)
    {
        // Add the suffix
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, value);
    }

    /**
     * Set the remote metadata value
     * 
     * @param name
     * @param values
     */
    public void setRemoteMetadata(String name, Date[] values)
    {
        // Add the namespace
        String metadataName = name + REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, values);
    }
}
