/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.catalog;

import java.util.ArrayList;
import java.util.List;

import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.commons.lang.StringUtils;

import org.ametys.cms.repository.ContentTypeExpression;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseListFactory;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.ModifiableTraversableAmetysObject;
import org.ametys.plugins.repository.TraversableAmetysObject;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.query.QueryHelper;
import org.ametys.plugins.repository.query.SortCriteria;
import org.ametys.plugins.repository.query.expression.AndExpression;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.StringExpression;

/**
 * Component to handle ODF catalogs
 */
public class CatalogsManager implements Serviceable, Component
{
    /** Avalon Role */
    public static final String ROLE = CatalogsManager.class.getName();
    
    private static final String __PLUGIN_NODE_NAME = "odf";
    
    private AmetysObjectResolver _resolver;
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) manager.lookup(AmetysObjectResolver.ROLE);
    }
    
    /**
     * Get the list of catalogs
     * @return the catalogs
     */
    public List<Catalog> getCatalogs()
    {
        List<Catalog> result = new ArrayList<Catalog>();

        TraversableAmetysObject catalogsNode = getCatalogsRootNode();
        
        AmetysObjectIterable<Catalog> catalogs = catalogsNode.getChildren();
        for (Catalog catalog : catalogs)
        {
            result.add(catalog);
        }
        
        return result;
    }

    /**
     * Get a catalog matching with the given name
     * @param name The name
     * @return a catalog, or null if not found
     */
    public Catalog getCatalog(String name)
    {
        ModifiableTraversableAmetysObject catalogsNode = getCatalogsRootNode();
        
        if (StringUtils.isNotEmpty(name) && catalogsNode.hasChild(name))
        {
            return catalogsNode.getChild(name);
        }
        
        // Not found
        return null;
    }
    
    /**
     * Get the root catalogs storage object.
     * @return the root catalogs node
     * @throws AmetysRepositoryException if a repository error occurs.
     */
    public ModifiableTraversableAmetysObject getCatalogsRootNode() throws AmetysRepositoryException
    {
        try
        {
            ModifiableTraversableAmetysObject rootNode = _resolver.resolve("/");
            ModifiableTraversableAmetysObject pluginsNode = _getOrCreateNode(rootNode, "ametys:plugins", "ametys:unstructured");
            ModifiableTraversableAmetysObject pluginNode = _getOrCreateNode(pluginsNode, __PLUGIN_NODE_NAME, "ametys:unstructured");
            
            return _getOrCreateNode(pluginNode, "catalogs", "ametys:unstructured");
        }
        catch (AmetysRepositoryException e)
        {
            throw new AmetysRepositoryException("Unable to get the ODF catalogs root node", e);
        }
    }
    
    /**
     * Create a new catalog
     * @param name The unique name
     * @param title The title of catalog
     * @return the created catalog
     */
    public Catalog createCatalog(String name, String title)
    {
        Catalog newCatalog = null;
        
        ModifiableTraversableAmetysObject catalogsNode = getCatalogsRootNode();
        
        newCatalog = catalogsNode.createChild(name, "ametys:catalog");
        newCatalog.setTitle(title);
        
        return newCatalog;
    }
    
    /**
     * Get programs linked to a catalog
     * @param catalog The code of catalog
     * @return The programs
     */
    public AmetysObjectIterable<Program> getPrograms (String catalog)
    {
        ContentTypeExpression cTypeExpr = new ContentTypeExpression(Operator.EQ, ProgramFactory.PROGRAM_CONTENT_TYPE);
        StringExpression catalogExpr = new StringExpression(Program.CATALOG, Operator.EQ, catalog);
        Expression programsExpression = new AndExpression(cTypeExpr, catalogExpr);
        
        // Add sort criteria to get size
        SortCriteria sortCriteria = new SortCriteria();
        sortCriteria.addCriterion(Catalog.TITLE, true, true);
        
        String programsQuery = QueryHelper.getXPathQuery(null, ProgramFactory.PROGRAM_NODETYPE, programsExpression, sortCriteria);
        return _resolver.query(programsQuery);
    }
    
    /**
     * Copy the programs and its hierarchy from a catalog to another.
     * The referenced courses are NOT copied.
     * @param catalog The new catalog to populate
     * @param catalogToCopy The catalog from which we copy the programs.
     */
    public void copyCatalog(Catalog catalog, Catalog catalogToCopy)
    {
        AmetysObjectIterable<Program> programs = getPrograms(catalogToCopy.getName());
        
        for (Program program : programs)
        {
            if (program.getParent() instanceof ModifiableTraversableAmetysObject)
            {
                Program newProgram = (Program) program.copyTo((ModifiableTraversableAmetysObject) program.getParent(), null);
                newProgram.getMetadataHolder().setMetadata(Program.CATALOG, catalog.getName());
                
                _copyContentChildren(program, newProgram);
            }
        }
    }
    
    /**
     * Copy the children of a content into another traversable content
     * @param srcContent the content to copy
     * @param targetContent the target content
     */
    protected void _copyContentChildren(TraversableAmetysObject srcContent, ModifiableTraversableAmetysObject targetContent)
    {
        for (AmetysObject child : srcContent.getChildren())
        {
            if (child instanceof SubProgram)
            {
                SubProgram subProgram = (SubProgram) child;
                SubProgram newSubProgram = (SubProgram) subProgram.copyTo(targetContent, null);
                _copyContentChildren(subProgram, newSubProgram);
            }
            else if (child instanceof Container)
            {
                Container container = (Container) child;
                Container newContainer = (Container) container.copyTo(targetContent, null);
                _copyContentChildren(container, newContainer);
            }
            else if (child instanceof CourseList)
            {
                CourseList courseList = (CourseList) child;
                
                CourseList newCourseList = (CourseList) targetContent.createChild(courseList.getName(), CourseListFactory.COURSE_LIST_NODETYPE);
                courseList.getMetadataHolder().copyTo(newCourseList.getMetadataHolder());
                
                newCourseList.saveChanges();
            }
        }
    }
    
    /**
     * Delete catalog
     * @param id the id of catalog
     */
    public void deleteCatalog (String id)
    {
        try
        {
            Catalog catalog = _resolver.resolveById(id);
            if (catalog != null)
            {
                catalog.remove();
                catalog.saveChanges();
            }
        }
        catch (UnknownAmetysObjectException e)
        {
            // Nothing
        }
        
    }
    
    private ModifiableTraversableAmetysObject _getOrCreateNode(ModifiableTraversableAmetysObject parentNode, String nodeName, String nodeType) throws AmetysRepositoryException
    {
        ModifiableTraversableAmetysObject definitionsNode;
        if (parentNode.hasChild(nodeName))
        {
            definitionsNode = parentNode.getChild(nodeName);
        }
        else
        {
            definitionsNode = parentNode.createChild(nodeName, nodeType);
            parentNode.saveChanges();
        }
        return definitionsNode;
    }
}
