/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.catalog.actions;

import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.acting.ServiceableAction;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.commons.lang.StringUtils;

import org.ametys.odf.catalog.Catalog;
import org.ametys.odf.catalog.CatalogsManager;

import com.ibm.icu.text.Transliterator;

/**
 * Action to create a new catalog. 
 * If an existing catalog is provided, it will be duplicated into the new catalog
 */
public class CatalogCreationAction extends ServiceableAction
{
    private static Transliterator __transliterator = Transliterator.getInstance("NFD; [:Nonspacing Mark:] Remove; NFC");
    private CatalogsManager _catalogsManager;
    
    @Override
    public void service(ServiceManager sManager) throws ServiceException
    {
        super.service(sManager);
        _catalogsManager = (CatalogsManager) sManager.lookup(CatalogsManager.ROLE);
    }

    @Override
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        @SuppressWarnings("unchecked")
        Map<String, Object> jsParameters = (Map<String, Object>) objectModel.get(ObjectModelHelper.PARENT_CONTEXT);
        Map<String, String> result = new HashMap<String, String>();
        
        String title = (String) jsParameters.get("title");
        
        String catalogNameToCopy = (String) jsParameters.get("catalogToCopy");
        
        String name = _filterName(title);
        
        Catalog catalog = _catalogsManager.getCatalog(name);
        if (catalog != null)
        {
            result.put("message", "already-exist");
            return result;
        }
        
        Catalog newCatalog = _catalogsManager.createCatalog(name, title);
        
        if (StringUtils.isNotEmpty(catalogNameToCopy))
        {
            Catalog catalogToCopy = _catalogsManager.getCatalog(catalogNameToCopy);
            
            if (catalogToCopy == null)
            {
                result.put("message", "not-found");
                return result;
            }

            _catalogsManager.copyCatalog(newCatalog, catalogToCopy);
        }
        
        newCatalog.saveChanges();
        
        result.put("id", newCatalog.getId());
        result.put("title", newCatalog.getTitle());
        
        return result;
    }
    
    private String _filterName(String name)
    {
        // FIXME CMS-5758 FilterNameHelper.filterName do not authorized name with number
        
        // Use lower case
        // then remove accents
        // then replace contiguous spaces with one dash
        // and finally remove non-alphanumeric characters except -
        String filteredName = __transliterator.transform(name.toLowerCase()).replaceAll(" +", "-").replaceAll("[^\\w-]", "-").replaceAll("-+", "-");
        
        // Remove characters '-' and '_' at the start and the end of the string
        return StringUtils.strip(filteredName, "-_");
    }
}
