/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.cdmfr;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Set;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.excalibur.source.SourceResolver;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.odf.catalog.Catalog;
import org.ametys.odf.catalog.CatalogsManager;
import org.ametys.odf.course.Course;
import org.ametys.odf.enumeration.OdfEnumerationConstant;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.person.Person;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.plugins.repository.metadata.BinaryMetadata;
import org.ametys.plugins.repository.metadata.ModifiableBinaryMetadata;
import org.ametys.plugins.repository.metadata.ModifiableCompositeMetadata;
import org.ametys.runtime.config.Config;
import org.ametys.runtime.util.URLEncoder;

/**
 * base CDMfr extension for Ametys
 */
public class AmetysCDMfrExtension implements CDMfrExtension, Serviceable
{
    private OdfEnumerationHelper _enumerationHelper;
    private CatalogsManager _catalogsManager;
    private SourceResolver _sourceResolver;
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        _enumerationHelper = (OdfEnumerationHelper) manager.lookup(OdfEnumerationHelper.ROLE);
        _catalogsManager = (CatalogsManager) manager.lookup(CatalogsManager.ROLE);
        _sourceResolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
    }
    
    @Override
    public void abstractProgram2CDM(ContentHandler contentHandler, AbstractProgram<? extends ProgramFactory> program, Set<String> persons, Set<String> orgUnits) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        // Passerelle et réorientation
        // <ametys-cdm:reorientation>...</ametys-cdm:reorientation>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + AbstractProgram.REORIENTATION, program.getReorientation(), _sourceResolver, attrs);
        
        // Stages
        // <ametys-cdm:training>
        //      <ametys-cdm:trainingInfo value="internship_mandatory">Obligatoire</ametys-cdm:trainingInfo>
        //      <ametys-cdm:trainingDuration>6 mois</ametys-cdm:trainingInfo>
        // </ametys-cdm:training>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "training");
        
        String internship = program.getInternship();
        if (!internship.isEmpty())
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute("value", internship);
            String internshipLabel = _enumerationHelper.getCDMfrValue(OdfEnumerationConstant.INTERNSHIP, internship);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "trainingInfo", attrs, internshipLabel);
        }
        else
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "trainingInfo");
        }
        
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "trainingDuration", program.getInternshipDuration());
        
        String internshipAbroad = program.getInternshipAbroad();
        if (!internshipAbroad.isEmpty())
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute("value", internshipAbroad);
            String internshipLabel = _enumerationHelper.getCDMfrValue(OdfEnumerationConstant.INTERNSHIP, internshipAbroad);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "trainingAbroadInfo", attrs, internshipLabel);
        }
        else
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "trainingAbroadInfo");
        }
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "trainingAbroadDuration", program.getAbroadInternshipDuration());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "training");
        
        // Etablissement partenaires
        // <ametys-cdm:partnerSchools/>
        if (program.getMetadataHolder().hasMetadata(AbstractProgram.PARTNER_SCHOOLS))
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + AbstractProgram.PARTNER_SCHOOLS);
            
            ModifiableCompositeMetadata partnerSchools = program.getMetadataHolder().getCompositeMetadata(AbstractProgram.PARTNER_SCHOOLS, true);
            String[] entryNames = partnerSchools.getMetadataNames();
            _sortRepeaterEntries(entryNames);
            
            for (String entryName : entryNames)
            {
                ModifiableCompositeMetadata entry = partnerSchools.getCompositeMetadata(entryName, true);
                String linkUrl = entry.getString("linkUrl", "");
                String linkLabel = entry.getString("linkLabel", "");
                
                if (!linkUrl.equals("") || !linkLabel.equals(""))
                {
                    XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "partnerSchool");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "linkUrl", linkUrl);
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "linkLabel", linkLabel);
                    XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "partnerSchool");
                }
            }
            
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + AbstractProgram.PARTNER_SCHOOLS);
        }
        
        // Laboratoires partenaires
        // <ametys-cdm:partnerLaboratories/>
        if (program.getMetadataHolder().hasMetadata(AbstractProgram.PARTNER_LABORATORIES))
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + AbstractProgram.PARTNER_LABORATORIES);
            
            ModifiableCompositeMetadata partnerLaboratories = program.getMetadataHolder().getCompositeMetadata(AbstractProgram.PARTNER_LABORATORIES, true);
            String[] entryNames = partnerLaboratories.getMetadataNames();
            _sortRepeaterEntries(entryNames);
            for (String entryName : entryNames)
            {
                ModifiableCompositeMetadata entry = partnerLaboratories.getCompositeMetadata(entryName, true);
                String linkUrl = entry.getString("linkUrl", "");
                String linkLabel = entry.getString("linkLabel", "");
                
                if (!linkUrl.equals("") || !linkLabel.equals(""))
                {
                    XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "partnerLaboratory");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "linkUrl", linkUrl);
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "linkLabel", linkLabel);
                    XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "partnerLaboratory");
                }
            }
            
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM +  AbstractProgram.PARTNER_LABORATORIES);
        }
        
        
        // Fichier attachées
        if (program.getMetadataHolder().hasMetadata(AbstractProgram.ATTACHMENTS))
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + AbstractProgram.ATTACHMENTS);
            
            ModifiableCompositeMetadata attachments = program.getMetadataHolder().getCompositeMetadata(AbstractProgram.ATTACHMENTS, true);
            String[] entryNames = attachments.getMetadataNames();
            _sortRepeaterEntries(entryNames);
            for (String entryName : entryNames)
            {
                ModifiableCompositeMetadata entry = attachments.getCompositeMetadata(entryName, true);
                
                if (entry.hasMetadata("attachment"))
                {
                    String url = null;
                    if (org.ametys.plugins.repository.metadata.CompositeMetadata.MetadataType.BINARY.equals(entry.getType("attachment")))
                    {
                        ModifiableBinaryMetadata file = entry.getBinaryMetadata("attachment");
                        url = _getBinaryAbsoluteUrl(program, file, "attachments/" + entryName + "/attachment");
                    }
                    
                    String text = entry.getString("attachment-text", "");
                    
                    XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "attachment");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "text", text);
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "href", url);
                    XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "attachment");
                }
            }
            
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + AbstractProgram.ATTACHMENTS);
        }
    }
    
    private void _sortRepeaterEntries(String[] entryNames)
    {
        Arrays.sort(entryNames, new Comparator<String>()
        {
            @Override
            public int compare(String s1, String s2)
            {
                try
                {
                    Integer i1 = new Integer(s1);
                    Integer i2 = new Integer(s2);

                    return i1.compareTo(i2);
                }
                catch (NumberFormatException e)
                {
                    return s1.compareTo(s2);
                }
            }
        });
    }
    
    private String _getBinaryAbsoluteUrl(AbstractProgram<? extends ProgramFactory> program, BinaryMetadata binary, String metadataPath)
    {
        StringBuilder resultPath = new StringBuilder();
            
        resultPath.append(Config.getInstance().getValueAsString("cms.url"))
              .append("/_odf")
              .append("/_contents")
              .append(_encodePath(program.getPath().substring(1)))
              .append("/_metadata/")
              .append(metadataPath)
              .append("/").append(URLEncoder.encode(binary.getFilename()))
              .append("?objectId=" + program.getId());
        
        return resultPath.toString();
    }
    
    private String _encodePath(String path)
    {
        StringBuffer sb = new StringBuffer();
        
        String[] parts = path.split("/");
        for (String part : parts)
        {
            sb.append("/");
            sb.append(URLEncoder.encode(part));
        }
        
        return sb.toString();
    }
    
    @Override
    public void program2CDM(ContentHandler contentHandler, Program program, Set<String> persons, Set<String> orgUnits) throws SAXException
    {
        // Catalogue de formation
        String catalogCode = program.getCatalog();
        if (StringUtils.isNotEmpty(catalogCode))
        {
            AttributesImpl attrs = new AttributesImpl();
            attrs.addCDATAAttribute("value", catalogCode);
            
            Catalog catalog = _catalogsManager.getCatalog(catalogCode);
            String catalogLabel = catalog != null ? catalog.getTitle() : catalogCode;
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_AMETYS_CDM + "catalog", attrs, catalogLabel);
        }
    }

    @Override
    public void course2CDM(ContentHandler contentHandler, Course course, Set<String> persons, Set<String> orgUnits) throws SAXException
    {
        // nothing
    }

    @Override
    public void orgunit2CDM(ContentHandler contentHandler, OrgUnit orgunit) throws SAXException
    {
        // nothing
    }

    @Override
    public void person2CDM(ContentHandler contentHandler, Person person) throws SAXException
    {
        // nothing
    }
}
