/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.ametys.odf.cdmfr;

import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;
import org.apache.excalibur.xml.sax.XMLizable;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseList.ChoiceType;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.metadata.RichText;

/**
 * Helper for export tasks.
 */
public final class CDMHelper
{
    /** The CDM date format. */
    public static final DateFormat CDM_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd");
    
    private CDMHelper()
    {
        // empty constructor
    }

    /**
     * SAX the course list structure
     * @param contentHandler The content handler to SAX into
     * @param cl The course list
     * @param courses References of saxed courses 
     * @param resolver The ametys resolver
     * @throws SAXException
     */
    public static void courseListToCDM (ContentHandler contentHandler, CourseList cl, Set<String> courses, AmetysObjectResolver resolver) throws SAXException
    {
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_STRUCTURE);
        
        courseListInfoToCDM (contentHandler, cl);
                
        for (String courseId : cl.getCourses())
        {
            Course course = resolver.resolveById(courseId);
            courses.add(course.getId());
            AttributesImpl attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, course.getCDMId());
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_REF_COURSE, attrs);
        }
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_STRUCTURE);
    }
    
    /**
     * Convert course list info to CDM
     * @param contentHandler
     * @param cl
     * @throws SAXException
     */
    public static void courseListInfoToCDM (ContentHandler contentHandler, CourseList cl) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("userDefined", "listCode");
        String code = cl.getCode();
        if (StringUtils.isEmpty(code))
        {
            code = org.ametys.runtime.util.StringUtils.generateKey().toUpperCase();
        }
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs, code);
        
        attrs.clear();
        attrs.addCDATAAttribute("userDefined", "listName");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs, cl.getTitle());
        
        ChoiceType type = cl.getType();
        if (type != null)
        {
            attrs.clear();
            attrs.addCDATAAttribute("userDefined", "listChoiceType");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs, cl.getType().toString().toLowerCase());
        }
        
        String ects = cl.getEcts();
        if (StringUtils.isNotEmpty(ects))
        {
            attrs.clear();
            attrs.addCDATAAttribute("userDefined", "ECTScredits");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs, ects);
        }
        
        if (type != null && type.equals(ChoiceType.CHOICE))
        {
            attrs.clear();
            attrs.addCDATAAttribute("userDefined", "listMinNumberOfCourses");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs, String.valueOf(cl.getMinNumberOfCourses()));
            
            attrs.clear();
            attrs.addCDATAAttribute("userDefined", "listMaxNumberOfCourses");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs, String.valueOf(cl.getMaxNumberOfCourses()));
        }
    }
    /**
     * Transform a docbook metadata in a CDM infoBlock way.
     * @param contentHandler the handler to SAX CDM.
     * @param tag the surrounding tag.
     * @param richText the metadata. If richText is null, nothing is SAXed. 
     * @param sourceResolver the Cocoon SourceResolver.
     * @throws SAXException if an error occurs. 
     */
    public static void richText2CDM(ContentHandler contentHandler, String tag, RichText richText, SourceResolver sourceResolver) throws SAXException
    {
        richText2CDM(contentHandler, tag, richText, sourceResolver, false, new AttributesImpl());
    }
    
    /**
     * Transform a docbook metadata in a CDM infoBlock way.
     * @param contentHandler the handler to SAX CDM.
     * @param tag the surrounding tag.
     * @param richText the metadata. If richText is null, nothing is SAXed. 
     * @param sourceResolver the Cocoon SourceResolver.
     * @param attrs The attributes to SAX
     * @throws SAXException if an error occurs. 
     */
    public static void richText2CDM(ContentHandler contentHandler, String tag, RichText richText, SourceResolver sourceResolver, AttributesImpl attrs) throws SAXException
    {
        richText2CDM(contentHandler, tag, richText, sourceResolver, false, attrs);
    }

    
    /**
     * Transform a docbook metadata in a CDM infoBlock way.
     * @param contentHandler the handler to SAX CDM.
     * @param tag the surrounding tag.
     * @param richText the metadata. If richText is null, nothing is SAXed. 
     * @param sourceResolver the Cocoon SourceResolver.
     * @param writeIfEmpty
     * @throws SAXException if an error occurs. 
     */
    public static void richText2CDM(ContentHandler contentHandler, String tag, RichText richText, SourceResolver sourceResolver, boolean writeIfEmpty) throws SAXException
    {
        richText2CDM(contentHandler, tag, richText, sourceResolver, writeIfEmpty, new AttributesImpl());
    }

    /**
     * Transform a docbook metadata in a CDM infoBlock way.
     * @param contentHandler the handler to SAX CDM.
     * @param tag the surrounding tag.
     * @param richText the metadata. If richText is null, nothing is SAXed
     * @param writeIfEmpty
     * @param attrs The attributes to SAX
     * @param sourceResolver the Cocoon SourceResolver.
     * @throws SAXException if an error occurs. 
     */
    public static void richText2CDM(ContentHandler contentHandler, String tag, RichText richText, SourceResolver sourceResolver, boolean writeIfEmpty, AttributesImpl attrs) throws SAXException
    {
        if (richText != null)
        {
            XMLUtils.startElement(contentHandler, tag, attrs);
            richText2CDM(contentHandler, richText, sourceResolver);
            XMLUtils.endElement(contentHandler, tag);
        }
        else if (attrs.getLength() > 0)
        {
            XMLUtils.createElement(contentHandler, tag, attrs);
        }
        else if (writeIfEmpty)
        {
            XMLUtils.createElement(contentHandler, tag, attrs);
        }
    }
    
    /**
     * Transform a docbook metadata in a CDM infoBlock way.
     * @param contentHandler the handler to SAX CDM.
     * @param richText the metadata. If richText is null, nothing is SAXed
     * @param sourceResolver the Cocoon SourceResolver.
     * @throws SAXException if an error occurs. 
     */
    public static void richText2CDM(ContentHandler contentHandler, RichText richText, SourceResolver sourceResolver) throws SAXException
    {
        if (richText != null)
        {
            InputStream is = null;
            try
            {
                is = richText.getInputStream();
                HashMap<String, Object> params = new HashMap<String, Object>();
                params.put("source", is);
                
                Source source = sourceResolver.resolveURI("cocoon://_plugins/odf/docbook2cdm", null, params);
                
                ((XMLizable) source).toSAX(contentHandler);
    
            }
            catch (IOException ex)
            {
                throw new RuntimeException(ex);
            }
            finally
            {
                IOUtils.closeQuietly(is);
            }
        }
    }
    
    /**
     * SAX a date in a CDM document.
     * @param contentHandler The content handler to SAX into.
     * @param tagName The element name.
     * @param value The date value.
     * @throws SAXException if an error occurs.
     */
    public static void date2CDM(ContentHandler contentHandler, String tagName, Date value) throws SAXException
    {
        if (value != null)
        {
            AttributesImpl attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_DATE, CDM_DATE_FORMAT.format(value));
            XMLUtils.createElement(contentHandler, tagName, attrs);
        }
    }
    
}
