/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.cdmfr;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Map;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;

import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.odf.program.Program;
import org.ametys.runtime.config.Config;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.FunctionProvider;
import com.opensymphony.workflow.WorkflowException;

/**
 * Generate and put CDM-fr file in an output folder
 *
 */
public class DepositCDMFRFunction extends AbstractContentWorkflowComponent implements FunctionProvider, Initializable
{
    private File _outputFolder;
    private boolean _isActive;
    private SourceResolver _sourceResolver;
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        super.service(manager);
        _sourceResolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
    }
    
    @Override
    public void initialize() throws Exception
    {
        _isActive = Config.getInstance().getValueAsBoolean("odf.publish.cdm-fr.output.folder");
        String outputFolderPath = Config.getInstance().getValueAsString("odf.publish.cdm-fr.output.folder.path");
        
        if (StringUtils.isNotEmpty(outputFolderPath))
        {
            _outputFolder = new File(outputFolderPath);
            if (!_outputFolder.exists())
            {
                _outputFolder.mkdirs();
            }
        }
    }

    @Override
    public void execute(Map transientVars, Map args, PropertySet ps) throws WorkflowException
    {
        if (!_isActive || _outputFolder == null)
        {
            // Do nothing
            return;
        }
        
        // Retrieve current content
        Program program = (Program) getContent(transientVars);
        
        OutputStream os = null;
        InputStream is = null;
        try
        {
            // Generate CDM-FR file
            Source cdmfrSource = _sourceResolver.resolveURI("cocoon://_plugins/odf/export-cdmfr.xml?id=" + program.getId());

            // Save file
            is = cdmfrSource.getInputStream();
            String filename = program.getCdmCode() + ".xml";
            
            // Delete existing file
            File file = new File(_outputFolder, filename);
            if (file.exists())
            {
                file.delete();
            }
         
            os = new FileOutputStream(file);
            IOUtils.copy(is, os);
        }
        catch (IOException e)
        {
            getMapToRender(transientVars).put("cdmfr-export-error", program.getTitle());
            _logger.error("Unable to generate and copy the CDM-fr file", e);
        }
        finally
        {
            IOUtils.closeQuietly(is);
            IOUtils.closeQuietly(os);
        }
    }
}
