/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.cdmfr;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.excalibur.source.SourceResolver;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.cms.contenttype.ContentTypeExtensionPoint;
import org.ametys.cms.repository.Content;
import org.ametys.odf.course.Course;
import org.ametys.odf.course.LOMSheet;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.enumeration.OdfEnumerationConstant;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.RootOrgUnitProvider;
import org.ametys.odf.person.ContactData;
import org.ametys.odf.person.Person;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.odf.program.ProgramPart;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.metadata.RichText;
import org.ametys.runtime.config.Config;
import org.ametys.runtime.util.I18nizableText;

/**
 * Component able to generate CDM-fr for a {@link CDMEntity}.
 */
public class ExportCDMfrManager implements Component, Serviceable, Contextualizable
{
    /** The Avalon role */
    public static final String ROLE = ExportCDMfrManager.class.getName();
    
    private static final String[] __FIXED_DOMAIN_NAMES = new String[]{"ALL", "DEG", "SHS", "STS", "STA"};
    
    private AmetysObjectResolver _resolver;
    private SourceResolver _sourceResolver;
    private OdfEnumerationHelper _enumerationHelper;
    private RootOrgUnitProvider _rootOrgUnitProvider;
    private Context _context;
    private CDMfrExtensionPoint _cdmFrExtensionPoint;
    private ContentTypeExtensionPoint _cTypeEP;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _sourceResolver = (SourceResolver) smanager.lookup(SourceResolver.ROLE);
        _enumerationHelper = (OdfEnumerationHelper) smanager.lookup(OdfEnumerationHelper.ROLE);
        _rootOrgUnitProvider = (RootOrgUnitProvider) smanager.lookup(RootOrgUnitProvider.ROLE);
        _cdmFrExtensionPoint = (CDMfrExtensionPoint) smanager.lookup(CDMfrExtensionPoint.ROLE);
        _cTypeEP = (ContentTypeExtensionPoint) smanager.lookup(ContentTypeExtensionPoint.ROLE);
    }
    
    @Override
    public void contextualize(Context context) throws ContextException
    {
        _context = context;
    }
    
    /**
     * Entry point to generate the CDM for a program
     * @param contentHandler 
     * @param program The program
     * @throws SAXException
     */
    public void generateCDM(ContentHandler contentHandler, Program program) throws SAXException
    {
        contentHandler.startDocument();
        contentHandler.startPrefixMapping("", "http://cdm-fr.fr/2013/CDM");
        contentHandler.startPrefixMapping("xsi", "http://www.w3.org/2001/XMLSchema-instance");
        contentHandler.startPrefixMapping("cdmfr", "http://cdm-fr.fr/2013/CDM-frSchema");
        contentHandler.startPrefixMapping("xhtml", "http://www.w3.org/1999/xhtml");
        contentHandler.startPrefixMapping("ametys-cdm", "http://www.ametys.org/cdm/1.0");
            
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("http://www.w3.org/2001/XMLSchema-instance", "schemaLocation", "xsi:schemaLocation", "http://cdm-fr.fr/2013/CDM http://cdm-fr.fr/2013/schemas/CDMFR.xsd");
        attrs.addCDATAAttribute("profile", "CDM-fr");
        attrs.addCDATAAttribute("language", program.getLanguage());
        XMLUtils.startElement(contentHandler, "CDM", attrs);
        
        saxHabilitation(contentHandler, program);
        
        XMLUtils.startElement(contentHandler, "properties");
        XMLUtils.createElement(contentHandler, "datasource", "Ametys v3");
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute("date", new SimpleDateFormat("yyyy-MM-dd").format(new Date()));
        XMLUtils.createElement(contentHandler, "datetime", attrs);
        XMLUtils.endElement(contentHandler, "properties");
        
        program2CDM(contentHandler, program);
        
        XMLUtils.endElement(contentHandler, "CDM");
        
        contentHandler.endDocument();
    }
    
    private void saxHabilitation(ContentHandler contentHandler, Program program) throws SAXException
    {
        /**
         * <cdmfr:habilitation>
         *      <cdmfr:habiliId>ID</cdmfr:habiliId>
         *      <cdmfr:cohabilitation>
         *          <cdmfr:exists>true|false</cdmfr:exists> // Habilité ou non
         *          <cdmfr:listOfOrgUnit>
         *              <cdmfr:habOrgUnit>
         *                  <cdmfr:refOrgUnit> // Etablisssement cohabilité
         *                  <cdmfr:domainName> 
         *                      <cdmfr:fixedDomain> // Domaine concerné par cette habilitation
         *                  </cdmfr:domainName>
         *              </<cdmfr:habOrgUnit>
         *          </cdmfr:listOfOrgUnit>
         *      </cdmfr:cohabilitation>
         *      <cdmfr:field>
         *          <fieldname>
         *              <free>MENTION</free>
         *          </fieldname>
         *          <speciality>
         *              <specialityId/>
         *              <specialityName/>
         *          </speciality>
         *      </cdmfr:field>
         *      <cdmfr:partnership>
         *          <cdmfr:training>
         *              <cdmfr:trainingStrategy /> // Stages
         *          </cdmfr:training>
         *      </cdmfr:partnership>
         * </cdmfr:habilitation>
         * 
         */
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID, "");
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_DIPLOMA_TYPE, getDiplomaType(program));
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_HABILITATION);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_HABILITATION_ID, program.getCDMId());
        
        String domain = program.getDomain();
        boolean isHabilited = StringUtils.isNotEmpty(domain) && !"HD".equals(domain);
        
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "cohabilitation");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "exists", String.valueOf(isHabilited));
        
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "listOfOrgUnit");
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "habOrgUnit");
        
        String rneCode = _rootOrgUnitProvider.getRoot().getUAICode();
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "refOrgUnit", rneCode);
        
        String domainName = _enumerationHelper.getItemCDMfrValue(OdfEnumerationConstant.DOMAIN, domain, false);
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "domainName");
        
        if (ArrayUtils.contains(__FIXED_DOMAIN_NAMES, domain))
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "fixedDomain", domainName);
        }
        else
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "openDomain", domainName);
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "domainName");
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "habOrgUnit");
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "listOfOrgUnit");
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "cohabilitation");
        
        // Etablissement cohabilité
        String[] jointOrgUnit = program.getJointOrgUnit();
        if (jointOrgUnit.length > 0)
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "cohabilitation");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "exists", String.valueOf(true));
            
            attrs.clear();
            attrs.addCDATAAttribute("cohabilitationRole", "cohabilité");
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "listOfOrgUnit", attrs);
            for (String uaiCode : jointOrgUnit)
            {
                if (StringUtils.isNotEmpty(uaiCode))
                {
                    XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "habOrgUnit");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "refOrgUnit", uaiCode);
                    I18nizableText ouLabel = _enumerationHelper.getItemLabel(OdfEnumerationConstant.JOIN_ORGUNIT, uaiCode);
                    if (ouLabel != null)
                    {
                        ouLabel.toSAX(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK);
                    }
                    XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "habOrgUnit");
                }
            }
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "listOfOrgUnit");
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "cohabilitation");
        }
        
        String speciality = program.getSpeciality();
        String mention = program.getMention();
        
        if (StringUtils.isNotEmpty(mention) || StringUtils.isNotEmpty(speciality))
        {
            // Formation LMD : Mention - Speciality
            /**
             * <cdmfr:field>
             *      <cdmfr:fieldName>
             *          <cdmfr:controlled fieldNameCode="code_mention">
             *              <cdmfr:registeredName>Libelle mention</cdmfr:registeredName>
             *          </cdmfr:controlled>
             *      </cdmfr:fieldName>
             *      <cdmfr:speciality>
             *          <cdmfr:specialityName/>
             *          <cdmfr:specialityId/>
             *          <cdmfr:refProgram/>
             *      </cdmfr:speciality>
             * </cdmfr:field>
             */
            
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "field");
            
            if (StringUtils.isNotEmpty(mention))
            {
                XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "fieldName");
               
                boolean isEnumerated = _cTypeEP.getExtension(program.getType()).getMetadataDefinition("mention").getEnumerator() != null;
                if (isEnumerated)
                {
                    String mentionLabel = getMentionLabel(program, mention);
                   
                    attrs.clear();
                    attrs.addCDATAAttribute("fieldNameCode", mention);
                    XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "controlled", attrs);
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "registeredName", mentionLabel);
                    XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "controlled");
                }
                else
                {
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "free", mention);
                }
               
               
                XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "fieldName");
            }
            
            if (StringUtils.isNotEmpty(speciality))
            {
                XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "speciality");
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "specialityName", speciality);
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "specialityId", program.getCDMId() + "SP01"); // FIXME "SP + n° ordre de la spécialité"
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "refProgram", program.getCDMId());
                XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "speciality");
            }
            
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "field");
        }
        else if (Config.getInstance().getValueAsString("odf.programs.degree.licensepro").equals(program.getDegree()))
        {
            // Licence professionelle
            /**
             * <cdmfr:licPro>
             *      <cdmfr:specialite>
             *          <cdm:text/>
             *      </cdmfr:specialite>
             * </cdmfr:licPro>
             */
            
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "licPro");
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "specialite");
            XMLUtils.createElement(contentHandler, "text", program.getSpeciality());
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "specialite");
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "licPro");
        }
        else
        {
            // Autre diplôme habilité ne s'inscrivant pas dans une structure mention-spécialité
            /**
             * <cdmfr:otherDiploma>
             *      <cdmfr:refProgram/>
             * </cdmfr:otherDiploma>
             */
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "otherDiploma");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "refProgram", program.getCDMId());
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "otherDiploma");
        }
        
        /**
         * <cdmfr:partnership>
         *     <cdmfr:training>
         *          <cdmfr:trainingStrategy />
         *     </cdmfr:training>
         * </cdmfr:partnership>
         */      
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "partnership");
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "training");
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "trainingStrategy", program.getRichText(AbstractProgram.TRAINING_STRATEGY), _sourceResolver);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "training");
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + "partnership");
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_HABILITATION);
    }
    
    /**
     * Get the type of diploma
     * @param program the program
     * @return diploma type
     */
    protected String getDiplomaType (Program program)
    {
        String degree = program.getDegree();
        if (Config.getInstance().getValueAsString("odf.programs.degree.license").equals(degree))
        {
            return "L";
        }
        else if (Config.getInstance().getValueAsString("odf.programs.degree.master").equals(degree))
        {
            return "M";
        }
        else if (Config.getInstance().getValueAsString("odf.programs.degree.licensepro").equals(degree))
        {
            return "Lpro";
        }
        return "autre";
    }

    /**
     * Get the label of the mention
     * @param program the program
     * @param mention the code of the mention
     * @return label
     */
    protected String getMentionLabel (Program program, String mention)
    {
        String degree = program.getDegree();
        if (Config.getInstance().getValueAsString("odf.programs.degree.license").equals(degree))
        {
            return _enumerationHelper.getCDMfrValue(OdfEnumerationConstant.MENTION_LICENCE, mention);
        }
        else if (Config.getInstance().getValueAsString("odf.programs.degree.master").equals(degree))
        {
            return _enumerationHelper.getCDMfrValue(OdfEnumerationConstant.MENTION_MASTER, mention);
        }
        else if (Config.getInstance().getValueAsString("odf.programs.degree.licensepro").equals(degree))
        {
            return _enumerationHelper.getCDMfrValue(OdfEnumerationConstant.MENTION_LICENCEPRO, mention);
        }
        return mention;
    }

    
    
    //***********************************************************************//
    //                              PROGRAM                                  //
    //***********************************************************************//
    
    /**
     * Export this entity in CDM-fr format.
     * @param contentHandler the target handler
     * @param program the program
     * @throws SAXException if an error occurs during export.
     */
    public void program2CDM(ContentHandler contentHandler, Program program) throws SAXException
    {
        boolean rootOrgUnitSaxed = false;
        
        Set<String> persons = new HashSet<String>();
        Set<String> orgUnits = new HashSet<String>();
        Set<String> coursesToSax = new HashSet<String>();
        
        abstractProgram2CDM(contentHandler, program, persons, orgUnits, coursesToSax);
        
        Set<String> coursesAlreadySax = new HashSet<String>();
        
        while (!coursesToSax.isEmpty())
        {
            Set<String> courses = new HashSet<String>();
            for (String courseId : coursesToSax)
            {
                coursesAlreadySax.add(courseId);
                Course course = _resolver.resolveById(courseId);
                courses.addAll(course2CDM(contentHandler, course, orgUnits, persons));
            }
            
            coursesToSax.clear();
            coursesToSax.addAll(courses);
            coursesToSax.removeAll(coursesAlreadySax);
        }

        for (String orgUnitId : orgUnits)
        {
            OrgUnit orgUnit = _resolver.resolveById(orgUnitId);
            if (orgUnitId.equals(_rootOrgUnitProvider.getRootId()))
            {
                rootOrgUnitSaxed = true;
            }
            
            orgunit2CDM(contentHandler, orgUnit, persons);
        }
        
        if (!rootOrgUnitSaxed)
        {
            orgunit2CDM(contentHandler, _rootOrgUnitProvider.getRoot(), persons);
        }
        
        for (String personId : persons)
        {
            Person person = _resolver.resolveById(personId);
            person2CDM(contentHandler, person);
        }
    }
    
    /**
     * Send the CDM-fr representation of their ProgramPart to the given Contenthandler.<br>
     * Also collects referenced orgUnits, persons and courses, so that they could be sent afterwards.
     * @param contentHandler the receiving contentHandler.
     * @param program 
     * @param persons collected {@link Person} ids.
     * @param orgUnits collected {@link OrgUnit} ids.
     * @param courses collected {@link Course} ids.
     * @throws SAXException if an error occurs during CDM processing.
     */
    public void abstractProgram2CDM(ContentHandler contentHandler, AbstractProgram<? extends ProgramFactory> program, Set<String> persons, Set<String> orgUnits, Set<String> courses) throws SAXException
    {
        String tagName = program.getTagName();
        
        AttributesImpl attrs = new AttributesImpl();
        
        // <program id="FRUAI{rootOrgUnitId}PR{code}">
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID, program.getCDMId());
        XMLUtils.startElement(contentHandler, tagName, attrs);
        
        // <programID>code</programID>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PROGRAM_ID, program.getCDMId());

        // <programName><text>name</text></programName>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_NAME);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_TEXT, program.getTitle());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_NAME);
        
        _abstractProgram2CDMCode(contentHandler, program);
        
        // <webLink>
        //      <href>href</href>
        //      <linkName>linkName</linkName>
        // </webLink>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_WEB_LINK);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_HREF, program.getProgramWebSiteUrl());
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_LINK_NAME, program.getProgramWebSiteLabel());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_WEB_LINK);
        
        // <programDescription>...</programDescription>
        attrs = new AttributesImpl();
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_NATURE, program.getEducationKind(_enumerationHelper));
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_DESCRIPTION, attrs);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, program.getPresentation(), _sourceResolver);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_DESCRIPTION);
        
        // <qualification>
        //      <qualificatioName><text></text></qualificationName>
        //      <qualificationDescription>
        //          <infoBlock>...</infoBlock>
        //          <statistics>success rate</statistics>
        //      </qualificationDescription>
        //      <degree degree=""/>
        //      <profession>...</profession>
        //      <studyQualification>...</studyQualification>
        // </qualification>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_QUALIFICATION);
        
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_QUALIFICATION_NAME);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEXT);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_QUALIFICATION_NAME);
        
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_QUALIFICATION_DESCRIPTION);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, program.getQualification(), _sourceResolver, true);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_STATISTICS, program.getSuccessRate());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_QUALIFICATION_DESCRIPTION);
        
        attrs = new AttributesImpl();
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_ECTS_CREDITS, program.getEcts());
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_CREDITS, attrs);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, program.getKnowledgeCheck(), _sourceResolver, true);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_CREDITS);

        attrs = new AttributesImpl();
        String cdmCode = _enumerationHelper.getItemCDMfrValue(OdfEnumerationConstant.DEGREE, program.getDegree(), false);
        if (StringUtils.isNotEmpty(cdmCode))
        {
            _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_DEGREE, cdmCode);
        }
        else
        {
            _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_DEGREE_NOT_LMD, program.getDegree());
        }
        
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_DEGREE, attrs);

        // <profession>
        //      <infoBlock>...</infoBlock>
        //      <romeData romeCode="code" romeLibel=""/>
        // </profession>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROFESSION);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, program.getJobOpportunities(), _sourceResolver, true);
        String[] romeCode = program.getRomeCode();
        for (String code : romeCode)
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute("romeCode", code);
            String label = _enumerationHelper.getCDMfrValue(OdfEnumerationConstant.CODE_ROME, code);
            if (StringUtils.isNotBlank(label))
            {
                attrs.addCDATAAttribute("romeLibel", label);
            }
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_ROME_DATA, attrs);
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROFESSION);
        
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_STUDY_QUALIFICATION);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, program.getFurtherStudy(), _sourceResolver, true);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_COMPETENCE_DESC);
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_LIMITED, "false");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_COMPETENCE_VALIDITY, attrs);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_STUDY_QUALIFICATION);
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_QUALIFICATION);
        
        // <ppalTeachingLanguage teachingLang="fr"/>
        String lang = program.getEducationLanguage();
        if (StringUtils.isNotEmpty(lang))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_TEACHING_LANG, lang);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PPAL_TEACHING_LANGUAGE, attrs);   
        }
        
        // <levelCode @codeSet="bac|bac+1|bac+2|bac+3|..."/>
        attrs = new AttributesImpl();
        String educationLevel = program.getEducationLevel();
        _addNotNullAttribute(attrs, "codeSet", _enumerationHelper.getCDMfrValue(OdfEnumerationConstant.LEVEL, educationLevel));
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_LEVEL_CODE, attrs);
        
        // <level @level="L|M|Doc"/>
        attrs = new AttributesImpl();
        if (educationLevel.startsWith("licence"))
        {
            _addNotNullAttribute(attrs, "level", "L");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_LEVEL, attrs);
        }
        else if (educationLevel.startsWith("master"))
        {
            _addNotNullAttribute(attrs, "level", "M");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_LEVEL, attrs);
        }
        else if (educationLevel.startsWith("doctorat"))
        {
            _addNotNullAttribute(attrs, "level", "Doc");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_LEVEL, attrs);
        }
        
        // <learningObjectives>...</learningObjectives>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_LEARNING_OBJECTIVES, program.getObjectives(), _sourceResolver);
        
        // <admissionInfo>
        //      <cdmfr:admissionDescription>...</cdmfr:admissionDescription>
        // </admissionInfo>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ADMISSION_INFO);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_ADMISSION_DESCRIPTION, program.getAccessCondition(), _sourceResolver);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_STUDENT_STATUS, "");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_ECTS_REQUIRED, "");
        String effectives = program.getEffectives();
        
        if (StringUtils.isNotEmpty(effectives))
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_STUDENT_PLACES, effectives);
        }
        
        Date teachingStart = program.getTeachingStart();
        if (teachingStart != null)
        {
            CDMHelper.date2CDM(contentHandler, CDMFRTagsConstants.TAG_TEACHING_START, teachingStart);
        }
        
        Date registrationDeadline = program.getRegistrationDeadline();
        if (registrationDeadline != null)
        {
            CDMHelper.date2CDM(contentHandler, CDMFRTagsConstants.TAG_REGISTRATION_DEADLINE, registrationDeadline);
        }
        
        Date registrationStart = program.getRegistrationStart();
        if (registrationStart != null)
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_REGISTRATION_DETAIL);
            
            CDMHelper.date2CDM(contentHandler, CDMFRTagsConstants.TAG_REGISTRATION_START, registrationStart);
            
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_REGISTRATION_DETAIL);
        }
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ADMISSION_INFO);
        
        // <recommendedPrerequisites>...</recommendedPrerequisites>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_RECOMMANDED_PREREQUISITES, program.getRecommendedPrerequisite(), _sourceResolver);

        // <formalPrerequisites>...</formalPrerequisites>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_FORMAL_PREREQUISITES, program.getNeededPrerequisite(), _sourceResolver, true);
        
        // <teachingPlace>
        //      <cdmfr:refOrgUnit />
        //      <adr>
        //          <pcode></pcode>
        //      </adr>
        // </teachingPlace>
        String[] places = program.getPlace();
        for (String postalCode : places)
        {
            if (StringUtils.isNotEmpty(postalCode))
            {
                XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_TEACHING_PLACE);
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_REF_ORG_UNIT);
                XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ADDRESS);
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PCODE, postalCode);
                XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ADDRESS);
                XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_TEACHING_PLACE);
            }
        }
        
        // <targetGroup>...</targetGroup>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_TARGET_GROUP, program.getTargetGroup(), _sourceResolver);

        // <formOfTeaching org="" method=""/>
        String formOfTeachingMethod = program.getDistanceLearning(_enumerationHelper);
        String[] formOfTeachingOrgs = program.getFormOfTeachingOrgs(_enumerationHelper);
        for (String formOfTeachingOrg: formOfTeachingOrgs)
        {
            attrs = new AttributesImpl();
            _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_METHOD, formOfTeachingMethod);
            _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_ORG, formOfTeachingOrg);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_FORM_OF_TEACHING, attrs);
        }
        
        // <studyAbroad>...</studoAbroad>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_STUDY_ABROAD, program.getStudyAbroad(), _sourceResolver);
        
        // <programDuration>...</programDuration>
        String duration = program.getDuration();
        if (StringUtils.isNotBlank(duration))
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PROGRAM_DURATION, duration);
        }
        
        // <cdmfr:programStructure>...</cdmfr:programStructure>
        RichText teachingOrganization = program.getTeachingOrganization();
        if (teachingOrganization != null && teachingOrganization.getLength() > 0)
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_STRUCTURE);
            CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, program.getTeachingOrganization(), _sourceResolver);
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_STRUCTURE);
        }
        
        
        // Sax courses list
        AmetysObjectIterable<AmetysObject> children = program.getChildren();
        for (AmetysObject child : children)
        {
            if (child instanceof CourseList)
            {
                CDMHelper.courseListToCDM(contentHandler, (CourseList) child, courses, _resolver);
            }
        }
        
        // <regulations/>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REGULATIONS);

        // <expenses>...</expenses>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_EXPENSES, program.getExpenses(), _sourceResolver);
        
        // <universalAjustment>...</universalAjustment>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_UNIVERSAL_ADJUSTMENT, program.getUniversalAdjustment(), _sourceResolver);

        // <contacts>
        //      <refPerson idRef="ID"/>
        //      <refOrgUnit idRef="ID"/>
        // </contacts>
        _abstractProgram2CDMContacts(contentHandler, program, persons, orgUnits);
        
        // <infoBlock>...</infoBlock>
        attrs = new AttributesImpl();
        if (tagName.equals(CDMFRTagsConstants.TAG_SUB_PROGRAM))
        {
            attrs.addCDATAAttribute("userDefined", "subProgram");
        }
        
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, program.getAdditionalInformations(), _sourceResolver, attrs);

        // <infoBlock userDefined="ametys-extension>...</infoBlock>
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute("userDefined", "ametys-extension");
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs);
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        for (String id : _cdmFrExtensionPoint.getExtensionsIds())
        {
            CDMfrExtension cdmFrExtension = _cdmFrExtensionPoint.getExtension(id);
            
            if (program instanceof Program)
            {
                cdmFrExtension.program2CDM(contentHandler, (Program) program, persons, orgUnits);
            }
            
            cdmFrExtension.abstractProgram2CDM(contentHandler, program, persons, orgUnits);
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK);
        
        // <programStructure>...</programStructure>
        _programPart2CDMSubPrograms(contentHandler, program, persons, orgUnits, courses);
        
        // <searchword>keyword</searchword>
        for (String keyword : program.getKeywords())
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_SEARCH_WORD, keyword);
        }
        
        // </program>
        XMLUtils.endElement(contentHandler, tagName);

        // Set content in request
        ContextHelper.getRequest(_context).setAttribute(Content.class.getName(), program);
    }
    
    private void _abstractProgram2CDMCode(ContentHandler contentHandler, AbstractProgram<? extends ProgramFactory> program) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        // <programCode codeSet="userDefined">code</programCode>
        String code = program.getCdmCode();
        if (StringUtils.isNotBlank(code))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "userDefined");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, code);
        }
        
        // <programCode codeSet="sectDGESIP">code</programCode>
        // <programCode libSet="sectDGESIP">label</programCode>
        for (String dgesipCode : program.getDGESIPCode())
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "sectDGESIP");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, dgesipCode);
            
            String codeLib = _enumerationHelper.getItemCDMfrValue(OdfEnumerationConstant.CODE_DGESIP, dgesipCode, false);
            if (StringUtils.isNotEmpty(codeLib))
            {
                attrs = new AttributesImpl();
                attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_LIB_SET, "sectDGESIP");
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, codeLib);
            }
        }
        
        // <programCode codeSet="ERASMUS">code</programCode>
        // <programCode libSet="ERASMUS">label</programCode>
        for (String erasmusCode : program.getErasmusCode())
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "ERASMUS");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, erasmusCode);
            
            String codeLib = _enumerationHelper.getItemCDMfrValue(OdfEnumerationConstant.CODE_ERASMUS, erasmusCode, false);
            if (StringUtils.isNotEmpty(codeLib))
            {
                attrs = new AttributesImpl();
                attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_LIB_SET, "ERASMUS");
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, codeLib);
            }
        }
        
        // <programCode codeSet="cite97">code</programCode>
        // <programCode libSet="cite97">label</programCode>
        for (String cite97Code : program.getCite97Code())
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "cite97");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, cite97Code);
            
            String codeLib = _enumerationHelper.getItemCDMfrValue(OdfEnumerationConstant.CODE_CITE97, cite97Code, false);
            if (StringUtils.isNotEmpty(codeLib))
            {
                attrs = new AttributesImpl();
                attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_LIB_SET, "cite97");
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, codeLib);
            }
        }
        
        // <programCode codeSet="FAP">code</programCode>
        // <programCode libSet="FAP">label</programCode>
        for (String fapCode : program.getFapCode())
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "FAP");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, fapCode);
            
            String codeLib = _enumerationHelper.getItemCDMfrValue(OdfEnumerationConstant.CODE_FAP, fapCode, false);
            if (StringUtils.isNotEmpty(codeLib))
            {
                attrs = new AttributesImpl();
                attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_LIB_SET, "FAP");
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, codeLib);
            }
        }
        
        // <programCode codeSet="codeSectDiscipSISE">code</programCode>
        // <programCode libSet="codeSectDiscipSISE">label</programCode>
        for (String siseCode : program.getSiseCode())
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "sectDiscipSISE");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, siseCode);
            
            String codeLib = _enumerationHelper.getItemCDMfrValue(OdfEnumerationConstant.CODE_SISE, siseCode, false);
            if (StringUtils.isNotEmpty(codeLib))
            {
                attrs = new AttributesImpl();
                attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_LIB_SET, "sectDiscipSISE");
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, codeLib);
            }
        }
    }
    
    private void _abstractProgram2CDMContacts(ContentHandler contentHandler, AbstractProgram<? extends ProgramFactory> program, Set<String> persons, Set<String> orgUnits) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_CONTACTS);
        
        for (String contactID : program.getPersonsInCharge())
        {
            if (!"".equals(contactID))
            {
                try
                {
                    Person person = _resolver.resolveById(contactID);
                    persons.add(person.getId());
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, person.getCDMId());
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ROLE, "responsible");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_PERSON, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // Contact does exist anymore, ignore it
                }
            }
        }
        
        for (String contactID : program.getContacts())
        {
            if (!"".equals(contactID))
            {
                try
                {
                    Person person = _resolver.resolveById(contactID);
                    persons.add(person.getId());
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, person.getCDMId());
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ROLE, "contact");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_PERSON, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // Contact does exist anymore, ignore it
                }
                
            }
        }

        for (String orgUnitId : program.getOrgUnits())
        {
            if (!"".equals(orgUnitId))
            {
                try
                {
                    OrgUnit orgUnit = _resolver.resolveById(orgUnitId);
                    orgUnits.add(orgUnit.getId());
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, orgUnit.getCDMId());
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_ORG_UNIT, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // OrgUnit does exist anymore, ignore it
                }
            }
        }
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_CONTACTS);
    }
    
    private void _programPart2CDMSubPrograms(ContentHandler contentHandler, ProgramPart programPart, Set<String> persons, Set<String> orgUnits, Set<String> courses) throws SAXException
    {
        for (AmetysObject child : programPart.getChildren())
        {
            if (child instanceof AbstractProgram)
            {
                abstractProgram2CDM(contentHandler, (AbstractProgram<?>) child, persons, orgUnits, courses);
            }
            else if (child instanceof Container)
            {
                container2CDM(contentHandler, (Container) child, persons, orgUnits, courses);
            }
        }
    }
    
    
    //***********************************************************************//
    //                             CONTAINER                                 //
    //***********************************************************************//
    
    /**
     * Send the CDM-fr representation of their ProgramPart to the given Contenthandler.<br>
     * Also collects referenced orgUnits, persons and courses, so that they could be sent afterwards.
     * @param contentHandler the receiving contentHandler.
     * @param container 
     * @param persons collected {@link Person} ids.
     * @param orgUnits collected {@link OrgUnit} ids.
     * @param courses collected {@link Course} ids.
     * @throws SAXException if an error occurs during CDM processing.
     */
    public void container2CDM(ContentHandler contentHandler, Container container, Set<String> persons, Set<String> orgUnits, Set<String> courses) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        // <program id="FR_RNE_{rootOrgUnitId}_PR_{code}">
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID, container.getCdmCode());
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_SUB_PROGRAM, attrs);
        
        // <programID>code</programID>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PROGRAM_ID, container.getCDMId());

        // <programName><text>name</text></programName>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_NAME);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_TEXT, container.getTitle());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_NAME);
        
        // <programCode codeSet="userDefined">code</programCode>
        String code = container.getCode();
        if (StringUtils.isNotBlank(code))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "userDefined");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROGRAM_CODE, attrs, code);
        }
        
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_ECTS_CREDITS, container.getEcts());
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_CREDITS, attrs);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, null, null, true);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_CREDITS);

        // <programStructure>...</programStructure>
        for (AmetysObject child : container.getChildren())
        {
            if (child instanceof CourseList)
            {
                CDMHelper.courseListToCDM(contentHandler, (CourseList) child, courses, _resolver);
            }
        }

        // <infoBlock>...</infoBlock>
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute("userDefined", "container");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs);
        
        // <subProgram>...</subProgram>
        _programPart2CDMSubPrograms(contentHandler, container, persons, orgUnits, courses);

        // </program>
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_SUB_PROGRAM);
    }
    
    
    //***********************************************************************//
    //                               COURSE                                  //
    //***********************************************************************//
    
    /**
     * Export this entity in CDM-fr format.
     * @param contentHandler the target handler
     * @param course The course
     * @throws SAXException if an error occurs during export.
     */
    public void course2CDM(ContentHandler contentHandler, Course course) throws SAXException
    {
        boolean rootOrgUnitSaxed = false;
        
        Set<String> orgUnits = new HashSet<String>();
        Set<String> persons = new HashSet<String>();
        
        Set<String> coursesAlreadySax = new HashSet<String>();
        Set<String> coursesToSax = new HashSet<String>();
        coursesToSax.add(course.getId());
        
        while (!coursesToSax.isEmpty())
        {
            Set<String> courses = new HashSet<String>();
            for (String courseId : coursesToSax)
            {
                coursesAlreadySax.add(courseId);
                Course courseToSax = _resolver.resolveById(courseId);
                courses.addAll(course2CDM(contentHandler, courseToSax, orgUnits, persons));
            }
            coursesToSax.clear();
            coursesToSax.addAll(courses);
            coursesToSax.removeAll(coursesAlreadySax);
        }
        
        for (String orgUnitId : orgUnits)
        {
            OrgUnit orgUnit = _resolver.resolveById(orgUnitId);
            if (orgUnitId.equals(_rootOrgUnitProvider.getRootId()))
            {
                rootOrgUnitSaxed = true;
            }
            orgunit2CDM(contentHandler, orgUnit, persons);
        }
        
        if (!rootOrgUnitSaxed)
        {
            OrgUnit orgUnit = _rootOrgUnitProvider.getRoot();
            orgunit2CDM(contentHandler, orgUnit, persons);
        }
        
        for (String personId : persons)
        {
            Person person = _resolver.resolveById(personId);
            person2CDM(contentHandler, person);
        }
        
        // Set content in request
        ContextHelper.getRequest(_context).setAttribute(Content.class.getName(), course);
    }
    
    /**
     * Export this entity in CDM-fr format.
     * @param contentHandler the content handler to SAX into
     * @param course the course
     * @param orgUnits the org units
     * @param persons the persons
     * @return the saxed coursed
     * @throws SAXException
     */
    public Set<String> course2CDM(ContentHandler contentHandler, Course course, Set<String> orgUnits, Set<String> persons) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        // <course id="FR_RNE_{rootOrgUnitId}_CO_{code}">
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID, course.getCDMId());
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_IDENT, course.getErasmusCode());
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_COURSE, attrs);
        
        // <courseID>code</courseID>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_COURSE_ID, course.getCDMId());
        
        // <courseName><text>name</text></courseName>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_COURSE_NAME);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEXT, course.getTitle());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_COURSE_NAME);

        // <courseCode codeSet="UE">code</couseCode>
        // FIXME : EC ou UE à définir selon la hiérarchie ?
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "UE");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_COURSE_CODE, attrs, course.getCdmCode());
        
        // <webLink>
        //      <href>href</href>
        //      <linkName>linkName</linkName>
        // </webLink>
        String webLinkURL = course.getWebLinkUrl();
        if (StringUtils.isNotEmpty(webLinkURL))
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_WEB_LINK);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_HREF, webLinkURL);
            String webLinkLabel = course.getWebLinkLabel();
            if (StringUtils.isNotEmpty(webLinkLabel))
            {
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_LINK_NAME, course.getWebLinkLabel());
            }
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_WEB_LINK);
        }

        // <level>level</level>
        attrs = new AttributesImpl();
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_LEVEL, course.getLevel(_enumerationHelper));
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_LEVEL, attrs);
        
        // <courseDescription>...</courseDescription>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_COURSE_DESCRIPTION);
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_DIGITAL_USE);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PERCENTAGE, "0");
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_LCMS);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_EXISTS, "false");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_RESOURCE_MGT);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PERC_OF_USE, "0");
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_LCMS);
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROD_PEDA);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_DIGIT_PROD_PERC, "0");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_DIGIT_PROD_UNT_PERC, "0");
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROD_PEDA);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_DIGITAL_USE);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_INFO_BLOCK, course.getDescription(), _sourceResolver, true);
        
        Set<String> courses = new HashSet<String>();
        
        for (CourseList coursesList : course.getCourseLists())
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_COURSE_CONTENTS);
            
            // SAX infos on course list. This DO NOT respect the CDM-fr schema
            CDMHelper.courseListInfoToCDM(contentHandler, coursesList);
            
            List<String> childCourses = coursesList.getCourses();
            for (String courseID : childCourses)
            {
                Course childCourse = _resolver.resolveById(courseID);
                courses.add(childCourse.getId());
                attrs = new AttributesImpl();
                attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, childCourse.getCDMId());
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_REF_COURSE, attrs);
            }
            
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_COURSE_CONTENTS);
        }
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_COURSE_DESCRIPTION);
        
        // <teachingTerm term="" timeOfDay="" start=""/>
        attrs = new AttributesImpl();
        Date startDate = course.getStartDate();
        if (startDate != null)
        {
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_START, new SimpleDateFormat("yyyy-MM-dd").format(startDate));
        }
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_TERM, course.getTeachingTerm(_enumerationHelper));
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_TIME_OF_DAY, course.getTimeSlot(_enumerationHelper));
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEACHING_TERM, attrs);

        // <credits ECTScredits="" totalWorkLoad="">
        //      <infoBlock>...</infoBlock>
        //      <globalVolume teachingType="CM/TP/TD">...</globalVolume>
        // </credits>
        attrs = new AttributesImpl();
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_TOTAL_WORK_LOAD, course.getNumberOfHours());
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_ECTS_CREDITS, course.getEcts());
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_CREDITS, attrs);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, null, _sourceResolver, true);
        if (StringUtils.isNotEmpty(course.getTotalDurationOfCM()))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_TEACHING_TYPE, "CM");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_GLOBAL_VOLUME, attrs, course.getTotalDurationOfCM());
        }
        if (StringUtils.isNotEmpty(course.getTotalDurationOfTD()))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_TEACHING_TYPE, "TD");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_GLOBAL_VOLUME, attrs, course.getTotalDurationOfTD());
        }
        if (StringUtils.isNotEmpty(course.getTotalDurationOfTP()))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_TEACHING_TYPE, "TP");
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_GLOBAL_VOLUME, attrs, course.getTotalDurationOfTP());
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_CREDITS);
        
        // <learningObjectives>...</learningObjectives>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_LEARNING_OBJECTIVES, course.getObjectives(), _sourceResolver);
        
        // <admissionInfo>
        //      <studentPlaces places=""/>
        //      <teachingStart date=""/>
        // </admissionInfo>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ADMISSION_INFO);

        String maxNumberOfStudents = course.getMaxNumberOfStudents();
        if (StringUtils.isNotEmpty(maxNumberOfStudents))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_PLACES, maxNumberOfStudents);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_STUDENT_PLACES, attrs);
        }
        
        if (startDate != null)
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_DATE, new SimpleDateFormat("yyyy-MM-dd").format(startDate));
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEACHING_START, attrs);
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ADMISSION_INFO);
        
        // <recommendedPrerequisites>...</recommendedPrerequisites>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_RECOMMANDED_PREREQUISITES, course.getRecommendedPrerequisite(), _sourceResolver);

        // <formalPrerequisites>...</formalPrerequisites>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_FORMAL_PREREQUISITES, course.getNeededPrerequisite(), _sourceResolver, true);

        // <teachingPlace>
        //      <cdmfr:refOrgUnit />
        //      <adr>
        //          <pcode></pcode>
        //      </adr>
        // </teachingPlace>
        String[] places = course.getTeachingLocation();
        for (String postalCode : places)
        {
            if (StringUtils.isNotEmpty(postalCode))
            {
                XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_TEACHING_PLACE);
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_REF_ORG_UNIT);
                XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ADDRESS);
                XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PCODE, postalCode);
                XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ADDRESS);
                XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_TEACHING_PLACE);
            }
        }
        
        // <formOfTeaching method="" org=""/>
        String[] teachingOrgs = course.getTeachingOrgs(_enumerationHelper);
        for (String teachingOrg : teachingOrgs)
        {
            attrs = new AttributesImpl();
            _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_METHOD, course.getTeachingMethod(_enumerationHelper));
            _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_ORG, teachingOrg);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_FORM_OF_TEACHING, attrs);
        }
        
        // <formOfAssessment>...</formOfAssessment>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_FORM_OF_ASSESSMENT, course.getFormOfAssessment(), _sourceResolver);
        
        // <benefits>...</benefits>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_BENEFITS, course.getBenefits(), _sourceResolver);
        
        // <instructionLanguage teachingLang=""/>
        String teachingLanguage = course.getTeachingLanguage();
        if (StringUtils.isNotEmpty(teachingLanguage))
        {
            attrs = new AttributesImpl();
            attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_TEACHING_LANG, teachingLanguage);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_INSTRUCTION_LANGUAGE, attrs);
        }
        
        // <syllabus>
        // ...
        //   <webLink role="learningObjectMetadata">
        //      <href>...</href>
        //      <linkName>...</linkName>
        //   </webLink>
        // </syllabus>
        RichText syllabus = course.getSyllabus();
        Set<LOMSheet> lomSheets = course.getLOMSheets();
        if (syllabus != null || !lomSheets.isEmpty())
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_SYLLABUS);
            if (syllabus != null)
            {
                CDMHelper.richText2CDM(contentHandler, course.getSyllabus(), _sourceResolver);
            }
            for (LOMSheet lomSheet : lomSheets)
            {
                lomSheet.toCDM(contentHandler);
            }
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_SYLLABUS);
        }
        
        // <teachingActivity method=""/>
        attrs = new AttributesImpl();
        _addNotNullAttribute(attrs, CDMFRTagsConstants.ATTRIBUTE_METHOD, course.getTeachingActivity(_enumerationHelper));
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEACHING_ACTIVITY, attrs);
        
        // <contacts>
        //      <refPerson idRef=ID/>
        // </contacts>
        _course2CDMContacts(contentHandler, course, persons, orgUnits);
        
        // <infoBlock>...</infoBlock>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, course.getAdditionalInformations(), _sourceResolver);
        
        // <infoBlock userDefined="ametys-extension>...</infoBlock>
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute("userDefined", "ametys-extension");
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs);
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        for (String id : _cdmFrExtensionPoint.getExtensionsIds())
        {
            CDMfrExtension cdmFrExtension = _cdmFrExtensionPoint.getExtension(id);
            cdmFrExtension.course2CDM(contentHandler, course, persons, orgUnits);
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK);
        
        // <searchword>keyword</searchword>
        for (String keyword : course.getKeywords())
        {
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_SEARCH_WORD, keyword);
        }
       
        // </course>
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_COURSE);
        
        return courses;
    }
    
    private void _course2CDMContacts(ContentHandler contentHandler, Course course, Set<String> persons, Set<String> orgUnits) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_CONTACTS);
        
        for (String contactID : course.getPersonsInCharge())
        {
            if (!"".equals(contactID))
            {
                try
                {
                    Person person = _resolver.resolveById(contactID);
                    persons.add(person.getId());
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, person.getCDMId());
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ROLE, "responsible");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_PERSON, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // Contact does exist anymore, ignore it
                }
                
            }
        }
        
        for (String contactID : course.getContacts())
        {
            if (!"".equals(contactID))
            {
                try
                {
                    Person person = _resolver.resolveById(contactID);
                    persons.add(person.getId());
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, person.getCDMId());
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ROLE, "contact");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_PERSON, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // Contact does exist anymore, ignore it
                }
            }
        }
        
        for (String orgUnitID : course.getOrgUnits())
        {
            if (!"".equals(orgUnitID))
            {
                try
                {
                    OrgUnit orgUnit = _resolver.resolveById(orgUnitID);
                    orgUnits.add(orgUnit.getId());
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, orgUnit.getCDMId());
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_ORG_UNIT, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // OrgUnit does exist anymore, ignore it
                }
            }
        }
        
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_CONTACTS);
    }
    
    
    //***********************************************************************//
    //                              ORGUNIT                                  //
    //***********************************************************************//
    
    /**
     * Export this entity in CDM-fr format.
     * @param contentHandler the target handler
     * @param orgunit the orgunit
     * @throws SAXException if an error occurs during export.
     */
    public void orgunit2CDM(ContentHandler contentHandler, OrgUnit orgunit) throws SAXException
    {
        Set<String> persons = new HashSet<String>();
        
        orgunit2CDM(contentHandler, orgunit, persons, true);
        
        for (String personId : persons)
        {
            Person person = _resolver.resolveById(personId);
            person2CDM(contentHandler, person);
        }
        
        if (!orgunit.getId().equals(_rootOrgUnitProvider.getRootId()))
        {
            OrgUnit rootOrgunit = _resolver.resolveById(_rootOrgUnitProvider.getRootId());
            orgunit2CDM(contentHandler, rootOrgunit, persons);
        }
        
        // Set content in request
        ContextHelper.getRequest(_context).setAttribute(Content.class.getName(), orgunit);
    }
    
    private void orgunit2CDM(ContentHandler contentHandler, OrgUnit orgunit, Set<String> persons) throws SAXException
    {
        orgunit2CDM(contentHandler, orgunit, persons, false);
    }
    
    private void orgunit2CDM(ContentHandler contentHandler, OrgUnit orgunit, Set<String> persons, boolean showChildren) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        // <orgunit id="FR_RNE_{rootOrgUnitId}_OR_{login}">
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID, orgunit.getCDMId());
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT, attrs);
        
        // <orgUnitID>RNE</orgUnitID>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT_ID, orgunit.getCDMId());
        
        // <orgUnitName><text>name</text></orgUnitID>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT_NAME);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEXT, orgunit.getTitle());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT_NAME);
        
        // <orgUnitAcronym>acronyme</orgUnitAcronym>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT_ACRONYM, orgunit.getAcronym());
        
        // <orgUnitCode codeSet="codeUAI">code</orgUnitCode>
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_CODE_SET, "codeUAI");
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT_CODE, attrs, orgunit.getUAICode());

        // <orgUnitKind/>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT_KIND);
        
        // <webLink>
        //      <href>href</href>
        //      <linkName>linkName</linkName>
        // </webLink>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_WEB_LINK);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_HREF, orgunit.getWebLinkURL());
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_LINK_NAME, orgunit.getWebLinkLabel());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_WEB_LINK);
        
        // <orgUnitDescription>...</orgUnitDescription>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT_DESCRIPTION, orgunit.getDescription(), _sourceResolver);
        
        // <admissionInfo>
        //      <admissionDescription>...</admissionDescription>
        // </admissionInfo>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ADMISSION_INFO);
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_ADMISSION_DESCRIPTION, orgunit.getAdmissionInfo(), _sourceResolver);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ADMISSION_INFO);
        
        // <regulations>...</regulations>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_REGULATIONS, orgunit.getRegulations(), _sourceResolver);
        
        // <expenses>...</expenses>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_EXPENSES, orgunit.getExpenses(), _sourceResolver);
        
        // <studentFacilities>...</studentFacilities>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_STUDENT_FACILITIES, orgunit.getStudentFacilities(), _sourceResolver);
        
        // <universalAjustment>...</universalAjustment>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_UNIVERSAL_ADJUSTMENT, orgunit.getUniversalAdjustment(), _sourceResolver);
        
        // <contacts>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_CONTACTS);
        
        for (String contactID : orgunit.getContacts())
        {
            if (StringUtils.isNotEmpty(contactID))
            {
                try
                {
                    Person person = _resolver.resolveById(contactID);
                    persons.add(person.getId());
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, person.getCDMId());
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ROLE, "contact");
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_PERSON, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // Contact does exist anymore, ignore it
                }
            }
        }
        
        // </contacts>
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_CONTACTS);
        
        // <infoBlock>...</infoBlock>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, orgunit.getAdditionnalInfos(), _sourceResolver);

        Set<OrgUnit> orgUnits = new HashSet<OrgUnit>();
        if (showChildren)
        {
            // <refOrgUnit IDref=""/>
            for (String orgUnitID : orgunit.getSubOrgUnits())
            {
                try
                {
                    OrgUnit orgUnit = _resolver.resolveById(orgUnitID);
                    orgUnits.add(orgUnit);
                    attrs = new AttributesImpl();
                    attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID_REF, orgUnit.getCDMId());
                    XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_REF_ORG_UNIT, attrs);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // OrgUnit does exist anymore, ignore it
                }
            }
        }
        
        // <infoBlock userDefined="ametys-extension>...</infoBlock>
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute("userDefined", "ametys-extension");
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs);
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        for (String id : _cdmFrExtensionPoint.getExtensionsIds())
        {
            CDMfrExtension cdmFrExtension = _cdmFrExtensionPoint.getExtension(id);
            cdmFrExtension.orgunit2CDM(contentHandler, orgunit);
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK);
                
        // </orgunit>
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ORG_UNIT);

        for (OrgUnit childOrgunit : orgUnits)
        {
            orgunit2CDM(contentHandler, childOrgunit, persons);
        }
    }
    
    
    //***********************************************************************//
    //                              PERSON                                   //
    //***********************************************************************//
    
    /**
     * Export this entity in CDM-fr format.
     * @param contentHandler the target handler
     * @param person the person
     * @throws SAXException if an error occurs during export.
     */
    public void person2CDM(ContentHandler contentHandler, Person person) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        
        // <person id="FRUAI{rootOrgUnitId}PE{login}">
        attrs.addCDATAAttribute(CDMFRTagsConstants.ATTRIBUTE_ID, person.getCDMId());
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_PERSON, attrs);
        
        // <personID>login</personID>
        String login = person.getLogin();
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PERSON_ID, StringUtils.isNotEmpty(login) ? login : person.getName());
        
        // <name>
        //      <given>prénom</given>
        //      <family>nom</family>
        // </name>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_NAME);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_GIVEN_NAME, person.getGivenName());
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_FAMILY_NAME, person.getLastName());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_NAME);

        // <title><text>titre</text></title>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_TITLE);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEXT, person.getPersonTitle());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_TITLE);
        
        // <role><text>role</text></role>
        for (String role : person.getRole())
        {
            XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ROLE);
            XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_TEXT, role);
            XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ROLE);
        }
        
        // <affiliation>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_AFFILIATION);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_REF_ORG_UNIT);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_PROFESS_FIELD);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_SECT_CNU);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.NAMESPACE_CDMFR + CDMFRTagsConstants.TAG_AFFILIATION);
        
        
        // <contactData>
        ContactData contactData = person.getContactData();
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_CONTACT_DATA);
        
        // <adr>
        //      <extadr>extadr</extadr>
        //      <street>address</street>
        //      <locality>locality</locality>
        //      <pcode>pcode</pcode>
        // </adr>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_ADDRESS);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_EXTADR, contactData.getAdditionalAddress());
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_STREET, contactData.getAddress());
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_LOCALITY, contactData.getTown());
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PCODE, contactData.getZipCode());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_ADDRESS);
        
        // <telephone>telephone</telephone>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_PHONE, contactData.getPhone());
        
        // <fax>fax</fax>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_FAX, contactData.getFax());
        
        // <email>email</email>
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_EMAIL, contactData.getMail());
        
        // <webLink>
        //      <href>href</href>
        //      <linkName>linkName</linkName>
        // </webLink>
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_WEB_LINK);
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_HREF, contactData.getWebLinkUrl());
        XMLUtils.createElement(contentHandler, CDMFRTagsConstants.TAG_LINK_NAME, contactData.getWebLinkLabel());
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_WEB_LINK);
        
        // </contactData>
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_CONTACT_DATA);
        
        // <infoBlock>...</infoBlock>
        CDMHelper.richText2CDM(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, person.getAdditionalInformations(), _sourceResolver);
        
        // <infoBlock userDefined="ametys-extension>...</infoBlock>
        attrs = new AttributesImpl();
        attrs.addCDATAAttribute("userDefined", "ametys-extension");
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK, attrs);
        XMLUtils.startElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        for (String id : _cdmFrExtensionPoint.getExtensionsIds())
        {
            CDMfrExtension cdmFrExtension = _cdmFrExtensionPoint.getExtension(id);
            cdmFrExtension.person2CDM(contentHandler, person);
        }
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_EXTENSION_BLOCK);
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_INFO_BLOCK);

        // </person>
        XMLUtils.endElement(contentHandler, CDMFRTagsConstants.TAG_PERSON);
    }
    
    
    //***********************************************************************//
    //                               MISC                                    //
    //***********************************************************************//
    
    private void _addNotNullAttribute(AttributesImpl attrs, String localName, String value)
    {
        if (StringUtils.isNotEmpty(value))
        {
            attrs.addCDATAAttribute(localName, value);
        }
    }
}
