/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.cdmfr;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;

import org.ametys.cms.repository.WorkflowAwareContent;
import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.odf.CallWSHelper;
import org.ametys.runtime.config.Config;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.FunctionProvider;
import com.opensymphony.workflow.WorkflowException;

/**
 * Send CDM-fr to a distant server
 *
 */
public class SendCDMFRFunction extends AbstractContentWorkflowComponent implements FunctionProvider, Initializable
{
    private boolean _isActive;
    private SourceResolver _sourceResolver;
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        super.service(manager);
        _sourceResolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
    }
    
    @Override
    public void initialize() throws Exception
    {
        _isActive = Config.getInstance().getValueAsBoolean("odf.publish.server");
    }

    @Override
    public void execute(Map transientVars, Map args, PropertySet ps) throws WorkflowException
    {
        if (!_isActive)
        {
            return;
        }
        
        // Retrieve current content
        WorkflowAwareContent content = getContent(transientVars);
        
        InputStream is = null;
        try
        {
            // Generate CDM-fr file
            Source cdmfrSource = _sourceResolver.resolveURI("cocoon://_plugins/odf/export-cdmfr.xml?id=" + content.getId());

            // Send to remote server
            is = cdmfrSource.getInputStream();
            
            Set<String> failedUrl = CallWSHelper.callWS("_odf-sync/upload-cdm", is, _logger);
            if (failedUrl == null || failedUrl.size() > 0)
            {
                getMapToRender(transientVars).put("cdmfr-publication-error", content.getTitle());
                if (failedUrl != null)
                {
                    getMapToRender(transientVars).put("cdmfr-publication-failed-url", StringUtils.join(failedUrl, ", "));
                }
                _logger.error("The program " + content.getId() + " can't be synchronized with portals" + (failedUrl != null ? " " + StringUtils.join(failedUrl, ", ") : ""));
            }
        }
        catch (IOException e)
        {
            getMapToRender(transientVars).put("cdmfr-publication-error", content.getTitle());
            _logger.error("Unable to publish CDM-fr on distant server", e);
        }
        finally
        {
            IOUtils.closeQuietly(is);
        }
    }
}
