/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.clientsideelement;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;

import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.odf.course.Course;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.RootOrgUnitProvider;
import org.ametys.odf.person.Person;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.lock.LockHelper;
import org.ametys.plugins.repository.lock.LockableAmetysObject;
import org.ametys.runtime.right.RightsManager.RightResult;
import org.ametys.runtime.ui.impl.StaticContextualClientSideElement;
import org.ametys.runtime.user.User;
import org.ametys.runtime.user.UsersManager;
import org.ametys.runtime.util.I18nizableText;

/**
 * This element creates an action button to delete a content
 */
public class DeleteContentClientSideElement extends StaticContextualClientSideElement implements Contextualizable
{
    /** Repository content */
    protected AmetysObjectResolver _resolver;

    /** Avalon context */
    protected Context _context;

    /** The users manager */
    protected UsersManager _usersManager;
    
    /** The root orgunit provider */
    protected RootOrgUnitProvider _rootOrgUnitProvider;

    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        super.service(manager);
        _resolver = (AmetysObjectResolver) manager.lookup(AmetysObjectResolver.ROLE);
        _usersManager = (UsersManager) manager.lookup(UsersManager.ROLE);
        _rootOrgUnitProvider = (RootOrgUnitProvider) manager.lookup(RootOrgUnitProvider.ROLE);
    }

    @Override
    public void contextualize(Context context) throws ContextException
    {
        _context = context;
    }

    @Override
    public Map<String, I18nizableText> getCurrentParameters(Map<String, Object> parameters)
    {
        Map<String, I18nizableText> results = new HashMap<String, I18nizableText>();

        List<String> contentsId = (List<String>) parameters.get("targets");

        int programCount = 0;
        int containerCount = 0;
        int courseCount = 0;
        int orgunitCount = 0;
        int personCount = 0;
        
        int unauthorizedOrgUnitCount = 0;
        int lockedCount = 0;
        int unauthorizedCount = 0;

        for (String contentId : contentsId)
        {
            Content content = _resolver.resolveById(contentId);

            boolean canDelete = _rightsManager.hasRight(_currentUserProvider.getUser(), _getRightId(content), _computeContext(content)) == RightResult.RIGHT_OK;

            if (!canDelete || !(content instanceof ModifiableContent))
            {
                List<String> unauthorizedI18nParameters = new ArrayList<String>();
                unauthorizedI18nParameters.add(content.getTitle());

                I18nizableText ed = this._initialParameters.get("unauthorized-content-description");
                I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), unauthorizedI18nParameters);

                results.put("unauthorized-content-description-" + unauthorizedCount, msg);
                results.put("unauthorized-content-" + unauthorizedCount, new I18nizableText(content.getId()));
                results.put("unauthorized-content-title-" + unauthorizedCount, new I18nizableText(content.getTitle()));

                unauthorizedCount++;
            }
            else
            {
                if (content instanceof LockableAmetysObject && isNotUnlockable(content))
                {
                    String currentLockerLogin = ((LockableAmetysObject) content).getLockOwner();
                    User currentLocker = currentLockerLogin != null ? _usersManager.getUser(currentLockerLogin) : null;
                    List<String> lockI18nParameters = new ArrayList<String>();
                    lockI18nParameters.add(content.getTitle());
                    lockI18nParameters.add(currentLocker != null ? currentLocker.getFullName() : "");
                    lockI18nParameters.add(currentLockerLogin != null ? currentLockerLogin : "Anonymous");

                    I18nizableText ed = this._initialParameters.get("locked-content-description");
                    I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), lockI18nParameters);

                    results.put("locked-content-description-" + lockedCount, msg);
                    results.put("locked-content-" + lockedCount, new I18nizableText(content.getId()));
                    results.put("locked-content-title-" + lockedCount, new I18nizableText(content.getTitle()));
                    lockedCount++;
                }
                else if (content instanceof Course)
                {
                    List<String> courseI18nParameters = new ArrayList<String>();
                    courseI18nParameters.add(content.getTitle());

                    I18nizableText ed = this._initialParameters.get("course-content-description");
                    I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), courseI18nParameters);

                    results.put("course-content-description-" + courseCount, msg);
                    results.put("course-content-" + courseCount, new I18nizableText(content.getId()));
                    results.put("course-content-title-" + courseCount, new I18nizableText(content.getTitle()));

                    courseCount++; 
                }
                else if (content instanceof SubProgram || content instanceof Container)
                {
                    List<String> containerI18nParameters = new ArrayList<String>();
                    containerI18nParameters.add(content.getTitle());

                    I18nizableText ed = this._initialParameters.get("container-content-description");
                    I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), containerI18nParameters);

                    results.put("container-content-description-" + containerCount, msg);
                    results.put("container-content-" + containerCount, new I18nizableText(content.getId()));
                    results.put("container-content-title-" + containerCount, new I18nizableText(content.getTitle()));

                    containerCount++;
                }
                else if (content instanceof Program)
                {
                    List<String> programI18nParameters = new ArrayList<String>();
                    programI18nParameters.add(content.getTitle());

                    I18nizableText ed = this._initialParameters.get("program-content-description");
                    I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), programI18nParameters);

                    results.put("program-content-description-" + programCount, msg);
                    results.put("program-content-" + programCount, new I18nizableText(content.getId()));
                    results.put("program-content-title-" + programCount, new I18nizableText(content.getTitle()));

                    programCount++;
                }
                else if (content instanceof OrgUnit)
                {
                    if (_rootOrgUnitProvider.isRoot(content.getId()))
                    {
                        List<String> unauthorizedI18nParameters = new ArrayList<String>();
                        unauthorizedI18nParameters.add(content.getTitle());

                        I18nizableText ed = this._initialParameters.get("unauthorized-orgunit-description");
                        I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), unauthorizedI18nParameters);

                        results.put("unauthorized-orgunit-description-" + unauthorizedCount, msg);
                        unauthorizedOrgUnitCount++;
                    }
                    else
                    {
                        List<String> orgunitI18nParameters = new ArrayList<String>();
                        orgunitI18nParameters.add(content.getTitle());

                        I18nizableText ed = this._initialParameters.get("orgunit-content-description");
                        I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), orgunitI18nParameters);

                        results.put("orgunit-content-description-" + orgunitCount, msg);
                        results.put("orgunit-content-" + orgunitCount, new I18nizableText(content.getId()));
                        results.put("orgunit-content-title-" + orgunitCount, new I18nizableText(content.getTitle()));
                        orgunitCount++;
                    }
                }
                else if (content instanceof Person)
                {
                    
                    List<String> personI18nParameters = new ArrayList<String>();
                    personI18nParameters.add(content.getTitle());

                    I18nizableText ed = this._initialParameters.get("person-content-description");
                    I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), personI18nParameters);

                    results.put("person-content-description-" + personCount, msg);
                    results.put("person-content-" + personCount, new I18nizableText(content.getId()));
                    results.put("person-content-title-" + personCount, new I18nizableText(content.getTitle()));

                    personCount++;
                }
            }
        }
        results.put("person-content-size", new I18nizableText(Integer.toString(personCount)));
        results.put("orgunit-content-size", new I18nizableText(Integer.toString(orgunitCount)));
        results.put("program-content-size", new I18nizableText(Integer.toString(programCount)));
        results.put("container-content-size", new I18nizableText(Integer.toString(containerCount)));
        results.put("course-content-size", new I18nizableText(Integer.toString(courseCount)));
        
        results.put("unauthorized-orgunit-content-size", new I18nizableText(Integer.toString(unauthorizedOrgUnitCount)));
        results.put("locked-content-size", new I18nizableText(Integer.toString(lockedCount)));
        results.put("unauthorized-content-size", new I18nizableText(Integer.toString(unauthorizedCount)));

        return results;
    }
    
    private String _computeContext(Content content)
    {
        String ctxSuffixe = content.getName();
        if (content instanceof SubProgram || content instanceof Container)
        {
            AmetysObject parent = content.getParent();
            while (parent instanceof Content)
            {
                ctxSuffixe = parent.getName() + "/" + ctxSuffixe;
                parent = parent.getParent(); 
            }
        }
        
        return "/contents/" + ctxSuffixe;
    }
    
    /**
     * Test if a content is an instance of {@link LockableAmetysObject} and is not unlockable by the current user.
     * @param content the content to test.
     * @return true if the content is an instance of LockableAmetysObject and can't be unlocked by the current user.
     */
    protected boolean isNotUnlockable(Content content)
    {
        boolean isNotUnlockable = false;
        
        if (content instanceof LockableAmetysObject)
        {
            LockableAmetysObject lockableContent = (LockableAmetysObject) content;
            boolean canUnlockAll = _rightsManager.hasRight(_currentUserProvider.getUser(), "CMS_Rights_UnlockAll", "/contributor") == RightResult.RIGHT_OK;
            
            isNotUnlockable = lockableContent.isLocked() && !LockHelper.isLockOwner(lockableContent, _currentUserProvider.getUser()) && !canUnlockAll;
        }
        
        return isNotUnlockable;
    }

    private String _getRightId (Content content)
    {
        if (content instanceof Course)
        {
            return "ODF_Rights_Course_Delete";
        }
        else if (content instanceof SubProgram || content instanceof Container)
        {
            return "ODF_Rights_Container_Delete";
        }
        else if (content instanceof Program)
        {
            return "ODF_Rights_Program_Delete";
        }
        else if (content instanceof Person)
        {
            return "ODF_Rights_Person_Delete";
        }
        else if (content instanceof OrgUnit)
        {
            return "ODF_Rights_OrgUnit_Delete";
        }
        return "Workflow_Rights_Delete";
    }
}
