/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.contenttype;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.configuration.DefaultConfigurationBuilder;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.commons.io.IOUtils;

import org.ametys.cms.contenttype.DefaultContentType;
import org.ametys.cms.contenttype.Fieldset;
import org.ametys.cms.contenttype.MetadataDefinition;
import org.ametys.cms.contenttype.MetadataDefinitionReference;
import org.ametys.cms.contenttype.MetadataSetElement;
import org.ametys.cms.contenttype.MetadataType;
import org.ametys.cms.repository.Content;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.runtime.right.RightsManager.RightResult;
import org.ametys.runtime.util.I18nizableText;

/**
 * Type of content to be used for ODF contents
 */
public class ODFContentType extends DefaultContentType
{
    /** Remote suffix */
    public static final String REMOTE_SUFFIX = "_remote";
    /** Synchro suffix */
    public static final String SYNC_SUFFIX = "_sync";
    /** Synchro widget */
    public static final String SYNCHRO_WIDGET = "synchro";
    
    private static final Collection<String> _ENUMERATED_EXCLUDED_IDS = Arrays.asList("/dc");
    
    /** The synchronized fields */
    protected Set<String> _synchronizedFields = new HashSet<String>();
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        super.service(manager);
        _contentIndexer = (ODFContentIndexer) manager.lookup(ODFContentIndexer.ROLE);
    }
    
    @Override
    public void configure(Configuration configuration) throws ConfigurationException
    {
        _id = configuration.getAttribute("id");
        
        // Get the synchronized fields before
        _parseSynchronizedFields ();
        
        super.configure(configuration);
    }
    
    @Override
    protected MetadataDefinition _parseMetadata(Configuration metadataConfiguration, MetadataAndRepeaterDefinitionParser defParser) throws ConfigurationException
    {
        MetadataDefinition metadataDefinition = super._parseMetadata(metadataConfiguration, defParser);
        
        String metadataName = metadataDefinition.getName();
        
        if (isSynchronized(metadataName))
        {
            _metadata.put(metadataName + REMOTE_SUFFIX, _getRemoteMetadataDefinition (metadataName, metadataDefinition));
            _metadata.put(metadataName + SYNC_SUFFIX, _getSyncMetadataDefinition (metadataName, metadataDefinition));
        }
        
        return metadataDefinition;
    }
    
    @Override
    protected void _parseMetadataSets(Configuration config, boolean allowSameName) throws ConfigurationException
    {
        for (String metadataName : _metadata.keySet())
        {
            if (isSynchronized(metadataName) && !metadataName.endsWith(REMOTE_SUFFIX) && !metadataName.endsWith(SYNC_SUFFIX))
            {
                _metadata.get(metadataName + REMOTE_SUFFIX).setEnumerator(_metadata.get(metadataName).getEnumerator());
            }
        }
        super._parseMetadataSets(config, allowSameName);
    }
    
    /**
     * Get the synchronized metadata from the configuration file
     * @throws ConfigurationException if the configuration is not valid.
     */
    protected void _parseSynchronizedFields () throws ConfigurationException
    {
        File synchroFile = new File (_cocoonContext.getRealPath("/WEB-INF/param/odf-synchro.xml"));
        
        if (synchroFile.exists())
        {
            InputStream is = null;
            try
            {
                is = new FileInputStream(synchroFile);
                Configuration configuration = new DefaultConfigurationBuilder().build(is);
                
                _configureSynchronizedFields(configuration);
            }
            catch (Exception e)
            {
                throw new ConfigurationException("Unable to read the configuration file odf-synchro.xml", e);
            }
            finally
            {
                IOUtils.closeQuietly(is);
            }
        }
    }
    
    /**
     * Get the synchronized metadata
     * @param configuration the content types set configuration to use.
     * @throws ConfigurationException  if the configuration is not valid.
     */
    protected void _configureSynchronizedFields (Configuration configuration) throws ConfigurationException
    {
        Configuration[] cTypesConf = configuration.getChildren("content-type");
        for (Configuration cTypeConf : cTypesConf)
        {
            if (_id.equals(cTypeConf.getAttribute("id")))
            {
                Configuration[] metaConf = cTypeConf.getChildren("metadata");
                for (Configuration metadata : metaConf)
                {
                    String metadataName = metadata.getAttribute("name");
                    _synchronizedFields.add(metadataName);
                }
            }
        }
    }
    
    @Override
    protected void _fillMetadataSetElement(Configuration metadataSetConfig, MetadataSetElement metadataSetElement, boolean isEdition) throws ConfigurationException
    {
        for (Configuration elementConfig : metadataSetConfig.getChildren())
        {
            String elementConfigName = elementConfig.getName();
            
            if (elementConfigName.equals("metadata-ref"))
            {
                String metadataName = elementConfig.getAttribute("name");
                MetadataDefinitionReference metadataDefRef = new MetadataDefinitionReference(metadataName);
                
                if (!metadataSetElement.addElement(metadataDefRef))
                {
                    throw new ConfigurationException("Metadata with name " + metadataName + " is already referenced", elementConfig);
                }
                
                if (isEdition && isSynchronized(metadataName))
                {
                    MetadataDefinitionReference remoteDefRef = new MetadataDefinitionReference(metadataName + REMOTE_SUFFIX);
                    metadataSetElement.addElement(remoteDefRef);
                    
                    MetadataDefinitionReference syncDefRef = new MetadataDefinitionReference(metadataName + SYNC_SUFFIX);
                    metadataSetElement.addElement(syncDefRef);
                }

                _fillMetadataSetElement(elementConfig, metadataDefRef, isEdition);
            }
            else if (elementConfigName.equals("fieldset"))
            {
                Fieldset fieldset = new Fieldset();
                I18nizableText label = _parseI18nizableText(elementConfig, "label");
                fieldset.setLabel(label);
                _fillMetadataSetElement(elementConfig, fieldset, isEdition);
                metadataSetElement.addElement(fieldset);
            }
            else if (elementConfigName.equals("dublin-core"))
            {
                _fillMetadataSetDublinCore(metadataSetElement);
            }
        }
    }

    
    private MetadataDefinition _getRemoteMetadataDefinition (String metadataName, MetadataDefinition metadataDefinition)
    {
        MetadataDefinition remoteMetaDef = new MetadataDefinition();
        remoteMetaDef.setName(metadataName + REMOTE_SUFFIX);
        remoteMetaDef.setId(metadataDefinition.getId() + REMOTE_SUFFIX);
        MetadataType type = metadataDefinition.getType();
        remoteMetaDef.setType(type);
        if (type == MetadataType.RICH_TEXT)
        {
            remoteMetaDef.setRichTextTransformer(metadataDefinition.getRichTextTransformer());
        }
        remoteMetaDef.setLabel(metadataDefinition.getLabel());
        remoteMetaDef.setDescription(metadataDefinition.getDescription());
        remoteMetaDef.setMultiple(metadataDefinition.isMultiple());
        remoteMetaDef.setWidget("empty");
        String remoteFieldName = "content.input." + metadataName + REMOTE_SUFFIX;
        remoteMetaDef.setWidgetParameters(Collections.singletonMap("copyValueTo", new I18nizableText(remoteFieldName)));
        
        return remoteMetaDef;
    }
    
    private MetadataDefinition _getSyncMetadataDefinition (String metadataName, MetadataDefinition metadataDefinition)
    {
        MetadataDefinition syncMetaDef = new MetadataDefinition();
        syncMetaDef.setId(metadataDefinition.getId() + SYNC_SUFFIX);
        syncMetaDef.setName(metadataName + SYNC_SUFFIX);
        syncMetaDef.setType(MetadataType.BOOLEAN);
        syncMetaDef.setLabel(metadataDefinition.getLabel());
        syncMetaDef.setDescription(metadataDefinition.getDescription());
        syncMetaDef.setWidget("synchro-button");
        
        return syncMetaDef;
    }
    
    @Override
    public MetadataDefinition getMetadataDefinition(String metadataName)
    {
        MetadataDefinition metaDef = super.getMetadataDefinition(metadataName);
        if (metaDef == null)
        {
            return null;
        }
        
        String widget = metaDef.getWidget();
        if (isSynchronized(metadataName))
        {   
            if (widget != null && widget.startsWith("multiselect"))
            {
                metaDef.setWidget("multiselect-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("sorted-multiselect"))
            {
                metaDef.setWidget("sorted-multiselect-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("grouped-enumeration"))
            {
                metaDef.setWidget("grouped-enumeration-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("multiple-grouped-enumeration"))
            {
                metaDef.setWidget("multiple-grouped-enumeration-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("multi-disciplines"))
            {
                metaDef.setWidget("multi-disciplines-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("orgunit"))
            {
                metaDef.setWidget("orgunit-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("odf-person"))
            {
                metaDef.setWidget("odf-person-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("small-richtext"))
            {
                metaDef.setWidget("small-richtext-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("degree-enumeration"))
            {
                metaDef.setWidget("degree-enumeration-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("mention-enumeration"))
            {
                metaDef.setWidget("mention-enumeration-" + SYNCHRO_WIDGET);
            }
            else if (widget != null && widget.startsWith("code-uai"))
            {
                metaDef.setWidget("code-uai-" + SYNCHRO_WIDGET);
            }
            else
            {
                metaDef.setWidget(SYNCHRO_WIDGET);
            }
        }
        else if (widget != null && widget.endsWith(SYNCHRO_WIDGET))
        {
            // the metadata is no more synchronized, use the default widget
            widget = widget.substring(0, widget.indexOf(SYNCHRO_WIDGET));
            metaDef.setWidget(widget.length() > 0 ? widget : null);
        }
        
        return metaDef;
    }
    
    /**
     * Get the synchronized fields for this content type instance.
     * @return a Set of the synchronized field names.
     */
    public Set<String> getSynchronizedFields()
    {
        return Collections.unmodifiableSet(_synchronizedFields);
    }
    
    /**
     * Determines if a metadata is synchronized
     * @param metadataName The metadata name
     * @return true if the metadata is synchronized
     */
    public boolean isSynchronized (String metadataName)
    {
        // TODO check metadata synchronized on content
        return _synchronizedFields.contains(metadataName);
    }
    
    /**
     * Get the enumerated metadatas.
     * @return the enumerated metadatas definitions, indexed by metadata path.
     */
    public Map<String, MetadataDefinition> getEnumeratedMetadatas()
    {
        Map<String, MetadataDefinition> enumeratedMetas = new LinkedHashMap<String, MetadataDefinition>();
        
        fillEnumeratedMetas(enumeratedMetas);
        
        return enumeratedMetas;
    }
    
    @Override
    protected boolean _hasRights(Content content, Set<String> rightLimitations)
    {
        if (rightLimitations.isEmpty())
        {
            return true;
        }

        String login = _currentUserProvider.getUser();
        
        Content contentForCtx = content;
        if (contentForCtx instanceof Container || contentForCtx instanceof SubProgram)
        {
            contentForCtx = content.getParent();
            while (!(contentForCtx instanceof Program))
            {
                contentForCtx = contentForCtx.getParent();
            }
        }

        for (String rightId : rightLimitations)
        {
            if (_rightsManager.hasRight(login, rightId, "/contents/" + contentForCtx.getName()) == RightResult.RIGHT_OK)
            {
                return true;
            }
        }

        return false;
    }
    
    /**
     * Fill the map of enumerated metadatas.
     * @param enumeratedMetas the map of to fill.
     */
    private void fillEnumeratedMetas(Map<String, MetadataDefinition> enumeratedMetas)
    {
        Set<String> metadataNames = getMetadataNames();
        for (String metadataName : metadataNames)
        {
            MetadataDefinition metaDef = getMetadataDefinition(metadataName);
            String metadataPath = metadataName;
            
            if (!metadataName.endsWith(REMOTE_SUFFIX) && !_ENUMERATED_EXCLUDED_IDS.contains(metaDef.getId()) && metaDef.getEnumerator() != null)
            {
                enumeratedMetas.put(metadataPath, metaDef);
                
                fillEnumeratedMetas(metaDef, enumeratedMetas, metadataPath + "/");
            }
        }
    }
    
    /**
     * Recursively fill the map of enumerated metadatas.
     * @param parentMetaDef the current metadata definition.
     * @param enumeratedMetas the map of enumerated metadatas to fill.
     * @param prefix the metadata prefix.
     */
    private void fillEnumeratedMetas(MetadataDefinition parentMetaDef, Map<String, MetadataDefinition> enumeratedMetas, String prefix)
    {
        Set<String> metadataNames = parentMetaDef.getMetadataNames();
        for (String metadataName : metadataNames)
        {
            MetadataDefinition metaDef = parentMetaDef.getMetadataDefinition(metadataName);
            String metadataPath = prefix + metadataName;
            
            if (!metadataName.endsWith(REMOTE_SUFFIX) && !_ENUMERATED_EXCLUDED_IDS.contains(metaDef.getId()) && metaDef.getEnumerator() != null)
            {
                enumeratedMetas.put(metadataPath, metaDef);
                
                fillEnumeratedMetas(metaDef, enumeratedMetas, metadataPath + "/");
            }
        }
    }
    
}
