/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.contenttype;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
import org.xml.sax.SAXException;

import org.ametys.cms.contenttype.ContentType;
import org.ametys.cms.contenttype.MetadataDefinition;
import org.ametys.cms.contenttype.MetadataSetElement;
import org.ametys.cms.lucene.FieldNames;
import org.ametys.cms.lucene.MetadataIndexer;
import org.ametys.cms.repository.Content;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.metadata.CompositeMetadata;

/**
 * Metadata indexer specific to ODF contents, which handles synchronized metadata values.
 */
public class ODFMetadataIndexer extends MetadataIndexer
{
    /** The avalon role. */
    @SuppressWarnings("hiding")
    public static final String ROLE = ODFMetadataIndexer.class.getName();
    
    @Override
    public void indexMetadata(MetadataSetElement metadataSet, CompositeMetadata metadata, String prefix, Content content, ContentType cType, Document document, MetadataSetElement element, String name, String fieldName, MetadataDefinition definition) throws SAXException, IOException, Exception
    {
        if (cType instanceof ODFContentType)
        {
            String metadataNameToIndex = _getMetadataNameToIndex ((ODFContentType) cType, metadata, name);
            super.indexMetadata(metadataSet, metadata, prefix, content, cType, document, element, metadataNameToIndex, name, definition);
        }
    }
    
    @Override
    protected boolean hasMetadata(CompositeMetadata metadata, String metadataName)
    {
        if (metadata.getBoolean(metadataName + ODFContentType.SYNC_SUFFIX, false))
        {
            return metadata.hasMetadata(metadataName + ODFContentType.REMOTE_SUFFIX);
        }
        return super.hasMetadata(metadata, metadataName);
    }
    
    /**
     * Get the name of metadata to index
     * @param cType the ODF content type
     * @param metadataHolder The metadata holder
     * @param metadataName The metadata name
     * @return The true metadata name to index
     */
    protected String _getMetadataNameToIndex (ODFContentType cType, CompositeMetadata metadataHolder, String metadataName)
    {
        if (cType.isSynchronized(metadataName) && metadataHolder.getBoolean(metadataName + ODFContentType.SYNC_SUFFIX, false))
        {
            return metadataName + ODFContentType.REMOTE_SUFFIX;
        }
        
        return metadataName;
    }
    
    @Override
    public void indexStringMetadata(CompositeMetadata metadata, String prefix, Content content, Document document, String name, String fieldName, MetadataDefinition definition) throws Exception
    {
        super.indexStringMetadata(metadata, prefix, content, document, name, fieldName, definition);
        
        Set<String> parentOrgUnitsToIndex = new HashSet<String>();
        
        if ("orgUnit".equals(name) || "orgUnit_remote".equals(name))
        {
            String[] strValues = metadata.getStringArray(name, new String[0]);
            for (String id : strValues)
            {
                if (StringUtils.isNotEmpty(id))
                {
                    try
                    {
                        OrgUnit ou = _resolver.resolveById(id);
                        OrgUnit parentOrgUnit = ou.getParentOrgUnit();
                        while (parentOrgUnit != null)
                        {
                            parentOrgUnitsToIndex.add(parentOrgUnit.getId());
                            parentOrgUnit = parentOrgUnit.getParentOrgUnit();
                        }
                    }
                    catch (UnknownAmetysObjectException ex)
                    {
                        _logger.warn("There's no orgunit to index for id : " + id + ". It is ignored.", ex);
                    }
                }
                
            }
        }
        
        for (String value : parentOrgUnitsToIndex)
        {
            document.add(new Field(FieldNames.FULL, value, Store.NO, Index.ANALYZED));
            document.add(new Field(fieldName, value, Store.YES, definition.getEnumerator() == null ? Index.ANALYZED : Index.NOT_ANALYZED));
            
            String[] words = StringUtils.split(value);
            for (String word : words)
            {
                document.add(new Field(FieldNames.ALL_NOT_ANALYZED, StringUtils.strip(word, ",?!:()[].{}=").toLowerCase(), Store.NO, Index.NOT_ANALYZED));
            }
        }
    }
    
}
