/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.course;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;

import org.apache.jackrabbit.util.Text;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.cdmfr.CDMEntity;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseListContainer;
import org.ametys.odf.courselist.CourseListFactory;
import org.ametys.odf.enumeration.OdfEnumerationConstant;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.odf.program.Program;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.CollectionIterable;
import org.ametys.plugins.repository.RepositoryIntegrityViolationException;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.jcr.JCRTraversableAmetysObject;
import org.ametys.plugins.repository.metadata.CompositeMetadata;
import org.ametys.plugins.repository.metadata.ModifiableCompositeMetadata;
import org.ametys.plugins.repository.metadata.RichText;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;
import org.ametys.plugins.repository.query.QueryHelper;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.StringExpression;

/**
 * Class representing a {@link Course}
 */
public class Course extends SynchronizableContent<CourseFactory> implements JCRTraversableAmetysObject, CourseListContainer, CDMEntity
{
    /** Constants for elpCode Metadata* */
    public static final String ELP_CODE = "elpCode";

    /** Mandatory Identifier to generate the CDM-fr id */
    public static final String CDM_CODE = "cdmCode";
    
    /** Constants for ects Metadata* */
    public static final String ECTS = "ects";

    /** Constants for level Metadata* */
    public static final String LEVEL = "level";

    /** Constants for description Metadata* */
    public static final String DESCRIPTION = "description";

    /** Constants for objectives Metadata* */
    public static final String OBJECTIVES = "objectives";

    /** Constants for nbHours Metadata* */
    public static final String NUMBER_OF_HOURS = "nbHours";

    /** Constants for totalDurationOfCM Metadata* */
    public static final String CM_DURATION = "totalDurationOfCM";

    /** Constants for totalDurationOfTP Metadata* */
    public static final String TP_DURATION = "totalDurationOfTP";

    /** Constants for totalDurationOfTD Metadata* */
    public static final String TD_DURATION = "totalDurationOfTD";

    /** Constants for neededPrerequisite Metadata* */
    public static final String NEEDED_PREREQUISITE = "neededPrerequisite";

    /** Constants for recommendedPrerequisite Metadata* */
    public static final String RECOMMENDED_PREREQUISITE = "recommendedPrerequisite";

    /** Constants for formOfAssessment Metadata* */
    public static final String FORM_OF_ASSESSMENT = "formOfAssessment";
    
    /** Constants for benefits Metadata* */
    public static final String BENEFITS = "benefits";

    /** Constants for syllabus Metadata* */
    public static final String SYLLABUS = "syllabus";

    /** Constants for additionalInformations Metadata* */
    public static final String ADDITIONAL_INFORMATIONS = "additionalInformations";

    /** Constants for erasmusCode Metadata* */
    public static final String ERASMUS_CODE = "erasmusCode";

    /** Constants for teachingLocation Metadata* */
    public static final String TEACHING_LOCATION = "teachingLocation";

    /** Constants for maxNumberOfStudents Metadata* */
    public static final String MAX_NUMBER_OF_STUDENTS = "maxNumberOfStudents";

    /** Constants for teachingTerm Metadata* */
    public static final String TEACHING_TERM = "teachingTerm";

    /** Constants for timeSlot Metadata* */
    public static final String TIME_SLOT = "timeSlot";

    /** Constants for trainingCourseDuration Metadata* */
    public static final String TRAINING_COURSE_DURATION = "trainingCourseDuration";

    /** Constants for teachingMethod Metadata* */
    public static final String TEACHING_METHOD = "formofteachingMethod";
    
    /** Constants for formofteachingOrg Metadata* */
    public static final String TEACHING_ORG = "formofteachingOrg";

    /** Constants for formOfTeaching Metadata* */
    public static final String TEACHING_ACTIVITY = "teachingActivity";

    /** Constants for teachingLanguage Metadata* */
    public static final String TEACHING_LANGUAGE = "teachingLanguage";

    /** Constants for startDate Metadata* */
    public static final String START_DATE = "startDate";

    /** Constants for keywords Metadata* */
    public static final String KEYWORDS = "keywords";

    /** Constants for webLinkLabel Metadata* */
    public static final String KNOW_MORE_WEB_LINK_LABEL = "webLinkLabel";

    /** Constants for webLinkUrl Metadata* */
    public static final String KNOW_MORE_WEB_LINK_URL = "webLinkUrl";

    /** Constants for lomSheets Metadata */
    public static final String LOM_SHEETS = "lomSheets";

    /** Constants for LOM Sheet URL metadata */
    public static final String LOM_SHEET_URL = "linkUrl";
    
    /** Constants for LOM Sheet label metadata */
    public static final String LOM_SHEET_LABEL = "linkLabel";
    
    /**
     * References
     */
    
    /** Constants for contentType Metadata* */
    public static final String ORG_UNITS_REFERENCES = "orgUnit";
    
    /** Constants for metadata 'personInCharge' */
    public static final String PERSONS_IN_CHARGE_REFERENCES = "personInCharge";

    /** Constants for metadata 'contact' */
    public static final String CONTACTS_REFERENCES = "contact";
    
    private String _contextPath;

    /**
     * Constructor.
     * @param node the JCR Node.
     * @param parentPath the parent path
     * @param factory the corresponding factory.
     */
    public Course(Node node, String parentPath, CourseFactory factory)
    {
        super(node, parentPath, factory);
    }

    /*@SuppressWarnings("unchecked")
    @Override
    public CourseList createChild(String name, String type) throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        return _getFactory().createChild(getPath(), getNode(), name, type);
    }*/
    
    @SuppressWarnings("unchecked")
    @Override
    public CourseList createChild(String name, String type) throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        return _getFactory().createChild(this, name, type);
    }
    
    @SuppressWarnings("unchecked")
    @Override
    public CourseList getChild(String path) throws AmetysRepositoryException, UnknownAmetysObjectException
    {
        return _getFactory().getChild(path, this);
    }

    @Override
    public AmetysObjectIterable<AmetysObject> getChildren() throws AmetysRepositoryException
    {
        return _getFactory().getChildren(this);
    }

    @Override
    public boolean hasChild(String name) throws AmetysRepositoryException
    {
        return _getFactory().hasChild(name, this);
    }
    
    @Override
    public CourseList createCourseList(String name)
    {
        String legalName = Text.escapeIllegalJcrChars(name);
        
        // Find unique name
        int index = 1;
        while (hasChild(legalName))
        {
            legalName = Text.escapeIllegalJcrChars(name + " " + (index++));
        }
        
        return this.createChild(legalName, CourseListFactory.COURSE_LIST_NODETYPE);
    }
    
    @Override
    public AmetysObjectIterable<CourseList> getCourseLists()
    {
        List<CourseList> list = new ArrayList<CourseList>();
        
        AmetysObjectIterable<AmetysObject> children = getChildren();
        for (AmetysObject child : children)
        {
            if (child instanceof CourseList)
            {
                list.add((CourseList) child);
            }
        }

        return new CollectionIterable<CourseList>(list);
    }

    /**
     * Test if this {@link Course} has an {@link CourseList} as child
     * @return A boolean
     */
    public boolean hasCourseList()
    {
        try
        {
            NodeIterator nodes = this.getNode().getNodes();
            while (nodes.hasNext())
            {
                Node child = nodes.nextNode();
                if (child.getPrimaryNodeType().getName().equals(CourseListFactory.COURSE_LIST_NODETYPE))
                {
                    return true;
                }
            }
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException(e);
        }

        return false;
    }
    
    /**
     * Get the referencing programs
     * @return the referencing programs
     */
    public List<Program> getReferencingPrograms ()
    {
        List<Program> programs = new ArrayList<Program>();
        
        Expression courseListExpression = new StringExpression(CourseList.COURSES_REFERENCES, Operator.EQ, getId());
        String courseListQuery = QueryHelper.getXPathQuery(null, "ametys:courseList", courseListExpression);
        
        AmetysObjectIterable<CourseList> lists = _getFactory()._getResolver().query(courseListQuery);
        
        for (CourseList list : lists)
        {
            programs.addAll(list.getReferencingPrograms());
        }
        
        return programs;
    }
    
    /**
     * Remove reference from local and remote metadata
     * @param metadataName The metadata name
     * @param value The value of reference to remove 
     */
    public void removeReference (String metadataName, String value)
    {
        String[] localReferences = getMetadataHolder().getStringArray(metadataName, new String[0]);
        List<String> localList = new ArrayList<String>(Arrays.asList(localReferences));
        if (localList.contains(value))
        {
            localList.remove(value);
        }
        getMetadataHolder().setMetadata(metadataName, localList.toArray(new String[localList.size()]));
        
        String[] remoteReferences = getMetadataHolder().getStringArray(metadataName + REMOTE_SUFFIX, new String[0]);
        List<String> remoteList = new ArrayList<String>(Arrays.asList(remoteReferences));
        if (remoteList.contains(value))
        {
            remoteList.remove(value);
        }
        getMetadataHolder().setMetadata(metadataName + REMOTE_SUFFIX, remoteList.toArray(new String[remoteList.size()]));
    }
    
    // --------------------------------------------------------------------------------------//
    // CONTACTS
    // --------------------------------------------------------------------------------------//

    /**
     * Return the list of Persons in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getPersonsInCharge()
    {
        String[] contacts = getStringArray(PERSONS_IN_CHARGE_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return the list of Contacts in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getContacts()
    {
        String[] contacts = getStringArray(CONTACTS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
   
    
    // --------------------------------------------------------------------------------------//
    // CONTEXT
    // --------------------------------------------------------------------------------------//
    /**
     * Set the parent path for links and breadcrumb
     * @param path the parent path
     */
    public void setContextPath (String path)
    {
        _contextPath = path;
    }
    
    /**
     * Get the parent path. Can be null.
     * @return the parent path
     */
    public String getContextPath ()
    {
        return _contextPath;
    }
    
    // --------------------------------------------------------------------------------------//
    // ORGUNITS
    // --------------------------------------------------------------------------------------//
    /**
     * Return the list of orgUnits binded to this course 
     * @return a list of uuid
     */
    public List<String> getOrgUnits()
    {
        String[] orgunits = getStringArray(ORG_UNITS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(orgunits));
    }

    /**
     * Bind a orgUnit element to this course
     * @param orgUnitId the uuid of a orgunit element
     */
    public void addOrgUnit(String orgUnitId)
    {
        String[] orgunits = getStringArray(ORG_UNITS_REFERENCES);
        
        List<String> orgunitsList = new ArrayList<String>(Arrays.asList(orgunits));
        orgunitsList.add(orgUnitId);
        
        getMetadataHolder().setMetadata(ORG_UNITS_REFERENCES, orgunitsList.toArray(new String[orgunitsList.size()]));
    }

    /**
     * Remove a orgunit from the list
     * @param orgUnitId the uuid of a orgunit element
     * @throws AmetysRepositoryException
     */
    public void removeOrgUnit(String orgUnitId) throws AmetysRepositoryException
    {
        String[] orgunits = getStringArray(ORG_UNITS_REFERENCES);
        
        List<String> orgunitsList = new ArrayList<String>(Arrays.asList(orgunits));
        if (orgunitsList.contains(orgUnitId))
        {
            orgunitsList.remove(orgUnitId);
        }
        
        getMetadataHolder().setMetadata(ORG_UNITS_REFERENCES, orgunitsList.toArray(new String[orgunitsList.size()]));
    }

    /**
     * Return the list of orgUnits binded to this course 
     * @return a list of uuid
     */
    public List<String> getLocalOrgUnits()
    {
        String[] orgunits = getMetadataHolder().getStringArray(ORG_UNITS_REFERENCES, new String[] {});
        return new ArrayList<String>(Arrays.asList(orgunits));
    }

    /**
     * Return the list of remote orgUnits binded to this course 
     * @return a list of uuid
     */
    public List<String> getRemoteOrgUnits()
    {
        String[] orgunits = getMetadataHolder().getStringArray(ORG_UNITS_REFERENCES + REMOTE_SUFFIX, new String[] {});
        return new ArrayList<String>(Arrays.asList(orgunits));
    }
    
    // --------------------------------------------------------------------------------------//
    // GETTERS AND SETTERS
    // --------------------------------------------------------------------------------------//

    /**
     * Get the code of the {@link Course}
     * @return the ELP code
     */
    public String getElpCode()
    {
        return getString(ELP_CODE);
    }

    /**
     * Set the ELP code
     * @param code the code to set
     * @throws AmetysRepositoryException
     */
    public void setElpCode(String code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.ELP_CODE, code);
    }
    
    /**
     * Get the CDM-fr of the {@link Course}
     * @return the CDM-fr code
     */
    public String getCdmCode()
    {
        return getString(CDM_CODE);
    }
    /**
     * Set the CDM-fr code
     * @param code the CDM-fr to set
     * @throws AmetysRepositoryException
     */
    public void setCdmCode(String code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.CDM_CODE, code);
    }
    
    /**
     * Get the description
     * @return the description or null
     */
    public RichText getDescription()
    {
        return getRichText(DESCRIPTION);
    }

    /**
     * Get the objectives
     * @return objectives
     */
    public RichText getObjectives()
    {
        return getRichText(OBJECTIVES);
    }

    /**
     * Get the CM duration
     * @return the CM duration
     */
    public String getTotalDurationOfCM()
    {
        return getString(CM_DURATION);
    }
    /**
     * Set the CM duration
     * @param cm the CM duration
     */
    public void setTotalDurationOfCM(String cm)
    {
        getMetadataHolder().setMetadata(CM_DURATION, cm);  
    }

    /**
     * Get the TD duration
     * @return the TD duration
     */
    public String getTotalDurationOfTD()
    {
        return getString(TD_DURATION);
    }
    /**
     * Set the TD duration
     * @param td 
     */
    public void setTotalDurationOfTD(String td)
    {
        getMetadataHolder().setMetadata(TD_DURATION, td);  
    }

    /**
     * Get the TP duration
     * @return the TP duration
     * @throws UnknownMetadataException
     * @throws AmetysRepositoryException
     */
    public String getTotalDurationOfTP() throws AmetysRepositoryException, UnknownMetadataException
    {
        return getString(TP_DURATION);
    }
    /**
     * Set the TP duration
     * @param tp 
     * @throws UnknownMetadataException
     * @throws AmetysRepositoryException
     */
    public void setTotalDurationOfTP(String tp) throws AmetysRepositoryException, UnknownMetadataException
    {
        getMetadataHolder().setMetadata(TP_DURATION, tp);
    }

    /**
     * Get the needed prerequisites
     * @return the needed prerequisites or null if not set
     */
    public RichText getNeededPrerequisite()
    {
        return getRichText(NEEDED_PREREQUISITE);
    }


    /**
     * Get the recommended prerequisites
     * @return  the recommended prerequisites or null if not set
     */
    public RichText getRecommendedPrerequisite()
    {
        return getRichText(RECOMMENDED_PREREQUISITE);
    }

    /**
     * Get the formOfAssessment metadata
     * @return the formOfAssessment or null if not set
     */
    public RichText getFormOfAssessment()
    {
        return getRichText(FORM_OF_ASSESSMENT);
    }
    
    /**
     * Get the benefits metadata
     * @return the benefits or null if not set
     */
    public RichText getBenefits()
    {
        return getRichText(BENEFITS);
    }

    /**
     * Get the syllabus
     * @return the syllabus or null if not set
     */
    public RichText getSyllabus()
    {
        return getRichText(SYLLABUS);
    }
    
    /**
     * Get the list of LOM sheets
     * @return the list of LOMsheets or an empty list
     */
    public Set<LOMSheet> getLOMSheets()
    {
        Set<LOMSheet> lomSheets = new HashSet<LOMSheet>();
        try
        {
            CompositeMetadata metaLomSheets = getMetadataHolder().getCompositeMetadata(LOM_SHEETS);
            for (String metadataName : metaLomSheets.getMetadataNames())
            {
                CompositeMetadata metaLomSheet = metaLomSheets.getCompositeMetadata(metadataName);
                LOMSheet lomSheet = new LOMSheet(metaLomSheet.getString(LOM_SHEET_URL, ""), metaLomSheet.getString(LOM_SHEET_LABEL, ""));
                lomSheets.add(lomSheet);
            }
        }
        catch (UnknownMetadataException e)
        {
            // Nothing to do
        }
        return lomSheets;
    }
    
    /**
     * Set the LOM sheets
     * @param sheets The LOM sheets
     */
    public void setLOMSheets (Set<LOMSheet> sheets)
    {
        ModifiableCompositeMetadata metaLomSheets = getMetadataHolder().getCompositeMetadata(LOM_SHEETS, true);
        
        // Remove old LOM sheets if exist
        String[] chilNames = metaLomSheets.getMetadataNames();
        for (String chilName : chilNames)
        {
            metaLomSheets.removeMetadata(chilName);
        }
        
        int i = 1;
        for (LOMSheet lomSheet : sheets)
        {
            ModifiableCompositeMetadata metaLomSheet = metaLomSheets.getCompositeMetadata(String.valueOf(i), true);
            metaLomSheet.setMetadata(LOM_SHEET_URL, lomSheet.getUrl());
            metaLomSheet.setMetadata(LOM_SHEET_LABEL, lomSheet.getLabel());
            i++;
        }
    }
    
    /**
     * Determines if the course has LOM sheet
     * @param lomSheet The LOM sheet to test
     * @return <code>true</code> if the course has LOM sheet
     */
    public boolean hasLOMSheet (LOMSheet lomSheet)
    {
        for (LOMSheet lom : getLOMSheets())
        {
            if (lom.getUrl().equals(lomSheet.getUrl()) && lom.getLabel().equals(lomSheet.getLabel()))
            {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * Get the additional information
     * @return the additional information
     */
    public RichText getAdditionalInformations()
    {
        return getRichText(ADDITIONAL_INFORMATIONS);
    }

    /**
     * Get the Erasmus code
     * @return the Erasmus code
     */
    public String getErasmusCode()
    {
        return getString(ERASMUS_CODE);
    }

    /**
     * Get the teaching location
     * @return the teaching location
     */
    public String[] getTeachingLocation()
    {
        return getStringArray(TEACHING_LOCATION);
    }

    /**
     * Set the teaching location
     * @param teachingLocation the teaching location to set
     * @throws AmetysRepositoryException
     */
    public void setTeachingLocation(String[] teachingLocation) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.TEACHING_LOCATION, teachingLocation);
    }

    /**
     * Get the ECTS
     * @return the ECTS
     */
    public String getEcts()
    {
        return getString(ECTS);
    }

    /**
     * Set the ECTS
     * @param ects the ECTS to set
     * @throws AmetysRepositoryException
     */
    public void setEcts(String ects) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.ECTS, ects);
    }

    /**
     * Get the number of hours
     * @return the number of hours
     */
    public String getNumberOfHours()
    {
        return getString(NUMBER_OF_HOURS);
    }

    /**
     * Set the number of hours
     * @param hours the number of hours to set
     * @throws AmetysRepositoryException
     */
    public void setNumberOfHours(String hours) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.NUMBER_OF_HOURS, hours);
    }

    /**
     * Get the effectives
     * @return the effectives
     * @throws UnknownMetadataException
     * @throws AmetysRepositoryException
     */
    public String getMaxNumberOfStudents()
    {
        return getString(MAX_NUMBER_OF_STUDENTS);
    }

    /**
     * Set the effectives
     * @param nb the effectives to set
     * @throws AmetysRepositoryException
     */
    public void setMaxNumberOfStudents(String nb) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.MAX_NUMBER_OF_STUDENTS, nb);
    }

    /**
     * Get the teaching term
     * @return the teaching term
     */
    public String getTeachingTerm()
    {
        return getString(TEACHING_TERM);
    }

    /**
     * Get the teaching term
     * @param provider the provider for CDM-fr enumeration values
     * @return the teaching term
     */
    public String getTeachingTerm(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.TEACHING_TERM, getTeachingTerm(), true);
    }

    /**
     * Set the teaching term
     * @param term the teaching term to set
     * @throws AmetysRepositoryException
     */
    public void setTeachingTerm(String term) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.TEACHING_TERM, term);
    }

    /**
     * Get the level
     * @return the level
     */
    public String getLevel()
    {
        return getString(LEVEL);
    }

    /**
     * Get the level
     * @param provider the provider for CDM-fr enumeration values
     * @return the level
     */
    public String getLevel(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.LEVEL, getLevel(), true);
    }

    /**
     * Set the level
     * @param level the level to set
     * @throws AmetysRepositoryException
     */
    public void setLevel(String level) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.LEVEL, level);
    }

    /**
     * Get the teaching method
     * @return the teaching method
     */
    public String getTeachingMethod()
    {
        return getString(TEACHING_METHOD);
    }

    /**
     * Get the teaching method
     * @param provider the provider for CDM-fr enumeration values
     * @return the teaching method
     */
    public String getTeachingMethod(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.FORMOFTEACHING_METHOD, getTeachingMethod(), false);
    }
    
    /**
     * Get the teaching organizations
     * @return the teaching organizations
     */
    public String[] getTeachingOrgs()
    {
        return getStringArray(TEACHING_ORG);
    }

    /**
     * Get the teaching organizations
     * @param provider the provider for CDM-fr enumeration values
     * @return the teaching method
     */
    public String[] getTeachingOrgs(OdfEnumerationHelper provider)
    {
        List<String> cdmValues = new ArrayList<String>();
        
        String[] teachingOrgs = getTeachingOrgs();
        for (String teachingOrg : teachingOrgs)
        {
            cdmValues.add(provider.getItemCDMfrValue(OdfEnumerationConstant.FORMOFTEACHING_ORG, teachingOrg, false));
        }
        return cdmValues.toArray(new String[cdmValues.size()]);
    }

    /**
     * Set the teaching method
     * @param teachingMethod the teaching method to set
     * @throws AmetysRepositoryException
     */
    public void setTeachingMethod(String teachingMethod) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.TEACHING_METHOD, teachingMethod);
    }

    /**
     * Get the teaching activity
     * @return the teaching activity
     */
    public String getTeachingActivity()
    {
        return getString(TEACHING_ACTIVITY);
    }

    /**
     * Get the teaching method
     * @param provider the provider for CDM-fr enumeration values
     * @return the teaching method
     */
    public String getTeachingActivity(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.TEACHING_ACTIVITY, getTeachingActivity(), false);
    }
    
    /**
     * Set the teaching activity
     * @param activity  the teaching activity to set
     * @throws AmetysRepositoryException
     */
    public void setTeachingActivity(String activity) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.TEACHING_ACTIVITY, activity);
    }

    /**
     * Get the teaching language
     * @return the teaching language
     */
    public String getTeachingLanguage()
    {
        return getString(TEACHING_LANGUAGE);
    }

    /**
     * Set the teaching language
     * @param language the teaching language to set
     * @throws AmetysRepositoryException
     */
    public void setTeachingLanguage(String language) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.TEACHING_LANGUAGE, language);
    }

    /**
     * Get the start date
     * @return the start date
     */
    public Date getStartDate()
    {
        return getDate(START_DATE);
    }

    /**
     * Set the start date
     * @param startDate the start date to set
     * @throws AmetysRepositoryException
     */
    public void setStartDate(Date startDate) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.START_DATE, startDate);
    }

    /**
     * Get the time slot
     * @return the time slot
     */
    public String getTimeSlot()
    {
        return getString(TIME_SLOT);
    }

    /**
     * Get the time slot
     * @param provider the provider for CDM-fr enumeration values
     * @return the time slot
     */
    public String getTimeSlot(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.TIME_SLOT, getTimeSlot(), false);
    }

    /**
     * Set the time slot
     * @param timeslot the time slot to set
     * @throws AmetysRepositoryException
     */
    public void setTimeSlot(String timeslot) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.TIME_SLOT, timeslot);
    }

    /**
     * Get the keywords
     * @return the keywords
     */
    public String[] getKeywords()
    {
        return getStringArray(KEYWORDS);
    }

    /**
     * Set the keywords
     * @param keywords the keywords to set
     * @throws AmetysRepositoryException
     */
    public void setKeywords(String[] keywords) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.KEYWORDS, keywords);
    }

    /**
     * Get the web link label
     * @return the web link label
     */
    public String getWebLinkLabel()
    {
        return getString(KNOW_MORE_WEB_LINK_LABEL);
    }

    /**
     * Set the web link label
     * @param label the web link label to set 
     * @throws AmetysRepositoryException
     */
    public void setWebLinkLabel(String label) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.KNOW_MORE_WEB_LINK_LABEL, label);
    }

    /**
     * Get the web link URL
     * @return the web link URL
     */
    public String getWebLinkUrl()
    {
        return getString(KNOW_MORE_WEB_LINK_URL);
    }

    /**
     * Set the web link URL
     * @param url the web link URL to set
     * @throws AmetysRepositoryException
     */
    public void setWebLinkUrl(String url) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Course.KNOW_MORE_WEB_LINK_URL, url);
    }

    // --------------------------------------------------------------------------------------//
    // CDM-fr
    // --------------------------------------------------------------------------------------//
    
    public String getCDMId()
    {
        return "FRUAI" + _getFactory()._getRootOrgUnitRNE() + "CO" + getCdmCode();
    }
    
    @Override
    @Deprecated
    public void toCDM(ContentHandler contentHandler, OdfEnumerationHelper provider, String rootOrgUnitId) throws SAXException
    {
        _getFactory()._getExportCDMfrManager().course2CDM(contentHandler, this);
    }
    
    /**
     * Export this entity in CDM-fr format.
     * @param contentHandler the content handler to SAX into
     * @param provider the constant provider
     * @param orgUnits the org units
     * @param persons the persons
     * @return the saxed coursed
     * @throws SAXException
     */
    @Deprecated
    public Set<String> toCDM(ContentHandler contentHandler, OdfEnumerationHelper provider, Set<String> orgUnits, Set<String> persons) throws SAXException
    {
        return _getFactory()._getExportCDMfrManager().course2CDM(contentHandler, this, orgUnits, persons);
    }
}
