/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.course.actions;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.SourceResolver;

import org.ametys.cms.observation.Event;
import org.ametys.cms.observation.ObservationConstants;
import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.ContentQueryHelper;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.SubProgram;
import org.ametys.odf.translation.TranslationHelper;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.lock.LockHelper;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.StringExpression;
import org.ametys.runtime.right.RightsManager;
import org.ametys.runtime.right.RightsManager.RightResult;

import com.opensymphony.workflow.WorkflowException;

/**
 * Abstract action to delete a ODF content
 */
public abstract class AbstractDeleteOdfAction extends AbstractOdfAction
{
    /** The rights manager */
    protected RightsManager _rightsManager;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _rightsManager = (RightsManager) smanager.lookup(RightsManager.ROLE);
    }
    
    @Override
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        Map<String, Object> jsParameters = (Map<String, Object>) objectModel.get(ObjectModelHelper.PARENT_CONTEXT);
        List<String> contentsId = (List<String>) jsParameters.get("contentId");

        String deletedContents = "";
        String stillReferencedContents = "";
        String failDeletedContents = "";

        for (String contentId : contentsId)
        {
            SynchronizableContent content = _resolver.resolveById(contentId);
            String contentTitle = content.getTitle();
            String contentName = content.getName();
            
            if (!hasReferences(content))
            {
                try
                {
                    if (content.isLocked())
                    {
                        boolean canUnlockAll = _rightsManager.hasRight(_getCurrentUser(), "CMS_Rights_UnlockAll", "/contributor") == RightResult.RIGHT_OK;
                        if (LockHelper.isLockOwner(content, _getCurrentUser()) || canUnlockAll)
                        {
                            content.unlock();
                        }
                    }
                    
                    Content parentProgram = null;
                    if (content instanceof SubProgram || content instanceof Container)
                    {
                        parentProgram = getParentProgramOrCourse (content);
                    }
                    
                    // Do specific
                    doSpecificBeforeDeletion(content);
                    // Notify observers that a deletion will be processed
                    _observationManager.notify(new Event(_getCurrentUser(), ObservationConstants.CONTENT_DELETING, content));
                    
                    // Remove content
                    content.remove();
                    
                    // Apply changes to parent Program
                    if (parentProgram != null)
                    {
                        applyChangesToParent(parentProgram);
                    }
                    
                    // persist changes
                    content.saveChanges();
                    
                    _observationManager.notify(new Event(_getCurrentUser(), ObservationConstants.CONTENT_DELETED, contentId, new String[]{contentName}));
                    
                    if (deletedContents.length() > 0)
                    {
                        deletedContents += ", ";
                    }
                    
                    deletedContents += contentTitle;
                }
                catch (WorkflowException we)
                {
                    getLogger().error("the workflow state of the program ", we);

                    if (failDeletedContents.length() > 0)
                    {
                        failDeletedContents += ", ";
                    }
                    
                    failDeletedContents += contentTitle;
                }
                catch (AmetysRepositoryException e)
                {
                    getLogger().error("Unable to delete content '" + contentId + "'", e);

                    if (failDeletedContents.length() > 0)
                    {
                        failDeletedContents += ", ";
                    }
                    
                    failDeletedContents += contentTitle;
                }
            }
            else
            {
                if (stillReferencedContents.length() > 0)
                {
                    stillReferencedContents += ", ";
                }
                stillReferencedContents += contentTitle;

                getLogger().warn("The user '" + _getCurrentUser() + "' try to delete the content '" + contentId + "' but he is still referenced");
            }
            
        }

        Map<String, String> result = new HashMap<String, String>();
        result.put("deleted-contents", deletedContents);
        result.put("still-referenced-contents", stillReferencedContents);
        result.put("fail-deleted-contents", failDeletedContents);
        return result;
    }
    
    /**
     * Do specific
     * @param content The content to delete
     * @throws RepositoryException
     */
    protected void doSpecificBeforeDeletion(Content content) throws RepositoryException
    {
        // Nothing to do
    }
    
    /**
     * Determines if content has still references
     * @param content The content to delete
     * @return true if content has still references
     * @throws RepositoryException
     */
    protected boolean hasReferences(Content content) throws RepositoryException
    {
        return false;
    }
    
    /**
     * Remove the properties that reference this program as a translation.
     * @param content the content.
     * @throws RepositoryException 
     */
    protected void removeTranslationsReferences(Content content) throws RepositoryException
    {
        String lang = content.getLanguage();
        
        String meta = TranslationHelper.TRANSLATIONS_METADATA + "/" + lang;
        Expression expression = new StringExpression(meta, Operator.EQ, content.getId());
        String query = ContentQueryHelper.getContentXPathQuery(expression);
        AmetysObjectIterable<ModifiableContent> contents = null;
        
        try
        {
            contents = _resolver.query(query);
            for (ModifiableContent translatedContent : contents)
            {
                Map<String, String> translations = TranslationHelper.getTranslations(translatedContent);
                
                translations.remove(lang);
                
                TranslationHelper.setTranslations(translatedContent, translations);
                
                translatedContent.saveChanges();
            }
        }
        catch (AmetysRepositoryException e)
        {
            if (contents != null)
            {
                contents.close();
            }
        }
    }
}   
