/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.course.actions;

import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;

import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.ContentQueryHelper;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.translation.TranslationHelper;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.StringExpression;

/**
 * Deletes a {Course}
 */
public class DeleteCourseAction extends AbstractDeleteOdfAction
{
    @Override
    protected boolean hasReferences(Content content)
    {
        String id = content.getId();
        if (content instanceof Course)
        {   
            // references from CourseList
            Expression expr = new StringExpression(CourseList.COURSES_REFERENCES, Operator.EQ, id);
            String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:courseList", expr, null);    
            AmetysObjectIterable<CourseList> it = _resolver.query(query);
            
            if (it.hasNext())
            {
                return true;
            }
        } 
        return false;
    }
    
    @Override
    protected void doSpecificBeforeDeletion(Content content) throws RepositoryException
    {
        // Remove the references to the course as a translation.
        removeTranslationsReferences(content);
    }
    
    /**
     * Remove the references of the deleted content
     * @param courseId the course id to delete
     */
    protected void removeReferencesFromCourseList (String courseId)
    {
        Expression expr = new StringExpression(CourseList.COURSES_REFERENCES, Operator.EQ, courseId);

        String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:courseList", expr, null);    
        AmetysObjectIterable<CourseList> it = _resolver.query(query);
        
        for (CourseList cl : it)
        {
            List<String> courses = cl.getCourses();
            if (courses.contains(courseId))
            {
                cl.removeCourse(courseId);
                cl.saveChanges();
            }
        }
    }
    
    /**
     * Remove the properties that reference this course as a translation.
     * @param content the Course content.
     * @throws RepositoryException 
     */
    protected void removeTranslations(SynchronizableContent content) throws RepositoryException
    {
        String lang = content.getLanguage();
        String meta = TranslationHelper.TRANSLATIONS_METADATA + "/" + lang;
        Expression expression = new StringExpression(meta, Operator.EQ, content.getId());
        String query = ContentQueryHelper.getContentXPathQuery(expression);
        AmetysObjectIterable<ModifiableContent> contents = null;
        
        try
        {
            contents = _resolver.query(query);
            for (ModifiableContent translatedContent : contents)
            {
                Map<String, String> translations = TranslationHelper.getTranslations(translatedContent);
                
                translations.remove(lang);
                
                TranslationHelper.setTranslations(translatedContent, translations);
                
                translatedContent.saveChanges();
            }
        }
        catch (AmetysRepositoryException e)
        {
            if (contents != null)
            {
                contents.close();
            }
        }
    }
    
}
