/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.course.generators;

import java.io.IOException;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.generation.ServiceableGenerator;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.xml.sax.SAXException;

import org.ametys.cms.repository.Content;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectResolver;

/**
 * Generates the child nodes of a {@link Course} or a {@link CourseList}
 */
public class CourseTreeGenerator extends ServiceableGenerator
{
    /** The ametys object resolver */
    protected AmetysObjectResolver _resolver;

    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
    }

    public void generate() throws IOException, SAXException, ProcessingException
    {
        Request request = ObjectModelHelper.getRequest(objectModel);
        String id = request.getParameter("id");
        
        contentHandler.startDocument();
        
        AmetysObject object = _resolver.resolveById(id);
        
        if (object instanceof Course)
        {
            saxCourse ((Course) object, true);
        }
        else if (object instanceof CourseList)
        {
            saxCourseList ((CourseList) object, true);
        }
        
        contentHandler.endDocument();
    }
    
    
    /**
     * SAX a {@link Course}
     * @param course the course to SAX
     * @param saxChildren <code>true</code> to SAX children
     * @throws SAXException if an error occurred while SAXing
     */
    protected void saxCourse (Course course, boolean saxChildren) throws SAXException
    {
        AttributesImpl attr = new AttributesImpl();
        attr.addCDATAAttribute("name", course.getName());
        attr.addCDATAAttribute("title", course.getTitle());
        String code = course.getCdmCode() != null ? course.getCdmCode() : "-";
        attr.addCDATAAttribute("code", code);
        attr.addCDATAAttribute("id", course.getId());
        attr.addCDATAAttribute("lang", course.getLanguage());
        attr.addCDATAAttribute("hasList", String.valueOf(course.hasCourseList()));
        
        XMLUtils.startElement(contentHandler, "course", attr);
        
        if (saxChildren)
        {
            for (AmetysObject child : course.getChildren())
            {
                if (child instanceof CourseList)
                {
                    saxCourseList((CourseList) child, false);
                }
            }
        }
       
        XMLUtils.endElement(contentHandler, "course");
    }
    
    /**
     * SAX a {@link CourseList}
     * @param cl the course list to SAX
     * @param saxChildCourses <code>true</code> to SAX children courses
     * @throws SAXException if an error occurred while SAXing
     */
    protected void saxCourseList (CourseList cl, boolean saxChildCourses) throws SAXException
    {
        AttributesImpl attr = new AttributesImpl();
        
        attr.addCDATAAttribute("name", cl.getName());
        String clCcode = cl.getCode() != null ? cl.getCode() : "-";
        attr.addCDATAAttribute("code", clCcode);
        attr.addCDATAAttribute("title", cl.getTitle());
        attr.addCDATAAttribute("id", cl.getId());
        attr.addCDATAAttribute("lang", _getLanguage(cl));
        
        boolean hasChild = cl.getCourses().size() > 0;
        attr.addCDATAAttribute("hasChild", String.valueOf(hasChild));
        XMLUtils.startElement(contentHandler, "courselist", attr);
        
        if (saxChildCourses)
        {
            for (String courseId : cl.getCourses())
            {
                Course course = _resolver.resolveById(courseId);
                saxCourse(course, false);
            }
        }
        
        XMLUtils.endElement(contentHandler, "courselist");
    }
    
    private String _getLanguage (CourseList cl)
    {
        AmetysObject parent = cl.getParent();
        while (parent != null)
        {
            if (parent instanceof Course || parent instanceof Container || parent instanceof SubProgram)
            {
                return ((Content) parent).getLanguage();
            }
            parent = parent.getParent();
        }
        
        return "";
    }
}
