/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.courselist;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.lock.Lock;
import javax.jcr.lock.LockManager;

import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.course.Course;
import org.ametys.odf.course.CourseContainer;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.MovableAmetysObject;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.repository.RepositoryIntegrityViolationException;
import org.ametys.plugins.repository.jcr.DefaultAmetysObject;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;

/**
 * Class representing a list of {@link Course}
 */
public class CourseList extends DefaultAmetysObject<CourseListFactory> implements CourseContainer, MovableAmetysObject
{
    /** Constants for contentType metadata* */
    public static final String CODE_LIST = "code";
    
    /** Constants for contentType metadata* */
    public static final String COURSE_LIST_TITLE = "title";
    
    /** Constants for contentType Metadata* */
    public static final String ECTS = "ects";
    
    /** Constants for type metadata */
    public static final String CHOICE_TYPE = "choiceType";
    /**
     * n : number of course to choose, this Metadata must be used only if the
     * type of the Course List is "choice"
     */
    public static final String MIN_COURSES = "min";

    /**
     * m : number of courses which can be chosen, this Metadata must be used
     * only if the type of the Course List is "choice"
     */
    public static final String MAX_COURSES = "max";

    /**
     * References of the {@link Course} contained in the list
     */
    public static final String COURSES_REFERENCES = "coursesReferences";

    /**
     * The type of the {@link CourseList}
     */
    public enum ChoiceType
    {
        /** mandatory type */
        MANDATORY,
        /** optional type */
        OPTIONAL,
        /** choice type */
        CHOICE
    }

    /**
     * Constructor
     * 
     * @param node
     * @param parentPath
     * @param factory
     */
    public CourseList(Node node, String parentPath, CourseListFactory factory)
    {
        super(node, parentPath, factory);
    }

    @Override
    public List<String> getCourses()
    {
        String[] courses = getMetadataHolder().getStringArray(COURSES_REFERENCES, new String[]{});
        return new ArrayList<String>(Arrays.asList(courses));
    }

    @Override
    public void orderCourseBefore(String idToMove, String sibling)
    {
        String[] courses = getMetadataHolder().getStringArray(COURSES_REFERENCES, new String[]{});
       
        String tmp = null;
        String[] tmpArray = new String[courses.length - 1];
        int index = 0;
        for (int i = 0; i < courses.length; i++)
        {
            if (courses[i].equals(idToMove))
            {
                tmp = courses[i];
            }
            else
            {
                tmpArray[index] = courses[i];
                index++;
            }
        }
        
        String[] reorderedCourses = new String[courses.length];
        if (sibling == null)
        {
            // Order to the end
            for (int i = 0; i < tmpArray.length; i++)
            {
                reorderedCourses[i] = tmpArray[i];
            }
            reorderedCourses[courses.length - 1] = tmp;
        }
        else
        {
            index = 0;
            for (int i = 0; i < tmpArray.length; i++)
            {
                if (tmpArray[i].equals(sibling))
                {
                    reorderedCourses[index] = tmp;
                    index++;
                }
                reorderedCourses[index] = tmpArray[i];
                index++;
            }
        }
        
        getMetadataHolder().setMetadata(COURSES_REFERENCES, reorderedCourses);
    }
    
    @Override
    public void addCourse(String courseId)
    {
        String[] courses = getMetadataHolder().getStringArray(COURSES_REFERENCES, new String[]{});
        
        List<String> courseList = new ArrayList<String>(Arrays.asList(courses));
        courseList.add(courseId);
        
        getMetadataHolder().setMetadata(COURSES_REFERENCES, courseList.toArray(new String[courseList.size()]));
    }

    @Override
    public void removeCourse(String courseId) throws AmetysRepositoryException
    {
        String[] courses = getMetadataHolder().getStringArray(CourseList.COURSES_REFERENCES, new String[]{});
        
        List<String> courseList = new ArrayList<String>(Arrays.asList(courses));
        if (courseList.contains(courseId))
        {
            courseList.remove(courseId);
        }
        
        getMetadataHolder().setMetadata(COURSES_REFERENCES, courseList.toArray(new String[courseList.size()]));
    }

    @Override
    public boolean hasCourse(String courseId)
    {
        String[] courses = getMetadataHolder().getStringArray(CourseList.COURSES_REFERENCES, new String[]{});
        
        List<String> courseList = new ArrayList<String>(Arrays.asList(courses));  
        return courseList.contains(courseId);
    }
    
    /**
     * Get the referencing programs
     * @return the referencing programs
     */
    public List<Program> getReferencingPrograms ()
    {
        List<Program> programs = new ArrayList<Program>();
        
        AmetysObject parent = getParent();
        if (parent instanceof Course)
        {
            programs.addAll(((Course) parent).getReferencingPrograms());
        }
        else
        {
            while (!(parent instanceof Program))
            {
                parent = parent.getParent();
            }
            
            programs.add((Program) parent);
        }
        
        return programs;
    }
    
    @Override
    public boolean canMoveTo(AmetysObject newParent) throws AmetysRepositoryException
    {
        return newParent instanceof Container || newParent instanceof AbstractProgram || newParent instanceof Course;
    }
    
    @Override
    public void moveTo(AmetysObject newParent, boolean renameIfExist) throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        if (!canMoveTo(newParent))
        {
            throw new AmetysRepositoryException("Can not move a CourseList " + getId() + " to something that is not a Container, Program, SubProgram or Course " + newParent.getId());
        }
        

        Node node = getNode();

        try
        {
            if (getParent().equals(newParent))
            {
                // Just order current node to the end
                _checkLock(node.getParent());
                node.getParent().orderBefore(node.getName(), null);
            }
            else
            {
                DefaultAmetysObject newParentAO = (DefaultAmetysObject) newParent;
                
                // Move node
                _checkLock(newParentAO.getNode());
                node.getSession().move(node.getPath(), newParentAO.getNode().getPath() + "/" + node.getName());
            }
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException("Can not move CourseList " + getId() + " to content " + newParent.getId(), e);
        }
        
    }
    
    @Override
    public void orderBefore(AmetysObject siblingNode) throws AmetysRepositoryException
    {
        Node node = getNode();
        try
        {
            _checkLock(node.getParent());
            node.getParent().orderBefore(node.getName(), siblingNode != null ? siblingNode.getName() : null);
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException(String.format("Unable to order CourseList '%s' before sibling '%s'", this, siblingNode != null ? siblingNode.getName() : ""), e);
        }
    }
    
    private static void _checkLock(Node node) throws RepositoryException
    {
        if (node.isLocked())
        {
            LockManager lockManager = node.getSession().getWorkspace().getLockManager();
            Lock lock = lockManager.getLock(node.getPath());
            Node lockHolder = lock.getNode();
            
            lockManager.addLockToken(lockHolder.getProperty(RepositoryConstants.METADATA_LOCKTOKEN).getString());
        }
    }
    
    // --------------------------------------------------------------------------------------//
    // GETTERS AND SETTERS
    // --------------------------------------------------------------------------------------//

    /**
     * Get the code of the {@link CourseList}
     * @return the code or <code>null</code>
     */
    public String getCode()
    {
        return getMetadataHolder().getString(CourseList.CODE_LIST, null);
    }

    /**
     * Set a code to the {@link CourseList}
     * @param code the code to set
     */
    public void setCode(String code)
    {
        getMetadataHolder().setMetadata(CourseList.CODE_LIST, code);
    }
    
    /**
     * Set the synchronization status
     * 
     * @param name
     * @param sync
     */
    public void synchronizeMetadata(String name, boolean sync)
    {
        getMetadataHolder().setMetadata(name + SynchronizableContent.SYNC_SUFFIX, sync);
    }

    /**
     * Get the label of the {@link CourseList}
     * @return the list's title
     */
    public String getTitle()
    {
        if (getMetadataHolder().getBoolean(CourseList.COURSE_LIST_TITLE + SynchronizableContent.SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getString(CourseList.COURSE_LIST_TITLE + SynchronizableContent.REMOTE_SUFFIX, "");
        }
        else
        {
            return getMetadataHolder().getString(CourseList.COURSE_LIST_TITLE, "");
        }
    }

    /**
     * Set a label to the {@link CourseList}
     * @param title the title to set
     */
    public void setTitle(String title)
    {
        getMetadataHolder().setMetadata(CourseList.COURSE_LIST_TITLE, title);
    }
    
    /**
     * Set a label to the {@link CourseList} with the synhronized value
     * @param title the title to set
     */
    public void setRemoteTitle(String title)
    {
        String metadataName = CourseList.COURSE_LIST_TITLE + SynchronizableContent.REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, title);
    }

    /**
     * Get the ECTS
     * @return the ECTS
     */
    public String getEcts()
    {
        if (getMetadataHolder().getBoolean(CourseList.ECTS + SynchronizableContent.SYNC_SUFFIX, false))
        {
            return getMetadataHolder().getString(CourseList.ECTS + SynchronizableContent.REMOTE_SUFFIX, "");
        }
        else
        {
            return getMetadataHolder().getString(CourseList.ECTS, "");
        }
    }

    /**
     * Set the ECTS
     * @param ects the ECTS to set
     * @throws AmetysRepositoryException
     */
    public void setEcts(String ects) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(CourseList.ECTS, ects);
    }
    /**
     * Set the ECTS  with the synhronized value
     * @param ects the ECTS to set
     * @throws AmetysRepositoryException
     */
    public void setRemoteEcts(String ects) throws AmetysRepositoryException
    {
        String metadataName = CourseList.ECTS + SynchronizableContent.REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, ects);
    }
    
    /**
     * If this set of teaching unit element is a type of choice this method
     * return the minimum number of teaching unit elements that the student must
     * choose else -1.
     * 
     * @return the minimum number of teaching unit element that the student must choose. 0 if not set
     */
    public int getMinNumberOfCourses()
    {
        if (getMetadataHolder().getBoolean(CourseList.MIN_COURSES + SynchronizableContent.SYNC_SUFFIX, false))
        {
            return (int) getMetadataHolder().getLong(CourseList.MIN_COURSES + SynchronizableContent.REMOTE_SUFFIX, 1);
        }
        else
        {
            return (int) getMetadataHolder().getLong(CourseList.MIN_COURSES, 1);
        }
    }

    /**
     * Set the minimum number of teaching unit element that the student must choose.
     * @param min the minimum number of teaching unit element that the student must choose.
     */
    public void setMinNumberOfCourses(int min)
    {
        getMetadataHolder().setMetadata(MIN_COURSES, min);
    }
    /**
     * Set the minimum number of teaching unit element that the student must choose,  with the synhronized value
     * @param min the minimum number of teaching unit element that the student must choose.
     */
    public void setRemoteMinNumberOfCourses(int min)
    {
        String metadataName = CourseList.MIN_COURSES + SynchronizableContent.REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, min);
    }
    
    /**
     * Get the remote minimum number of teaching unit elements
     * @return the remote minimum number of teaching unit elements
     */
    public long getRemoteMinNumberOfCourse ()
    {
        return (int) getMetadataHolder().getLong(CourseList.MIN_COURSES + SynchronizableContent.REMOTE_SUFFIX, 1);
    }

    /**
     * If this set of teaching unit element is a type of choice this method
     * return the maximum number of teaching unit element that this set of
     * teaching unit element must contains else -1.
     * 
     * @return the maximum number of teaching unit element that this set of
     *         teaching unit element must contains.
     */
    public int getMaxNumberOfCourses()
    {
        if (getMetadataHolder().getBoolean(CourseList.MAX_COURSES + SynchronizableContent.SYNC_SUFFIX, false))
        {
            return (int) getMetadataHolder().getLong(CourseList.MAX_COURSES + SynchronizableContent.REMOTE_SUFFIX, 2);
        }
        else
        {
            return (int) getMetadataHolder().getLong(CourseList.MAX_COURSES, 2);
        }
    }
    
    /**
     * Get the remote maximun number of teaching unit elements
     * @return the remote maximun number of teaching unit elements
     */
    public long getRemoteMaxNumberOfCourse ()
    {
        return (int) getMetadataHolder().getLong(CourseList.MAX_COURSES + SynchronizableContent.REMOTE_SUFFIX, 2);
    }

    /**
     * Set the maximum number of teaching unit element that this set of teaching
     * unit element must contains.
     * @param max the maximum number of teaching unit element that this set of
     *            teaching unit element must contains.
     */
    public void setMaxNumberOfCourses(int max)
    {
        getMetadataHolder().setMetadata(MAX_COURSES, max);
    }
    /**
     * Set the maximum number of teaching unit element that this set of teaching
     * unit element must contains,  with the synhronized value
     * @param max the maximum number of teaching unit element that this set of
     *            teaching unit element must contains.
     */
    public void setRemoteMaxNumberOfCourses(int max)
    {
        String metadataName = CourseList.MAX_COURSES + SynchronizableContent.REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, max);
    }

    /**
     * Indicates whether the teaching unit elements contained are mandatory,
     * optional or a choice.
     * 
     * @return the type of the teaching unit elements contained.
     */
    public ChoiceType getType()
    {
        try
        {
            if (getMetadataHolder().getBoolean(CourseList.CHOICE_TYPE + SynchronizableContent.SYNC_SUFFIX, false))
            {
                return ChoiceType.valueOf(getMetadataHolder().getString(CourseList.CHOICE_TYPE + SynchronizableContent.REMOTE_SUFFIX, ""));
            }
            else
            {
                return ChoiceType.valueOf(getMetadataHolder().getString(CourseList.CHOICE_TYPE, ""));
            }
        }
        catch (IllegalArgumentException e)
        {
            // No corresponding choice
            return null;
        }
        catch (UnknownMetadataException e)
        {
            return null;
        }
    }

    /**
     * Set the type.
     * @param type the type to set
     */
    public void setType(ChoiceType type)
    {
        getMetadataHolder().setMetadata(CourseList.CHOICE_TYPE, type.name());
    }
    
    /**
     * Set the type.
     * @param type the type to set
     */
    public void setRemoteType(ChoiceType type)
    {
        String metadataName = CourseList.CHOICE_TYPE + SynchronizableContent.REMOTE_SUFFIX;
        getMetadataHolder().setMetadata(metadataName, type.name());
    }
}
