/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.courselist.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.SourceResolver;

import org.ametys.cms.observation.AbstractNotifierAction;
import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.WorkflowAwareContent;
import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseList.ChoiceType;
import org.ametys.odf.courselist.CourseListContainer;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.lock.LockHelper;
import org.ametys.plugins.repository.lock.LockableAmetysObject;
import org.ametys.plugins.workflow.Workflow;
import org.ametys.runtime.user.UsersManager;
import org.ametys.runtime.util.I18nizableText;
import org.ametys.runtime.util.cocoon.ActionResultGenerator;

import com.opensymphony.workflow.WorkflowException;
import com.opensymphony.workflow.spi.Step;

/**
 * Action to create a {@link CourseList} to an {@link Course}
 */
public class AddCourseListAction extends AbstractNotifierAction
{
    
    /** Workflow instance. */
    private Workflow _workflow;
    private UsersManager _usersManager;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _workflow = (Workflow) smanager.lookup(Workflow.ROLE);
        _usersManager = (UsersManager) smanager.lookup(UsersManager.ROLE);
    }
    
    @Override
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        Request request = ObjectModelHelper.getRequest(objectModel);
        Map<String, Object> result = new HashMap<String, Object>();

        String parentId = request.getParameter("parentId");
        String name = request.getParameter("name");
        String type = request.getParameter("type");
        String ects = request.getParameter("ects");
        String availableChoices = request.getParameter("available-choices");
        String nbChoices = request.getParameter("nb-choices");

        assert parentId != null;

        Content parent = _resolver.resolveById(parentId);

        LockableAmetysObject lockableContent = null;
        if (parent instanceof LockableAmetysObject)
        {
            lockableContent = (LockableAmetysObject) parent;
            if (lockableContent.isLocked() && !LockHelper.isLockOwner(lockableContent, _getCurrentUser()))
            {
                getLogger().error("User '" + _getCurrentUser() + "' try to edit content '" + parent.getName() + "' but it is locked by another user");
                
                String lockOwner = lockableContent.getLockOwner();
                
                List<String> i18nParams = new ArrayList<String>();
                i18nParams.add(parent.getTitle());
                i18nParams.add(_usersManager.getUser(lockOwner).getFullName());
                i18nParams.add(lockOwner);
                result.put("message", new I18nizableText("plugin.odf", "PLUGINS_ODF_COURSE_LIST_LOCKED_ERROR", i18nParams));
                result.put("locked", "true");
                
                request.setAttribute(ActionResultGenerator.MAP_REQUEST_ATTR, result);
                return EMPTY_MAP;
            }
        }
        
        if (parent instanceof CourseListContainer)
        {
            // Add list
            CourseList list = ((CourseListContainer) parent).createCourseList(name);

            list.setCode(org.ametys.runtime.util.StringUtils.generateKey());
            
            // Initialize properties of the ELPList
            list.setTitle(name);
            list.setEcts(ects);
            
            ChoiceType choiceType = ChoiceType.valueOf(type);
            list.setType(ChoiceType.valueOf(type));
            
            if (choiceType.equals(ChoiceType.CHOICE))
            {
                list.setMaxNumberOfCourses(Integer.parseInt(availableChoices));
                list.setMinNumberOfCourses(Integer.parseInt(nbChoices));
            }
            
            // Save changes
            list.saveChanges();

            result.put("parentId", parent.getId());
            result.put("courseListId", list.getId());
            result.put("courseListName", list.getName());
            result.put("courseListTitle", list.getTitle());
            result.put("courseListLanguage", _getLang(list));

            // Notify listeners
            Map<String, Object> eventParams = new HashMap<String, Object>();
            eventParams.put("name", list.getName());
            eventParams.put("path", list.getPath());
        }

        // change the workflow state of the program root, if the parent of the added list is a subProg, a container or the program
        changeProgramState(parent);
        
        request.setAttribute(ActionResultGenerator.MAP_REQUEST_ATTR, result);
        return EMPTY_MAP;
    }
    
    private String _getLang (CourseList list)
    {
        AmetysObject parent = list.getParent();
        while (parent != null)
        {
            if (parent instanceof Course || parent instanceof Container || parent instanceof SubProgram)
            {
                return ((Content) parent).getLanguage();
            }
            parent = parent.getParent();
        }
        
        return "";
    }
    
    /**
     * Change the workflow state of the program root, if the parent of the added list is a subProg, a container or the program
     * @param content
     * @throws WorkflowException 
     */
    private void changeProgramState(AmetysObject content) throws WorkflowException
    {
        AmetysObject parent = null;
        
        if (content instanceof Program)
        {
            parent = content;
        }
        else if (content instanceof SubProgram || content instanceof Container)
        {
            // retrieve the program parent
            parent = content.getParent();
            while (!(parent instanceof Program))
            {
                parent = parent.getParent();
            }
        }
        
        if (parent != null)
        {
            try
            {
                if (((WorkflowAwareContent) parent).getCurrentStepId() != 1)
                {
                    Map<String, Object> inputs = new HashMap<String, Object>();
                    inputs.put(AbstractContentWorkflowComponent.RESULT_MAP_KEY, new HashMap<String, Object>());
                    inputs.put(AbstractContentWorkflowComponent.CONTENT_KEY, parent);
                    _workflow.doAction(((WorkflowAwareContent) parent).getWorkflowId(), 22, inputs);
                    
                    Step currentStep = (Step) _workflow.getCurrentSteps(((WorkflowAwareContent) parent).getWorkflowId()).iterator().next();
                    ((Program) parent).setCurrentStepId(currentStep.getStepId());
                    
                    ((Program) parent).saveChanges();
                }
            }
            catch (AmetysRepositoryException e)
            {
                // FIXME : le current step n'existe pas si la formation viens d'etre créée.
            }
        }
    }    
}
