/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.courselist.actions;

import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.acting.ServiceableAction;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.commons.lang.StringUtils;
import org.apache.jackrabbit.util.Text;

import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.WorkflowAwareContent;
import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseList.ChoiceType;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.workflow.Workflow;

import com.opensymphony.workflow.WorkflowException;
import com.opensymphony.workflow.spi.Step;

/**
 * Action to edit a {@link CourseList}
 */
public class EditCourseListAction extends ServiceableAction
{
    private AmetysObjectResolver _resolver;
    
    /** Workflow instance. */
    private Workflow _workflow;
    
    @Override
    public void service(ServiceManager sManager) throws ServiceException
    {
        super.service(sManager);
        _resolver = (AmetysObjectResolver) sManager.lookup(AmetysObjectResolver.ROLE);
        _workflow = (Workflow) sManager.lookup(Workflow.ROLE);
    }

    @Override
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        Request request = ObjectModelHelper.getRequest(objectModel);
        Map<String, String> result = new HashMap<String, String>();

        String id = request.getParameter("id");
        String name = request.getParameter("name");
        boolean nameSync = Boolean.parseBoolean(request.getParameter("nameSync"));
        String ects = request.getParameter("ects");
        boolean ectsSync = Boolean.parseBoolean(request.getParameter("ectsSync"));
        String type = request.getParameter("type");
        boolean typeSync = Boolean.parseBoolean(request.getParameter("typeSync"));
        String availableChoices = request.getParameter("available-choices");
        boolean availableChoicesSync = Boolean.parseBoolean(request.getParameter("availableChoicesSync"));
        String nbChoices = request.getParameter("nb-choices");
        boolean nbChoicesSync = Boolean.parseBoolean(request.getParameter("nbChoicesSync"));
        
        assert id != null;

        String legalName = Text.escapeIllegalJcrChars(name);

        CourseList list = _resolver.resolveById(id);
        if (!nameSync)
        {
            list.setTitle(legalName);
        }
        list.getMetadataHolder().setMetadata(CourseList.COURSE_LIST_TITLE + SynchronizableContent.SYNC_SUFFIX, nameSync);
        
        if (!ectsSync && ects.matches("^([0-9]+)(\\.[0-9]{1})?$"))
        {
            list.setEcts(ects);
        }
        list.getMetadataHolder().setMetadata(CourseList.ECTS + SynchronizableContent.SYNC_SUFFIX, ectsSync);
        
        if (!typeSync)
        {
            ChoiceType choiceType = ChoiceType.valueOf(type);
            list.setType(ChoiceType.valueOf(type));
            
            if (choiceType.equals(ChoiceType.CHOICE))
            {
                if (StringUtils.isNotEmpty(nbChoices))
                {
                    list.setMinNumberOfCourses(Integer.parseInt(nbChoices));
                }
                if (StringUtils.isNotEmpty(availableChoices))
                {
                    list.setMaxNumberOfCourses(Integer.parseInt(availableChoices));
                }
            }
        }
        else
        {
            ChoiceType typeRemote = ChoiceType.valueOf(list.getMetadataHolder().getString(CourseList.CHOICE_TYPE + SynchronizableContent.REMOTE_SUFFIX, ""));
            if (typeRemote.equals(ChoiceType.CHOICE))
            {
                if (!nbChoicesSync && StringUtils.isNotEmpty(nbChoices))
                {
                    list.setMinNumberOfCourses(Integer.parseInt(nbChoices));
                }
                list.getMetadataHolder().setMetadata(CourseList.MIN_COURSES + SynchronizableContent.SYNC_SUFFIX, nbChoicesSync);
                if (!availableChoicesSync && StringUtils.isNotEmpty(availableChoices))
                {
                    list.setMaxNumberOfCourses(Integer.parseInt(availableChoices));
                }
                list.getMetadataHolder().setMetadata(CourseList.MAX_COURSES + SynchronizableContent.SYNC_SUFFIX, availableChoicesSync);
            }
        }
        list.getMetadataHolder().setMetadata(CourseList.CHOICE_TYPE + SynchronizableContent.SYNC_SUFFIX, typeSync);
        
        // save changes
        list.saveChanges();
        
        // Change the workflow state of the program root, if the parent of the edited list is a subProg, a container or the program
        changeProgramState(list);
        
        result.put("courseListId", list.getId());
        result.put("courseListName", list.getName());
        result.put("courseListTitle", list.getTitle());
        result.put("courseListLanguage", _getLang(list));
        
        return result;
    }
    
    private String _getLang (CourseList list)
    {
        AmetysObject parent = list.getParent();
        while (parent != null)
        {
            if (parent instanceof Course || parent instanceof Container || parent instanceof SubProgram)
            {
                return ((Content) parent).getLanguage();
            }
            parent = parent.getParent();
        }
        
        return "";
    }
    
    /**
     * Change the workflow state of the program root, if the parent of the edited list is a subProg, a container or the program
     * @param content
     * @throws WorkflowException 
     */
    private void changeProgramState(CourseList list) throws WorkflowException
    {
        AmetysObject parent = list.getParent();
        if (parent != null)
        {
            if (!(parent instanceof Course))
            {
                if (parent instanceof Program)
                {
                    // ok
                }
                else if (parent instanceof SubProgram || parent instanceof Container)
                {
                    // retrieve the program parent
                    parent = parent.getParent();
                    while (!(parent instanceof Program))
                    {
                        parent = parent.getParent();
                    }
                }
           
                try
                {
                    if (((WorkflowAwareContent) parent).getCurrentStepId() != 1)
                    {
                        Map<String, Object> inputs = new HashMap<String, Object>();
                        inputs.put(AbstractContentWorkflowComponent.RESULT_MAP_KEY, new HashMap<String, Object>());
                        inputs.put(AbstractContentWorkflowComponent.CONTENT_KEY, parent);
                        _workflow.doAction(((WorkflowAwareContent) parent).getWorkflowId(), 22, inputs);
                        
                        Step currentStep = (Step) _workflow.getCurrentSteps(((WorkflowAwareContent) parent).getWorkflowId()).iterator().next();
                        ((Program) parent).setCurrentStepId(currentStep.getStepId());
                        
                        ((Program) parent).saveChanges();
                    }
                }
                catch (AmetysRepositoryException e)
                {
                    // FIXME : le current step n'existe pas si la formation viens d'etre créée.
                }
            }
        }
    }
}
