/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.enumeration;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.configuration.DefaultConfigurationBuilder;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.cms.contenttype.MetadataDefinition;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.runtime.util.I18nizableText;
import org.ametys.runtime.util.parameter.Enumerator;

/**
 * This component handles ODF enumerations 
 *
 */
public class OdfEnumerationHelper extends AbstractLogEnabled implements Component, Configurable, Serviceable
{
    /** Avalon Role */
    public static final String ROLE = OdfEnumerationHelper.class.getName();
    
    private static final String __ITEM_TAG = "item";
    private static final String __ITEM_CODE_ATTRIBUTE = "code";
    private static final String __ITEM_I18NKEY_ATTRIBUTE = "i18n-key";
    private static final String __ITEM_CDMFR_ATTRIBUTE = "cdmValue";
    private static final String __SUBITEM_TAG = "subitem";
    private static final String __I18NCATALOGUE_ATTRIBUTE = "i18n-catalogue";
    
    private Map<String, Map<String, OdfEnumerationItem>> _items;
    private SourceResolver _srcResolver;       
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        _srcResolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
    } 
    
    @Override
    public void configure(Configuration configuration) throws ConfigurationException
    {
        _items = new LinkedHashMap<String, Map<String, OdfEnumerationItem>>();
        
        Configuration[] itemsConfiguration = configuration.getChildren("item");
        for (Configuration itemConfig : itemsConfiguration)
        {
            String key = itemConfig.getAttribute("key");
            String location = itemConfig.getAttribute("location");
            
            Map<String, OdfEnumerationItem> items = _parseFile(location);
            
            _items.put(key, items);
        }
    }
    
    /**
     * Get the full list of enumerations.
     * @return the enumeration keys.
     */
    public Set<String> getEnumerationKeys()
    {
        return Collections.unmodifiableSet(_items.keySet());
    }
    
    /**
     * Get the CDM-fr value associated with the given code
     * @param key The constant key
     * @param code The code
     * @return The CDM-fr value or empty string if not found
     */
    public String getCDMfrValue (String key, String code)
    {
        Map<String, OdfEnumerationItem> items = _items.get(key);
        
        if (items != null)
        {
            OdfEnumerationItem item = items.get(code);
            if (item != null)
            {
                return item.getCDMValue();
            }
            
            // Search on subitems
            for (OdfEnumerationItem it : items.values())
            {
                Map<String, OdfEnumerationItem> subitems = it.getChildren();
                OdfEnumerationItem subitem = subitems.get(code);
                if (subitem != null)
                {
                    return subitem.getCDMValue();
                }
            }
        }
        
        return "";
    }
    
    /**
     * Get all items of an enumeration
     * @param key The enumeration key
     * @return items of enumeration
     */
    public Map<String, OdfEnumerationItem> getItems (String key)
    {
        return _items.get(key);
    }
    
    /**
     * Get all items of an enumeration and their label
     * @param key The enumeration key
     * @return items of enumeration
     */
    public Map<Object, I18nizableText> getItemsLabel (String key)
    {
        Map<Object, I18nizableText> itemEnum = new LinkedHashMap<Object, I18nizableText>();
        
        Map<String, OdfEnumerationItem> items = _items.get(key);
        if (items != null)
        {
            for (OdfEnumerationItem item : items.values())
            {
                itemEnum.put(item.getCode(), item.getLabel());
            }
        }
        
        return itemEnum;
    }
    
    /**
     * Returns the item label
     * @param key The enumeration key
     * @param code The item code
     * @return  the item label or <code>null</code>
     */
    public I18nizableText getItemLabel (String key, String code)
    {
        Map<String, OdfEnumerationItem> items = _items.get(key);
        if (items != null)
        {
            OdfEnumerationItem item = items.get(code);
            if (item != null)
            {
                return item.getLabel();
            }
            
            // Search on subitems
            for (OdfEnumerationItem it : items.values())
            {
                Map<String, OdfEnumerationItem> subitems = it.getChildren();
                OdfEnumerationItem subitem = subitems.get(code);
                if (subitem != null)
                {
                    return subitem.getLabel();
                }
            }
        }
        return null;
    }
    
    
    /**
     * Get all sub-items of an enumeration and their label
     * @param key The enumeration key
     * @return items of enumeration
     */
    public Map<Object, I18nizableText> getSubitemsLabel(String key)
    {
        Map<Object, I18nizableText> itemEnum = new LinkedHashMap<Object, I18nizableText>();
        
        Map<String, OdfEnumerationItem> items = _items.get(key);
        if (items != null)
        {
            for (OdfEnumerationItem item : items.values())
            {
                Map<String, OdfEnumerationItem> subitems = item.getChildren();
                if (subitems != null)
                {
                    for (OdfEnumerationItem subitem : subitems.values())
                    {
                        itemEnum.put(subitem.getCode(), subitem.getLabel());
                    }
                }
            }
        }
        
        return itemEnum;
    }
    
    /**
     * Returns the CMD-fr value corresponding to the given code
     * @param key The enumeration key
     * @param code The code of item
     * @param returnCodeIfEmpty <code>true</code> to return the code if CDM-fr value is empty
     * @return the CDM-fr value
     */
    public String getItemCDMfrValue (String key, String code, boolean returnCodeIfEmpty)
    {
        String value = getCDMfrValue(key, code);
        if (StringUtils.isEmpty(value) && returnCodeIfEmpty)
        {
            return code;
        }
        return value;
    }
    
    /**
     * Returns the code corresponding to the given CMD-fr value
     * @param key The enumeration key
     * @param cdmfrValue The CDM-fr value
     * @return the code or the CDM-fr value if not found
     */
    public String getItemCode (String key, String cdmfrValue)
    {
        if (StringUtils.isEmpty(cdmfrValue))
        {
            return cdmfrValue;
        }
        
        Map<String, OdfEnumerationItem> items = _items.get(key);
        
        if (items != null)
        {
            for (OdfEnumerationItem item : items.values())
            {
                if (cdmfrValue.equals(item.getCDMValue()))
                {
                    return item.getCode();
                }
                
                // Search on subitems
                Map<String, OdfEnumerationItem> subitems = item.getChildren();
                for (OdfEnumerationItem subitem : subitems.values())
                {
                    if (cdmfrValue.equals(subitem.getCDMValue()))
                    {
                        return item.getCode();
                    }
                }
                
            }
        }
        
        return cdmfrValue;
    }
    
    /**
     * SAX enumerations of a ODF content type
     * @param contentHandler the content handler to sax into
     * @param cType the ODF content type
     * @throws Exception
     */
    public void saxEnumerations (ContentHandler contentHandler, ODFContentType cType) throws Exception
    {
        Map<String, MetadataDefinition> enumeratedMetadatas = cType.getEnumeratedMetadatas();
        
        XMLUtils.startElement(contentHandler, "enumerated-metadata");
        
        for (MetadataDefinition metadataDef : enumeratedMetadatas.values())
        {
            saxEnumeration(contentHandler, metadataDef);
        }
        
        XMLUtils.endElement(contentHandler, "enumerated-metadata");
    }
    
    /**
     * SAX enumeration
     * @param contentHandler the content handler to sax into
     * @param metaDef the metadata definition
     * @throws Exception
     */
    public void saxEnumeration (ContentHandler contentHandler, MetadataDefinition metaDef) throws Exception
    {
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("metadataName", metaDef.getName());
        attrs.addCDATAAttribute("metadataPath", metaDef.getId().substring(1));
        
        XMLUtils.startElement(contentHandler, "items", attrs);
        
        Enumerator enumerator = metaDef.getEnumerator();
        if (enumerator != null)
        {
            Map<Object, I18nizableText> entries = enumerator.getEntries();
            
            for (Object code : entries.keySet())
            {
                String codeStr = code.toString();
                if (StringUtils.isNotEmpty(codeStr))
                {
                    I18nizableText value = entries.get(code);
                    
                    AttributesImpl valueAttrs = new AttributesImpl();
                    valueAttrs.addCDATAAttribute("code", codeStr);
                    
                    XMLUtils.startElement(contentHandler, "item", valueAttrs);
                    value.toSAX(contentHandler);
                    XMLUtils.endElement(contentHandler, "item");
                }
            }
        }
        
        XMLUtils.endElement(contentHandler, "items");
    }

    private Map<String, OdfEnumerationItem> _parseFile (String uri)
    {
        Map<String, OdfEnumerationItem> items = new LinkedHashMap<String, OdfEnumerationItem>(); // ODF-763
        
        Source source = null;
        InputStream is = null;
        try
        {
            source = _srcResolver.resolveURI(uri);
            if (source.exists())
            {
                is = source.getInputStream();
                Configuration configuration = new DefaultConfigurationBuilder().build(is);
                
                String i18nCatalogue = configuration.getAttribute(__I18NCATALOGUE_ATTRIBUTE, "application");
                
                for (Configuration itemConf : configuration.getChildren(__ITEM_TAG))
                {
                    OdfEnumerationItem item = _parseItem (itemConf, i18nCatalogue);
                    items.put(item.getCode(), item);
                }
            }
        }
        catch (IOException e)
        {
            getLogger().error("Unable to read file " + uri, e);
        }
        catch (ConfigurationException e)
        {
            getLogger().error("Unable to parse file " + uri, e);
        }
        catch (SAXException e)
        {
            getLogger().error("Unable parse file " + uri, e);
        }
        finally
        {
            IOUtils.closeQuietly(is);
            _srcResolver.release(source);
        }
        
        return items;
    }
    
    private OdfEnumerationItem _parseItem (Configuration configuration, String i18nCatalogue) throws ConfigurationException
    {
        String code = configuration.getAttribute(__ITEM_CODE_ATTRIBUTE);
        String i18nKey = configuration.getAttribute(__ITEM_I18NKEY_ATTRIBUTE);
        String cdmValue = configuration.getAttribute(__ITEM_CDMFR_ATTRIBUTE, "");
        
        Map<String, OdfEnumerationItem> subItems = new LinkedHashMap<String, OdfEnumerationItem>();
        
        for (Configuration subItemConf : configuration.getChildren(__SUBITEM_TAG))
        {
            OdfEnumerationItem subItem = _parseItem(subItemConf, i18nCatalogue);
            subItems.put(subItem.getCode(), subItem);
        }
        
        return new OdfEnumerationItem(code, new I18nizableText(i18nCatalogue, i18nKey), cdmValue, subItems);
    }
}
