/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.ametys.odf.enumeration;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;

import org.ametys.runtime.util.I18nUtils;
import org.ametys.runtime.util.I18nizableText;

/**
 * Helper component to cache translations of odf constants.
 */
public class OdfEnumerationI18nCache implements Component, Serviceable
{
    /** Avalon role */
    public static final String ROLE = OdfEnumerationI18nCache.class.getName();

    // Map<EnumerationName, Map<Lang, Map<Code, Label>>>
    private Map<String, Map<String, Map<String, String>>> _cache = new HashMap<String, Map<String, Map<String, String>>>();
    
    private OdfEnumerationHelper _enumerationHelper;
    private I18nUtils _i18nUtils;
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        _enumerationHelper = (OdfEnumerationHelper) manager.lookup(OdfEnumerationHelper.ROLE);
        _i18nUtils = (I18nUtils) manager.lookup(I18nUtils.ROLE);
    }
    
    /**
     * Returns the translated value associated with a key or null if the key does not exist for the given constant.
     * @param constantKey the odf constant.
     * @param language the language.
     * @param key the key of the value to be be translated.
     * @return a translated value.
     */
    public String getTranslatedValue(String constantKey, String language, String key)
    {
        if (!_cache.containsKey(constantKey))
        {
            _cache.put(constantKey, new HashMap<String, Map<String, String>>());
        }
        
        Map<String, Map<String, String>> languages = _cache.get(constantKey);
        
        if (!languages.containsKey(language))
        {
            languages.put(language, new HashMap<String, String>());
        }
        
        Map<String, String> values = languages.get(language);
        
        if (!values.containsKey(key))
        {
            I18nizableText text = _enumerationHelper.getItemLabel(constantKey, key);
            
            if (text == null)
            {
                return null;
            }
            
            values.put(key, _i18nUtils.translate(text, language));
        }

        return values.get(key);
    }
    
    /**
     * Returns the translated values associated with a key
     * @param constantKey the odf constant.
     * @param language the language.
     * @return the translated values
     */
    public Map<String, String> getTranslatedValues (String constantKey, String language)
    {
        Map<String, String> translatedValues = new LinkedHashMap<String, String>();
        
        Map<Object, I18nizableText> allItems = _enumerationHelper.getItemsLabel(constantKey);
        for (Object key : allItems.keySet())
        {
            String translatedValue = getTranslatedValue(constantKey, language, (String) key);
            translatedValues.put((String) key, translatedValue);
        }
        return translatedValues;
    }
}
