/*
 *  Copyright 2014 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.export.pdf;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Map;

import org.apache.avalon.framework.parameters.ParameterException;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.components.source.impl.SitemapSource;
import org.apache.cocoon.generation.ServiceableGenerator;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.xml.sax.SAXException;

import org.ametys.cms.contenttype.ContentType;
import org.ametys.cms.contenttype.ContentTypeExtensionPoint;
import org.ametys.cms.contenttype.MetadataDefinition;
import org.ametys.cms.contenttype.MetadataDefinitionReference;
import org.ametys.cms.contenttype.MetadataSet;
import org.ametys.cms.repository.ContentTypeExpression;
import org.ametys.cms.repository.LanguageExpression;
import org.ametys.odf.catalog.Catalog;
import org.ametys.odf.catalog.CatalogsManager;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.odf.program.generators.ODFMetadataManager;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.query.QueryHelper;
import org.ametys.plugins.repository.query.SortCriteria;
import org.ametys.plugins.repository.query.expression.AndExpression;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.StringExpression;
import org.ametys.runtime.util.IgnoreRootHandler;


/**
 * Generator producing the SAX events for the catalogue summary 
 */ 
public class FOProgramsGenerator extends ServiceableGenerator
{
    /** The Ametys object resolver */
    protected AmetysObjectResolver _ametysObjectResolver;
    /** The content type extension point */
    protected ContentTypeExtensionPoint _ctypeEP;
    /** ODF metadata manager */
    protected ODFMetadataManager _metadataManager;
    /** The ODf enumeration helper */
    protected OdfEnumerationHelper _odfEnumHelper;
    /** The catalog manager */
    protected CatalogsManager _catalogManager;
    
    @Override
    public void service(ServiceManager sManager) throws ServiceException
    {
        super.service(sManager);
        _ametysObjectResolver = (AmetysObjectResolver) sManager.lookup(AmetysObjectResolver.ROLE);
        _ctypeEP = (ContentTypeExtensionPoint) sManager.lookup(ContentTypeExtensionPoint.ROLE);
        _metadataManager = (ODFMetadataManager) sManager.lookup(ODFMetadataManager.ROLE);
        _odfEnumHelper = (OdfEnumerationHelper) sManager.lookup(OdfEnumerationHelper.ROLE);
        _catalogManager = (CatalogsManager) sManager.lookup(CatalogsManager.ROLE);
    }
    
    public void generate() throws IOException, SAXException, ProcessingException
    {
        contentHandler.startDocument();
        XMLUtils.startElement(contentHandler, "programs");
        
        Catalog catalog = null;
        if ("_default".equals(source) && _catalogManager.getCatalogs().size() > 0)
        {
            catalog = _catalogManager.getCatalogs().get(0);
        }
        else
        {
            catalog = _catalogManager.getCatalog(source);
        }
        
        if (catalog == null)
        {
            throw new IllegalArgumentException ("Failed to generated PDF of unknown catalog '" + source + "'");
        }
        
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("name", source);
        XMLUtils.createElement(contentHandler, "catalog", attrs, catalog.getTitle());
        
        try
        {
            _saxPrograms(catalog.getName(), parameters.getParameter("lang"));
        }
        catch (ParameterException e)
        {
            throw new IllegalArgumentException ("Missing lang parameter", e);
        }
        
        try
        {
            ContentType programCtype = _ctypeEP.getExtension(ProgramFactory.PROGRAM_CONTENT_TYPE);
            _odfEnumHelper.saxEnumerations(contentHandler, (ODFContentType) programCtype);
        }
        catch (Exception e)
        {
            throw new ProcessingException("Unable to sax enumerated metadata", e);
        }
        
        XMLUtils.endElement(contentHandler, "programs");
        contentHandler.endDocument();
    }
    
    /**
     * SAX programs
     * @param catalog the catalog's name
     * @param lang the programs' language
     * @throws MalformedURLException
     * @throws IOException
     * @throws SAXException
     */
    protected void _saxPrograms(String catalog, String lang) throws MalformedURLException, IOException, SAXException
    {
        AmetysObjectIterable<Program> programs = _getPrograms(catalog, lang);
        
        while (programs.hasNext())
        {
            Program program = programs.next();
            
            AttributesImpl attrs = new AttributesImpl();
            attrs.addCDATAAttribute("id", program.getId());
            attrs.addCDATAAttribute("name", program.getName());
            attrs.addCDATAAttribute("title", program.getTitle());
            
            XMLUtils.startElement(contentHandler, "program", attrs);
            
            _saxEnumeratedMetadata (program);
            
            XMLUtils.startElement(contentHandler, "fo");
            SitemapSource src = null;      
            
            try
            {
                String uri = "cocoon://_plugins/odf/_content/" + program.getName() + ".fo";
                
                src = (SitemapSource) resolver.resolveURI(uri);
                
                src.toSAX(new IgnoreRootHandler(contentHandler));
            }
            catch (UnknownAmetysObjectException e)
            {
                // The content may be archived
            }
            finally
            {
                resolver.release(src);
            }
            XMLUtils.endElement(contentHandler, "fo");
            
            XMLUtils.endElement(contentHandler, "program");
        }
    }
    
    /**
     * SAX enumerated values of a metadata 
     * @param program
     * @throws AmetysRepositoryException
     * @throws SAXException
     * @throws IOException
     */
    protected void _saxEnumeratedMetadata(Program program) throws AmetysRepositoryException, SAXException, IOException
    {
        ContentType cType = _ctypeEP.getExtension(program.getType());
        Map<String, MetadataDefinition> enumeratedMetadatas = ((ODFContentType) cType).getEnumeratedMetadatas();
        
        MetadataSet rootMetadataSet = new MetadataSet();
        
        for (String metadataPath : enumeratedMetadatas.keySet())
        {
            String[] pathSegments = org.apache.commons.lang.StringUtils.split(metadataPath, '/');
            
            MetadataDefinitionReference metadataDefRef = new MetadataDefinitionReference(pathSegments[0]);
            rootMetadataSet.addElement(metadataDefRef);
            
            for (int i = 1; i < pathSegments.length; i++)
            {
                MetadataDefinitionReference subMetadataDefRef = new MetadataDefinitionReference(pathSegments[i]);
                metadataDefRef.addElement(subMetadataDefRef);
                
                metadataDefRef = subMetadataDefRef;
            }
        }
        
        XMLUtils.startElement(contentHandler, "metadata");
        _metadataManager.saxMetadata(contentHandler, program, rootMetadataSet);
        XMLUtils.endElement(contentHandler, "metadata");
    }

    private AmetysObjectIterable<Program> _getPrograms(String catalog, String lang)
    {
        Expression programExpression = new ContentTypeExpression(Operator.EQ, ProgramFactory.PROGRAM_CONTENT_TYPE);
        Expression catalogExpr = new StringExpression(Program.CATALOG, Operator.EQ, catalog);
        Expression langExpr = new LanguageExpression(Operator.EQ, lang);
        
        SortCriteria sortCriteria = new SortCriteria();
        sortCriteria.addCriterion("title", true, true);
        String programQuery = QueryHelper.getXPathQuery(null, ProgramFactory.PROGRAM_NODETYPE, new AndExpression(programExpression, catalogExpr, langExpr), sortCriteria);
        AmetysObjectIterable<Program> programs = _ametysObjectResolver.query(programQuery);
        
        return programs;
    }
}
