/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.observation;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;

import org.ametys.cms.observation.Event;
import org.ametys.cms.observation.ObservationConstants;
import org.ametys.cms.observation.Observer;
import org.ametys.cms.repository.ContentTypeExpression;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.odf.course.Course;
import org.ametys.odf.course.CourseFactory;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.OrgUnitFactory;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.odf.program.SubProgramFactory;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.query.expression.AndExpression;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.OrExpression;
import org.ametys.plugins.repository.query.expression.StringExpression;

/**
 * Observes {@link OrgUnit} content deletion in order to update references in others ODF contents.
 */
public class UpdateReferenceOnOrgUnitDeletionObserver implements Observer, Serviceable
{
    /** The Ametys resolver */
    protected AmetysObjectResolver _resolver;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
    }
    
    @Override
    public boolean supports(Event event)
    {
        return event.getId().equals(ObservationConstants.CONTENT_DELETING) && event.getTarget() instanceof OrgUnit;
    }
    
    @Override
    public int getPriority(Event event)
    {
        // Will be processed before live synchronization observers
        return MAX_PRIORITY;
    }

    @Override
    public void observe(Event event)
    {
        OrgUnit ou = (OrgUnit) event.getTarget();
        String ouId = ou.getId();
        
        removeReferencesFromOrgUnits(ouId);
        removeReferencesFromCourses(ouId);
        removeReferencesFromAbstractPrograms(ouId);
    }

    /**
     * Remove references from {@link OrgUnit}s
     * @param ouId The reference to delete
     */
    protected void removeReferencesFromOrgUnits (String ouId)
    {
        Expression cTypeExpr = new ContentTypeExpression(Operator.EQ, OrgUnitFactory.ORGUNIT_CONTENT_TYPE);
        
        Expression ouLocalExpr = new StringExpression(OrgUnit.ORGUNITS_REFERENCES, Operator.EQ, ouId);
        Expression ouRemoteExpr = new StringExpression(OrgUnit.ORGUNITS_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, ouId);
        Expression ouExpr = new OrExpression(ouLocalExpr, ouRemoteExpr);
        
        Expression expr = new AndExpression(cTypeExpr, ouExpr);
        
        String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);    
        AmetysObjectIterable<OrgUnit> ouIt = _resolver.query(query);
        
        for (OrgUnit ou : ouIt)
        {
            ou.removeReference(OrgUnit.ORGUNITS_REFERENCES, ouId);
            ou.saveChanges();
        }
    }
    
    /**
     * Remove references from {@link AbstractProgram}s
     * @param ouId The reference to delete
     */
    protected  void removeReferencesFromAbstractPrograms (String ouId)
    {
        Expression cTypeExpr1 = new ContentTypeExpression(Operator.EQ, ProgramFactory.PROGRAM_CONTENT_TYPE);
        Expression cTypeExpr2 = new ContentTypeExpression(Operator.EQ, SubProgramFactory.SUBPROGRAM_CONTENT_TYPE);
        Expression cTypeExpr = new OrExpression(cTypeExpr1, cTypeExpr2);
        
        Expression ouLocalExpr = new StringExpression(AbstractProgram.ORG_UNITS_REFERENCES, Operator.EQ, ouId);
        Expression ouRemoteExpr = new StringExpression(AbstractProgram.ORG_UNITS_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, ouId);
        Expression ouExpr = new OrExpression(ouLocalExpr, ouRemoteExpr);
        
        Expression expr = new AndExpression(cTypeExpr, ouExpr);
        
        String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);    
        AmetysObjectIterable<Program> programIt = _resolver.query(query);
        
        for (AbstractProgram program : programIt)
        {
            program.removeReference(AbstractProgram.ORG_UNITS_REFERENCES, ouId);
            program.saveChanges();
        }
    }
    
    /**
     * Remove references from {@link Course}s
     * @param ouId The reference to delete
     */
    protected void removeReferencesFromCourses (String ouId)
    {
        Expression cTypeExpr = new ContentTypeExpression(Operator.EQ, CourseFactory.COURSE_CONTENT_TYPE);
        
        Expression ouLocalExpr = new StringExpression(Course.ORG_UNITS_REFERENCES, Operator.EQ, ouId);
        Expression ouRemoteExpr = new StringExpression(Course.ORG_UNITS_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, ouId);
        Expression ouExpr = new OrExpression(ouLocalExpr, ouRemoteExpr);
        
        Expression expr = new AndExpression(cTypeExpr, ouExpr);
        
        String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);    
        AmetysObjectIterable<Course> courseIt = _resolver.query(query);
        
        for (Course course : courseIt)
        {
            course.removeReference(Course.ORG_UNITS_REFERENCES, ouId);
            course.saveChanges();
        }
    }
}
