/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.observation;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;

import org.ametys.cms.observation.Event;
import org.ametys.cms.observation.ObservationConstants;
import org.ametys.cms.observation.Observer;
import org.ametys.cms.repository.ContentTypeExpression;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.odf.course.Course;
import org.ametys.odf.course.CourseFactory;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.OrgUnitFactory;
import org.ametys.odf.person.Person;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.odf.program.SubProgramFactory;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.query.expression.AndExpression;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.OrExpression;
import org.ametys.plugins.repository.query.expression.StringExpression;

/**
 * Observes {@link Person} content deletion in order to update references in others ODF contents.
 */
public class UpdateReferenceOnPersonDeletionObserver implements Observer, Serviceable
{
    /** The Ametys resolver */
    protected AmetysObjectResolver _resolver;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
    }
    
    @Override
    public boolean supports(Event event)
    {
        return event.getId().equals(ObservationConstants.CONTENT_DELETING) && event.getTarget() instanceof Person;
    }
    
    @Override
    public int getPriority(Event event)
    {
        // Will be processed before live synchronization observers
        return MAX_PRIORITY;
    }

    @Override
    public void observe(Event event)
    {
        Person person = (Person) event.getTarget();
        String personId = person.getId();
        
        removeReferencesFromOrgUnits(personId);
        removeReferencesFromCourses(personId);
        removeReferencesFromAbstractPrograms(personId);
    }

    /**
     * Remove references from {@link OrgUnit}s
     * @param id The reference to delete
     */
    protected void removeReferencesFromOrgUnits (String id)
    {
        Expression cTypeExpr = new ContentTypeExpression(Operator.EQ, OrgUnitFactory.ORGUNIT_CONTENT_TYPE);
        
        Expression localExpr = new StringExpression(OrgUnit.CONTACTS_REFERENCES, Operator.EQ, id);
        Expression remoteExpr = new StringExpression(OrgUnit.CONTACTS_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, id);
        Expression expr1 = new OrExpression(localExpr, remoteExpr);
        
        Expression expr = new AndExpression(cTypeExpr, expr1);
        
        String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);    
        AmetysObjectIterable<OrgUnit> it = _resolver.query(query);
        
        for (OrgUnit orgUnit : it)
        {
            orgUnit.removeReference(OrgUnit.CONTACTS_REFERENCES, id);
            orgUnit.saveChanges();
        }
    }
    
    /**
     * Remove references from {@link AbstractProgram}s
     * @param id The reference to delete
     */
    protected void removeReferencesFromAbstractPrograms (String id)
    {
        Expression cTypeExpr1 = new ContentTypeExpression(Operator.EQ, ProgramFactory.PROGRAM_CONTENT_TYPE);
        Expression cTypeExpr2 = new ContentTypeExpression(Operator.EQ, SubProgramFactory.SUBPROGRAM_CONTENT_TYPE);
        
        Expression cTypeExpr = new OrExpression(cTypeExpr1, cTypeExpr2);
        
        Expression localExpr1 = new StringExpression(AbstractProgram.CONTACTS_REFERENCES, Operator.EQ, id);
        Expression remoteExpr1 = new StringExpression(AbstractProgram.CONTACTS_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, id);
        Expression expr1 = new OrExpression(localExpr1, remoteExpr1);
        
        Expression localExpr2 = new StringExpression(AbstractProgram.PERSONS_IN_CHARGE_REFERENCES, Operator.EQ, id);
        Expression remoteExpr2 = new StringExpression(AbstractProgram.PERSONS_IN_CHARGE_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, id);
        Expression expr2 = new OrExpression(localExpr2, remoteExpr2);
        
        Expression orExpr = new OrExpression(expr1, expr2);
        
        Expression expr = new AndExpression(cTypeExpr, orExpr);
        
        String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);    
        AmetysObjectIterable<AbstractProgram> it = _resolver.query(query);
        
        for (AbstractProgram program : it)
        {
            program.removeReference(AbstractProgram.CONTACTS_REFERENCES, id);
            program.removeReference(AbstractProgram.PERSONS_IN_CHARGE_REFERENCES, id);
            program.saveChanges();
        }
    }
    
    /**
     * Remove references from {@link Course}s
     * @param id The reference to delete
     */
    protected void removeReferencesFromCourses (String id)
    {
        Expression cTypeExpr = new ContentTypeExpression(Operator.EQ, CourseFactory.COURSE_CONTENT_TYPE);
        
        Expression localExpr1 = new StringExpression(Course.CONTACTS_REFERENCES, Operator.EQ, id);
        Expression remoteExpr1 = new StringExpression(Course.CONTACTS_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, id);
        Expression expr1 = new OrExpression(localExpr1, remoteExpr1);
        
        Expression localExpr2 = new StringExpression(Course.PERSONS_IN_CHARGE_REFERENCES, Operator.EQ, id);
        Expression remoteExpr2 = new StringExpression(Course.PERSONS_IN_CHARGE_REFERENCES + ODFContentType.REMOTE_SUFFIX, Operator.EQ, id);
        Expression expr2 = new OrExpression(localExpr2, remoteExpr2);
        
        Expression orExpr = new OrExpression(expr1, expr2);
        
        Expression expr = new AndExpression(cTypeExpr, orExpr);
        
        String query = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);    
        AmetysObjectIterable<Course> it = _resolver.query(query);
        
        for (Course course : it)
        {
            course.removeReference(Course.CONTACTS_REFERENCES, id);
            course.removeReference(Course.PERSONS_IN_CHARGE_REFERENCES, id);
            course.saveChanges();
        }
    }
}
