/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.orgunit;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.cms.repository.ContentTypeExpression;
import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.cdmfr.CDMEntity;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.metadata.RichText;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;
import org.ametys.plugins.repository.query.expression.AndExpression;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.StringExpression;

/**
 * OrgUnit java object
 */
public class OrgUnit extends SynchronizableContent<OrgUnitFactory> implements CDMEntity
{
    /** code property. */
    public static final String CODE = "code";
    
    /** prefix for the code property */
    public static final String CODE_PREFIX = "orgunit-";

    /** code RNE property. */
    public static final String CODE_UAI = "codeUAI";
    
    /** acronym property. */
    public static final String ACRONYM = "acronym";

    /** description property. */
    public static final String DESCRIPTION = "description";

    /** admission description property. */
    public static final String ADMISSION_INFO = "admissionInfo";

    /** regulations property. */
    public static final String REGULATIONS = "regulations";

    /** expenses property. */
    public static final String EXPENSES = "expenses";

    /** universal adjustement property. */
    public static final String UNIVERSAL_ADJUSTMENT = "universalAdjustment";

    /** student facilities property. */
    public static final String STUDENT_FACILITIES = "studentFacilities";

    /** additional data property. */
    public static final String ADDITIONNAL_INFOS = "additionalInfos";

    /** web link property. */
    public static final String WEB_LINK_LABEL = "webLinkLabel";
    
    /** web link  url property. */
    public static final String WEB_LINK_URL = "webLinkUrl";
    
    /**
     * References of contacts
     */
    public static final String CONTACTS_REFERENCES = "contactsReferences";

    /**
     * References of children (subOrgUnits)
     */
    public static final String ORGUNITS_REFERENCES = "orgunitsReferences";

    /**
     * Constructor
     * @param node
     * @param parentPath
     * @param factory
     */
    public OrgUnit(Node node, String parentPath, OrgUnitFactory factory)
    {
        super(node, parentPath, factory);
    }

    /**
     * Remove reference from local and remote metadata
     * @param metadataName The metadata name
     * @param value The value of reference to remove 
     */
    public void removeReference (String metadataName, String value)
    {
        String[] localReferences = getMetadataHolder().getStringArray(metadataName, new String[0]);
        List<String> localList = new ArrayList<String>(Arrays.asList(localReferences));
        if (localList.contains(value))
        {
            localList.remove(value);
        }
        getMetadataHolder().setMetadata(metadataName, localList.toArray(new String[localList.size()]));
        
        String[] remoteReferences = getMetadataHolder().getStringArray(metadataName + REMOTE_SUFFIX, new String[0]);
        List<String> remoteList = new ArrayList<String>(Arrays.asList(remoteReferences));
        if (remoteList.contains(value))
        {
            remoteList.remove(value);
        }
        getMetadataHolder().setMetadata(metadataName + REMOTE_SUFFIX, remoteList.toArray(new String[remoteList.size()]));
    }
    
    // --------------------------------------------------------------------------------------//
    //
    // SUB ORGUNITS
    //
    // --------------------------------------------------------------------------------------//

    /**
     * Return a List of orgUnits IDs up to date, Each ID is checked to
     * remove deleted elements
     * @return List&lt;String&gt;
     */
    public List<String> getSubOrgUnits()
    {
        String[] orgunits = getStringArray(ORGUNITS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(orgunits));
    }

    /**
     * Add a sub orgnit
     * @param id the sub orgunit id to add
     */
    public void addSubOrgUnit (String id)
    {
        String[] orgunits = getMetadataHolder().getStringArray(ORGUNITS_REFERENCES, new String[]{});
        
        List<String> orgunitsList = new ArrayList<String>(Arrays.asList(orgunits));
        orgunitsList.add(id);
        
        getMetadataHolder().setMetadata(ORGUNITS_REFERENCES, orgunitsList.toArray(new String[orgunitsList.size()]));
    }
    
    /**
     * Delete a sub orgnit
     * @param id the sub orgunit id to delete
     */
    public void removeSubOrgUnit (String id)
    {
        String[] orgunits = getMetadataHolder().getStringArray(ORGUNITS_REFERENCES, new String[]{});
        
        List<String> orgunitsList = new ArrayList<String>(Arrays.asList(orgunits));
        if (orgunitsList.contains(id))
        {
            orgunitsList.remove(id);
        }
        
        getMetadataHolder().setMetadata(ORGUNITS_REFERENCES, orgunitsList.toArray(new String[orgunitsList.size()]));
    }
    
    /**
     * Get the parent {@link OrgUnit}
     * @return the parent {@link OrgUnit} or null;
     */
    public OrgUnit getParentOrgUnit ()
    {
        Expression cTypeExpr = new ContentTypeExpression(Operator.EQ, "org.ametys.plugins.odf.Content.orgunit");
        Expression expr = new AndExpression(cTypeExpr, new StringExpression(ORGUNITS_REFERENCES, Operator.EQ, getId()));
        
        String xpathQuery = org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);
        
        try
        {
            AmetysObjectIterable<OrgUnit> it = _getFactory()._getResolver().query(xpathQuery, getNode().getSession());
            if (it.hasNext())
            {
                return it.next();
            }
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException("Unable to parent orgunit of orgunit " + getPath(), e);
        }
        
        return null;
    }
    
    // --------------------------------------------------------------------------------------//
    //
    // CONTACTS
    //
    // --------------------------------------------------------------------------------------//

    /**
     * Return a List of contact IDs
     * @return a list of uuid
     */
    public List<String> getContacts()
    {
        String[] contacts = getStringArray(CONTACTS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return a List of lopcal contact IDs
     * @return a list of uuid
     */
    public List<String> getLocalContacts()
    {
        String[] contacts = getMetadataHolder().getStringArray(CONTACTS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return a List of lopcal contact IDs
     * @return a list of uuid
     */
    public List<String> getRemoteContacts()
    {
        String[] contacts = getMetadataHolder().getStringArray(CONTACTS_REFERENCES + REMOTE_SUFFIX);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    
    // --------------------------------------------------------------------------------------//
    //
    // GETTERS AND SETTERS
    //
    // --------------------------------------------------------------------------------------//

    /**
     * Get the metadata Code
     * 
     * @return String
     * @throws UnknownMetadataException
     * @throws AmetysRepositoryException
     */
    public String getCode() throws UnknownMetadataException, AmetysRepositoryException
    {
        return getString(OrgUnit.CODE);
    }

    /**
     * Set the metadata code
     * 
     * @param code
     * @throws AmetysRepositoryException
     */
    public void setCode(String code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(OrgUnit.CODE, code);
    }

    /**
     * Get the metadata RneCode
     * 
     * @return String
     * @throws UnknownMetadataException
     * @throws AmetysRepositoryException
     */
    public String getUAICode() throws UnknownMetadataException, AmetysRepositoryException
    {
        return getString(OrgUnit.CODE_UAI);
    }

    /**
     * Set the metadata RneCode
     * 
     * @param rne
     * @throws AmetysRepositoryException
     */
    public void setUAICode(String rne) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(OrgUnit.CODE_UAI, rne);
    }
    
    /**
     * Return the metadata code
     * 
     * @return the acronym
     */
    public String getAcronym()
    {
        return getString(OrgUnit.ACRONYM);
    }

    /**
     * Set the metadata acronym
     * 
     * @param value
     */
    public void setAcronym(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.ACRONYM, value);
    }

    /**
     * Return the metadata description
     * 
     * @return the description
     */
    public RichText getDescription()
    {
        return getRichText(OrgUnit.DESCRIPTION);
    }

    /**
     * Set the metadata description
     * 
     * @param value
     */
    public void setDescription(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.DESCRIPTION, value);
    }

    /**
     * Return the metadata admission_info
     * 
     * @return the admission_info
     */
    public RichText getAdmissionInfo()
    {
        return getRichText(OrgUnit.ADMISSION_INFO);
    }

    /**
     * Set the metadata admission_info
     * 
     * @param value
     */
    public void setAdmissionInfo(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.ADMISSION_INFO, value);
    }

    /**
     * Return the metadata
     * 
     * @return the regulations
     */
    public RichText getRegulations()
    {
        return getRichText(OrgUnit.REGULATIONS);
    }

    /**
     * Set the metadata regulations
     * 
     * @param value
     * 
     */
    public void setRegulations(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.REGULATIONS, value);
    }

    /**
     * Return the metadata Expenses
     * 
     * @return the Expenses
     */
    public RichText getExpenses()
    {
        return getRichText(OrgUnit.EXPENSES);
    }

    /**
     * Set the metadata Expenses
     * 
     * @param value
     * 
     */
    public void setExpenses(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.EXPENSES, value);
    }

    /**
     * Return the metadata universalAdjustment
     * 
     * @return the universalAdjustment
     */
    public RichText getUniversalAdjustment()
    {
        return getRichText(OrgUnit.UNIVERSAL_ADJUSTMENT);
    }

    /**
     * Set the metadata universalAdjustment
     * @param value
     * 
     */
    public void setUniversalAdjustment(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.UNIVERSAL_ADJUSTMENT, value);
    }

    /**
     * Return the metadata student_facilities
     * 
     * @return the student_facilities
     */
    public RichText getStudentFacilities()
    {
        return getRichText(OrgUnit.STUDENT_FACILITIES);
    }

    /**
     * Set the metadata student_facilities
     * 
     * @param value
     * 
     */
    public void setStudentFacilities(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.STUDENT_FACILITIES, value);
    }

    /**
     * Return the metadata additionnal_infos
     * 
     * @return the additionnal_infos
     */
    public RichText getAdditionnalInfos()
    {
        return getRichText(OrgUnit.ADDITIONNAL_INFOS);
    }

    /**
     * Set the metadata additionnal_infos
     * 
     * @param value
     * 
     */
    public void setAdditionnalInfos(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.ADDITIONNAL_INFOS, value);
    }

    /**
     * Return the metadata web_link
     * 
     * @return the web_link
     */
    public String getWebLinkLabel()
    {
        return getString(OrgUnit.WEB_LINK_LABEL);
    }

    /**
     * Set the metadata web_link
     * 
     * @param value
     * 
     */
    public void setWebLinkLabel(String value)
    {
        getMetadataHolder().setMetadata(OrgUnit.WEB_LINK_LABEL, value);
    }

    /**
     * Get the web link URL
     * @return the web link URL or null
     */
    public String getWebLinkURL()
    {
        return getString(WEB_LINK_URL);
    }

    /**
     * Set the web link URL
     * @param url the web link URL to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setWebLinkURL(String url) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(WEB_LINK_URL, url);
    }
    
    // --------------------------------------------------------------------------------------//
    //
    // CDM EXPORT
    //
    // --------------------------------------------------------------------------------------//
    
    public String getCDMId() 
    {
        return "FRUAI" + _getFactory()._getRootOrgUnitUAI() + "OU" + getUAICode();
    }
    
    @Override
    @Deprecated
    public void toCDM(ContentHandler contentHandler, OdfEnumerationHelper provider, String rootOrgUnitId) throws SAXException
    {
        _getFactory()._getExportCDMfrManager().orgunit2CDM(contentHandler, this);
    }
}
