/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.orgunit;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.commons.lang.StringUtils;

import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.runtime.util.I18nizableText;
import org.ametys.runtime.util.parameter.Enumerator;

/**
 * Enumerator for {@link OrgUnit}s
 *
 */
public class OrgUnitEnumerator implements Enumerator, Serviceable, Configurable
{
    /** The Ametys object resolver */
    protected AmetysObjectResolver _resolver;
    /** The OrgUnit root provider */
    protected RootOrgUnitProvider _orgUnitProvider;
    
    /** True if a value must be selected, false to allow the empty value. */
    protected boolean _mandatory;
    /** All option : 'disabled' to disable the 'all' option, 
     * 'blank' to use a blank value for 'all' option,
     * or 'concat' to concat all content types' ids for 'all' option
     **/
    protected String _allOption;
    
    @Override
    public void configure(Configuration configuration) throws ConfigurationException
    {
        _mandatory = configuration.getChild("enumeration").getChild("custom-enumerator").getChild("mandatory").getValueAsBoolean(true);
        Configuration allOptConf = configuration.getChild("all-option", false);
        if (allOptConf == null)
        {
            allOptConf = configuration.getChild("enumeration").getChild("custom-enumerator").getChild("all-option", false);
        }
        _allOption = allOptConf != null ? allOptConf.getValue("blank") : "";
    }
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _orgUnitProvider = (RootOrgUnitProvider) smanager.lookup(RootOrgUnitProvider.ROLE);
    }

    @Override
    public Map<Object, I18nizableText> getEntries() throws Exception
    {
        Map<Object, I18nizableText> entries = new HashMap<Object, I18nizableText>();
        
        if (!_mandatory)
        {
            entries.put("", new I18nizableText("plugin.odf", "PLUGINS_ODF_ORGUNIT_ENUMERATOR_VALUE_ALL"));
        }
        
        OrgUnit orgUnit = _orgUnitProvider.getRoot();
        entries.putAll(_getSubOrgUnits(orgUnit));
        
        // All orgunits
        if (_allOption.equals("blank"))
        {
            entries.put("", new I18nizableText("plugin.cms", "WIDGET_COMBOBOX_ALL_OPTIONS"));
        }
        else if (_allOption.equals("concat"))
        {
            entries.put(StringUtils.join(entries.keySet(), ','), new I18nizableText("plugin.cms", "WIDGET_COMBOBOX_ALL_OPTIONS"));
        }
        
        return entries;
    }
    
    private Map<Object, I18nizableText> _getSubOrgUnits (OrgUnit orgUnit)
    {
        Map<Object, I18nizableText> entries = new HashMap<Object, I18nizableText>();
        
        List<String> ids = orgUnit.getSubOrgUnits();
        for (String id : ids)
        {
            try
            {
                OrgUnit child = _resolver.resolveById(id);
                String title = child.getTitle();
                
                entries.put(id, new I18nizableText(title));
                
                entries.putAll(_getSubOrgUnits (child));
            }
            catch (UnknownAmetysObjectException e)
            {
                // Ignore.
            }
        }
        
        return entries;
    }

    @Override
    public I18nizableText getEntry(String value) throws Exception
    {
        I18nizableText entry = null;
        
        if (StringUtils.isEmpty(value))
        {
            entry = new I18nizableText("plugin.odf", "PLUGINS_ODF_ORGUNIT_ENUMERATOR_VALUE_ALL");
        }
        else
        {
            try
            {
                OrgUnit orgunit = _resolver.resolveById(value);
                entry = new I18nizableText(orgunit.getTitle());
            }
            catch (UnknownAmetysObjectException e)
            {
                // Ignore.
            }
        }
        
        return entry;
    }

}
