/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.orgunit.actions;

import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.environment.Redirector;
import org.apache.commons.lang.StringUtils;

import org.ametys.cms.observation.Event;
import org.ametys.cms.observation.ObservationConstants;
import org.ametys.cms.observation.ObservationManager;
import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.cms.workflow.InitializeContentWorkflowAction;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.RootOrgUnitProvider;
import org.ametys.odf.workflow.CreateODFContentFunction;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.jcr.DefaultAmetysObject;
import org.ametys.plugins.repository.version.VersionableAmetysObject;
import org.ametys.runtime.user.CurrentUserProvider;

import com.opensymphony.workflow.InvalidInputException;
import com.opensymphony.workflow.WorkflowException;
import com.opensymphony.workflow.spi.Step;

/**
 * Action for creating a new org unit
 */
public class AddOrgUnitAction extends InitializeContentWorkflowAction
{
    /** Constant for parent identifier */
    public static final String CONTENT_PARENT_ID = CreateODFContentFunction.class.getName() + "$parentId";
    
    /** Ametys object resolver available to subclasses. */
    protected AmetysObjectResolver _resolver;
    /** Observation manager available to subclasses. */
    protected ObservationManager _observationManager;
    private RootOrgUnitProvider _rootOUProvider;
    private CurrentUserProvider _currentUserProvider;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _observationManager = (ObservationManager) smanager.lookup(ObservationManager.ROLE);
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _rootOUProvider = (RootOrgUnitProvider) smanager.lookup(RootOrgUnitProvider.ROLE);
        _currentUserProvider = (CurrentUserProvider) smanager.lookup(CurrentUserProvider.ROLE);
    }
    
    @Override
    protected Map _act(Redirector redirector, Map objectModel, String source, Parameters parameters, int actionId, Map inputs) throws InvalidInputException, WorkflowException
    {
        Map result = super._act(redirector, objectModel, source, parameters, actionId, inputs);
        
        Map<String, Object> workflowResult = (Map<String, Object>) inputs.get(AbstractContentWorkflowComponent.RESULT_MAP_KEY);
        String contentId = (String) workflowResult.get("contentId");
        
        String parentId = _getParameters(objectModel).get(CONTENT_PARENT_ID);
        
        if (StringUtils.isNotEmpty(parentId))
        {
            DefaultAmetysObject parent = _resolver.resolveById(parentId);
            if (parent instanceof OrgUnit)
            {
                OrgUnit ou = (OrgUnit) parent;
                ou.addSubOrgUnit(contentId);
                
                applyChangesToParent (ou);
            }
        }
        else
        {
            // Attach new OrgUnit to root OrgUnit
            OrgUnit rootOU = _rootOUProvider.getRoot();
            if (rootOU != null && !rootOU.getId().equals(contentId))
            {
                rootOU.addSubOrgUnit(contentId);

                applyChangesToParent (rootOU);
            }
        }
        
        return result;
    }
    
    /**
     * Apply changes to parent {@link OrgUnit}
     * @param ou The parent orgunit
     * @throws WorkflowException
     */
    protected void applyChangesToParent(OrgUnit ou) throws WorkflowException
    {
        // Save changes
        ou.saveChanges();
        // Create new version
        ((VersionableAmetysObject) ou).checkpoint();
        
        // Notify listeners
        _observationManager.notify(new Event(_currentUserProvider.getUser(), ObservationConstants.CONTENT_MODIFIED, ou));
       
        // Do workflow action
        Map<String, Object> inputs = new HashMap<String, Object>();
        inputs.put(AbstractContentWorkflowComponent.RESULT_MAP_KEY, new HashMap<String, Object>());
        inputs.put(AbstractContentWorkflowComponent.CONTENT_KEY, ou);
        _workflow.doAction(ou.getWorkflowId(), 22, inputs);
        
        Step currentStep = (Step) _workflow.getCurrentSteps(ou.getWorkflowId()).iterator().next();
        ou.setCurrentStepId(currentStep.getStepId());
        ou.saveChanges();
    }

}
