/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.orgunit.generators;

import java.io.IOException;
import java.util.Collection;
import java.util.Map;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.generation.AbstractGenerator;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.lang.StringUtils;
import org.xml.sax.SAXException;

import org.ametys.cms.repository.Content;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.RootOrgUnitProvider;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;

/**
 * Generate the structure tree of OrgUnits. The tree root is the main OrgUnit
 */
public class OrgUnitTreeGenerator extends AbstractGenerator implements Serviceable
{
    private AmetysObjectResolver _resolver;
    private RootOrgUnitProvider _orgUnitProvider;
    
    public void service(ServiceManager manager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) manager.lookup(AmetysObjectResolver.ROLE);
        _orgUnitProvider = (RootOrgUnitProvider) manager.lookup(RootOrgUnitProvider.ROLE);
    }

    @Override
    public void setup(SourceResolver res, Map objModel, String src, Parameters params) throws ProcessingException, SAXException, IOException
    {
        super.setup(res, objModel, src, params);
    }

    public void generate() throws IOException, SAXException, ProcessingException
    {
        String id = parameters.getParameter("id", "");
        String parentPath = parameters.getParameter("parentPath", "");

        // to generate a part of the structure tree, limited to theses selected orgunits
        String selected = parameters.getParameter("selected", "");
        
        contentHandler.startDocument();
        if (selected.equals(""))
        {
            saxOrgUnit(id, parentPath, true);
        }
        else
        {
            String[] list = selected.split(",");
            for (String idOrgunit : list)
            {
                saxOrgUnit(idOrgunit, parentPath, false);
            }
        }
        contentHandler.endDocument();
    }

    /**
     * Sax a Orgunit
     * @param id
     * @param parentPath
     * @throws SAXException
     */
    private void saxOrgUnit(String id, String parentPath, boolean saxChildren) throws SAXException
    {
        String path = parentPath;
        OrgUnit ou = null;
        boolean isRoot = false;
        
        if (StringUtils.isNotEmpty(id) && StringUtils.startsWith(id, "orgunit"))
        {
            ou = _resolver.resolveById(id);
            isRoot = _orgUnitProvider.isRoot(ou);
        }
        else
        {
            ou = _orgUnitProvider.getRoot();
            isRoot = true;
        }
        
        if (StringUtils.isEmpty(path))
        {
            path = getOrgUnitContext(ou);
        }
        
        if (ou != null)
        {
            AttributesImpl attr = new AttributesImpl();
            attr.addCDATAAttribute("title", ou.getTitle());
            attr.addCDATAAttribute("id", ou.getId());
            attr.addCDATAAttribute("name", ou.getName());
            if (isRoot)
            {
                attr.addCDATAAttribute("isRoot", "true"); 
            }
            attr.addCDATAAttribute("path", isRoot ? "" : path);
            
            
            XMLUtils.startElement(contentHandler, "orgunit", attr);
            if (saxChildren)
            {
                Collection<String> childrenIDs = ou.getSubOrgUnits();
       
                if (childrenIDs != null)
                {
                    saxChildren(childrenIDs, isRoot ? "" : path);
                }
            }
            XMLUtils.endElement(contentHandler, "orgunit");
        }
    }

    private void saxChildren(Collection<String> ids, String parentPath) throws SAXException
    {
        if (!ids.isEmpty())
        {
            for (String orgUnitId : ids)
            {
                try
                {
                    Content subOrgUnit = _resolver.resolveById(orgUnitId);

                    AttributesImpl attr = new AttributesImpl();
                    attr.addCDATAAttribute("name", subOrgUnit.getName());
                    attr.addCDATAAttribute("title", subOrgUnit.getTitle());
                    attr.addCDATAAttribute("id", subOrgUnit.getId());
                    attr.addCDATAAttribute("path", parentPath + "/" + subOrgUnit.getName());
                    
                    Collection<String> childrenIDs = ((OrgUnit) subOrgUnit).getSubOrgUnits();
                    if (!childrenIDs.isEmpty())
                    {
                        attr.addCDATAAttribute("hasChild", "true");
                    }
                    XMLUtils.createElement(contentHandler, "orgunit", attr);
                }
                catch (UnknownMetadataException e)
                {
                    // The orgunit referenced may be not exists anymore or be not present in live
                }
            }
        }
    }

    /**
     * Get the orgunit context
     * @param ou The orgunit to get context of
     * @return The context
     */
    protected String getOrgUnitContext (OrgUnit ou)
    {
        if (_orgUnitProvider.isRoot(ou))
        {
            return "";
        }
        
        String context = "/" + ou.getName();
        
        OrgUnit parent = ou.getParentOrgUnit();
        while (parent != null && !_orgUnitProvider.isRoot(parent))
        {
            context = "/" + parent.getName() + context;
            parent = parent.getParentOrgUnit();
        }
        return context;
    }
}
