/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.person;

import javax.jcr.Node;

import org.apache.commons.lang.StringUtils;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.cdmfr.CDMEntity;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.metadata.RichText;

/**
 * Class representing a {@link Person}
 */
public class Person extends SynchronizableContent<PersonFactory> implements CDMEntity
{
    /** ldap uid */
    public static final String LDAP_UID = "ldapUid";
    
    /** Mandatory Identifier to generate the CDM-fr id */
    public static final String LOGIN = "login";

    /** Name property. */
    public static final String NAME = "lastName";

    /** Given name property. */
    public static final String GIVEN_NAME = "givenName";

    /** TITLE LABEL property. */
    public static final String TITLE = "personTitle";

    /** ROLE LABEL property. */
    public static final String ROLE = "role";

    /** Additionnal informations property. */
    public static final String ADDITIONAL_INFORMATIONS = "additionalInformations";

    /**
     * Constructor
     * @param node
     * @param parentPath
     * @param factory
     */
    public Person(Node node, String parentPath, PersonFactory factory)
    {
        super(node, parentPath, factory);
    }

    // --------------------------------------------------------------------------------------//
    //
    // GETTERS AND SETTERS
    //
    // --------------------------------------------------------------------------------------//

    /**
     * Get the login of the person
     * This identifier will be the CDM-fr id of the person
     * @return the login or null
     */
    public String getLogin()
    {
        return getString(Person.LOGIN);
    }
    /**
     * Set the login of the person
     * This identifier will be the CDM-fr id of the person
     * @param login the login to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setLogin(String login) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Person.LOGIN, login);
    }
    
    /**
     * Get the lastname of the person
     * @return the Name or null
     */
    public String getLastName()
    {
        return getString(NAME);
    }
    /**
     * Set the lastname of the person
     * @param name the Name to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setLastName(String name) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(NAME, name);
    }
    
    /**
     * Get the given name of the person
     * @return the given name or null
     */
    public String getGivenName()
    {
        return getString(GIVEN_NAME);
    }

    /**
     * Set the given name of the person
     * @param name the Name to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setGivenName(String name) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(GIVEN_NAME, name);
    }
    
    /**
     * Get the contact data informations
     * @return ContactData
     */
    public ContactData getContactData()
    {
        ContactData contact = new ContactData();
        contact.setAddress(getString(ContactData.ADDRESS));
        contact.setAdditionalAddress(getString(ContactData.ADDITIONAL_ADDRESS));
        contact.setZipCode(getString(ContactData.ZIP_CODE));
        contact.setTown(getString(ContactData.TOWN));
        contact.setPhone(getString(ContactData.PHONE));
        contact.setFax(getString(ContactData.FAX));
        contact.setMail(getString(ContactData.MAIL));
        contact.setWebLinkLabel(getString(ContactData.WEB_LINK_LABEL));
        contact.setWebLinkUrl(getString(ContactData.WEB_LINK_URL));
        return contact;
    }
    
    /**
     * Get the Title posessed by the person
     * @return ContactData
     */
    public String getPersonTitle()
    {
        return getString(TITLE);
    }
    /**
     * Set the Title posessed by the person
     * @param title 
     */
    public void setPersonTitle(String title)
    {
        getMetadataHolder().setMetadata(TITLE, title);
    }
    
    /**
     * Get the role informations, a person may have different roles in different contexts, e.g. lecturer.
     * @return role
     */
    public String[] getRole()
    {
        return getStringArray(ROLE);
    }
    /**
     * Set the role
     * A person may have different roles in different contexts, e.g. lecturer.
     * @param role 
     */
    public void setRole(String[] role)
    {
        getMetadataHolder().setMetadata(ROLE, role);
    }
    
    /**
     * Return the additionnals informations
     * @return the description
     */
    public RichText getAdditionalInformations()
    {
        return getRichText(Person.ADDITIONAL_INFORMATIONS);
    }
    
    @Override
    public String getCDMId()
    {
        String login = getLogin();
        return "FRUAI" + _getFactory()._getRootOrgUnitRNE() + "PE" + (StringUtils.isNotEmpty(login) ? login : getName());
    }
    
    @Override
    @Deprecated
    public void toCDM(ContentHandler contentHandler, OdfEnumerationHelper provider, String rootOrgUnitId) throws SAXException
    {
        _getFactory()._getExportCDMfrManager().person2CDM(contentHandler, this);
    }
}
