/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.person.ldap;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.Control;
import javax.naming.ldap.InitialLdapContext;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.SortControl;

import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.lang.StringUtils;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.ContentTypeExpression;
import org.ametys.odf.person.ContactManager;
import org.ametys.odf.person.Person;
import org.ametys.odf.person.PersonFactory;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.query.expression.AndExpression;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.StringExpression;
import org.ametys.runtime.plugins.core.util.ldap.AbstractLDAPConnector;

/**
 *  Use a ldap server for getting the list of contacts.
 */
public class LdapContactManager extends AbstractLDAPConnector implements ContactManager, Serviceable, ThreadSafe, Component
{
    /** Relative DN for users. */
    protected String _relativeDN;
    /** Filter for limiting the search. */
    protected String _objectFilter;
    /** The scope used for search. */
    protected int _searchScope;
    /** Name of the login attribute. */
    protected String _loginAttribute;
    /** Name of the title attribute. */
    protected String _titleAttribute;
    /** Name of the role attribute. */
    protected String _roleAttribute;
    /** Name of the first name attribute. */
    protected String _firstnameAttribute;
    /** Name of the last name attribute. */
    protected String _lastnameAttribute;
    /** Name of the email attribute. */
    protected String _emailAttribute;
    /** To know if email is a mandatory attribute */
    protected boolean _emailIsMandatory;
    /** Name of the phone attribute. */
    protected String _phoneAttribute;
    /** Name of the fax attribute. */
    protected String _faxAttribute;
    /** Name of the web link label attribute. */
    protected String _webLinkLabelAttribute;
    /** Name of the web link attribute. */
    protected String _webLinkAttribute;
    /** Name of the address attribute. */
    protected String _addressAttribute;
    /** Name of the street attribute. */
    protected String _streetAttribute;
    /** Name of the zip code attribute. */
    protected String _zipCodeAttribute;
    /** Name of the town attribute. */
    protected String _townAttribute;
    
    /** complete list of attribute to retrieve for a person */
    protected String[] _attrsList;
    
    /** Ametys object resovler. */
    protected AmetysObjectResolver _resolver;


    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        _resolver = (AmetysObjectResolver) manager.lookup(AmetysObjectResolver.ROLE);
    }
    
    @Override
    public void configure(Configuration configuration) throws ConfigurationException
    {
        super.configure(configuration);
        
        List<String> attrsList = new ArrayList<String>();
        
        _relativeDN = _getConfigParameter(configuration, "RelativeDN");
        _objectFilter = _getFilter(configuration, "Filter");
        _searchScope = _getSearchScope(configuration, "SearchScope");
        _loginAttribute = _getConfigParameter(configuration, "Login");
        
        _firstnameAttribute = _getConfigParameter(configuration, "GivenName");
        if (StringUtils.isNotEmpty(_firstnameAttribute))
        {
            attrsList.add(_firstnameAttribute);
        }
        else
        {
            _firstnameAttribute = null;
        }
        
        _lastnameAttribute = _getConfigParameter(configuration, "LastName");
        attrsList.add(_lastnameAttribute);
        _emailAttribute = _getConfigParameter(configuration, "Email");
        attrsList.add(_emailAttribute);
        _emailIsMandatory = configuration.getChild("Email").getAttributeAsBoolean("mandatory", false);
        
        _phoneAttribute = _getConfigParameter(configuration, "Phone");
        if (StringUtils.isEmpty(_phoneAttribute))
        {
            _phoneAttribute = null;
        }
        else
        {
            attrsList.add(_phoneAttribute);
        }

        _faxAttribute = _getConfigParameter(configuration, "Fax");
        if (StringUtils.isEmpty(_faxAttribute))
        {
            _faxAttribute = null;
        }
        else
        {
            attrsList.add(_faxAttribute);
        }

        _streetAttribute = _getConfigParameter(configuration, "Street");
        if (StringUtils.isEmpty(_streetAttribute))
        {
            _streetAttribute = null;
        }
        else
        {
            attrsList.add(_streetAttribute);
        }
        
        _addressAttribute = _getConfigParameter(configuration, "Address");
        if (StringUtils.isEmpty(_addressAttribute))
        {
            _addressAttribute = null;
        }
        else
        {
            attrsList.add(_addressAttribute);
        }

        _zipCodeAttribute = _getConfigParameter(configuration, "ZipCode");
        if (StringUtils.isEmpty(_zipCodeAttribute))
        {
            _zipCodeAttribute = null;
        }
        else
        {
            attrsList.add(_zipCodeAttribute);
        }

        _townAttribute = _getConfigParameter(configuration, "Town");
        if (StringUtils.isEmpty(_townAttribute))
        {
            _townAttribute = null;
        }
        else
        {
            attrsList.add(_townAttribute);
        }

        _webLinkLabelAttribute = _getConfigParameter(configuration, "WebLinkLabel");
        if (StringUtils.isEmpty(_webLinkLabelAttribute))
        {
            _webLinkLabelAttribute = null;
        }
        else
        {
            attrsList.add(_webLinkLabelAttribute);
        }

        _webLinkAttribute = _getConfigParameter(configuration, "WebLink");
        if (StringUtils.isEmpty(_webLinkAttribute))
        {
            _webLinkAttribute = null;
        }
        else
        {
            attrsList.add(_webLinkAttribute);
        }

        _titleAttribute = _getConfigParameter(configuration, "Title");
        if (StringUtils.isEmpty(_titleAttribute))
        {
            _titleAttribute = null;
        }
        else
        {
            attrsList.add(_titleAttribute);
        }

        _roleAttribute = _getConfigParameter(configuration, "Role");
        if (StringUtils.isEmpty(_roleAttribute))
        {
            _roleAttribute = null;
        }
        else
        {
            attrsList.add(_roleAttribute);
        }
        
        _attrsList = attrsList.toArray(new String[attrsList.size()]);
    }
    
    /**
     * Sax the user list.
     * 
     * @param handler The content handler to sax in.
     * @param count The maximum number of users to sax. (-1 to sax all) Warning,
     *            if LDAP contains not valuable objects (with missing mandatory
     *            attributes) they will be ignored but the search scope is still
     *            limited to <code>count</code> : in other words you may
     *            receive less than 'count' results even if the ldap contains
     *            more matching entries
     * @param offset The offset to start with, first is 0.
     * @param parameters Parameters containing a pattern to match :
     *            <ul>
     *            <li>"pattern" => The pattern to match (String) or null to get
     *            all the users.
     *            </ul>
     * @throws SAXException If an error occurs while saxing.
     */
    @Override
    public void toSAX(ContentHandler handler, int count, int offset, Map parameters) throws SAXException
    {
        String pattern = (String) parameters.get("pattern");
        if (pattern != null && pattern.length() == 0)
        {
            pattern = null;
        }
        boolean hideExisting = "true".equals(parameters.get("hide"));

        XMLUtils.startElement(handler, "users");
        if (count != 0)
        {
            Map<String, Map<String, Object>> entries = new LinkedHashMap<String, Map<String, Object>>();
            
            int currentOffset = _toSAXInternal(handler, entries, count, offset >= 0 ? offset : 0, pattern, 0, hideExisting);
            if (count == -1 || currentOffset == -1)
            {
                String total = "" + currentOffset;
                XMLUtils.createElement(handler, "total", total);
            }
        }
        XMLUtils.endElement(handler, "users");
    }

    /**
     * Retrieve a person with its login
     * @param login
     * @return LdapEntiy
     */
    @Override
    public LdapEntity getPerson(String login)
    {
        LdapEntity person = null;
        DirContext context = null;
        NamingEnumeration results = null;
        try
        {
            // Connexion au serveur ldap
            context = new InitialDirContext(_getContextEnv());

            // Créer le filtre de recherche en échappant le login
            String filter = "(&" + _objectFilter + "(" + _loginAttribute + "={0}))";
            Object[] params = new Object[] {login};

            // Effectuer la recherche
            results = context.search(_relativeDN, filter, params, _getSearchConstraint(true, 0));

            // Chercher l'utilisateur voulu
            if (results.hasMoreElements())
            {
                Map<String, Object> attributes = _getAttributes((SearchResult) results.nextElement());
                if (attributes != null)
                {
                    person = new LdapEntity();
                    person.setLogin((String) attributes.get(_loginAttribute));                    
                    _setAttributes(person, attributes);
                }
            }

            if (results.hasMoreElements())
            {
                // Annuler le résultat car plusieurs correspondances pour un login
                person = null;
                getLogger().error("Multiple matches for attribute '" + _loginAttribute + "' and value = '" + login + "'");
            }

        }
        catch (IllegalArgumentException e)
        {
            getLogger().error("Error missing at least one attribute or attribute value", e);
        }
        catch (NamingException e)
        {
            getLogger().error("Error communication with ldap server", e);
        }

        finally
        {
            // Fermer les ressources de connexion
            _cleanup(context, results);
        }

        // Retourner l'utilisateur trouvé ou null
        return person;
    }

    /**
     * Set the  {@link LdapEntity} attributes
     * @param person
     * @param attributes
     */
    private void _setAttributes(LdapEntity person, Map<String, Object> attributes)
    {
        if (_firstnameAttribute != null)
        {
            person.setFirstName((String) attributes.get(_firstnameAttribute));
        }
        if (_lastnameAttribute != null)
        {
            person.setLastName((String) attributes.get(_lastnameAttribute));
        }
        if (_emailAttribute != null)
        {
            person.setMail((String) attributes.get(_emailAttribute));
        }
        if (_phoneAttribute != null)
        {
            person.setPhone((String) attributes.get(_phoneAttribute));
        }
        if (_faxAttribute != null)
        {
            person.setFax((String) attributes.get(_faxAttribute));
        }
        if (_streetAttribute != null)
        {
            person.setStreet((String) attributes.get(_streetAttribute));
        }
        if (_addressAttribute != null)
        {
            person.setAddress((String) attributes.get(_addressAttribute));
        }
        if (_zipCodeAttribute != null)
        {
            person.setZipCode((String) attributes.get(_zipCodeAttribute));
        }
        if (_townAttribute != null)
        {
            person.setTown((String) attributes.get(_townAttribute));
        }
        if (_webLinkLabelAttribute != null)
        {
            person.setWebLinkLabel((String) attributes.get(_webLinkLabelAttribute));
        }
        if (_webLinkAttribute != null)
        {
            person.setWebLink((String) attributes.get(_webLinkAttribute));
        }
        if (_titleAttribute != null)
        {
            person.setTitle((String) attributes.get(_titleAttribute));
        }
        if (_roleAttribute != null)
        {
            person.setRole((String) attributes.get(_roleAttribute));
        }
    }

    /**
     * Get a list of users
     * @return Collection of {@link LdapEntity}
     */
    @Override
    public Collection<LdapEntity> getPersons()
    {
        // Créer une liste d'utilisateurs
        List<LdapEntity> users = new ArrayList<LdapEntity>();
        
        DirContext context = null;
        NamingEnumeration results = null;

        try
        {
            // Connexion au serveur ldap
            context = new InitialDirContext(_getContextEnv());

            // Effectuer la recherche
            results = context.search(_relativeDN, _objectFilter, _getSearchConstraint(false, 0));

            // Remplir la liste des utilisateurs
            while (results.hasMoreElements())
            {
                Map<String, Object> attributes = _getAttributes((SearchResult) results.nextElement());
                if (attributes != null)
                {
                    LdapEntity user = new LdapEntity();
                    user.setLogin((String) attributes.get(_loginAttribute));     

                    _setAttributes(user, attributes);

                    users.add(user);
                }
            }
        }
        catch (IllegalArgumentException e)
        {
            getLogger().error("Error missing at least one attribute or attribute value", e);
        }
        catch (NamingException e)
        {
            getLogger().error("Error communication with ldap server", e);
        }
        finally
        {
            // Fermer les ressources de connexion
            _cleanup(context, results);
        }

        // Retourner la liste des utilisateurs sous forme de collection
        // d'utilisateurs, éventuellement vide
        return users;
    }
   
    /**
     * Sax the user list.
     * 
     * @param handler The content handler to sax in.
     * @param entries Where to store entries
     * @param count The maximum number of users to sax. Cannot be 0. Can be -1 to all.
     * @param offset The results to ignore
     * @param pattern The pattern to match.
     * @param possibleErrors This number will be added to count to set the max of the request, but count results will still be returned. The difference stands for errors.
     * @param hideExisting
     * @return the final offset
     * @throws SAXException If an error occurs while saxing.
     */
    protected int _toSAXInternal(ContentHandler handler, Map<String, Map<String, Object>> entries, int count, int offset, String pattern, int possibleErrors, boolean hideExisting) throws SAXException
    {        
        LdapContext context = null;
        NamingEnumeration results = null;

        try
        {
            // Connexion au serveur ldap
            context = new InitialLdapContext(_getContextEnv(), null);
            context.setRequestControls(_getSortControls());

            Map filter = _getPatternFilter(pattern);

            // Effectuer la recherche
            results = context.search(_relativeDN, 
                                    (String) filter.get("filter"), 
                                    (Object[]) filter.get("params"), 
                                    _getSearchConstraint(false, count == -1 ? 0 : (count + offset + possibleErrors)));

            // Sax results
            return _sax(handler, entries, count, offset, pattern, results, possibleErrors, hideExisting);
        }
        catch (IllegalArgumentException e)
        {
            getLogger().error("Error missing at least one attribute or value", e);
            return -1;
        }
        catch (NamingException e)
        {
            getLogger().error("Error durring the communication with ldap server", e);
            return -1;
        }
        finally
        {
            // Fermer les ressources de connexion
            _cleanup(context, results);
        }
    }
    
    /**
     * Get the sort control.
     * @return the sort controls. May be empty if a small error occurs
     * @throws SAXException if a fatal error occurs
     */
    protected Control[] _getSortControls() throws SAXException
    {
        try
        {
            SortControl sortControl = new SortControl(new String[] {_lastnameAttribute, _firstnameAttribute, _loginAttribute}, Control.NONCRITICAL);
            return new Control[] {sortControl};
        }
        catch (IOException e)
        {
            getLogger().warn("Cannot sort request on LDAP", e);
            return new Control[0];
        }
    }        
  
    /**
     * Get the filter from a pattern.
     * 
     * @param pattern The pattern to match.
     * @return The result as a Map containing the filter and the parameters.
     */
    protected Map _getPatternFilter(String pattern)
    {
        Map<String, Object> result = new HashMap<String, Object>();

        // Vérifier si l'on a un motif
        if (pattern == null)
        {
            result.put("filter", _objectFilter);
            result.put("params", new Object[0]);
        }
        else
        {
            // Créer le filtre de recherche en échappant les variables
            StringBuffer filter = new StringBuffer("(&" + _objectFilter + "(|(");
            Object[] params = null;

            if (_firstnameAttribute == null)
            {
                filter.append(_loginAttribute);
                filter.append("=*{0}*)(");
                filter.append(_lastnameAttribute);
                filter.append("=*{1}*)(");
                filter.append(_emailAttribute);
                filter.append("=*{2}*)))");
                params = new Object[] {pattern, pattern, pattern};
            }
            else
            {
                filter.append(_loginAttribute);
                filter.append("=*{0}*)(");
                filter.append(_firstnameAttribute);
                filter.append("=*{1}*)(");
                filter.append(_lastnameAttribute);
                filter.append("=*{2}*)(");
                filter.append(_emailAttribute);
                filter.append("=*{3}*)))");
                params = new Object[] {pattern, pattern, pattern, pattern};
            }

            result.put("filter", filter.toString());
            result.put("params", params);
        }
        return result;
    }    
    
    
    /**
     * Get constraints for a search.
     * @param getAllAttrs 
     * 
     * @param maxResults The maximum number of items that will be retrieve (0
     *            means all)
     * @return The constraints as a SearchControls.
     */
    protected SearchControls _getSearchConstraint(boolean getAllAttrs, int maxResults)
    {
        // Paramètres de recherche
        SearchControls constraints = new SearchControls();
        if (!getAllAttrs)
        {
            int attributesCount = 4;
            int index = 0;
    
            if (_firstnameAttribute == null)
            {
                attributesCount--;
            }
    
            // Positionner les attributs voulus
            String[] attrs = new String[attributesCount];
    
            attrs[index++] = _loginAttribute;
            if (_firstnameAttribute != null)
            {
                attrs[index++] = _firstnameAttribute;
            }
            attrs[index++] = _lastnameAttribute;
            attrs[index++] = _emailAttribute;
            
            constraints.setReturningAttributes(attrs);
        }
        else
        {
            constraints.setReturningAttributes(null);
        }

        // Choisir la profondeur de la recherche
        constraints.setSearchScope(_searchScope);

        if (maxResults > 0)
        {
            constraints.setCountLimit(maxResults);
        }

        return constraints;
    }
    
    
    /**
     * Sax LDAP results
     * @param handler The content handler to sax in
     * @param entries where to store entries
     * @param count The count limit (cannot be 0, but can be -1)
     * @param offset The offset for starting search
     * @param pattern The pattern
     * @param results The LDAP seach result
     * @param possibleErrors The number of additionnal results that stand for errors
     * @return the current offset at the end
     * @throws SAXException if an error occured
     * @throws IllegalArgumentException if an error occured
     */
    private int _sax(ContentHandler handler, Map<String, Map<String, Object>> entries, int count, int offset, String pattern, NamingEnumeration results, int possibleErrors, boolean hideExisting) throws SAXException
    {
        int nbResults = 0;
        
        boolean hasMoreElement = results.hasMoreElements();
        
        // First loop on the items to ignore (before the offset)
        while (nbResults < offset && hasMoreElement)
        {
            nbResults++;
            
            // FIXME we should check that this element has really attributes to count it as an real offset
            results.nextElement();

            hasMoreElement = results.hasMoreElements();
        }
        
        // Second loop to work
        while ((count == -1 || entries.size() < count) && hasMoreElement)
        {
            nbResults++;
            
            // Passer à l'entrée suivante
            SearchResult result = (SearchResult) results.nextElement();
            Map<String, Object> attrs = _getAttributes(result);
            if (attrs != null)
            {
                entries.put((String) attrs.get(_loginAttribute), attrs);
            }

            hasMoreElement = results.hasMoreElements();
        }


        // If we have less results than expected
        // can be due to errors (null attributes)
        // can be due to max results is less than wanted results
        if (entries.size() < count && nbResults == count + offset + possibleErrors)
        {
            double nbErrors = count + possibleErrors - entries.size();
            double askedResultsSize = possibleErrors + count;
            int newPossibleErrors = Math.max(possibleErrors + count - entries.size(), (int) Math.ceil((nbErrors / askedResultsSize + 1) * nbErrors));
            return _toSAXInternal(handler, entries, count, offset, pattern, newPossibleErrors, hideExisting);
        }
        else
        {
            for (Map<String, Object> attributes : entries.values())
            {
                _entryToSAX(handler, attributes, hideExisting);
            }
            return entries.size();
        }
    }
   
    /**
     * Sax a particular person
     * 
     * @param login the login of the person. Cannot be null.
     * @param handler The content handler to sax in.
     * @throws SAXException If an error occurs while saxing.
     */
    @Override
    public void saxPerson(String login, ContentHandler handler) throws SAXException
    {
        DirContext context = null;
        NamingEnumeration results = null;

        try
        {
            // Connexion au serveur ldap
            context = new InitialDirContext(_getContextEnv());

            // Créer le filtre de recherche en échappant le login
            String filter = "(&" + _objectFilter + "(" + _loginAttribute + "={0}))";
            Object[] params = new Object[] {login};

            // Effectuer la recherche
            results = context.search(_relativeDN, filter, params, _getSearchConstraint(true, 0));

            // Chercher la personne voulu
            if (results.hasMoreElements())
            {
                SearchResult result = (SearchResult) results.nextElement();
                _entryToSAX(handler, _getAttributes(result), false);
            }

            if (results.hasMoreElements())
            {
                // Annuler le résultat car plusieurs correspondances pour un login
                String errorMessage = "Multiple matches for attribute '" + _loginAttribute + "' and value = '" + login + "'";
                getLogger().error(errorMessage);
                throw new IllegalArgumentException(errorMessage);
            }
        }
        catch (NamingException e)
        {
            String errorMessage = "Error communication with ldap server";
            getLogger().error(errorMessage, e);
            throw new RuntimeException(errorMessage, e);
        }
        finally
        {
            // Fermer les ressources de connexion
            _cleanup(context, results);
        }
    }
    
    /**
     * Sax an ldap entry.
     * @param handler The content handler to sax in.
     * @param attributes The ldap attributes of the entry to sax.
     * @param hideExisting true if already imported entries should be hidden
     * @return true is the entry is correct
     * @throws IllegalArgumentException If a needed attribute is missing in the entry.
     * @throws SAXException If a problem occurs while saxing.
     */
    protected boolean _entryToSAX(ContentHandler handler, Map<String, Object> attributes, boolean hideExisting) throws SAXException
    {
        if (attributes == null)
        {
            return false;
        }
        
        String login = (String) attributes.get(_loginAttribute);
        String xpathQuery = getXPathQueryForPerson(login);
        AmetysObjectIterable<Content> it = _resolver.query(xpathQuery);
        boolean imported = it.hasNext();
        
        if (!hideExisting || (!imported && hideExisting))
        {
            AttributesImpl attr = new AttributesImpl();
            attr.addAttribute("", "login", "login", "CDATA", login);
            XMLUtils.startElement(handler, "user", attr);
            
            if (imported)
            {
                XMLUtils.createElement(handler, "imported", "true");
            }
            
            if (_firstnameAttribute != null)
            {
                String firstName = (String) attributes.get(_firstnameAttribute);
                XMLUtils.createElement(handler, "firstname", firstName != null ? firstName : "");
            }
            if (_lastnameAttribute != null)
            {
                String lastName = (String) attributes.get(_lastnameAttribute);
                XMLUtils.createElement(handler, "lastname", lastName != null ? lastName : "");
            }
            if (_emailAttribute != null)
            {
                String email = (String) attributes.get(_emailAttribute);
                XMLUtils.createElement(handler, "email", email != null ? email : "");
            }
            if (_roleAttribute != null)
            {
                String role = (String) attributes.get(_roleAttribute);
                XMLUtils.createElement(handler, "role", role != null ? role : "");
            }
            if (_titleAttribute != null)
            {
                String title = (String) attributes.get(_titleAttribute);
                XMLUtils.createElement(handler, "title", title != null ? title : "");
            }
            if (_phoneAttribute != null)
            {
                String phone = (String) attributes.get(_phoneAttribute);
                XMLUtils.createElement(handler, "phone", phone != null ? phone : "");
            }
            if (_faxAttribute != null)
            {
                String fax = (String) attributes.get(_faxAttribute);
                XMLUtils.createElement(handler, "fax", fax != null ? fax : "");
            }
            if (_streetAttribute != null)
            {
                String street = (String) attributes.get(_streetAttribute);
                XMLUtils.createElement(handler, "street", street != null ? street : "");
            }
            if (_addressAttribute != null)
            {
                String address = (String) attributes.get(_addressAttribute);
                XMLUtils.createElement(handler, "additionalAddress", address != null ? address : "");
            }
            if (_zipCodeAttribute != null)
            {
                String zipCode = (String) attributes.get(_zipCodeAttribute);
                XMLUtils.createElement(handler, "zipCode", zipCode != null ? zipCode : "");
            }
            if (_townAttribute != null)
            {
                String town = (String) attributes.get(_townAttribute);
                XMLUtils.createElement(handler, "town", town != null ? town : "");
            }
            if (_webLinkLabelAttribute != null)
            {
                String webLinkLabel = (String) attributes.get(_webLinkLabelAttribute);
                XMLUtils.createElement(handler, "webLinkLabel", webLinkLabel != null ? webLinkLabel : "");
            }
            if (_webLinkAttribute != null)
            {
                String webLink = (String) attributes.get(_webLinkAttribute);
                XMLUtils.createElement(handler, "webLink", webLink != null ? webLink : "");
            }
            handler.endElement("", "user", "user");
            
            return true;
        }

        return false;
    }    
    
    /**
     * Get attributes from a ldap entry.
     * 
     * @param entry The ldap entry to get attributes from.
     * @return The attributes in a map.
     * @throws IllegalArgumentException If a needed attribute is missing.
     */
    protected Map<String, Object> _getAttributes(SearchResult entry)
    {
        try
        {
            Map<String, Object> result = new HashMap<String, Object>();

            // Récupérer les attributs de l'entrée
            Attributes attrs = entry.getAttributes();

            // Récupérer le login
            Attribute ldapAttr = attrs.get(_loginAttribute);
            if (ldapAttr == null)
            {
                if (getLogger().isWarnEnabled())
                {
                    getLogger().debug("Missing login attribute : '" + _loginAttribute + "'");
                }
                return null;
            }
            result.put(_loginAttribute, ldapAttr.get());

            _fillMapAttribute(result, _firstnameAttribute, attrs);
            _fillMapAttribute(result, _lastnameAttribute, attrs);
            _fillMapAttribute(result, _titleAttribute, attrs);
            _fillMapAttribute(result, _roleAttribute, attrs);
            _fillMapAttribute(result, _emailAttribute, attrs);
            _fillMapAttribute(result, _phoneAttribute, attrs);
            _fillMapAttribute(result, _faxAttribute, attrs);
            _fillMapAttribute(result, _streetAttribute, attrs);
            _fillMapAttribute(result, _addressAttribute, attrs);
            _fillMapAttribute(result, _zipCodeAttribute, attrs);
            _fillMapAttribute(result, _townAttribute, attrs);
            _fillMapAttribute(result, _webLinkLabelAttribute, attrs);
            _fillMapAttribute(result, _webLinkAttribute, attrs);

            return result;
        }
        catch (NamingException e)
        {
            throw new IllegalArgumentException("Missing at least one value for an attribute in an ldap entry", e);
        }
    }   
    
    private void _fillMapAttribute(Map<String, Object> map, String attrName, Attributes attrs) throws NamingException
    {
        Attribute ldapAttr;

        if (attrName != null)
        {
            // Récupére l'attribut
            ldapAttr = attrs.get(attrName);
            if (ldapAttr == null)
            {
                map.put(attrName, "");
                getLogger().debug("Missing attribute : '" + attrName + "', for user '" + map.get(_loginAttribute) + "'.");
            }
            else
            {
                map.put(attrName, ldapAttr.get());
            }
        }
    }
    
    
    
    @Override
    public String getXPathQueryForPerson(String login)
    {
        Expression cTypeExpr = new ContentTypeExpression(Operator.EQ, PersonFactory.PERSON_CONTENT_TYPE);
        Expression loginExpr = new StringExpression(Person.LOGIN, Operator.EQ, login);
        Expression expr = new AndExpression(cTypeExpr, loginExpr);
        
        return org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, "ametys:content", expr, null);
    }
}
