/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.RepositoryException;
import javax.jcr.version.OnParentVersionAction;

import org.apache.jackrabbit.util.Text;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseListFactory;
import org.ametys.odf.enumeration.OdfEnumerationConstant;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.CollectionIterable;
import org.ametys.plugins.repository.RemovableAmetysObject;
import org.ametys.plugins.repository.RepositoryIntegrityViolationException;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.metadata.RichText;

/**
 * Abstract common superclass for {@link Program} and {@link SubProgram}.
 * @param <F> The actual type of the factory
 */
public abstract class AbstractProgram<F extends ProgramFactory> extends SynchronizableContent<F> implements ProgramPart
{
    /** already imported attribute */
    public static final String ALREADY_IMPORTED_ATTR = "imported";

    // ------------------------------------------------//
    //
    // PROGRAM METADATA
    //
    // -----------------------------------------------//
    /** Constants for metadata 'code' */
    public static final String CODE = "code";
    
    /** Mandatory Identifier to generate the CDM-fr id */
    public static final String CDM_CODE = "cdmCode";

    /** Constants for metadata 'degree' */
    public static final String DEGREE = "degree";

    /** Constants for metadata 'domain' */
    public static final String DOMAIN = "domain";
    
    /** Constants for metadata 'mention' */
    public static final String MENTION = "mention";

    /** Constants for metadata 'speciality' */
    public static final String SPECIALITY = "speciality";

    /** Constants for metadata 'educationLevel' */
    public static final String LEVEL = "educationLevel";

    /** Constants for metadata 'duration' */
    public static final String DURATION = "duration";
    
    /** Constants for metadata 'educationLanguage' */
    public static final String EDUC_LANGUAGE = "educationLanguage";

    /** Constants for metadata 'presentation' */
    public static final String PRESENTATION = "presentation";

    /** Constants for metadata 'objectives' */
    public static final String OBJECTIVES = "objectives";

    /** Constants for metadata 'qualification' */
    public static final String QUALIFICATION = "qualification";

    /** Constants for metadata 'teachingOrganization' */
    public static final String TEACHING_ORGANIZATION = "teachingOrganization";

    /** Constants for metadata 'accessCondition' */
    public static final String ACCESS_CONDITION = "accessCondition";

    /** Constants for metadata 'neededPrerequisite' */
    public static final String NEEDED_PREREQUISITE = "neededPrerequisite";

    /** Constants for metadata 'recommendedPrerequisite' */
    public static final String RECOMMENDED_PREREQUISITE = "recommendedPrerequisite";

    /** Constants for metadata 'furtherStudy' */
    public static final String FURTHER_STUDY = "furtherStudy";

    /** Constants for metadata 'studyAbroad' */
    public static final String STUDY_ABROAD = "studyAbroad";

    /** Constants for metadata 'targetGroup' */
    public static final String TARGET_GROUP = "targetGroup";

    /** Constants for metadata 'jobOpportunities' */
    public static final String JOB_OPPORTUNITIES = "jobOpportunities";

    /** Constants for metadata 'trainingStrategy' */
    public static final String TRAINING_STRATEGY = "trainingStrategy";
    
    /** Constants for metadata 'knowledgeCheck' */
    public static final String KNOWLEDGE_CHECK = "knowledgeCheck";

    /** Constants for metadata 'universalAdjustment' */
    public static final String UNIVERSAL_ADJUSTMENT = "universalAdjustment";

    /** Constants for metadata 'expenses' */
    public static final String EXPENSES = "expenses";

    /** Constants for metadata 'additionalInformations' */
    public static final String ADDITIONNAL_INFORMATIONS = "additionalInformations";

    /** Constants for metadata 'jointOrgUnit' */
    public static final String JOINT_ORGUNIT = "jointOrgUnit";

    /** Constants for metadata 'place' */
    public static final String PLACE = "place";

    /** Constants for metadata 'distanceLearning' */
    public static final String DISTANCE_LEARNING = "distanceLearning";

    /** Constants for metadata 'educationKind' */
    public static final String EDUCATION_KIND = "educationKind";

    /** Constant for metadata 'formofteachingOrg' */
    public static final String FORM_OF_TEACHING_ORG = "formofteachingOrg";

    /** Constants for metadata 'ects' */
    public static final String ECTS = "ects";

    /** Constants for metadata 'internship' */
    public static final String INTERNSHIP = "internship";

    /** Constants for metadata 'internshipDuration' */
    public static final String INTERNSHIP_DURATION = "internshipDuration";

    /** Constants for metadata 'internshipAbroad' */
    public static final String INTERNSHIP_ABROAD = "internshipAbroad";

    /** Constants for metadata 'internshipAbroadDuration' */
    public static final String INTERNSHIP_ABROAD_DURATION = "internshipAbroadDuration";
    
    /** Constants for metadata 'registrationStart' */
    public static final String REGISTRATION_START = "registrationStart";
    
    /** Constants for metadata 'registrationDeadline' */
    public static final String REGISTRATION_DEADLINE = "registrationDeadline";
    
    /** Constants for metadata 'teachingStart' */
    public static final String TEACHING_START = "teachingStart";

    /** Constants for metadata 'partnerSchools' */
    public static final String PARTNER_SCHOOLS = "partnerSchools";

    /** Constants for metadata 'partnerLaboratories' */
    public static final String PARTNER_LABORATORIES = "partnerLaboratories";

    /** Constants for metadata 'rncpCode' */
    public static final String RNCP_CODE = "rncpCode";

    /** Constants for metadata 'siseCode' */
    public static final String SISE_CODE = "siseCode";

    /** Constants for metadata 'Cite 97' */
    public static final String CITE97_CODE = "cite97Code";

    /** Constants for metadata 'erasmusCode' */
    public static final String ERASMUS_CODE = "erasmusCode";
    
    /** Constants for metadata 'erasmusCode' */
    public static final String DGESIP_CODE = "dgesipCode";
    
    /** Constants for metadata 'romeCode' */
    public static final String ROME_CODE = "romeCode";
    
    /** Constants for metadata 'romeCode' */
    public static final String FAP_CODE = "fapCode";
    
    /** Constants for metadata 'programWebSiteLabel' */
    public static final String PROGRAM_WEBSITE_LABEL = "programWebSiteLabel";

    /** Constants for metadata 'programWebSiteUrl' */
    public static final String PROGRAM_WEBSITE_URL = "programWebSiteUrl";

    /** Constants for metadata 'attachment' */
    public static final String ATTACHMENTS = "attachments";

    /** Constants for metadata 'numberOfStudents' */
    public static final String NUMBER_OF_STUDENTS = "numberOfStudents";
    
    /** Constants for metadata 'successRate' */
    public static final String SUCCESSRATE = "successRate";

    /** Constants for metadata 'reorientation' */
    public static final String REORIENTATION = "reorientation";
    
    /** Constants for metadata 'keywords' */
    public static final String KEYWORDS = "keywords";
    
    /**
     * References
     */
       
    /** Constants for metadata 'orgUnits' */
    public static final String ORG_UNITS_REFERENCES = "orgUnit";

    /** Constants for metadata 'personInCharge' */
    public static final String PERSONS_IN_CHARGE_REFERENCES = "personInCharge";

    /** Constants for metadata 'contact' */
    public static final String CONTACTS_REFERENCES = "contact";
    
    private String _contextPath;

    // ------------------------------------------------//
    //
    // PROGRAM METHODS
    //
    // -----------------------------------------------//
    /**
     * Constructor
     * 
     * @param node
     * @param parentPath
     * @param factory
     */
    public AbstractProgram(Node node, String parentPath, F factory)
    {
        super(node, parentPath, factory);
    }
    
    @Override
    public void remove() throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        AmetysObjectIterable<RemovableAmetysObject> children = this.getChildren();
        
        for (RemovableAmetysObject child : children)
        {
            child.remove();
        }
        
        super.remove();
    }
    
    @Override
    protected void restoreFromNode(Node node) throws RepositoryException
    {
        AmetysObjectIterable<AmetysObject> children = this.getChildren();
        for (AmetysObject child : children)
        {
            if (child instanceof SynchronizableContent)
            {
                ((SynchronizableContent) child).remove();
            }
        }
        
        // Copy the node with properties and sub-nodes (except sub-contents).
        restoreFromNodeWithoutSubcontents(getBaseNode(), node, false);
        
        // Then, restore sub-contents.
        restoreSubcontents(getBaseNode(), node);
    }
    
    /**
     * Restore sub-contents from an old version of the node.
     * @param parentNode the destination parent node.
     * @param srcNode the source node.
     * @throws RepositoryException if an error occurs.
     */
    private void restoreSubcontents(Node parentNode, Node srcNode) throws RepositoryException
    {
        NodeIterator subNodeIt = srcNode.getNodes();
        while (subNodeIt.hasNext())
        {
            Node subNode = subNodeIt.nextNode();
            
            // Test if the sub-node is a content by its "workflowRef" property.
            if (subNode.hasProperty("ametys-internal:workflowRef"))
            {
                // Copy the node with properties and sub-nodes (except sub-contents).
                Node destNode = parentNode.addNode(subNode.getName(), subNode.getProperty("jcr:frozenPrimaryType").getString());
                
                // Restore the node, again without its sub-contents.
                restoreFromNodeWithoutSubcontents(destNode, subNode, true);
                
                // Recreate the content workflow.
                _getFactory().recreateContentWorkflow(destNode);
                
                // Restore the sub-contents.
                restoreSubcontents(destNode, subNode);
            }
        }
    }
    
    /**
     * Restore from a node
     * @param destNode The destination node.
     * @param srcNode The node to restore.
     * @throws RepositoryException If error occurs
     */
    private void restoreFromNodeWithoutSubcontents(Node destNode, Node srcNode, boolean excludeWorkflowRef) throws RepositoryException
    {
        // Remove all properties and nodes of the current node (except jcr and OnParentVersion=IGNORE).
        PropertyIterator propIt = destNode.getProperties();
        while (propIt.hasNext())
        {
            Property prop = propIt.nextProperty();
            String propName = prop.getName();
            if (!propName.startsWith("jcr:") && prop.getDefinition().getOnParentVersion() != OnParentVersionAction.IGNORE)
            {
                prop.remove();
            }
        }
        
        NodeIterator nodeIt = destNode.getNodes();
        while (nodeIt.hasNext())
        {
            Node childNode = nodeIt.nextNode();
            String nodeName = childNode.getName();
            if (!nodeName.startsWith("jcr:") && childNode.getDefinition().getOnParentVersion() != OnParentVersionAction.IGNORE && !childNode.getReferences().hasNext())
            {
                childNode.remove();
            }
        }
        
        // Copy all properties and nodes from the given node (except jcr).
        PropertyIterator newPropIt = srcNode.getProperties();
        while (newPropIt.hasNext())
        {
            Property newProp = newPropIt.nextProperty();
            
            if (!newProp.getName().startsWith("jcr:") && !(excludeWorkflowRef && newProp.getName().equals("ametys-internal:workflowRef")))
            {
                if (newProp.getDefinition().isMultiple())
                {
                    destNode.setProperty(newProp.getName(), newProp.getValues());
                }
                else
                {
                    destNode.setProperty(newProp.getName(), newProp.getValue());
                }
            }
        }
        
        NodeIterator newNodeIt = srcNode.getNodes();
        while (newNodeIt.hasNext())
        {
            Node newNode = newNodeIt.nextNode();
            
            // Copy all nodes except sub-contents
            if (!newNode.getName().startsWith("jcr:") && !newNode.hasProperty("ametys-internal:workflowRef"))
            {
                copyNode(destNode, newNode);
            }
        }
    }
    
    public <A extends AmetysObject> A getChild(String path) throws AmetysRepositoryException, UnknownAmetysObjectException
    {
        return (A) _getFactory().getChild(this, path);
    }

    @Override
    public <A extends AmetysObject> AmetysObjectIterable<A> getChildren() throws AmetysRepositoryException
    {
        return _getFactory().getChildren(this);
    }

    @Override
    public boolean hasChild(String name) throws AmetysRepositoryException
    {
        return _getFactory().hasChild(this, name);
    }

    @Override
    public <A extends AmetysObject> A createChild(String name, String type) throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        return (A) _getFactory().createChild(this, name, type);
    }
    
    /**
     * Set the parent path for links and breadcrumb
     * @param path the parent path
     */
    public void setContextPath (String path)
    {
        _contextPath = path;
    }
    
    /**
     * Get the parent path. Can be null.
     * @return the parent path
     */
    public String getContextPath ()
    {
        return _contextPath;
    }
    
    /**
     * Remove reference from local and remote metadata
     * @param metadataName The metadata name
     * @param value The value of reference to remove 
     */
    public void removeReference (String metadataName, String value)
    {
        String[] localReferences = getMetadataHolder().getStringArray(metadataName, new String[0]);
        List<String> localList = new ArrayList<String>(Arrays.asList(localReferences));
        if (localList.contains(value))
        {
            localList.remove(value);
        }
        getMetadataHolder().setMetadata(metadataName, localList.toArray(new String[localList.size()]));
        
        String[] remoteReferences = getMetadataHolder().getStringArray(metadataName + REMOTE_SUFFIX, new String[0]);
        List<String> remoteList = new ArrayList<String>(Arrays.asList(remoteReferences));
        if (remoteList.contains(value))
        {
            remoteList.remove(value);
        }
        getMetadataHolder().setMetadata(metadataName + REMOTE_SUFFIX, remoteList.toArray(new String[remoteList.size()]));
    }
    
    // --------------------------------------------------------------------------------------//
    // COURSES LIST 
    // --------------------------------------------------------------------------------------//
    
    @Override
    public CourseList createCourseList(String name)
    {
        String legalName = Text.escapeIllegalJcrChars(name);
        
        // Find unique name
        int index = 1;
        while (hasChild(legalName))
        {
            legalName = Text.escapeIllegalJcrChars(name + " " + (index++));
        }
        
        return this.createChild(legalName, CourseListFactory.COURSE_LIST_NODETYPE);
        
    }
    
    @Override
    public AmetysObjectIterable<CourseList> getCourseLists()
    {
        List<CourseList> list = new ArrayList<CourseList>();
        
        AmetysObjectIterable<AmetysObject> children = getChildren();
        for (AmetysObject child : children)
        {
            if (child instanceof CourseList)
            {
                list.add((CourseList) child);
            }
        }

        return new CollectionIterable<CourseList>(list);
    }
    
    // --------------------------------------------------------------------------------------//
    // ORGUNITS
    // --------------------------------------------------------------------------------------//

    /**
     * Return the list of orgUnits binded to this program 
     * @return a list of uuid
     */
    public List<String> getOrgUnits()
    {
        String[] orgunits = getStringArray(ORG_UNITS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(orgunits));
    }
    
    /**
     * Bind a orgUnit element to this program
     * @param orgUnitId the uuid of a orgunit element
     */
    public void addOrgUnit(String orgUnitId)
    {
        String[] orgunits = getStringArray(ORG_UNITS_REFERENCES);
        
        List<String> orgunitsList = new ArrayList<String>(Arrays.asList(orgunits));
        orgunitsList.add(orgUnitId);
        
        getMetadataHolder().setMetadata(ORG_UNITS_REFERENCES, orgunitsList.toArray(new String[orgunitsList.size()]));
    }
    
    /**
     * Remove a orgunit from the list
     * @param orgUnitId the uuid of a orgunit element
     * @throws AmetysRepositoryException
     */
    public void removeOrgUnit(String orgUnitId) throws AmetysRepositoryException
    {
        removeReference (ORG_UNITS_REFERENCES, orgUnitId);
    }
    
    /**
     * Return the list of remote orgUnits binded to this program 
     * @return a list of uuid
     */
    public List<String> getLocalOrgUnits()
    {
        String[] orgunits = getMetadataHolder().getStringArray(ORG_UNITS_REFERENCES, new String[]{});
        return new ArrayList<String>(Arrays.asList(orgunits));
    }
    
    /**
     * Return the list of remote orgUnits binded to this program 
     * @return a list of uuid
     */
    public List<String> getRemoteOrgUnits()
    {
        String[] orgunits = getMetadataHolder().getStringArray(ORG_UNITS_REFERENCES + REMOTE_SUFFIX, new String[]{});
        return new ArrayList<String>(Arrays.asList(orgunits));
    }
    
    // --------------------------------------------------------------------------------------//
    // CONTACTS
    // --------------------------------------------------------------------------------------//

    /**
     * Return the list of Persons in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getPersonsInCharge()
    {
        String[] contacts = getStringArray(PERSONS_IN_CHARGE_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return the list of Persons in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getLocalPersonsInCharge()
    {
        String[] contacts = getMetadataHolder().getStringArray(PERSONS_IN_CHARGE_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return the list of Persons in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getRemotePersonsInCharge()
    {
        String[] contacts = getMetadataHolder().getStringArray(PERSONS_IN_CHARGE_REFERENCES + REMOTE_SUFFIX);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return the list of Contacts in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getContacts()
    {
        String[] contacts = getStringArray(CONTACTS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return the list of Contacts in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getLocalContacts()
    {
        String[] contacts = getMetadataHolder().getStringArray(CONTACTS_REFERENCES);
        return new ArrayList<String>(Arrays.asList(contacts));
    }
    
    /**
     * Return the list of Contacts in charge binded to this program 
     * @return a list of uuid
     */
    public List<String> getRemoteContacts()
    {
        String[] contacts = getMetadataHolder().getStringArray(CONTACTS_REFERENCES + REMOTE_SUFFIX);
        return new ArrayList<String>(Arrays.asList(contacts));
    }

    // --------------------------------------------------------------------------------------//
    // GETTERS AND SETTERS
    // --------------------------------------------------------------------------------------//

    /**
     * Get the code of this {@link Program}
     * @return the code or null if not set
     */
    public String getCode()
    {
        return getString(CODE);
    }

    /**
     * Set the code
     * @param code the code to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setCode(String code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(CODE, code);
    }

    /**
     * Get the CDM-fr of the {@link Program}
     * @return the CDM-fr code
     */
    public String getCdmCode()
    {
        return getString(CDM_CODE);
    }
    /**
     * Set the CDM-fr code
     * @param code the CDM-fr to set
     * @throws AmetysRepositoryException
     */
    public void setCdmCode(String code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(CDM_CODE, code);
    }

    /**
     * Get the degree
     * @return the degree or null
     */
    public String getDegree()
    {
        return getString(DEGREE);
    }

    /**
     * Get the degree
     * @param provider the {@link OdfEnumerationHelper}.
     * @return the degree or null
     */
    public String getDegree(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.DEGREE, getDegree(), false);
    }

    /**
     * Set the degree
     * @param degree the degree to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setDegree(String degree) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(DEGREE, degree);
    }

    /**
     * Get the domain
     * @return the domain or null
     */
    public String getDomain()
    {
        return getString(DOMAIN);
    }

    /**
     * Set the domain
     * @param domain the domain to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setDomain(String domain) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(DOMAIN, domain);
    }
    
    /**
     * Get the education presentation
     * @return the education presentation or null if not set
     */
    public RichText getPresentation()
    {
        return getRichText(PRESENTATION);
    }

    /**
     * Get the education objectives
     * @return the education objectives or null if not set
     */
    public RichText getObjectives()
    {
        return getRichText(OBJECTIVES);
    }

    /**
     * Get the education qualification
     * @return the education qualification or null if not set
     */
    public RichText getQualification()
    {
        return getRichText(QUALIFICATION);
    }

    /**
     * Get the education organization
     * @return the education organization or null if not set
     */
    public RichText getTeachingOrganization()
    {
        return getRichText(TEACHING_ORGANIZATION);
    }

    /**
     * Get the access conditions
     * @return  the access conditions  or null if not set
     */
    public RichText getAccessCondition()
    {
        return getRichText(ACCESS_CONDITION);
    }

    /**
     * Get the education requirements
     * @return the education requirements or null if not set
     */
    public RichText getNeededPrerequisite()
    {
        return getRichText(NEEDED_PREREQUISITE);
    }

    /**
     * Get the education recommended prerequisite
     * @return the education recommended prerequisite or null if not set
     */
    public RichText getRecommendedPrerequisite()
    {
        return getRichText(RECOMMENDED_PREREQUISITE);
    }

    /**
     * Get the education further study
     * @return the education further study or null if not set
     */
    public RichText getFurtherStudy()
    {
        return getRichText(FURTHER_STUDY);
    }

    /**
     * Get the education study abroard
     * @return the education study abroard or null if not set
     */
    public RichText getStudyAbroad()
    {
        return getRichText(STUDY_ABROAD);
    }

    /**
     * Get the education target
     * @return the education target or null if not set
     */
    public RichText getTargetGroup()
    {
        return getRichText(TARGET_GROUP);
    }

    /**
     * Get the job opportunities
     * @return the job opportunities or null if not set
     */
    public RichText getJobOpportunities()
    {
        return getRichText(JOB_OPPORTUNITIES);
    }

    /**
     * Get acknowledgments
     * @return acknowledgments or null if not set
     */
    public RichText getKnowledgeCheck()
    {
        return getRichText(KNOWLEDGE_CHECK);
    }

    /**
     * Get universal adjustment
     * @return universal adjustment or null if not set
     */
    public RichText getUniversalAdjustment()
    {
        return getRichText(UNIVERSAL_ADJUSTMENT);
    }

    /**
     * Get the additional informations
     * @return the additional informations or null if not set
     */
    public RichText getAdditionalInformations()
    {
        return getRichText(ADDITIONNAL_INFORMATIONS);
    }

    /**
     * Get the education level
     * @return the education level or null
     */
    public String getEducationLevel()
    {
        return getString(LEVEL);
    }

    /**
     * Get the level
     * @param provider the {@link OdfEnumerationHelper}.
     * @return the level
     */
    public String getEducationLevel(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.LEVEL, getEducationLevel(), true);
    }
    
    /**
     * Set the education level
     * @param level the education level to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setEducationLevel(String level) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(LEVEL, level);
    }

    /**
     * Get the RNCP code
     * @return the RNCP code
     */
    public String[] getRncpCode()
    {
        return getStringArray(RNCP_CODE);
    }

    /**
     * Set the RNCP code
     * @param code the RNCP code to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setRncpCode(String[] code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(RNCP_CODE, code);
    }

    /**
     * Get the SISE code
     * @return the SISE code
     */
    public String[] getSiseCode()
    {
        return getStringArray(SISE_CODE);
    }

    /**
     * Set the SISE code
     * @param code the SISE code to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setSiseCode(String[] code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(SISE_CODE, code);
    }

    /**
     * Get the CITE97 code
     * @return the CITE97 code
     */
    public String[] getCite97Code()
    {
        return getStringArray(CITE97_CODE);
    }

    /**
     * Set the CITE97 code
     * @param code the CITE97 code to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setCite97Code(String[] code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(CITE97_CODE, code);
    }
    
    /**
     * Get the Erasmus code
     * @return the Erasmus code
     */
    public String[] getDGESIPCode()
    {
        return getStringArray(DGESIP_CODE);
    }

    /**
     * Set the Erasmus code
     * @param code the Erasmus code to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setDGESIPCode(String[] code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(DGESIP_CODE, code);
    }

    /**
     * Get the Erasmus code
     * @return the Erasmus code
     */
    public String[] getErasmusCode()
    {
        return getStringArray(ERASMUS_CODE);
    }

    /**
     * Set the Erasmus code
     * @param code the Erasmus code to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setErasmusCode(String[] code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(ERASMUS_CODE, code);
    }
    
    /**
     * Set the ROME code
     * @param code the ROME codes to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setRomeCode(String[] code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(ROME_CODE, code);
    }

    /**
     * Get the ROME code
     * @return the ROME code
     */
    public String[] getRomeCode()
    {
        return getStringArray(ROME_CODE);
    }
    
    /**
     * Get the ROME code
     * @return the ROME code
     */
    public String[] getFapCode()
    {
        return getStringArray(FAP_CODE);
    }
    
    /**
     * Get the FAP code
     * @param fap the FAP codes to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setFapCode(String[] fap) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(FAP_CODE, fap);
    }
    
    /**
     * Get the mention
     * @return the mention or null
     */
    public String getMention()
    {
        return getString(MENTION);
    }

    /**
     * Get the mention
     * @param mention the mention to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setMention(String mention) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(MENTION, mention);
    }

    /**
     * Get the speciality
     * @return the speciality or null
     */
    public String getSpeciality()
    {
        return getString(SPECIALITY);
    }

    /**
     * Set the metadata Speciality
     * 
     * @param speciality
     * @throws AmetysRepositoryException
     */
    public void setSpeciality(String speciality) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(SPECIALITY, speciality);
    }

    /**
     * Get the org units
     * @return the org units
     */
    public String[] getJointOrgUnit()
    {
        return getStringArray(JOINT_ORGUNIT);
    }

    /**
     * Set the org unit
     * @param orgUnit the org unit to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setJointOrgUnit(String[] orgUnit) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(JOINT_ORGUNIT, orgUnit);
    }

    /**
     * Get the places
     * @return the places as an Array of String
     */
    public String[] getPlace()
    {
        return getStringArray(PLACE);
    }

    /**
     * Set the places
     * @param places the places to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setPlace(String[] places) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(PLACE, places);
    }

    /**
     * Get the site web label
     * @return the site web label or null
     */
    public String getProgramWebSiteLabel()
    {
        return getString(PROGRAM_WEBSITE_LABEL);
    }

    /**
     * Set the site web label
     * @param label the site web label to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setProgramWebSiteLabel(String label) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(PROGRAM_WEBSITE_LABEL, label);
    }

    /**
     * Get the site web url
     * @return the site web url or null
     */
    public String getProgramWebSiteUrl()
    {
        return getString(PROGRAM_WEBSITE_URL);
    }

    /**
     * Set the site web url
     * @param url the site web url to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setProgramWebSiteUrl(String url) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(PROGRAM_WEBSITE_URL, url);
    }

    /**
     * Get the ECTS credits
     * @return the ECTS credits
     */
    public String getEcts()
    {
        return getString(ECTS);
    }

    /**
     * Set the ECTS credits
     * @param ects the ECTS credits to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setEcts(String ects) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(ECTS, ects);
    }

    /**
     * Get the kind of education
     * @return the kind of education or null
     */
    public String getEducationKind()
    {
        return getString(EDUCATION_KIND);
    }


    /**
     * Get the level
     * @param provider the {@link OdfEnumerationHelper}.
     * @return the level
     */
    public String getEducationKind(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.PROGRAM_TYPE, getEducationKind(), true);
    }
    
    /**
     * Set the kind of education
     * @param kind the kind of education to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setEducationKind(String kind) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(EDUCATION_KIND, kind);
    }

    /**
     * Get the duration
     * @return the duration 
     */
    public String getDuration()
    {
        return getString(DURATION);
    }

    /**
     * Set the duration
     * @param duration the duration
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setDuration(String duration) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(DURATION, duration);
    }
    
    /**
     * Get the education language
     * @return the education language 
     */
    public String getEducationLanguage()
    {
        return getString(EDUC_LANGUAGE);
    }

    /**
     * Set the education language
     * @param lang the education language
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setEducationLanguage(String lang) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(EDUC_LANGUAGE, lang);
    }

    /**
     * Get the effectives
     * @return the effectives or 0 if not set
     */
    public String getEffectives()
    {
        return getString(NUMBER_OF_STUDENTS);
    }

    /**
     * Set the effectives
     * @param effective the effectives to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setEffectives(String effective) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(NUMBER_OF_STUDENTS, effective);
    }

    /**
     * Get the SuccessRate
     * @return the SuccessRate 
     */
    public String getSuccessRate()
    {
        return getString(SUCCESSRATE);
    }
    
    /**
     * Get the expenses
     * @return the expenses
     */
    public RichText getExpenses()
    {
        return getRichText(EXPENSES);
    }
    
    /**
     * Set the school fee
     * @param fee the school fee to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setExpenses(String fee) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(EXPENSES, fee);
    }
    
    /**
     * Get the form of teaching organization
     * @return the form of teaching organization
     */
    public String[] getFormOfTeachingOrgs()
    {
        return getStringArray(FORM_OF_TEACHING_ORG);
    }
    
    /**
     * Get the forms of teaching organization
     * @param provider the {@link OdfEnumerationHelper}.
     * @return the forms of teaching organization
     */
    public String[] getFormOfTeachingOrgs(OdfEnumerationHelper provider)
    {
        List<String> cdmValues = new ArrayList<String>();
        
        String[] teachingOrgs = getFormOfTeachingOrgs();
        for (String teachingOrg : teachingOrgs)
        {
            cdmValues.add(provider.getItemCDMfrValue(OdfEnumerationConstant.FORMOFTEACHING_ORG, teachingOrg, false));
        }
        return cdmValues.toArray(new String[cdmValues.size()]);
    }
    
    /**
     * Set the form of teaching organization.
     * @param formOfTeachingOrg the kind of education to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setFormOfTeachingOrg(String[] formOfTeachingOrg) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(FORM_OF_TEACHING_ORG, formOfTeachingOrg);
    }
    
    /**
     * Get the REORIENTATION 
     * @return RichText
     */
    public RichText getReorientation()
    {
        return getRichText(REORIENTATION);
    }
    
    /**
     * Get the distance learning
     * @return the distance learning or null
     */
    public String getDistanceLearning()
    {
        return getString(DISTANCE_LEARNING);
    }

    /**
     * Get the distance learning
     * @param provider the {@link OdfEnumerationHelper}.
     * @return the distance learning or null
     */
    public String getDistanceLearning(OdfEnumerationHelper provider)
    {
        return provider.getItemCDMfrValue(OdfEnumerationConstant.DISTANCE_LEARNING_MODALITIES, getDistanceLearning(), false);
    }
    
    /**
     * Set the distance learning
     * @param distance the distance learning to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setDistanceLearning(String distance) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(DISTANCE_LEARNING, distance);
    }

    /**
     * Get the internship
     * @return the internship or null
     */
    public String getInternship()
    {
        return getString(INTERNSHIP);
    }

    /**
     * Set the internship
     * @param internship the internship to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setInternship(String internship) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(INTERNSHIP, internship);
    }

    /**
     * Get the internship duration
     * @return the internship duration or null
     */
    public String getInternshipDuration()
    {
        return getString(INTERNSHIP_DURATION);
    }

    /**
     * Set the internship duration
     * @param duration the internship duration to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setInternshipDuration(String duration) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(INTERNSHIP_DURATION, duration);
    }

    /**
     * Get the internship abroad
     * @return the internship abroad or null
     */
    public String getInternshipAbroad()
    {
        return getString(INTERNSHIP_ABROAD);
    }

    /**
     * Set the internship abroad
     * @param abroad the internship abroad to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setInternshipAbroad(String abroad) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(INTERNSHIP_ABROAD, abroad);
    }

    /**
     * Get the internship abroad duration
     * @return  the internship abroad duration or null
     */
    public String getAbroadInternshipDuration()
    {
        return getString(INTERNSHIP_ABROAD_DURATION);
    }

    /**
     * Set the internship abroad duration
     * @param duration the internship abroad duration to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setAbroadInternshipDuration(String duration) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(INTERNSHIP_ABROAD_DURATION, duration);
    }

    /**
     * Get the registration start date.
     * @return The registration start date, can be null.
     */
    public Date getRegistrationStart()
    {
        return getDate(REGISTRATION_START);
    }
    
    /**
     * Set the registration start date.
     * @param registrationStart The registration start date.
     * @throws AmetysRepositoryException if an error occurred.
     */
    public void setRegistrationStart(Date registrationStart) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(REGISTRATION_START, registrationStart);
    }
    
    /**
     * Get the registration deadline date.
     * @return The registration deadline date, can be null.
     */
    public Date getRegistrationDeadline()
    {
        return getDate(REGISTRATION_DEADLINE);
    }
    
    /**
     * Set the registration deadline date.
     * @param registrationDeadline The registration deadline date.
     * @throws AmetysRepositoryException if an error occurred.
     */
    public void setRegistrationDeadline(Date registrationDeadline) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(REGISTRATION_DEADLINE, registrationDeadline);
    }
    
    /**
     * Get the teaching start date.
     * @return The teaching start date, can be null.
     */
    public Date getTeachingStart()
    {
        return getDate(TEACHING_START);
    }
    
    /**
     * Set the teaching start date.
     * @param teachingStart The teaching start date.
     * @throws AmetysRepositoryException if an error occurred.
     */
    public void setTeachingStart(Date teachingStart) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(TEACHING_START, teachingStart);
    }
    
    /**
     * Get the keywords
     * @return the keywords
     */
    public String[] getKeywords()
    {
        return getStringArray(KEYWORDS);
    }

    /**
     * Set the keywords
     * @param keywords the keywords to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setKeywords(String[] keywords) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(KEYWORDS, keywords);
    }
    
    // --------------------------------------------------------------------------------------//
    // CDM-fr
    // --------------------------------------------------------------------------------------//
    
    public String getCDMId() 
    {
        return "FRUAI" + _getFactory()._getRootOrgUnitRNE() + "PR" + getCdmCode();
    }
    
    /**
     * Returns the surrounding tag name in the CDM-fr representation.
     * @return the surrounding tag name.
     */
    public abstract String getTagName();
    
    @Override
    @Deprecated
    public void toCDM(ContentHandler contentHandler, OdfEnumerationHelper provider, Set<String> persons, Set<String> orgUnits, Set<String> courses) throws SAXException
    {
        _getFactory()._getExportCDMfrManager().abstractProgram2CDM(contentHandler, this, persons, orgUnits, courses);
    }
}
