/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.lock.Lock;
import javax.jcr.lock.LockManager;

import org.apache.jackrabbit.util.Text;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.cdmfr.CDMFRTagsConstants;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseListFactory;
import org.ametys.odf.enumeration.OdfEnumerationHelper;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.CollectionIterable;
import org.ametys.plugins.repository.MovableAmetysObject;
import org.ametys.plugins.repository.RemovableAmetysObject;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.repository.RepositoryIntegrityViolationException;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.jcr.DefaultAmetysObject;
import org.ametys.plugins.repository.metadata.RichText;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;

/**
 * Container java object
 */
public class Container extends SynchronizableContent<ContainerFactory> implements ProgramPart, MovableAmetysObject, CDMFRTagsConstants
{

    /** Constants for contentType Metadata* */
    public static final String CODE = "code";
    
    /** Mandatory Identifier to generate the CDM-fr id */
    public static final String CDM_CODE = "cdmCode";

    /** Constants for contentType Metadata* */
    public static final String ECTS = "ects";

    /** Constants for contentType Metadata* */
    public static final String DESCRIPTION = "description";

    /**
     * Constructor
     * 
     * @param node
     * @param parentPath
     * @param factory
     */
    public Container(Node node, String parentPath, ContainerFactory factory)
    {
        super(node, parentPath, factory);
    }
    
    @Override
    public void lock() throws AmetysRepositoryException
    {
        getParentProgram().lock();
    }
    
    @Override
    public void unlock() throws AmetysRepositoryException
    {
        getParentProgram().unlock();
    }
    
    @Override
    public boolean isLocked() throws AmetysRepositoryException
    {
        return getParentProgram().isLocked();
    }
    
    @Override
    public String getLockOwner() throws AmetysRepositoryException
    {
        return getParentProgram().getLockOwner();
    }
    
    /**
     * Get the parent {@link Program}
     * @return the parent {@link Program}
     * @throws AmetysRepositoryException
     */
    public Program getParentProgram()  throws AmetysRepositoryException
    {
        AmetysObject parent = this.getParent();
        
        while (!(parent instanceof Program))
        {
            parent = parent.getParent();
        }
        
        return (Program) parent;
    }
    
    // --------------------------------------------------------------------------------------//
    // Structure Management
    // --------------------------------------------------------------------------------------//
    
    @Override
    public void remove() throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        AmetysObjectIterable<RemovableAmetysObject> children = this.getChildren();
        
        for (RemovableAmetysObject child : children)
        {
            child.remove();
        }
        
        super.remove();
    }
    
    public <A extends AmetysObject> A getChild(String path) throws AmetysRepositoryException, UnknownAmetysObjectException
    {
        return (A) _getFactory().getChild(this, path);
    }

    @Override
    public <A extends AmetysObject> AmetysObjectIterable<A> getChildren() throws AmetysRepositoryException
    {
        return _getFactory().getChildren(this);
    }

    @Override
    public boolean hasChild(String name) throws AmetysRepositoryException
    {
        return _getFactory().hasChild(this, name);
    }

    @Override
    public <A extends AmetysObject> A createChild(String name, String type) throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        return (A) _getFactory().createChild(this, name, type);
    }
    
    @Override
    public CourseList createCourseList(String name)
    {
        String legalName = Text.escapeIllegalJcrChars(name);
        
        // Find unique name
        int index = 1;
        while (hasChild(legalName))
        {
            legalName = Text.escapeIllegalJcrChars(name + " " + (index++));
        }
        
        return this.createChild(legalName, CourseListFactory.COURSE_LIST_NODETYPE);
    }
    
    @Override
    public AmetysObjectIterable<CourseList> getCourseLists()
    {
        List<CourseList> list = new ArrayList<CourseList>();
        
        AmetysObjectIterable<AmetysObject> children = getChildren();
        for (AmetysObject child : children)
        {
            if (child instanceof CourseList)
            {
                list.add((CourseList) child);
            }
        }

        return new CollectionIterable<CourseList>(list);
    }
    
    
    @Override
    public boolean canMoveTo(AmetysObject newParent) throws AmetysRepositoryException
    {
        return newParent instanceof Container || newParent instanceof AbstractProgram;
    }
    
    @Override
    public void moveTo(AmetysObject newParent, boolean renameIfExist) throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        if (!canMoveTo(newParent))
        {
            throw new AmetysRepositoryException("Can not move a Container " + getId() + " to something that is not a Container or Program " + newParent.getId());
        }
        

        Node node = getNode();

        try
        {
            if (getParent().equals(newParent))
            {
                // Just order current node to the end
                _checkLock(node.getParent());
                node.getParent().orderBefore(node.getName(), null);
            }
            else
            {
                DefaultAmetysObject newParentAO = (DefaultAmetysObject) newParent;
                
                // Move node
                _checkLock(newParentAO.getNode());
                node.getSession().move(node.getPath(), newParentAO.getNode().getPath() + "/" + node.getName());
            }
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException("Can not move Container " + getId() + " to content " + newParent.getId(), e);
        }
    }
    
    @Override
    public void orderBefore(AmetysObject siblingNode) throws AmetysRepositoryException
    {
        Node node = getNode();
        try
        {
            _checkLock(node.getParent());
            node.getParent().orderBefore(node.getName(), siblingNode != null ? siblingNode.getName() : null);
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException(String.format("Unable to order Container '%s' before sibling '%s'", this, siblingNode != null ? siblingNode.getName() : ""), e);
        }
    }
    
    private static void _checkLock(Node node) throws RepositoryException
    {
        if (node.isLocked())
        {
            LockManager lockManager = node.getSession().getWorkspace().getLockManager();
            Lock lock = lockManager.getLock(node.getPath());
            Node lockHolder = lock.getNode();
            
            lockManager.addLockToken(lockHolder.getProperty(RepositoryConstants.METADATA_LOCKTOKEN).getString());
        }
    }
    
    // --------------------------------------------------------------------------------------//
    // GETTERS AND SETTERS
    // --------------------------------------------------------------------------------------//
   
    /**
     * Get the code 
     * @return the code or null if not set
     */
    public String getCode()
    {
        return getString(CODE);
    }

    /**
     * Set the code 
     * @param code the code to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setCode(String code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(CODE, code);
    }

    /**
     * Get the CDM-fr of the {@link Container}
     * @return the CDM-fr code
     */
    public String getCdmCode()
    {
        return getString(CDM_CODE);
    }
    /**
     * Set the CDM-fr code
     * @param code the CDM-fr to set
     * @throws AmetysRepositoryException
     */
    public void setCdmCode(String code) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(Container.CDM_CODE, code);
    }
    
    /**
     * Get the ECTS credits
     * @return the ECTS credits 
     * @throws UnknownMetadataException
     * @throws AmetysRepositoryException if an error occurred
     */
    public String getEcts() throws AmetysRepositoryException, UnknownMetadataException
    {
        return getString(ECTS);
    }

    /**
     * Set the ECTS credits
     * @param ects the ECTS credits to set
     * @throws AmetysRepositoryException if an error occurred
     */
    public void setEcts(String ects) throws AmetysRepositoryException
    {
        getMetadataHolder().setMetadata(ECTS, ects);
    }

    /**
     * Get the description
     * @return the description or null if not set
     */
    public RichText getDescription()
    {
        return getRichText(DESCRIPTION);
    }

    @Override
    public String getCDMId()
    {
        return getCdmCode();
    }

    @Override
    @Deprecated
    public void toCDM(ContentHandler contentHandler, OdfEnumerationHelper provider, Set<String> persons, Set<String> orgUnits, Set<String> courses) throws SAXException
    {
        _getFactory()._getExportCDMfrManager().container2CDM(contentHandler, this, persons, orgUnits, courses);
    }
}
