/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program;

import java.util.Collections;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;

import org.ametys.cms.repository.WorkflowAwareContentHelper;
import org.ametys.odf.SynchronizableTraversableContentFactory;
import org.ametys.odf.catalog.CatalogsManager;
import org.ametys.odf.cdmfr.ExportCDMfrManager;
import org.ametys.odf.course.CourseFactory;
import org.ametys.odf.orgunit.RootOrgUnitProvider;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.workflow.Workflow;
import org.ametys.plugins.workflow.store.JackrabbitWorkflowStore;

import com.opensymphony.workflow.WorkflowException;
import com.opensymphony.workflow.spi.Step;

/**
 * {@link ProgramFactory} for handling {@link Program}
 */
public class ProgramFactory extends SynchronizableTraversableContentFactory
{
    /** {@link Program} nodetype for resources collection */
    public static final String PROGRAM_NODETYPE = RepositoryConstants.NAMESPACE_PREFIX + ":programContent";

    /** {@link Program} content type */
    public static final String PROGRAM_CONTENT_TYPE = "org.ametys.plugins.odf.Content.program";

    private RootOrgUnitProvider _rootOrgUnitProvider;
    private CatalogsManager _catalogsManager;
    private ExportCDMfrManager _exportCDMfrManager;
    private Workflow _workflow;
    
    @Override
    public void service(ServiceManager manager) throws ServiceException
    {
        super.service(manager);
        _rootOrgUnitProvider = (RootOrgUnitProvider) manager.lookup(RootOrgUnitProvider.ROLE);
        _exportCDMfrManager = (ExportCDMfrManager) manager.lookup(ExportCDMfrManager.ROLE);
        _catalogsManager = (CatalogsManager) manager.lookup(CatalogsManager.ROLE);
        _workflow = (Workflow) manager.lookup(Workflow.ROLE);
    }
    
    @Override
    public AbstractProgram getAmetysObject(Node node, String parentPath) throws AmetysRepositoryException
    {
        return new Program(node, parentPath, this);
    }
    
    String _getRootOrgUnitRNE()
    {
        return _rootOrgUnitProvider.getRoot().getUAICode();
    }
    
    ExportCDMfrManager _getExportCDMfrManager()
    {
        return _exportCDMfrManager;
    }
    
    CatalogsManager _getCatalogEnumerationHelper()
    {
        return _catalogsManager;
    }
    
    Workflow _getWorkflow()
    {
        return _workflow;
    }
    
    void recreateContentWorkflow(Node contentNode)
    {
        try
        {
            String workflowName = null;
            if (contentNode.isNodeType(ProgramFactory.PROGRAM_NODETYPE)
                || contentNode.isNodeType(SubProgramFactory.SUBPROGRAM_NODETYPE)
                || contentNode.isNodeType(ContainerFactory.CONTAINER_NODETYPE))
            {
                workflowName = "content";
            }
            else if (contentNode.isNodeType(CourseFactory.COURSE_NODETYPE))
            {
                workflowName = "course";
            }
            
            if (workflowName != null)
            {
                _createContentWorkflow(contentNode, workflowName);
            }
        }
        catch (WorkflowException e)
        {
            getLogger().error("Impossible to recreate a workflow instance for content " + contentNode, e);
        }
        catch (RepositoryException e)
        {
            getLogger().error("Impossible to recreate a workflow instance for content " + contentNode, e);
        }
    }
    
    private void _createContentWorkflow(Node contentNode, String workflowName) throws WorkflowException, RepositoryException
    {
        long workflowId = _workflow.initialize(workflowName, 0, Collections.emptyMap());
        
        Session session = contentNode.getSession();
        
        contentNode.setProperty(RepositoryConstants.NAMESPACE_PREFIX_INTERNAL + ':' + WorkflowAwareContentHelper.METADATA_WORKFLOW_ID, workflowId);
        
        JackrabbitWorkflowStore workflowStore = (JackrabbitWorkflowStore) _workflow.getConfiguration().getWorkflowStore();
        Node workflowEntryNode = workflowStore.getEntryNode(session, workflowId);
        
        // Link workflow entry to the content
        workflowEntryNode.setProperty("ametys-internal:contentRef", contentNode);
        contentNode.setProperty("ametys-internal:workflowRef", workflowEntryNode);
        
        // Update current step property
        Step currentStep = (Step) _workflow.getCurrentSteps(workflowId).iterator().next();
//        content.setCurrentStepId(currentStep.getStepId());
        contentNode.setProperty("ametys-internal:currentStepId", currentStep.getStepId());
    }
    
}
