/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.lock.Lock;
import javax.jcr.lock.LockManager;

import org.ametys.odf.cdmfr.CDMFRTagsConstants;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.MovableAmetysObject;
import org.ametys.plugins.repository.RepositoryConstants;
import org.ametys.plugins.repository.RepositoryIntegrityViolationException;
import org.ametys.plugins.repository.jcr.DefaultAmetysObject;

/**
 * SubProgram java object
 */
public class SubProgram extends AbstractProgram<SubProgramFactory> implements MovableAmetysObject
{
    /**
     * Constructor
     * @param node
     * @param parentPath
     * @param factory
     */
    public SubProgram(Node node, String parentPath, SubProgramFactory factory)
    {
        super(node, parentPath, factory);
    }
        
    @Override
    public void lock() throws AmetysRepositoryException
    {
        getParentProgram().lock();
    }
    
    @Override
    public void unlock() throws AmetysRepositoryException
    {
        getParentProgram().unlock();
    }
    
    @Override
    public boolean isLocked() throws AmetysRepositoryException
    {
        return getParentProgram().isLocked();
    }
    
    @Override
    public String getLockOwner() throws AmetysRepositoryException
    {
        return getParentProgram().getLockOwner();
    }
    
    /**
     * Get the parent {@link Program}
     * @return the parent {@link Program}
     * @throws AmetysRepositoryException
     */
    public Program getParentProgram()
    {
        AmetysObject parent = this.getParent();
        
        while (!(parent instanceof Program))
        {
            parent = parent.getParent();
        }
        
        return (Program) parent;
    }
    
    @Override
    public boolean canMoveTo(AmetysObject newParent) throws AmetysRepositoryException
    {
        return newParent instanceof ProgramPart;
    }
    
    @Override
    public void moveTo(AmetysObject newParent, boolean renameIfExist) throws AmetysRepositoryException, RepositoryIntegrityViolationException
    {
        if (!canMoveTo(newParent))
        {
            throw new AmetysRepositoryException("Can not move a SubProgram " + getId() + " to something that is not a Container or Program " + newParent.getId());
        }
        

        Node node = getNode();

        try
        {
            if (getParent().equals(newParent))
            {
                // Just order current node to the end
                _checkLock(node.getParent());
                node.getParent().orderBefore(node.getName(), null);
            }
            else
            {
                DefaultAmetysObject newParentAO = (DefaultAmetysObject) newParent;
                
                // Move node
                _checkLock(newParentAO.getNode());
                node.getSession().move(node.getPath(), newParentAO.getNode().getPath() + "/" + node.getName());
            }
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException("Can not move CourseList " + getId() + " to content " + newParent.getId(), e);
        }
    }
    
    @Override
    public void orderBefore(AmetysObject siblingNode) throws AmetysRepositoryException
    {
        Node node = getNode();
        try
        {
            _checkLock(node.getParent());
            node.getParent().orderBefore(node.getName(), siblingNode != null ? siblingNode.getName() : null);
        }
        catch (RepositoryException e)
        {
            throw new AmetysRepositoryException(String.format("Unable to order SubProgram '%s' before sibling '%s'", this, siblingNode != null ? siblingNode.getName() : ""), e);
        }
    }
    
    @Override
    public String getTagName()
    {
        return CDMFRTagsConstants.TAG_SUB_PROGRAM;
    }
    
    private static void _checkLock(Node node) throws RepositoryException
    {
        if (node.isLocked())
        {
            LockManager lockManager = node.getSession().getWorkspace().getLockManager();
            Lock lock = lockManager.getLock(node.getPath());
            Node lockHolder = lock.getNode();
            
            lockManager.addLockToken(lockHolder.getProperty(RepositoryConstants.METADATA_LOCKTOKEN).getString());
        }
    }
}
