/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program.actions;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.acting.ServiceableAction;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.commons.lang.StringUtils;

import org.ametys.cms.repository.WorkflowAwareContent;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.person.Person;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.workflow.Workflow;
import org.ametys.runtime.cocoon.JSonReader;

import com.opensymphony.workflow.spi.Step;

/** 
 * This action determines if a program and its sub-elements are in validated workflow step
 *
 */
public class IsProgramValidAction extends ServiceableAction
{
    private static final int __VALIDATED_STEP_ID = 3;
    
    /** The Ametys object resolver */
    protected AmetysObjectResolver _resolver;
    /** The Workflow */
    protected Workflow _workflow;
    
    /** The Set of invalid courses */
    protected Set<String> _invalidCourses;
    /** The Set of invalid orgunits */
    protected Set<String> _invalidOrgUnits;
    /** The Set of invalid persons */
    protected Set<String> _invalidPersons;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _workflow = (Workflow) smanager.lookup(Workflow.ROLE);
    }
    
    @Override
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        Map<String, Object> result = new HashMap<String, Object>();
        
        Map<String, Object> jsParameters = (Map<String, Object>) objectModel.get(ObjectModelHelper.PARENT_CONTEXT);
        List<String> contentsId = (List<String>) jsParameters.get("contentIds");
        
        Set<String> invalidPrograms = new HashSet<String>();
        Set<String> validPrograms = new HashSet<String>();
        _invalidCourses = new HashSet<String>();
        _invalidOrgUnits = new HashSet<String>();
        _invalidPersons = new HashSet<String>();
        
        for (String id : contentsId)
        {
            Program program = _resolver.resolveById(id);
            if (!isProgramValid(program))
            {
                invalidPrograms.add(program.getTitle());
            }
            else
            {
                validPrograms.add(program.getId());
            }
        }
        
        result.put("validPrograms", validPrograms);
        
        if (invalidPrograms.size() > 0)
        {
            result.put("invalidPrograms", invalidPrograms);
        }
        if (_invalidCourses.size() > 0)
        {
            result.put("invalidCourses", _invalidCourses);
        }
        if (_invalidPersons.size() > 0)
        {
            result.put("invalidPersons", _invalidPersons);
        }
        if (_invalidOrgUnits.size() > 0)
        {
            result.put("invalidOrgUnits", _invalidOrgUnits);
        }
        
        Request request = ObjectModelHelper.getRequest(objectModel);
        request.setAttribute(JSonReader.MAP_TO_READ, result);
        
        return EMPTY_MAP;
    }
    
    /**
     * Determines if a Program is valid in its entirety 
     * @param program the program
     * @return <code>true</code> if the program is valid
     */
    protected boolean isProgramValid (AbstractProgram<? extends ProgramFactory> program)
    {
        boolean isValid = true;
        
        // Check OrgUnits
        for (String id : program.getOrgUnits())
        {
            isValid = isOrgUnitValid(id) && isValid;
        }
        
        // Check contacts
        for (String id : program.getContacts())
        {
            isValid = isPersonValid(id) && isValid;
        }
        
        // Check persons in charge
        for (String id : program.getPersonsInCharge())
        {
            isValid = isPersonValid(id) && isValid;
        }
        
        // Check children
        AmetysObjectIterable<AmetysObject> children = program.getChildren();
        for (AmetysObject ao : children)
        {
            if (ao instanceof SubProgram)
            {
                isValid = isProgramValid((SubProgram) ao) && isValid;
            }
            else if (ao instanceof Container)
            {
                isValid = isContainerValid((Container) ao) && isValid;
            }
            else if (ao instanceof CourseList)
            {
                for (String courseId : ((CourseList) ao).getCourses())
                {
                    Course childCourse = _resolver.resolveById(courseId);
                    isValid = isCourseValid(childCourse) && isValid;
                }
            }
        }
        
        return isValid;
    }
    
    
    
    /**
     * Determines if a container is valid in its entirety
     * @param container the container
     * @return <code>true</code> if the container is valid
     */
    protected boolean isContainerValid (Container container)
    {
        boolean isValid = true;
        
        AmetysObjectIterable<AmetysObject> children = container.getChildren();
        
        for (AmetysObject ao : children)
        {
            if (ao instanceof SubProgram)
            {
                isValid = isProgramValid((SubProgram) ao) && isValid;
            }
            else if (ao instanceof Container)
            {
                isValid = isContainerValid((Container) ao) && isValid;
            }
            else if (ao instanceof CourseList)
            {
                for (String courseId : ((CourseList) ao).getCourses())
                {
                    Course childCourse = _resolver.resolveById(courseId);
                    isValid = isCourseValid(childCourse) && isValid;
                }
            }
        }
        
        
        return isValid;
    }
    
    /**
     * Determines if a course is valid in its entirety
     * @param course the course
     * @return <code>true</code> if the course is valid
     */
    protected boolean isCourseValid (Course course)
    {
        boolean isValid = true;
        
        if (!isContentInValidedStep(course))
        {
            isValid = false;
            _invalidCourses.add(course.getTitle());
        }
        
        // Check OrgUnits
        for (String id : course.getOrgUnits())
        {
            isValid = isOrgUnitValid(id) && isValid;
        }
        
        // Check contacts
        for (String id : course.getContacts())
        {
            isValid = isPersonValid(id) && isValid;
        }
        
        // Check persons in charge
        for (String id : course.getPersonsInCharge())
        {
            isValid = isPersonValid(id) && isValid;
        }
        
        // Check children courses recursively
        AmetysObjectIterable<AmetysObject> children = course.getChildren();
        for (AmetysObject ao : children)
        {
            if (ao instanceof CourseList)
            {
                for (String courseId : ((CourseList) ao).getCourses())
                {
                    Course childCourse = _resolver.resolveById(courseId);
                    isValid = isCourseValid(childCourse) && isValid;
                }
            }
        }
        
        return isValid;
    }
    
    /**
     * Determines if a OrgUnit is valid in its entirety
     * @param ouId the org unit id
     * @return <code>true</code> if the org unit is valid
     */
    protected boolean isOrgUnitValid (String ouId)
    {
        boolean isValid = true;
        
        if (StringUtils.isNotEmpty(ouId))
        {
            OrgUnit orgUnit = _resolver.resolveById(ouId);
            
            if (!isContentInValidedStep(orgUnit))
            {
                isValid = false;
                _invalidOrgUnits.add(orgUnit.getTitle());
            }
            
            // Check contacts
            for (String id : orgUnit.getContacts())
            {
                isValid = isPersonValid(id) && isValid;
            }
        }
        
        return isValid;
    }
    
    /**
     * Determines if a Person is valid in its entirety
     * @param personId the person id
     * @return <code>true</code> if the person is valid
     */
    protected boolean isPersonValid (String personId)
    {
        if (StringUtils.isNotEmpty(personId))
        {
            Person person = _resolver.resolveById(personId);
            
            if (!isContentInValidedStep(person))
            {
                _invalidPersons.add(person.getTitle());
                return false;
            }
        }
        
        return true;
    }
    
    /**
     * Determines if a content is in the validated workflow step
     * @param content the content to check
     * @return <code>true</code> if the content is in validated step
     */
    protected boolean isContentInValidedStep (WorkflowAwareContent content)
    {
        Step currentStep = (Step) _workflow.getCurrentSteps(content.getWorkflowId()).iterator().next();
        return currentStep.getStepId() == __VALIDATED_STEP_ID;
    }
    
}
