/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.commons.collections.map.HashedMap;
import org.apache.commons.lang.StringUtils;

import org.ametys.cms.repository.Content;
import org.ametys.odf.course.Course;
import org.ametys.odf.course.actions.AbstractOdfAction;
import org.ametys.odf.courselist.CourseList;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.MovableAmetysObject;
import org.ametys.plugins.repository.lock.LockHelper;
import org.ametys.plugins.repository.lock.LockableAmetysObject;
import org.ametys.runtime.user.UsersManager;
import org.ametys.runtime.util.I18nizableText;
import org.ametys.runtime.util.cocoon.ActionResultGenerator;

/**
 * Move 
 */
public class MoveAction extends AbstractOdfAction
{
    private UsersManager _usersManager;

    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _usersManager = (UsersManager) smanager.lookup(UsersManager.ROLE);
    }
    
    @Override
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        Request request = ObjectModelHelper.getRequest(objectModel);

        String id = request.getParameter("id");
        String oldParentId = request.getParameter("oldParent");
        String newParentId = request.getParameter("newParent");
        String sibling = request.getParameter("sibling");
        
        AmetysObject ao = _resolver.resolveById(id);
        AmetysObject oldParent = _resolver.resolveById(oldParentId);
        AmetysObject newParent = _resolver.resolveById(newParentId);
        
        if (ao instanceof Course)
        {
            if (oldParentId.equals(newParentId) && sibling != null)
            {
                CourseList parentCl = (CourseList) newParent;
                
                if (_isLocked(parentCl, request))
                {
                    return EMPTY_MAP;
                }
                
                parentCl.orderCourseBefore(id, sibling);
                
                parentCl.saveChanges();
            }
            else
            {
                CourseList oldParentCl = (CourseList) oldParent;
                if (_isLocked(oldParentCl, request))
                {
                    return EMPTY_MAP;
                }
                
                oldParentCl.removeCourse(ao.getId());
                
                CourseList newParentCl = (CourseList) newParent;
                if (_isLocked(newParentCl, request))
                {
                    return EMPTY_MAP;
                }
                
                newParentCl.addCourse(ao.getId());
                
                oldParentCl.saveChanges();
                newParentCl.saveChanges();
            }
        }
        else if (ao instanceof MovableAmetysObject)
        {
            if (_isLocked(ao, request) || _isLocked(ao.getParent(), request))
            {
                return EMPTY_MAP;
            }
            
            if (!oldParentId.equals(newParentId))
            {
                ((MovableAmetysObject) ao).moveTo(newParent, true);
            }
            
            if (StringUtils.isNotEmpty(sibling))
            {
                AmetysObject brother = _resolver.resolveById(sibling);
                
                ((MovableAmetysObject) ao).orderBefore(brother);
            }
            else
            {
                ((MovableAmetysObject) ao).orderBefore(null);
            }
        }
        
        applyChangesToParent(newParent);
        if (!oldParentId.equals(newParentId))
        {
            applyChangesToParent(oldParent);
        }
        
        return EMPTY_MAP;
    }
    
    private boolean _isLocked (AmetysObject ao, Request request)
    {
        LockableAmetysObject lockableContent = null;
        if (ao instanceof LockableAmetysObject)
        {
            lockableContent = (LockableAmetysObject) ao;
            if (lockableContent.isLocked() && !LockHelper.isLockOwner(lockableContent, _getCurrentUser()))
            {
                getLogger().error("User '" + _getCurrentUser() + "' try to move content '" + ao.getName() + "' but it is locked by another user");
                
                String lockOwner = lockableContent.getLockOwner();
                
                List<String> i18nParams = new ArrayList<String>();
                i18nParams.add(ao instanceof Content ? ((Content) ao).getTitle() : ao.getName());
                i18nParams.add(_usersManager.getUser(lockOwner).getFullName());
                i18nParams.add(lockOwner);
                
                Map<String, Object> result = new HashMap<String, Object>();
                result.put("message", new I18nizableText("plugin.odf", "PLUGINS_ODF_PROGRAM_TREE_LOCKED_ERROR", i18nParams));
                result.put("locked", "true");
                
                request.setAttribute(ActionResultGenerator.MAP_REQUEST_ATTR, result);
                return true;
            }
        }
        
        return false;
    }
}
