/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program.actions;

import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.SourceResolver;

import org.ametys.cms.contenttype.ContentType;
import org.ametys.cms.contenttype.ContentTypeExtensionPoint;
import org.ametys.cms.observation.AbstractNotifierAction;
import org.ametys.cms.observation.Event;
import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.cms.repository.ModifiableWorkflowAwareContent;
import org.ametys.cms.repository.WorkflowAwareContent;
import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.cms.workflow.CreateContentFunction;
import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseListContainer;
import org.ametys.odf.observation.OdfObservationConstants;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.odf.translation.ContentCopyHelper;
import org.ametys.odf.translation.TranslationHelper;
import org.ametys.odf.workflow.CreateODFContentFunction;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.TraversableAmetysObject;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.metadata.CompositeMetadata;
import org.ametys.plugins.repository.metadata.ModifiableCompositeMetadata;
import org.ametys.plugins.workflow.Workflow;

import com.opensymphony.workflow.WorkflowException;

/**
 * Action which creates a translated version of a program.
 */
public class TranslateProgramAction extends AbstractNotifierAction
{
    /** The ametys object resolver. */
    protected AmetysObjectResolver _ametysResolver;
    /** The workflow */
    protected Workflow _workflow;
    /** The content type extension point. */
    protected ContentTypeExtensionPoint _cTypeEP;
    
    @Override
    public void service(ServiceManager serviceManager) throws ServiceException
    {
        super.service(serviceManager);
        _ametysResolver = (AmetysObjectResolver) serviceManager.lookup(AmetysObjectResolver.ROLE);
        _workflow = (Workflow) serviceManager.lookup(Workflow.ROLE);
        _cTypeEP = (ContentTypeExtensionPoint) serviceManager.lookup(ContentTypeExtensionPoint.ROLE);
    }
    
    @Override
    public Map act(Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters) throws Exception
    {
        String contentId = parameters.getParameter("contentId");
        String lang = parameters.getParameter("language");
        boolean fullCopy = parameters.getParameterAsBoolean("full", false);
        
        Map<String, String> results = new HashMap<String, String>();
        
        Program program = _ametysResolver.resolveById(contentId);
        
        if (program.getLanguage().equals(lang))
        {
            return Collections.singletonMap("error", "same-language");
        }
        
        // Get existing translations
        Map<String, String> translations = TranslationHelper.getTranslations(program);
        
        boolean alreadyExists = false;
        String translatedProgramId = null;
        
        if (!translations.containsKey(lang))
        {
            ModifiableWorkflowAwareContent translatedProgram = translateContent(program, lang, 1, null, fullCopy);
            translatedProgramId = translatedProgram.getId();
        }
        else
        {
            translatedProgramId = translations.get(lang);
            // Check if content exists
            try
            {
                _resolver.resolveById(translatedProgramId);
                alreadyExists = true;
            }
            catch (UnknownAmetysObjectException e) 
            {
                ModifiableWorkflowAwareContent translatedProgram = translateContent(program, lang, 1, null, fullCopy);
                translatedProgramId = translatedProgram.getId();
            }
            
        }
        
        results.put("already-exists", Boolean.toString(alreadyExists));
        results.put("translated-program-id", translatedProgramId);
        
        return results;
    }
    
    
    /**
     * Translate a program: create a second program object, and copy all the metadata to it.
     * @param content the content to translate.
     * @param lang the language to translate the program into.
     * @param actionId The action id for creation
     * @param parentId The parent id. Can be null.
     * @param fullCopy true to translate all the program structure.
     * @return the translated Program object.
     * @throws WorkflowException if an error occurs creating the Program translation.
     */
    protected ModifiableWorkflowAwareContent translateContent(SynchronizableContent content, String lang, int actionId, String parentId, boolean fullCopy) throws WorkflowException
    {
        ModifiableWorkflowAwareContent translatedContent = createTranslatedContent(content, lang, actionId, parentId);
        
        copyContentMetadata(content, translatedContent);
        
        if (fullCopy)
        {
            copyStructure((TraversableAmetysObject) content, translatedContent, lang);
        }
        
        translatedContent.setCreationDate(new Date());
        translatedContent.setLastModified(new Date());
        translatedContent.setCreator(_getCurrentUser());
        
        linkTranslations(content, translatedContent);
        
        content.saveChanges();
        translatedContent.saveChanges();
        
        _observationManager.notify(new Event(_getCurrentUser(), OdfObservationConstants.ODF_CONTENT_TRANSLATED, content));
        
        return translatedContent;
    }
    
    /**
     * Translate a course list: create a second CourseList object, and copy all the metadata to it.
     * @param courseList the course list to translate.
     * @param container the parent CourseListContainer.
     * @param lang the language to translate the course list to.
     * @return the translated CourseList.
     * @throws WorkflowException if an error occurs creating the CourseList translation.
     */
    protected CourseList translateCourseList(CourseList courseList, CourseListContainer container, String lang) throws WorkflowException
    {
        CourseList translatedCL = container.createCourseList(courseList.getName());
        
        copyCourseListMetadata(courseList, translatedCL);
        
        copyCourseListStructure(courseList, translatedCL, lang);
        
        courseList.saveChanges();
        translatedCL.saveChanges();
        
        return translatedCL;
    }
    
    /**
     * Create the translated version of a {@link Content}.
     * @param content the content to translate.
     * @param lang the language of translation
     * @param actionId Id of workflow action
     * @param parentId The parent id. Can be null
     * @return the translated content.
     * @throws WorkflowException if an error occurs creating the translated content.
     */
    protected ModifiableWorkflowAwareContent createTranslatedContent(WorkflowAwareContent content, String lang, int actionId, String parentId) throws WorkflowException
    {
        String contentName = content.getName() + "-" + lang;
        String contentTitle = content.getTitle();
        String cType = content.getType();
        
        long programWorkflowId = content.getWorkflowId();
        String workflowName = _workflow.getWorkflowName(programWorkflowId);
        
        // Create content.
        Map<String, Object> workflowResult = new HashMap<String, Object>();
        
        Map<String, Object> inputs = new HashMap<String, Object>();
        inputs.put(AbstractContentWorkflowComponent.RESULT_MAP_KEY, workflowResult);
        inputs.put(CreateContentFunction.CONTENT_NAME_KEY, contentName);
        inputs.put(CreateContentFunction.CONTENT_TITLE_KEY, contentTitle);
        inputs.put(CreateContentFunction.CONTENT_TYPE_KEY, cType);
        inputs.put(CreateContentFunction.CONTENT_LANGUAGE_KEY, lang);
        
        if (parentId != null)
        {
            inputs.put(CreateODFContentFunction.CONTENT_PARENT_ID, parentId);
        }
        
        _workflow.initialize(workflowName, actionId, inputs);
        
        String translatedContentId = (String) workflowResult.get("contentId");
        
        ModifiableWorkflowAwareContent translatedContent = _ametysResolver.resolveById(translatedContentId);
        
        return translatedContent;
    }
    
    /**
     * Copy the program structure.
     * @param object the object of which to copy the structure.
     * @param translatedContent the Content under which the objects are to be created.
     * @param language the desired language.
     * @throws WorkflowException 
     */
    protected void copyStructure(TraversableAmetysObject object, ModifiableContent translatedContent, String language) throws WorkflowException
    {
        for (AmetysObject child : object.getChildren())
        {
            if (child instanceof SubProgram)
            {
                translateContent((SubProgram) child, language, 11, translatedContent.getId(), true);
            }
            else if (child instanceof Container)
            {
                translateContent((Container) child, language, 12, translatedContent.getId(), true);
            }
            else if (child instanceof CourseList && translatedContent instanceof CourseListContainer)
            {
                translateCourseList((CourseList) child, (CourseListContainer) translatedContent, language);
            }
        }
    }
    
    /**
     * Copy a course list structure.
     * @param originalCourseList the course list to copy.
     * @param translatedCourseList the destination course list.
     * @param lang the language to translate into.
     * @throws WorkflowException
     */
    protected void copyCourseListStructure(CourseList originalCourseList, CourseList translatedCourseList, String lang) throws WorkflowException
    {
        for (String courseId : originalCourseList.getCourses())
        {
            Course originalCourse = _ametysResolver.resolveById(courseId);
            
            Course translatedCourse = null;
            Map<String, String> translations = TranslationHelper.getTranslations(originalCourse);
            if (translations.containsKey(lang))
            {
                translatedCourse = _ametysResolver.resolveById(translations.get(lang));
            }
            else
            {
                translatedCourse = (Course) translateContent(originalCourse, lang, 1, null, true);
            }
            translatedCourseList.addCourse(translatedCourse.getId());
        }
        
        translatedCourseList.saveChanges();
    }
    
    /**
     * Store links to the other translations in all the translated objects.
     * @param originalContent
     * @param translatedContent
     */
    protected void linkTranslations(ModifiableContent originalContent, ModifiableContent translatedContent)
    {
        Map<String, String> translations = TranslationHelper.getTranslations(originalContent);
        
        // Add the original and translated references.
        translations.put(originalContent.getLanguage(), originalContent.getId());
        translations.put(translatedContent.getLanguage(), translatedContent.getId());
        
        for (String contentId : translations.values())
        {
            ModifiableContent content = _ametysResolver.resolveById(contentId);
            
            Map<String, String> otherTranslations = new HashMap<String, String>(translations);
            otherTranslations.remove(content.getLanguage());
            
            TranslationHelper.setTranslations(content, otherTranslations);
        }
    }
    
    /**
     * Copy all the metadata of the source content to the new one.
     * @param originalContent the source Content.
     * @param translatedContent the destination Content.
     * @throws AmetysRepositoryException if an error occurs copying the metadata.
     */
    protected void copyContentMetadata(Content originalContent, ModifiableContent translatedContent) throws AmetysRepositoryException
    {
        ContentType contentType = _cTypeEP.getExtension(originalContent.getType());
        if (contentType instanceof ODFContentType)
        {
            ContentCopyHelper.copyMetadatas(originalContent.getMetadataHolder(), translatedContent.getMetadataHolder(), contentType);
        }
    }
    
    /**
     * Copy data from a CourseList to another one.
     * @param originalCourseList the source CourseList.
     * @param translatedCourseList the destination CourseList.
     */
    protected void copyCourseListMetadata(CourseList originalCourseList, CourseList translatedCourseList)
    {
        CompositeMetadata originalMeta = originalCourseList.getMetadataHolder();
        ModifiableCompositeMetadata translatedMeta = translatedCourseList.getMetadataHolder();
        
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.COURSE_LIST_TITLE);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.COURSE_LIST_TITLE + SynchronizableContent.REMOTE_SUFFIX);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.COURSE_LIST_TITLE + SynchronizableContent.SYNC_SUFFIX);
        
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.CODE_LIST);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.CODE_LIST + SynchronizableContent.REMOTE_SUFFIX);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.CODE_LIST + SynchronizableContent.SYNC_SUFFIX);
        
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.ECTS);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.ECTS + SynchronizableContent.REMOTE_SUFFIX);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.ECTS + SynchronizableContent.SYNC_SUFFIX);
        
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.CHOICE_TYPE);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.CHOICE_TYPE + SynchronizableContent.REMOTE_SUFFIX);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.CHOICE_TYPE + SynchronizableContent.SYNC_SUFFIX);
        
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.MIN_COURSES);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.MIN_COURSES + SynchronizableContent.REMOTE_SUFFIX);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.MIN_COURSES + SynchronizableContent.SYNC_SUFFIX);
        
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.MAX_COURSES);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.MAX_COURSES + SynchronizableContent.REMOTE_SUFFIX);
        ContentCopyHelper.copySimpleMetadata(originalMeta, translatedMeta, CourseList.MAX_COURSES + SynchronizableContent.SYNC_SUFFIX);
    }
    
}
