/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program.generators;

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.io.IOUtils;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import org.ametys.cms.contenttype.MetadataDefinition;
import org.ametys.cms.contenttype.MetadataManager;
import org.ametys.cms.repository.Content;
import org.ametys.cms.transformation.RichTextTransformer;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.metadata.CompositeMetadata;
import org.ametys.plugins.repository.metadata.ModifiableRichText;
import org.ametys.plugins.repository.metadata.RichText;
import org.ametys.runtime.user.User;
import org.ametys.runtime.util.IgnoreRootHandler;
import org.ametys.runtime.util.parameter.Enumerator;
import org.ametys.runtime.util.parameter.ParameterHelper;

/**
 * This manager handle synchronized metadata for ODF contents
 */
public class ODFMetadataManager extends MetadataManager implements Contextualizable
{
    /** Avalon Role. */
    @SuppressWarnings("hiding")
    public static final String ROLE = ODFMetadataManager.class.getName();
    
    /** Request attribute for edition mode */
    public static final String EDITION_MODE_REQUEST_ATTR = ODFMetadataManager.class.getName() + "$isEditionMode";

    private Context _context;
    
    @Override
    public void contextualize(Context context) throws ContextException
    {
        _context = context;
    }
    
    @Override
    protected boolean hasMetadata(CompositeMetadata metadata, String metadataName)
    {
        if (!_isEditionMode() && metadata.getBoolean(metadataName + ODFContentType.SYNC_SUFFIX, false))
        {
            return metadata.hasMetadata(metadataName + ODFContentType.REMOTE_SUFFIX);
        }
        return super.hasMetadata(metadata, metadataName);
    }
    
    @Override
    protected void _saxSingleStringMetadata(ContentHandler contentHandler, Content content, CompositeMetadata metadatas, String metadataName, String prefix, boolean checkRead, Enumerator enumerator) throws AmetysRepositoryException, SAXException
    {
        String metadataForValues = _getMetadataName(metadatas, metadataName);
        
        if (!checkRead || _canRead(content, prefix + metadataName))
        {
            String[] values = metadatas.getStringArray(metadataForValues);
            
            if (values != null)
            {
                if (enumerator != null)
                {
                    _saxEnumeratedStringMetadata (contentHandler, content, metadataName, enumerator, values);
                }
                else
                {
                    for (String value : values)
                    {
                        XMLUtils.createElement(contentHandler, metadataName, value);
                    }
                }
            }      
        }
    }
    
    @Override
    protected void _saxRichTextMetadata(ContentHandler contentHandler, Content content, CompositeMetadata metadata, MetadataDefinition metadataDef, String metadataName, String prefix, boolean checkRead, boolean editionRendering) throws AmetysRepositoryException, SAXException, IOException
    {
        if (!checkRead || _canRead(content, prefix + metadataName))
        {
            String metadataNameForValue = _getMetadataName(metadata, metadataName);
            
            RichText richText = metadata.getRichText(metadataNameForValue);
            AttributesImpl attrs = new AttributesImpl();
            String mimeType = richText.getMimeType();
            
            attrs.addCDATAAttribute("mime-type", richText.getMimeType());
            attrs.addCDATAAttribute("lastModified", ParameterHelper.valueToString(richText.getLastModified()));
            
            XMLUtils.startElement(contentHandler, metadataName, attrs);
            
            if (editionRendering)
            {
                if (!(richText instanceof ModifiableRichText))
                {
                    throw new SAXException("");
                }   
                
                RichTextTransformer richTextTransformer = metadataDef.getRichTextTransformer();
                StringBuilder result = new StringBuilder(2048);
                
                try
                {
                    richTextTransformer.transformForEditing((ModifiableRichText) richText, result);
                }
                catch (IOException e)
                {
                    throw new AmetysRepositoryException("Unable to transform a rich text into a string", e);
                }
                
                XMLUtils.data(contentHandler, result.toString());
            }
            else
            {
                if (mimeType.equals("text/xml"))
                {
                    RichTextTransformer richTextTransformer = metadataDef.getRichTextTransformer();
                    richTextTransformer.transformForRendering(richText, new IgnoreRootHandler(contentHandler));
                }
                else if (mimeType.equals("text/plain"))
                {
                    String encoding = richText.getEncoding();
                    InputStream is = richText.getInputStream();
                    
                    try
                    {
                        XMLUtils.data(contentHandler, IOUtils.toString(is, encoding));
                    }
                    finally
                    {
                        IOUtils.closeQuietly(is);
                    }
                }
                else
                {
                    throw new AmetysRepositoryException("Mime-type " + mimeType + " is not supported for rich text rendering");
                }
            }
            
            XMLUtils.endElement(contentHandler, metadataName);
        }
    }
    
    @Override
    protected void _saxSingleDateMetadata(ContentHandler contentHandler, Content content, CompositeMetadata metadatas, String metadataName, String prefix, boolean checkRead) throws AmetysRepositoryException, SAXException
    {
        if (!checkRead || _canRead(content, prefix + metadataName))
        {
            String metadataNameForValues = _getMetadataName(metadatas, metadataName);
            Date[] values = metadatas.getDateArray(metadataNameForValues);
            
            if (values != null)
            {
                for (Date value : values)
                {
                    XMLUtils.createElement(contentHandler, metadataName, ParameterHelper.valueToString(value));
                }
            }
        }
    }
    
    @Override
    protected void _saxUserMetadata (ContentHandler contentHandler, Content content, CompositeMetadata metadata, String metadataName, String prefix, boolean checkRead) throws AmetysRepositoryException, SAXException
    {
        if (!checkRead || _canRead(content, prefix + metadataName))
        {
            String metadataNameForValues = _getMetadataName(metadata, metadataName);
            String[] logins = metadata.getStringArray(metadataNameForValues);
            if (logins != null)
            {
                for (String login : logins)
                {
                    User user = _usersManager.getUser(login);
                    if (user != null)
                    {
                        AttributesImpl attrs = new AttributesImpl();
                        attrs.addAttribute("", "login", "login", "CDATA", login);
                        attrs.addAttribute("", "email", "email", "CDATA", user.getEmail());
                        XMLUtils.createElement(contentHandler, metadataName, attrs, user.getFullName());
                    }
                }
            }
        }
    }
    
    private boolean _isEditionMode ()
    {
        Request request = ContextHelper.getRequest(_context);
        
        if (request.getAttribute(EDITION_MODE_REQUEST_ATTR) != null)
        {
            return (Boolean) request.getAttribute(EDITION_MODE_REQUEST_ATTR);
        }
        return false;
    }
    
    private String _getMetadataName (CompositeMetadata metadataHolder, String metadataName)
    {
        if (!_isEditionMode() && metadataHolder.getBoolean(metadataName + ODFContentType.SYNC_SUFFIX, false))
        {
            return metadataName + ODFContentType.REMOTE_SUFFIX;
        }
        return metadataName;
    }
}
