/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program.generators;

import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.components.source.impl.SitemapSource;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.generation.Generator;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.excalibur.source.SourceResolver;
import org.xml.sax.SAXException;

import org.ametys.cms.contenttype.ContentType;
import org.ametys.cms.contenttype.MetadataDefinition;
import org.ametys.cms.contenttype.MetadataSet;
import org.ametys.cms.repository.Content;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseList.ChoiceType;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.person.Person;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.odf.translation.TranslationHelper;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.runtime.util.IgnoreRootHandler;
import org.ametys.runtime.util.parameter.ParameterHelper;

/**
 * {@link Generator} for rendering raw content data for a {@link Program}.
 */
public class ProgramContentGenerator extends ODFContentGenerator
{
    /** The source resolver */
    protected SourceResolver _srcResolver;
    
    @Override
    public void service(ServiceManager serviceManager) throws ServiceException
    {
        super.service(serviceManager);
        _srcResolver = (SourceResolver) serviceManager.lookup(SourceResolver.ROLE);
    }
    
    @Override
    protected void _saxOtherData(Content content) throws SAXException, ProcessingException
    {
        super._saxOtherData(content);
        
        boolean isEditionMetadataSet = parameters.getParameterAsBoolean("isEditionMetadataSet", false);
        if (!isEditionMetadataSet)
        {
            if (content instanceof AbstractProgram)
            {
                AbstractProgram program = (AbstractProgram) content;
                
                // Sax all the string metadatas of widget "odf-person".
                Set<String> personMetas = _getPersonMetadatas(content);
                for (String personMeta : personMetas)
                {
                    String[] personIds = program.getStringArray(personMeta);
                    for (String personId : personIds)
                    {
                        if (StringUtils.isNotEmpty(personId))
                        {
                            saxPersonContent(personMeta, personId);
                        }
                    }
                }
                
                // Children
                AmetysObjectIterable<AmetysObject> children = program.getChildren();
                for (AmetysObject ao : children)
                {
                    if (ao instanceof SubProgram)
                    {
                        saxSubProgram((SubProgram) ao, program.getContextPath());
                    }
                    else if (ao instanceof Container)
                    {
                        saxContainer ((Container) ao, program.getContextPath());
                    }
                    else if (ao instanceof CourseList)
                    {
                        saxCourseList((CourseList) ao, program.getContextPath());
                    }
                }
                
                List<String> orgUnits = program.getOrgUnits();
                saxOrgUnits(orgUnits);
                
                // Sax existing translations
                Map<String, String> translations = TranslationHelper.getTranslations(program);
                if (!translations.isEmpty())
                {
                    saxTranslations(translations);
                }
            }
        }
        
        Request request = ObjectModelHelper.getRequest(objectModel);
        request.setAttribute(Content.class.getName(), content);
    }
    
    /**
     * Get the metadatas of type "odf-person" for this content.
     * @param content the content.
     * @return a Set of the "odf-person" metadata names.
     */
    protected Set<String> _getPersonMetadatas(Content content)
    {
        Set<String> metadataNames = new HashSet<String>();
        
        ContentType contentType = _contentTypeExtensionPoint.getExtension(content.getType());
        
        for (String metaName : contentType.getMetadataNames())
        {
            MetadataDefinition def = contentType.getMetadataDefinition(metaName);
            if ("odf-person".equals(def.getWidget()) || "odf-person-synchro".equals(def.getWidget()))
            {
                metadataNames.add(metaName);
            }
        }
        
        return metadataNames;
    }

    /**
     * SAX a container
     * @param container the container to SAX
     * @param parentPath the parent path
     * @throws SAXException
     */
    protected void saxContainer (Container container, String parentPath) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("title", container.getTitle());
        attrs.addCDATAAttribute("code", container.getCode());
        
        XMLUtils.startElement(contentHandler, "container", attrs);
        
        // Children
        AmetysObjectIterable<AmetysObject> children = container.getChildren();
        for (AmetysObject ao : children)
        {
            if (ao instanceof SubProgram)
            {
                saxSubProgram((SubProgram) ao, parentPath);
            }
            else if (ao instanceof Container)
            {
                saxContainer ((Container) ao, parentPath);
            }
            else if (ao instanceof CourseList)
            {
                saxCourseList((CourseList) ao, parentPath);
            }
        }
        
        XMLUtils.endElement(contentHandler, "container");
    }
    
    /**
     * SAX a sub program
     * @param subProgram the sub program to SAX
     * @param parentPath the parent path
     * @throws SAXException
     */
    protected void saxSubProgram (SubProgram subProgram, String parentPath) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("title", subProgram.getTitle());
        attrs.addCDATAAttribute("code", subProgram.getCode());
        attrs.addCDATAAttribute("id", subProgram.getId());
        if (parentPath != null)
        {
            attrs.addCDATAAttribute("path", parentPath + "/" + subProgram.getName());
        }
        XMLUtils.startElement(contentHandler, "subprogram", attrs);
        XMLUtils.endElement(contentHandler, "subprogram");
    }
    
    /**
     * SAX a course list
     * @param courseList The course list to SAX
     * @param parentPath the parent path
     * @throws SAXException
     */
    protected void saxCourseList (CourseList courseList, String parentPath) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("title", courseList.getTitle());
        
        String code = courseList.getCode();
        if (code != null)
        {
            attrs.addCDATAAttribute("code", code);
        }
   
        String ects = courseList.getEcts();
        if (ects != null)
        {
            attrs.addCDATAAttribute("ects", ects);
        }
        
        ChoiceType type = courseList.getType();
        if (type != null)
        {
            attrs.addCDATAAttribute("type", courseList.getType().toString());
        }
        
        if (ChoiceType.CHOICE.equals(type))
        {
            attrs.addCDATAAttribute("min", String.valueOf(courseList.getMinNumberOfCourses()));
            attrs.addCDATAAttribute("max", String.valueOf(courseList.getMaxNumberOfCourses()));
        }
        
        XMLUtils.startElement(contentHandler, "courseList", attrs);
        
        List<String> courseIds = courseList.getCourses();
        for (String courseId : courseIds)
        {
            try
            {
                Course course = _resolver.resolveById(courseId);
                saxCourse(course, parentPath);
            }
            catch (UnknownAmetysObjectException e)
            {
                // TODO
            }
        }
        XMLUtils.endElement(contentHandler, "courseList");
    }
    
    /**
     * SAX a course
     * @param course the course to SAX
     * @param parentPath the parent path
     * @throws SAXException
     */
    protected void saxCourse (Course course, String parentPath) throws SAXException
    {
        AttributesImpl attrs = new AttributesImpl();
        attrs.addCDATAAttribute("title", course.getTitle());
        attrs.addCDATAAttribute("code", course.getElpCode());
        attrs.addCDATAAttribute("id", course.getId());
        attrs.addCDATAAttribute("name", course.getName());
        
        if (parentPath != null)
        {
            attrs.addCDATAAttribute("path", parentPath + "/" + course.getName() + "-" + course.getElpCode());
        }
        
        ContentType contentType = _contentTypeExtensionPoint.getExtension(course.getType());
        MetadataSet metadataSet = contentType.getMetadataSetForView("courseList");
        
        XMLUtils.startElement(contentHandler, "course", attrs);
        
        try
        {
            if (metadataSet != null)
            {
                XMLUtils.startElement(contentHandler, "metadata");
                _metadataManager.saxMetadata(contentHandler, course, metadataSet);
                XMLUtils.endElement(contentHandler, "metadata");
            }
        }
        catch (IOException e)
        {
            throw new SAXException(e);
        }
        
        // Course list
        AmetysObjectIterable<AmetysObject> children = course.getChildren();
        for (AmetysObject ao : children)
        {
            if (ao instanceof CourseList)
            {
                saxCourseList((CourseList) ao, parentPath);
            }
        }
        
        XMLUtils.endElement(contentHandler, "course");
    }
    
    /**
     * SAX OrgUnits
     * @param orgUnits The OrgUnit ids
     * @throws SAXException
     */
    protected void saxOrgUnits (List<String> orgUnits) throws SAXException
    {
        XMLUtils.startElement(contentHandler, "orgunits");
        for (String orgUnitId : orgUnits)
        {
            if (!orgUnitId.isEmpty())
            {
                try
                {
                    OrgUnit orgUnit = _resolver.resolveById(orgUnitId);
                    
                    XMLUtils.startElement(contentHandler, "orgunit");
                    saxContent(orgUnit, "link");
                    XMLUtils.endElement(contentHandler, "orgunit");
                }
                catch (IOException e)
                {
                    throw new SAXException(e);
                }
                catch (UnknownAmetysObjectException e)
                {
                    // TODO
                }
            }
        }
        
        XMLUtils.endElement(contentHandler, "orgunits");
    }
    
    /**
     * SAX a metadata of type "odf-person".
     * @param metaName the metadata name.
     * @param personId the contact id.
     * @throws SAXException
     */
    protected void saxPersonContent(String metaName, String personId) throws SAXException
    {
        try
        {
            Person person = _resolver.resolveById(personId);
            
            XMLUtils.startElement(contentHandler, metaName);
            saxContent(person, "abstract");
            XMLUtils.endElement(contentHandler, metaName);
        }
        catch (IOException e)
        {
            throw new SAXException(e);
        }
        catch (UnknownAmetysObjectException e)
        {
            // The program can reference a non-existing person: ignore the exception.
        }
    }
    
    
    /**
     * SAX the HTML content of a {@link Content}
     * @param content the content
     * @param metadataSetName the metadata set name
     * @throws SAXException
     * @throws IOException
     */
    protected void saxContent (Content content, String metadataSetName) throws SAXException, IOException
    {
        String format = parameters.getParameter("output-format", "html");
        if (StringUtils.isEmpty(format))
        {
            format = "html";
        }
        
        SitemapSource src = null;      
        try
        {
            String uri = "cocoon://_content." + format + "?contentId=" + content.getId() + "&metadataSetName=" + metadataSetName + "&output-format=" + format;
            
            src = (SitemapSource) _srcResolver.resolveURI(uri);
            
            AttributesImpl attrs = new AttributesImpl();
            attrs.addCDATAAttribute("id", content.getId());
            attrs.addCDATAAttribute("name", content.getName());
            attrs.addCDATAAttribute("title", content.getTitle());
            attrs.addCDATAAttribute("lastModifiedAt", ParameterHelper.valueToString(content.getLastModified()));
            
            XMLUtils.startElement(contentHandler, "content", attrs);
            
            src.toSAX(new IgnoreRootHandler(contentHandler));
            
            XMLUtils.endElement(contentHandler, "content");
        }
        catch (UnknownAmetysObjectException e)
        {
            // The content may be archived
        }
        finally
        {
            _srcResolver.release(src);
        }
        
        
    }
}
