/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program.generators;

import java.util.Date;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.environment.Request;

import org.ametys.cms.contenttype.MetadataManager;
import org.ametys.cms.contenttype.MetadataType;
import org.ametys.cms.repository.ContentTypeExpression;
import org.ametys.cms.repository.SearchGenerator;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.plugins.repository.query.SortCriteria;
import org.ametys.plugins.repository.query.expression.AndExpression;
import org.ametys.plugins.repository.query.expression.BooleanExpression;
import org.ametys.plugins.repository.query.expression.DoubleExpression;
import org.ametys.plugins.repository.query.expression.Expression;
import org.ametys.plugins.repository.query.expression.Expression.Operator;
import org.ametys.plugins.repository.query.expression.LongExpression;
import org.ametys.plugins.repository.query.expression.OrExpression;
import org.ametys.plugins.repository.query.expression.StringExpression;
import org.ametys.plugins.repository.query.expression.StringWildcardExpression;
import org.ametys.runtime.util.parameter.ParameterHelper;
import org.ametys.runtime.util.parameter.ParameterHelper.ParameterType;

/**
 * Search the {@link Program}s matching criteria passed in request parameters
 * 
 */
public class ProgramSearchGenerator extends SearchGenerator
{
    @Override
    public void service(ServiceManager serviceManager) throws ServiceException
    {
        super.service(serviceManager);
        
        // Use specific metadata manager for synchronizable metadata
        _metadataManager = (MetadataManager) serviceManager.lookup(ODFMetadataManager.ROLE);
    }
    
    @Override
    protected String getXPathQuery(Request request)
    {
        Expression expr = createExpression(request);
        SortCriteria sortCriteria = getSortCriteria(request);

        return org.ametys.plugins.repository.query.QueryHelper.getXPathQuery(null, ProgramFactory.PROGRAM_NODETYPE, expr, sortCriteria);
    }
    
    @Override
    protected Expression createContentTypeExpressions(Request request)
    {
        return new ContentTypeExpression(Operator.EQ, ProgramFactory.PROGRAM_CONTENT_TYPE);
    }

    @Override
    protected Expression getWildcardExpression(MetadataType type, String metadataName, String value)
    {
        String[] wildcardValues = _createWildcardValues(value);
        
        if (wildcardValues == null || wildcardValues.length == 0)
        {
            return null;
        }
        
        Expression[] locExprs = new Expression[wildcardValues.length];
        Expression[] remExprs = new Expression[wildcardValues.length];
        for (int i = 0; i < wildcardValues.length; i++)
        {
            String wildcardValue = wildcardValues[i].trim();
            
            locExprs[i] = new StringWildcardExpression(metadataName, wildcardValue, true);
            
            remExprs[i] = new StringWildcardExpression(metadataName + ODFContentType.REMOTE_SUFFIX, wildcardValue, true);
        }
        
        Expression syncExpr = new BooleanExpression(metadataName + ODFContentType.SYNC_SUFFIX, Operator.EQ, true);
        Expression locExpr = new OrExpression(locExprs);
        Expression remExpr = new OrExpression(remExprs);
        
        return new OrExpression(locExpr, new AndExpression(syncExpr, remExpr));
    }
    
    @Override
    protected Expression getExpression(MetadataType type, String metadataName, String value, Operator testOperator)
    {
        Expression syncExpr = new BooleanExpression(metadataName + ODFContentType.SYNC_SUFFIX, Operator.EQ, true);

        if (type == MetadataType.STRING)
        {
            Expression locExpr = new StringExpression(metadataName, testOperator, value);
            Expression remExpr = new StringExpression(metadataName + ODFContentType.REMOTE_SUFFIX, testOperator, value);
            
            return new OrExpression(locExpr, new AndExpression(syncExpr, remExpr));
        }
        else if (type == MetadataType.DATE)
        {
            Date dateValue = (Date) ParameterHelper.castValue(value, ParameterType.DATE);
            if (dateValue != null)
            {
                Expression locExpr = createExpression(metadataName, dateValue, testOperator);
                Expression remExpr = createExpression(metadataName + ODFContentType.REMOTE_SUFFIX, dateValue, testOperator);
                
                return new OrExpression(locExpr, new AndExpression(syncExpr, remExpr));
            }
            else
            {
                return null;
            }
        }
        else if (type == MetadataType.LONG)
        {
            Long longValue = (Long) ParameterHelper.castValue(value, ParameterType.LONG);
            if (longValue != null)
            {
                Expression locExpr = new LongExpression(metadataName, testOperator, longValue);
                Expression remExpr = new LongExpression(metadataName + ODFContentType.REMOTE_SUFFIX, testOperator, longValue);
                
                return new OrExpression(locExpr, new AndExpression(syncExpr, remExpr));
            }
            else
            {
                return null;
            }
        }
        else if (type == MetadataType.DOUBLE)
        {
            Double doubleValue = (Double) ParameterHelper.castValue(value, ParameterType.DOUBLE);
            if (doubleValue != null)
            {
                Expression locExpr = new DoubleExpression(metadataName, testOperator, doubleValue);
                Expression remExpr = new DoubleExpression(metadataName + ODFContentType.REMOTE_SUFFIX, testOperator, doubleValue);
                
                return new OrExpression(locExpr, new AndExpression(syncExpr, remExpr));
            }
            else
            {
                return null;
            }
        }
        else if (type == MetadataType.BOOLEAN)
        {
            Boolean boolValue = (Boolean) ParameterHelper.castValue(value, ParameterType.BOOLEAN);
            if (boolValue != null)
            {
                Expression locExpr = new BooleanExpression(metadataName, testOperator, boolValue);
                Expression remExpr = new BooleanExpression(metadataName + ODFContentType.REMOTE_SUFFIX, testOperator, boolValue);
                
                return new OrExpression(locExpr, new AndExpression(syncExpr, remExpr));
            }
            else
            {
                return null;
            }
        }
        
        return null;
    }
}
