/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.program.generators;

import java.io.IOException;

import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.xml.AttributesImpl;
import org.apache.cocoon.xml.XMLUtils;
import org.apache.commons.lang.StringUtils;
import org.xml.sax.SAXException;

import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.course.Course;
import org.ametys.odf.course.generators.CourseTreeGenerator;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.TraversableAmetysObject;

/**
 * Generates the program organization
 * 
 */
public class ProgramStructureGenerator extends CourseTreeGenerator
{
    @Override
    public void generate() throws IOException, SAXException, ProcessingException
    {
        Request request = ObjectModelHelper.getRequest(objectModel);
        String id = request.getParameter("id");

        assert id != null;

        contentHandler.startDocument();

        AmetysObject object = _resolver.resolveById(id);

        if (object instanceof Program)
        {
            saxContent ((Program) object, "program", true);
        }
        else if (object instanceof SubProgram)
        {
            saxContent ((SubProgram) object, "subprogram", true);
        }
        else if (object instanceof Container)
        {
            saxContent ((Container) object, "container", true);
        }
        else if (object instanceof Course)
        {
            saxContent ((Course) object, "course", true);
        }
        else if (object instanceof CourseList)
        {
            saxCourseList ((CourseList) object, true);
        }
        
        contentHandler.endDocument();
    }
    
    /**
     * SAX content information
     * @param content The synchronizable content 
     * @param tagName The tag name to use
     * @param saxChildren <code>true</code> to SAX children
     * @throws SAXException
     */
    protected void saxContent (SynchronizableContent content, String tagName, boolean saxChildren) throws SAXException
    {
        AttributesImpl attr = new AttributesImpl();
        attr.addCDATAAttribute("name", content.getName());
        attr.addCDATAAttribute("title", content.getTitle());
        attr.addCDATAAttribute("code", getCode(content));
        attr.addCDATAAttribute("id", content.getId());
        attr.addCDATAAttribute("lang", content.getLanguage());
        
        boolean hasChild = ((TraversableAmetysObject) content).getChildren().getSize() > 0;
        attr.addCDATAAttribute("hasChild", String.valueOf(hasChild));
        
        XMLUtils.startElement(contentHandler, tagName, attr);
        
        if (saxChildren && content instanceof TraversableAmetysObject)
        {
            saxChildren ((TraversableAmetysObject) content);
        }
       
        XMLUtils.endElement(contentHandler, tagName);
    }
    
    /**
     * Get code of a synchronizable content
     * @param content The content 
     * @return The CDM unique code
     */
    protected String getCode (SynchronizableContent content)
    {
        String code = null;
        if (content instanceof AbstractProgram)
        {
            code = ((AbstractProgram) content).getCdmCode();
        }
        else if (content instanceof Container)
        {
            code = ((Container) content).getCdmCode();
        }
        else if (content instanceof Course)
        {
            code = ((Course) content).getCdmCode();
        }
        
        if (StringUtils.isNotEmpty(code))
        {
            return code;
        }
        else
        {
            return "-";
        }
    }
    
    private void saxChildren (TraversableAmetysObject object) throws SAXException
    {
        for (AmetysObject childAO : object.getChildren())
        {
            if (childAO instanceof Program)
            {
                saxContent ((Program) childAO, "program", false);
            }
            else if (childAO instanceof SubProgram)
            {
                saxContent ((SubProgram) childAO, "subprogram", false);
            }
            else if (childAO instanceof Container)
            {
                saxContent ((Container) childAO, "container", false);
            }
            else if (childAO instanceof Course)
            {
                saxContent ((Course) childAO, "course", false);
            }
            else if (childAO instanceof CourseList)
            {
                saxCourseList ((CourseList) childAO, false);
            }
        }
    }
}
