/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.rights;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.commons.lang.StringUtils;

import org.ametys.cms.repository.Content;
import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.course.Course;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.RootOrgUnitProvider;
import org.ametys.odf.person.Person;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.provider.AbstractRepository;
import org.ametys.runtime.config.Config;
import org.ametys.runtime.plugins.core.right.RightContextConvertor;


/**
 * This implementation convert the contexts '/contents/[content_id]' to '/orgunits/[lang]/[orgunit_path]' for odf contents
 */
public class ODFRightsContextConvertor extends AbstractLogEnabled implements RightContextConvertor, Serviceable
{
    /** The content right context prefix */
    public static final String CONTENT_RIGHT_CONTEXT_PREFIX = "/contents";
    
    private static Pattern _contentContext = Pattern.compile("^(/[^/]+)?" + CONTENT_RIGHT_CONTEXT_PREFIX + "/(.+)$");
    
    private AmetysObjectResolver _resolver;
    private RootOrgUnitProvider _ouProvider;

    private Repository _repository;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {   
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _ouProvider = (RootOrgUnitProvider) smanager.lookup(RootOrgUnitProvider.ROLE);
        _repository = (Repository) smanager.lookup(AbstractRepository.ROLE);
    }
    
    @Override
    public Set<String> convertContext(String initialContext)
    {
        String ouLang = Config.getInstance().getValueAsString("odf.programs.lang");
        
        Set<String> contexts = new HashSet<String>();
        
        Content content = getContentFromContext (initialContext);
        if (content != null)
        {
            if (content instanceof OrgUnit)
            {
                contexts.add("/orgunits/" + ouLang + "/" + getOrgUnitContext((OrgUnit) content));
            }
            else if (content instanceof Person)
            {
                contexts.add("/contributor");
            }
            else if (content instanceof SynchronizableContent)
            {
                List<String> ids = getOrgUnits(content);
                for (String id : ids)
                {
                    if (StringUtils.isNotEmpty(id))
                    {
                        Session session = null;
                        try
                        {
                            // Force default workspace to get orgunit (we can be in archives workspace)
                            session = _repository.login("default");
                            
                            OrgUnit ou = _resolver.resolveById(id, session);
                            contexts.add("/orgunits/" + ouLang + "/" + getOrgUnitContext(ou));
                        }
                        catch (RepositoryException e)
                        {
                            getLogger().error("Unable to get rights context for unknown orgunit of id '" + id + "'", e);
                        }
                        catch (UnknownAmetysObjectException e)
                        {
                            getLogger().error("Unable to get rights context for unknown orgunit of id '" + id + "'", e);
                        }
                        finally
                        {
                            if (session != null)
                            {
                                session.logout();
                            }
                        }
                    }
                }
                
                if (ids.size() == 0)
                {
                    contexts.add("/orgunits/" + ouLang);
                }
            }
        }
    
        return contexts;
    }
    
    /**
     * Get the orgunits for the content
     * @param content the content
     * @return the orgunits in a List
     */
    protected List<String> getOrgUnits (Content content)
    {
        List<String> ouIds = new ArrayList<String>();
        
        if (content instanceof Program)
        {
            ouIds = ((Program) content).getOrgUnits();
        }
        else if (content instanceof SubProgram)
        {
            ouIds = ((SubProgram) content).getOrgUnits();
            Program program = ((SubProgram) content).getParentProgram();
            ouIds.addAll(program.getOrgUnits());
        }
        else if (content instanceof Container)
        {
            Program program = ((Container) content).getParentProgram();
            ouIds = program.getOrgUnits();
        }
        else if (content instanceof Course)
        {
            ouIds = ((Course) content).getOrgUnits();
        }
        
        // Clear empty ids
        List<String> ids = new ArrayList<String>();
        for (String id : ouIds)
        {
            if (StringUtils.isNotEmpty(id))
            {
                ids.add(id);
            }
        }
        return ids;
    }

    /**
     * Get the orgunit context for right query
     * @param ou The orgunit to get context of
     * @return The context
     */
    protected String getOrgUnitContext (OrgUnit ou)
    {
        if (_ouProvider.isRoot(ou))
        {
            return "";
        }
        
        String context = ou.getName();
        
        OrgUnit parent = ou.getParentOrgUnit();
        while (parent != null && !_ouProvider.isRoot(parent))
        {
            context = parent.getName() + "/" + context;
            parent = parent.getParentOrgUnit();
        }
        return context;
    }
    
    /**
     * Get the content concerned by the context (if possible)
     * @param context a right context
     * @return the content determined or null
     */
    protected Content getContentFromContext (String context)
    {
        if (context == null)
        {
            return null;
        }
        
        Matcher matcher = _contentContext.matcher(context);
        if (matcher.matches())
        {
            String contentPath = matcher.group(2);
            try
            {
                return _resolver.resolve("ametys:odf/ametys:contents/" + contentPath);
            }
            catch (AmetysRepositoryException e)
            {
                return null;
            }
        }
        return null;
    }

}
