/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.translation;

import org.ametys.cms.contenttype.ContentType;
import org.ametys.odf.SynchronizableContent;
import org.ametys.odf.contenttype.ODFContentType;
import org.ametys.plugins.repository.metadata.BinaryMetadata;
import org.ametys.plugins.repository.metadata.CompositeMetadata;
import org.ametys.plugins.repository.metadata.CompositeMetadata.MetadataType;
import org.ametys.plugins.repository.metadata.File;
import org.ametys.plugins.repository.metadata.Folder;
import org.ametys.plugins.repository.metadata.ModifiableBinaryMetadata;
import org.ametys.plugins.repository.metadata.ModifiableCompositeMetadata;
import org.ametys.plugins.repository.metadata.ModifiableFile;
import org.ametys.plugins.repository.metadata.ModifiableFolder;
import org.ametys.plugins.repository.metadata.ModifiableResource;
import org.ametys.plugins.repository.metadata.ModifiableRichText;
import org.ametys.plugins.repository.metadata.Resource;
import org.ametys.plugins.repository.metadata.RichText;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;

/**
 * Helper methods to fully copy a content's metadata into another.
 */
public final class ContentCopyHelper
{

    private ContentCopyHelper()
    {
        // Hide the default constructor.
    }
    
    /**
     * Copy all metadata of a content's metadata holder to another one.
     * @param originalMeta the source metadata holder.
     * @param translatedMeta the destination metadata holder.
     * @param contentType the content type.
     */
    public static void copyMetadatas(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, ContentType contentType)
    {
        originalMeta.copyTo(translatedMeta);
    }

    /**
     * Copy all synchronized metadata of an ODF content's metadata holder to another one.
     * @param originalMeta the source metadata holder.
     * @param translatedMeta the destination metadata holder.
     * @param cType the ODF content type.
     */
    public static void copySynchronizedMetadatas(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, ODFContentType cType)
    {
        for (String metadataName : originalMeta.getMetadataNames())
        {
            // Copy the remote metadata if exist
            if (metadataName.endsWith(SynchronizableContent.REMOTE_SUFFIX))
            {
                copyMetadata(originalMeta, translatedMeta, metadataName, cType);
            }
        }
    }
    
    /**
     * Copy a metadata value.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     * @param cType the ODF content type.
     */
    public static void copyMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName, ODFContentType cType)
    {
        try
        {
            MetadataType type = originalMeta.getType(metadataName);
            boolean isMultiple = originalMeta.isMultiple(metadataName);
            
            switch (type)
            {
                case STRING:
                    copyStringMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case DATE:
                    copyDateMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case LONG:
                    copyLongMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case BOOLEAN:
                    copyBooleanMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case DOUBLE:
                    copyDoubleMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case BINARY:
                    copyBinaryMetadata(originalMeta, translatedMeta, metadataName);
                    break;
                case RICHTEXT:
                    copyRichTextMetadata(originalMeta, translatedMeta, metadataName);
                    break;
                case COMPOSITE:
                    copySynchronizedMetadatas (originalMeta.getCompositeMetadata(metadataName), translatedMeta.getCompositeMetadata(metadataName, true), cType);
                    break;
                default:
                    break;
            }
        }
        catch (UnknownMetadataException e)
        {
            // Ignore.
        }
    }
    
    /**
     * Copy a simple metadata value (not usable for repeaters, composite metadata, and so on.)
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     */
    public static void copySimpleMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName)
    {
        try
        {
            MetadataType type = originalMeta.getType(metadataName);
            boolean isMultiple = originalMeta.isMultiple(metadataName);
            
            switch (type)
            {
                case STRING:
                    copyStringMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case DATE:
                    copyDateMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case LONG:
                    copyLongMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case BOOLEAN:
                    copyBooleanMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case DOUBLE:
                    copyDoubleMetadata(originalMeta, translatedMeta, metadataName, isMultiple);
                    break;
                case BINARY:
                    copyBinaryMetadata(originalMeta, translatedMeta, metadataName);
                    break;
                case RICHTEXT:
                    copyRichTextMetadata(originalMeta, translatedMeta, metadataName);
                    break;
                default:
                    break;
            }
        }
        catch (UnknownMetadataException e)
        {
            // Ignore.
        }
    }
    
    /**
     * Copy a String metadata.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     * @param isMultiple true if the metadata is multiple.
     */
    public static void copyStringMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName, boolean isMultiple)
    {
        if (isMultiple)
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getStringArray(metadataName));
        }
        else
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getString(metadataName));
        }
    }
    
    /**
     * Copy a Date metadata.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     * @param isMultiple true if the metadata is multiple.
     */
    public static void copyDateMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName, boolean isMultiple)
    {
        if (isMultiple)
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getDateArray(metadataName));
        }
        else
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getDate(metadataName));
        }
    }
    
    /**
     * Copy a Long metadata.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     * @param isMultiple true if the metadata is multiple.
     */
    public static void copyLongMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName, boolean isMultiple)
    {
        if (isMultiple)
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getLongArray(metadataName));
        }
        else
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getLong(metadataName));
        }
    }
    
    /**
     * Copy a Boolean metadata.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     * @param isMultiple true if the metadata is multiple.
     */
    public static void copyBooleanMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName, boolean isMultiple)
    {
        if (isMultiple)
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getBooleanArray(metadataName));
        }
        else
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getBoolean(metadataName));
        }
    }
    
    /**
     * Copy a Double metadata.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     * @param isMultiple true if the metadata is multiple.
     */
    public static void copyDoubleMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName, boolean isMultiple)
    {
        if (isMultiple)
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getDoubleArray(metadataName));
        }
        else
        {
            translatedMeta.setMetadata(metadataName, originalMeta.getDouble(metadataName));
        }
    }
    
    /**
     * Copy a Binary metadata.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     */
    public static void copyBinaryMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName)
    {
        BinaryMetadata originalBinary = originalMeta.getBinaryMetadata(metadataName);
        ModifiableBinaryMetadata translatedBinary = translatedMeta.getBinaryMetadata(metadataName, true);
        
        // Copy the binary properties and content.
        translatedBinary.setEncoding(originalBinary.getEncoding());
        translatedBinary.setFilename(originalBinary.getFilename());
        translatedBinary.setMimeType(originalBinary.getMimeType());
        translatedBinary.setLastModified(originalBinary.getLastModified());
        translatedBinary.setInputStream(originalBinary.getInputStream());
    }
    
    /**
     * Copy a RichText metadata.
     * @param originalMeta the source composite metadata.
     * @param translatedMeta the destination composite metadata.
     * @param metadataName the metadata name.
     */
    public static void copyRichTextMetadata(CompositeMetadata originalMeta, ModifiableCompositeMetadata translatedMeta, String metadataName)
    {
        RichText originalRichText = originalMeta.getRichText(metadataName);
        ModifiableRichText translatedRichText = translatedMeta.getRichText(metadataName, true);
        
        // Copy the rich text properties and content.
        translatedRichText.setEncoding(originalRichText.getEncoding());
        translatedRichText.setMimeType(originalRichText.getMimeType());
        translatedRichText.setLastModified(originalRichText.getLastModified());
        translatedRichText.setInputStream(originalRichText.getInputStream());
        
        // Copy the rich-text additional data folder.
        copyFolder(originalRichText.getAdditionalDataFolder(), translatedRichText.getAdditionalDataFolder());
    }
    
    /**
     * Copy a folder.
     * @param originalFolder the folder to copy.
     * @param translatedFolder the folder to copy to.
     */
    public static void copyFolder(Folder originalFolder, ModifiableFolder translatedFolder)
    {
        // First copy all the sub-folders.
        for (Folder originalSubFolder : originalFolder.getFolders())
        {
            ModifiableFolder translatedSubFolder = translatedFolder.addFolder(originalSubFolder.getName());
            copyFolder(originalSubFolder, translatedSubFolder);
        }
        
        // Then copy all the files.
        for (File file : originalFolder.getFiles())
        {
            ModifiableFile translatedFile = translatedFolder.addFile(file.getName());
            Resource originalResource = file.getResource();
            ModifiableResource translatedResource = translatedFile.getResource();
            
            translatedResource.setEncoding(originalResource.getEncoding());
            translatedResource.setMimeType(originalResource.getMimeType());
            translatedResource.setLastModified(originalResource.getLastModified());
            translatedResource.setInputStream(originalResource.getInputStream());
        }
    }
}
