/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.translation;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.plugins.repository.metadata.CompositeMetadata;
import org.ametys.plugins.repository.metadata.ModifiableCompositeMetadata;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;

/**
 * Translation helper: get and set translations for a content.
 */
public final class TranslationHelper
{

    /** The translations metadata name. */
    public static final String TRANSLATIONS_METADATA = "translations";
    
    private TranslationHelper()
    {
        // Hide the default constructor.
    }
    
    /**
     * Get the translations of a content.
     * @param content the Content.
     * @return the translations as a Map of language -> content ID.
     */
    public static Map<String, String> getTranslations(Content content)
    {
        Map<String, String> translations = new HashMap<String, String>();
        
        try
        {
            CompositeMetadata translationsMeta = content.getMetadataHolder().getCompositeMetadata(TRANSLATIONS_METADATA);
            for (String language : translationsMeta.getMetadataNames())
            {
                String contentId = translationsMeta.getString(language, "");
                if (StringUtils.isNotBlank(contentId))
                {
                    translations.put(language, contentId);
                }
            }
        }
        catch (UnknownMetadataException e)
        {
            // Ignore, just return an empty map.
        }
        
        return translations;
    }
    
    /**
     * Set the translations on a content.
     * @param content the Content.
     * @param translations the translations to set, as a Map of language -> content ID.
     */
    public static void setTranslations(ModifiableContent content, Map<String, String> translations)
    {
        ModifiableCompositeMetadata translationsMeta = content.getMetadataHolder().getCompositeMetadata(TRANSLATIONS_METADATA, true);
        
        for (String language : translationsMeta.getMetadataNames())
        {
            translationsMeta.removeMetadata(language);
        }
        
        for (String language : translations.keySet())
        {
            String contentId = translations.get(language);
            
            translationsMeta.setMetadata(language, contentId);
        }
    }
        
}
