/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.workflow;

import java.util.Date;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import org.ametys.cms.FilterNameHelper;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.cms.repository.ModifiableWorkflowAwareContent;
import org.ametys.cms.workflow.CreateContentFunction;
import org.ametys.plugins.repository.RepositoryIntegrityViolationException;
import org.ametys.plugins.repository.collection.AmetysObjectCollection;
import org.ametys.runtime.config.Config;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.WorkflowException;

/**
 * Abstract OSWorkflow function for creating a ODF content.
 */
public abstract class CreateODFContentFunction extends CreateContentFunction
{
    /** Constant for parent identifier */
    public static final String CONTENT_PARENT_ID = CreateODFContentFunction.class.getName() + "$parentId";
    
    @Override
    protected AmetysObjectCollection< ? , ModifiableWorkflowAwareContent> _getContentRoot(Map transientVars) throws WorkflowException
    {
        return _resolver.resolve("/ametys:odf/ametys:contents");
    }
    
    @Override
    public void execute(Map transientVars, Map args, PropertySet ps) throws WorkflowException
    {
        if (transientVars.get(CONTENT_LANGUAGE_KEY) == null)
        {
            // Override the content language with the odf main language
            String odfLang = Config.getInstance().getValueAsString("odf.programs.lang");
            transientVars.put(CONTENT_LANGUAGE_KEY, odfLang);
        }
        
        super.execute(transientVars, args, ps);
    }
    
    @Override
    protected ModifiableWorkflowAwareContent _createContent(String desiredContentName, AmetysObjectCollection< ? , ModifiableWorkflowAwareContent> contentsNode)
    {
        ModifiableWorkflowAwareContent content = null;
        
        String prefix = _getContentNamePrefix();
        String prefixedContentName = prefix + desiredContentName;
        String contentName = FilterNameHelper.filterName(prefixedContentName);
        
        if (contentName.substring(prefix.length()).contains(prefix))
        {
            String afterPrefix = contentName.substring(prefix.length());
            afterPrefix = StringUtils.replace(afterPrefix, prefix, prefix.substring(0, prefix.length() - 1) + "1-", -1);
            contentName = prefix + afterPrefix;
        }
        
        int errorCount = 0;
        do
        {
            if (errorCount != 0)
            {
                contentName = FilterNameHelper.filterName(prefixedContentName + " " + (errorCount + 1));
            }
            try
            {
                content = contentsNode.createChild(contentName, _getNodeType());
            }
            catch (RepositoryIntegrityViolationException e)
            {
                // Content name is already used
                errorCount++;
            }
        }
        while (content == null);
        
        return content;
    }
    
    /**
     * Get the prefix for content name
     * @return the prefix
     */
    protected abstract String _getContentNamePrefix ();
    
    /**
     * Get the content node type
     * @return the content node type
     */
    protected abstract String _getNodeType ();
    
    @Override
    protected void _populateContent(Map transientVars, ModifiableContent content) throws WorkflowException
    {
        String login = getUser(transientVars);
        if (login == null)
        {
            // FIXME Login can be null if odf creation is processed by global synchronization
            login = "Anonymous";
        }
        
        // Ajout des meta-donnees standard
        content.setCreator(login);
        content.setCreationDate(new Date());
        content.setLastContributor(login);
        content.setLastModified(new Date());
    }
}
