/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.workflow;

import java.util.Map;

import org.ametys.cms.FilterNameHelper;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.cms.repository.ModifiableWorkflowAwareContent;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.SubProgramFactory;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.ModifiableTraversableAmetysObject;
import org.ametys.plugins.repository.RepositoryIntegrityViolationException;
import org.ametys.plugins.repository.collection.AmetysObjectCollection;
import org.ametys.runtime.util.StringUtils;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.WorkflowException;

/**
 * OSWorkflow function for creating a odfContent.
 */
public class CreateSubProgramFunction extends CreateODFContentFunction
{
    /** Content name prefix for programs */
    public static final String CONTENT_NAME_PREFIX = "subprogram-";
    
    private ModifiableTraversableAmetysObject _parentContent;
    
    @Override
    protected String _getContentNamePrefix()
    {
        return CONTENT_NAME_PREFIX;
    }
    
    @Override
    protected String _getNodeType()
    {
        return SubProgramFactory.SUBPROGRAM_NODETYPE;
    }
    
    @Override
    public void execute(Map transientVars, Map args, PropertySet ps) throws WorkflowException
    {
        try
        {
            String parentId = _getNonNullVar(transientVars, CreateODFContentFunction.CONTENT_PARENT_ID, "Missing parent ID");
            _parentContent = _resolver.resolveById(parentId);
        }
        catch (AmetysRepositoryException e)
        {
            throw new WorkflowException("Unable to create the content", e);
        }
        
        super.execute(transientVars, args, ps);
    }
    
    @Override
    protected ModifiableWorkflowAwareContent _createContent(String desiredContentName, AmetysObjectCollection< ? , ModifiableWorkflowAwareContent> contentsNode)
    {
        ModifiableWorkflowAwareContent content = null;
        
        String prefixedContentName = _getContentNamePrefix() + desiredContentName;
        String contentName = FilterNameHelper.filterName(prefixedContentName);
        int errorCount = 0;
        do
        {
            if (errorCount != 0)
            {
                contentName = FilterNameHelper.filterName(prefixedContentName + " " + (errorCount + 1));
            }
            try
            {
                content = _parentContent.createChild(contentName, _getNodeType());
            }
            catch (RepositoryIntegrityViolationException e)
            {
                // Content name is already used
                errorCount++;
            }
        }
        while (content == null);
        
        return content;
    }
    
    @Override
    protected void _populateAdditionalData(Map transientVars, ModifiableContent content) throws WorkflowException
    {
        // Set a unique CDM code if not exist
        String cdmCode = content.getMetadataHolder().getString(AbstractProgram.CDM_CODE, null);
        if (org.apache.commons.lang.StringUtils.isEmpty(cdmCode))
        {
            content.getMetadataHolder().setMetadata(AbstractProgram.CDM_CODE, StringUtils.generateKey().toUpperCase());
        }
    }
}
