/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.workflow;

import java.util.List;
import java.util.Map;

import org.ametys.cms.observation.Event;
import org.ametys.cms.observation.ObservationConstants;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramPart;
import org.ametys.plugins.repository.AmetysObject;
import org.ametys.plugins.repository.AmetysObjectIterable;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.WorkflowException;

/**
 * OSWorkflow function for validating a content globally (ie with all it children recursively).
 */
public class GlobalValidateProgramFunction extends ValidateODFContentFunction
{
    @Override
    public void execute(Map transientVars, Map args, PropertySet ps) throws WorkflowException
    {
        // Validate program
        super.execute(transientVars, args, ps);
        
        // Validate recursively the whole program structure
        transientVars.put(VALIDATE_RECURSIVELY, true);
        
        Program program = (Program) getContent(transientVars);
        // Validate program's structure
        _validateProgramPart (program, transientVars);
        
        // Validate OrgUnits
        _validateContents(program.getOrgUnits(), transientVars);
        
        // Validate contacts
        _validateContents(program.getContacts(), transientVars);
        _validateContents(program.getPersonsInCharge(), transientVars);
        
        // Notify observers that the content has been validated
        _observerManager.notify(new Event(getUser(transientVars), ObservationConstants.CONTENT_VALIDATED, program));
    }
    
    private void _validateProgramPart(ProgramPart programPart, Map transientVars) throws WorkflowException
    {
        AmetysObjectIterable<AmetysObject> children = programPart.getChildren();
        for (AmetysObject childObject : children)
        {
            if (childObject instanceof ProgramPart)
            {
                _validateProgramPart((ProgramPart) childObject, transientVars);
            }
        }
        
        AmetysObjectIterable<CourseList> courseLists = programPart.getCourseLists();
        for (CourseList courseList : courseLists)
        {
            _validateCourseList(courseList, transientVars);
        }
    }
    
    
    private void _validateCourseList(CourseList courseList, Map transientVars) throws WorkflowException
    {
        List<String> courseIds = courseList.getCourses();
        for (String id : courseIds)
        {
            Course course = _resolver.resolveById(id);
            // Validate course
            _doValidateWorkflowAction(course, transientVars);
            
            AmetysObjectIterable<CourseList> courseLists = course.getCourseLists();
            for (CourseList list : courseLists)
            {
                _validateCourseList(list, transientVars);
            }
        }
    }
}
