/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.workflow;

import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.activity.Initializable;
import org.apache.cocoon.environment.Request;
import org.apache.commons.lang.ArrayUtils;

import org.ametys.cms.contenttype.ContentType;
import org.ametys.cms.contenttype.ContentTypeExtensionPoint;
import org.ametys.cms.contenttype.MetadataDefinition;
import org.ametys.cms.contenttype.MetadataType;
import org.ametys.cms.contenttype.RepeaterDefinition;
import org.ametys.cms.repository.Content;
import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.odf.SynchronizableContent;
import org.ametys.plugins.repository.metadata.UnknownMetadataException;
import org.ametys.plugins.workflow.AbstractWorkflowComponent;
import org.ametys.runtime.util.parameter.Errors;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.Condition;
import com.opensymphony.workflow.WorkflowException;

/**
 * OSWorkflow condition to check metadata of a synchronizable content are valid.<br>
 * For now, API allows to check only metadata of first level (not composite metadata nor repeaters) 
 */
public class ValidateMetadataCondition extends AbstractWorkflowComponent implements Condition, Initializable
{
    private ContentTypeExtensionPoint _cTypeEP;
    
    @Override
    public void initialize() throws Exception
    {
        _cTypeEP = (ContentTypeExtensionPoint) _manager.lookup(ContentTypeExtensionPoint.ROLE);
    }
    
    @Override
    public boolean passesCondition(Map transientVars, Map args, PropertySet ps) throws WorkflowException
    {
        Content content = getContent(transientVars);
        
        ContentType contentType = _cTypeEP.getExtension(content.getType());
        
        if (content instanceof SynchronizableContent)
        {
            Set<String> metadataNames = contentType.getMetadataNames();
            for (String metadataName : metadataNames)
            {
                MetadataDefinition metadataDef = contentType.getMetadataDefinition(metadataName);
                if (!validateMetadata((SynchronizableContent) content, metadataDef, metadataName, new Errors()))
                {
                    return false;
                }
            }
        }
        
        return true;
    }
    
    /**
     * Validate a metadata of a {@link SynchronizableContent}
     * @param content The synchronizable content
     * @param metadataDef The metadata definition
     * @param metadataName The metadata name
     * @param errors The errors
     * @return <code>true</code> if the metadata is valid
     */
    protected boolean validateMetadata(SynchronizableContent content, MetadataDefinition metadataDef, String metadataName, Errors errors)
    {
        MetadataType type = metadataDef.getType();
        
        if (metadataDef instanceof RepeaterDefinition || type.equals(MetadataType.COMPOSITE))
        {
            // FIXME API does not allow to check repeater or composite metadata for ODF content ... suppose it is valid
            return true;
        }
        else
        {
            if (metadataDef.getValidator() == null)
            {
                // no validator, the metadata is valid
                return true;
            }
            
            // Get the synchronized or local value
            Object value = getValue(content, metadataDef, metadataName);
            metadataDef.getValidator().validate(value, errors);
            // The metadata is valid is there is no errors
            return !errors.hasErrors();
        }
    }
    
    /**
     * Get the value, synchronized or not, of a metadata
     * @param content The synchronizable content
     * @param metadataDef The metadata definition
     * @param metadataName The metadata name
     * @return The value of metadata
     */
    protected Object getValue(SynchronizableContent content, MetadataDefinition metadataDef, String metadataName)
    {
        MetadataType type = metadataDef.getType();
        
        try
        {
            switch (type)
            {
                case STRING:
                case USER:
                case REFERENCE:
                    return content.getStringArray(metadataName);
                case DOUBLE:
                    return ArrayUtils.toObject(content.getDoubleArray(metadataName));
                case LONG:
                    return ArrayUtils.toObject(content.getLongArray(metadataName));
                case BOOLEAN:
                    return content.getBoolean(metadataName);
                case DATE:
                case DATETIME:
                    return content.getDate(metadataName);
                case BINARY:
                    return content.getBinaryMetadata(metadataName);
                case FILE:
                    return content.getMetadataHolder().getType(metadataName) == org.ametys.plugins.repository.metadata.CompositeMetadata.MetadataType.BINARY ? content.getBinaryMetadata(metadataName) :  content.getString(metadataName);
                case RICH_TEXT:
                    return content.getRichText(metadataName);
                default:
                    return null;
            }
        }
        catch (UnknownMetadataException e)
        {
            return null;
        }
    }
    
    /**
     * Retrieve the content associated with the workflow.
     * @param transientVars the parameters from the call.
     * @return the content.
     * @throws WorkflowException if the content is not found.
     */
    protected Content getContent(Map transientVars) throws WorkflowException
    {
        Content content = (Content) transientVars.get(AbstractContentWorkflowComponent.CONTENT_KEY);
        
        if (content == null)
        {
            Request request = getRequest();
            if (request == null)
            {
                throw new WorkflowException("The request is null, unable to retrieve content");
            }
            
            // FIXME use CMSContextHandler for retrieving the current matched content
            content = (Content) request.getAttribute(AbstractContentWorkflowComponent.CONTENT_KEY);
            
            if (content == null)
            {
                throw new WorkflowException("Unable to retrieve content");
            }
        }
        
        // Found in transient variables map
        return content;
    }
}
