/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.odf.workflow;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jcr.RepositoryException;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.cocoon.environment.Redirector;
import org.apache.commons.lang.StringUtils;

import org.ametys.cms.observation.Event;
import org.ametys.cms.observation.ObservationConstants;
import org.ametys.cms.observation.ObservationManager;
import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.ModifiableWorkflowAwareContent;
import org.ametys.cms.repository.WorkflowAwareContent;
import org.ametys.cms.workflow.AbstractContentFunction;
import org.ametys.cms.workflow.AbstractContentWorkflowComponent;
import org.ametys.odf.course.Course;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.AmetysRepositoryException;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.plugins.repository.version.VersionableAmetysObject;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.workflow.InvalidActionException;
import com.opensymphony.workflow.WorkflowException;
import com.opensymphony.workflow.spi.Step;

/**
 * OSWorkflow function for validating a ODF content.
 */
public class ValidateODFContentFunction extends AbstractContentFunction
{
    /** Label for the validated version of contents. */
    public static final String VALID_LABEL = "Live";
    
    /** Label for the validated version of contents. */
    public static final String VALIDATE_RECURSIVELY = ValidateODFContentFunction.class.getName() + "$recursively";
    
    /** The action id */
    public static final int VALIDATE_ACTION_ID = 4;
    /** The validate step id */
    public static final int VALIDATED_STEP_ID = 3;
    
    /** The Ametys object resolver */
    protected AmetysObjectResolver _resolver;
    /** Observer manager. */
    protected ObservationManager _observerManager;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _resolver = (AmetysObjectResolver) smanager.lookup(AmetysObjectResolver.ROLE);
        _observerManager = (ObservationManager) smanager.lookup(ObservationManager.ROLE);
    }
    
    @Override
    public void execute(Map transientVars, Map args, PropertySet ps) throws WorkflowException
    {
        _logger.info("Performing content validation");
        WorkflowAwareContent content = getContent(transientVars);
        
        if (!(content instanceof ModifiableWorkflowAwareContent))
        {
            throw new IllegalArgumentException("The provided content " + content.getId() + " is not a ModifiableWorkflowAwareContent.");
        }
        
        ModifiableWorkflowAwareContent modifiableContent = (ModifiableWorkflowAwareContent) content;
        
        try
        {
            _validateContent(modifiableContent);
            
            if (transientVars.get(VALIDATE_RECURSIVELY) != null)
            {
                // Validate recursively the referenced contents
                _validateRecursively(content, transientVars);
            }
            
            // Set the current step ID.
            _setCurrentStepId(modifiableContent);
            // Create a new version.
            _createVersion(modifiableContent);
            // Add the valid label on the newly created version.
            _addLabel(modifiableContent, VALID_LABEL);
        }
        catch (RepositoryException e)
        {
            throw new WorkflowException("Unable to link the workflow to the content", e);
        }
        catch (AmetysRepositoryException e)
        {
            throw new WorkflowException("Unable to validate the content", e);
        }
        
        // Notify observers that the content has been validated
        _observerManager.notify(new Event(getUser(transientVars), ObservationConstants.CONTENT_VALIDATED, content));
    }

    /**
     * Validates the content.
     * @param content the content.
     * @throws WorkflowException if an error occurs.
     * @throws RepositoryException if an error occurs.
     */
    protected void _validateContent(ModifiableWorkflowAwareContent content) throws WorkflowException, RepositoryException
    {
        if (!(content instanceof VersionableAmetysObject))
        {
            throw new WorkflowException("Invalid content implementation: " + content);
        }
        
        Date validationDate = new Date();
        boolean isValid = Arrays.asList(((VersionableAmetysObject) content).getAllLabels()).contains(VALID_LABEL);
        if (!isValid)
        {
            content.setLastMajorValidationDate(validationDate);
        }
        content.setLastValidationDate(validationDate);
        // Remove the proposal date.
        content.setProposalDate(null);
        
        content.saveChanges();
    }
    
    /**
     * Validate the referenced contents recursively
     * @param content The validated content
     * @param transientVars The parameters from the call.
     * @throws WorkflowException
     */
    protected void _validateRecursively (WorkflowAwareContent content, Map transientVars) throws WorkflowException
    {
        if (content instanceof Course)
        {
            // Validate orgunits recursively
            _validateContents(((Course) content).getOrgUnits(), transientVars);
            // Validate contacts recursively
            _validateContents(((Course) content).getContacts(), transientVars);
            // Validate person in charge recursively
            _validateContents(((Course) content).getPersonsInCharge(), transientVars);
        }
        else if (content instanceof OrgUnit)
        {
            // Validate contacts recursively
            _validateContents(((OrgUnit) content).getContacts(), transientVars);
        }
    }
    
    /**
     * Validate the list of contents
     * @param contentIds The id of contents to validate
     * @param transientVars The parameters from the call.
     * @throws WorkflowException
     */
    protected void _validateContents (List<String> contentIds, Map transientVars) throws WorkflowException
    {
        for (String id : contentIds)
        {
            try
            {
                if (StringUtils.isNotEmpty(id))
                {
                    WorkflowAwareContent content = _resolver.resolveById(id);
                    _doValidateWorkflowAction (content, transientVars);
                }
            }
            catch (UnknownAmetysObjectException e)
            {
                // Nothing
            }
        }
    }
    
    /**
     * Validate a content if not already validated
     * @param content The content to validate
     * @param transientVars The parameters from the call.
     * @throws WorkflowException
     */
    protected void _doValidateWorkflowAction (WorkflowAwareContent content, Map transientVars) throws WorkflowException
    {
        long workflowId = content.getWorkflowId();
        if (!_isValidated (workflowId))
        {
            try
            {
                _workflow.doAction(workflowId, VALIDATE_ACTION_ID, _getInputs(content, transientVars));
            }
            catch (InvalidActionException e)
            {
                if (!getMapToRender(transientVars).containsKey("invalidated-contents"))
                {
                    getMapToRender(transientVars).put("invalidated-contents", new HashSet<String>());
                }
                
                Set<String> invalidatedContents = (Set<String>) getMapToRender(transientVars).get("invalidated-contents");
                invalidatedContents.add(content.getTitle() + " (" + content.getName() + ")");
                _logger.warn("Unable to validate content " + content.getTitle() + " (" + content.getName() + ") : mandatory metadata are probably missing", e);
            }
            
            // Update current step
            _setCurrentStepId(content);
        }
    }
    
    /**
     * Determines if a content is already in validated step
     * @param workflowId The content workflow id
     * @return true if the content is already in validated step
     */
    protected boolean _isValidated (long workflowId)
    {
        List<Step> steps = _workflow.getCurrentSteps(workflowId);
        return VALIDATED_STEP_ID == steps.iterator().next().getStepId();
    }
    
    /**
     * Get inputs
     * @param content The content to validate
     * @param transientVars the parameters from the call.
     * @return The inputs
     */
    protected Map<String, Object> _getInputs (Content content, Map transientVars)
    {
        Map<String, Object> inputs = new HashMap<String, Object>(1); 
        
        // Provide the redirector
        Redirector redirector = (Redirector) transientVars.get(Redirector.class.getName());
        inputs.put(Redirector.class.getName(), redirector);
        
        // Provide the content key 
        inputs.put(AbstractContentWorkflowComponent.CONTENT_KEY, content); 
        
        // Validate recursively
        if (transientVars.get(VALIDATE_RECURSIVELY) != null)
        {
            inputs.put(VALIDATE_RECURSIVELY, true); 
        }
        
        // Provide a map for providing data to the generator 
        Map<String, Object> result = new HashMap<String, Object>(); 
        inputs.put(AbstractContentWorkflowComponent.RESULT_MAP_KEY, result); 
        
        return inputs;
    }
    
}
